/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let crypto = require('crypto')

let log = require('../../../log').m('Access')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

//const Session = require('./session')
const PasswordField = require('./password')
const UserInGroupAccessTest = require('./useringroup.test')
const UserIsGuestAccessTest = require('./userisguest.test')
const UserCanAccessTest = require('./usercan.test')

class Access extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._secrets = {
      salt: 'aaa'
    }
    this.M('field').registerFieldType(PasswordField)
    this.M('logic').registerLogicTest(UserInGroupAccessTest)
    this.M('logic').registerLogicTest(UserIsGuestAccessTest)
    this.M('logic').registerLogicTest(UserCanAccessTest)
  }

  _update01to011()
  {
    return Rx.of(this.M('bundle').bundle('session')).pipe(RxO.mergeMap(b=>{
      if (b.hasField('iface'))
        return Rx.of(b)
      log.info('updating session bundle with iface field')
      b.addField({ name:g18n.t('Interface'), code: 'iface', type: 'string', flags: ['system'] })
      return b.save()
    }))
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1'&&to=='0.1.1')
          return this._update01to011()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }


  /*
  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      //this._secrets = this._core._confs.app.secrets
      //this._core.module('bundle').bundle('session').setNodeClass(Session)
      //this._sess = this._core.module('bundle').collection('session')
      //this._users = this._core.module('bundle').collection('user')
      log.log('inited')
      obs.next()
      obs.complete()
    })
  }
  */

  installIface(iface)
  {
    return Rx.Observable.create(obs=>{
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      //this._secrets = this._core._confs.install.secrets

      let $t = []
 
      //session bundle

      let b = this._core.module('bundle').bundle('session',true)
      b.name(g18n.t('Session'))
      b.desc(g18n.t('System user sessions'))
      b.flags(['system','locked'])
      b.addField({ name:g18n.t('User'), code: 'user', type: 'relation', flags: ['system','required'] })
      b.addField({ name:g18n.t('Login code'), code: 'code', type: 'string', flags: ['system'] })
      b.addField({ name:g18n.t('Interface'), code: 'iface', type: 'string', flags: ['system'] })
      b.addField({ name:g18n.t('Session token'), code: 'token', type: 'string', flags: ['system'] })
      b.addField({ name:g18n.t('Open'), code: 'open', type: 'boolean', flags:['system'], def:false })
      b.addField({ name:g18n.t('Closed'), code: 'closed', type: 'boolean', flags:['system'], def:false })
      b.addField({ name:g18n.t('Created'), code: 'created', type: 'datetime', flags:['system'], generator: 'now' })

      log.log('installing bundle',b.code())
      $t.push(b.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        //this._core.module('bundle').collection('session').setNodeClass(Session)
        log.success('bundle installed')
        obs.next()
        obs.complete()
      })

    })
  }

  salt(msg)
  {
    let hash = crypto.createHash('sha256')
    hash.update(msg)
    let hash1 = hash.digest('hex')+this._secrets.salt
    let hash2 = crypto.createHash('sha256')
    hash2.update(hash1)
    return hash2.digest('hex')
  }

  saltVerify(msg,salt)
  {
    let hash = this.salt(msg)
    return (hash === salt)
  }

  closeDashboardSessions(user)
  {
    return Rx.Observable.create(obs=>{
      if (!user.can('auth.dashboard'))
        return this.obsEnd(obs,null)
      this.C('session').removeNodes({user:user.id()}).subscribe(res=>{
        //console.log(res)
        this.obsEnd(obs,res)
      },err=>{
        log.err('session deleting FAILED')
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }


  generateDashboardSession(user)
  {
    return Rx.Observable.create(obs=>{
      if (!user.can('auth.dashboard'))
        return this.obsEnd(obs,null)
      let code = crypto.randomBytes(8).toString('hex')
      let token = crypto.randomBytes(32).toString('hex')
      this.C('session').initNode().subscribe($sess=>{
        $sess.f('user',user)
        $sess.f('code',code)
        $sess.f('token',token)
        $sess.f('iface','dashboard')
        $sess.save().subscribe(s=>{
          log.info('new session generated',code)
          this.obsEnd(obs,code)
        },err=>{
          log.err('saving new session FAILS')
          console.log(err)
          this.obsErr(obs,err)
        })
      },err=>{
        log.err('session node initialization FAILED')
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }

  dashboardCodeUser(code)
  {
    return Rx.Observable.create(obs=>{
      this.C('session').findNode({code:code,iface:'dashboard'}).subscribe(sess=>{
        //console.log(['search sesss by code',code,sess])
        if (!sess)
          return this.obsEnd(obs,null)
        if (sess.f('open') || sess.f('closed'))
        {
          obs.error({CODE:'SESSION_FAILED'})
          return this.obsEnd(obs,null)
        }
        this.C('user').findNode({_id:sess.f('user')}).subscribe(u=>{
          if (!u)
            return this.obsEnd(obs,null)
            sess.f('open',true)
            sess.save().subscribe(s=>{
              this.obsEnd(obs,{
                user: u,
                sess: s
              })
            },err=>{
              log.err('search session user failed',sess.tgid)
              console.log(err)
              this.obsEnd(obs,null)
            })
        },err=>{
          log.err('search session user failed',sess.fp('user').toObject())
          console.log(err)
          this.obsEnd(obs,null)
        })
      },err=>{
        log.err('search session with code failed',code)
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }
 
  dashboardTokenUser(token)
  {
    if (typeof token != 'string')
      return Rx.of(null)

    return this.C('session').findNode({token:token,iface:'dashboard'}).pipe(
      RxO.mergeMap(sess=>{
        if(!sess||!sess.f('open')||sess.f('closed'))
          return Rx.of(null)
        return this.C('user').findNode({_id:sess.f('user')}).pipe(
          RxO.mergeMap(u=>Rx.iif(_=>!u,Rx.of(null),u.populateAccess()))
        )
      })
    )
  }

  userAccess(user)
  {
    return Rx.Observable.create(obs=>{
      //log.test('populating user access',user.titles())
      //this.C('usersgroup').findNodes({$or:[{users:user.id()},{code:'guests'}]}).subscribe(groups=>{
      let ugm = this.C('usersgroup')._model
      let rlm = this.C('role')._model
      let q = ugm.find({$or:[{users:user.id()},{code:'guests'}]}).populate({
        path: 'roles',
        model: rlm
      })
      Rx.from(q.exec()).subscribe(groups=>{
        //console.log(['ACCESS user access',user.titles(),groups])
        //let ids = []
        let roles = []
        let grps = []
        groups.forEach(g=>{
          grps.push(g.code[0])
          //console.log(g.code[0])
          if (g&&Array.isArray(g.roles))
            g.roles.forEach(r=>{
              roles.push(this.C('role').loadNode(r))
            })
          //let rls = g.field('roles').all()
          //console.log(['ACCESS group roles',rls])
          //rls.forEach(r=>ids.push(r))
        })

        //this.C('role').findNodes({_id:{$in:ids}}).subscribe(roles=>{
          //log.test('populated',user.titles())
          obs.next({roles:roles,gcodes:grps})
          obs.complete()
        //})
      },err=>{
        log.err('search groups fail')
        console.log(err)
        obs.error(err)
      })
    })
  }


  updateAdminsAccess()
  {
    return Rx.Observable.create(obs=>{
      log.info('updating admins access')
      this.C('usersgroup').findNode({code:'admins'}).subscribe($grp=>{
        this.C('role').findNodes({}).subscribe($list=>{
          $grp.field('roles').clear()
          $list.forEach(role=>$grp.field('roles').push(role))
          $grp.save().subscribe(g=>{
            this.obsEnd(obs)
          },err=>this.obsErr(obs,err))
        },err=>this.obsErr(obs,err))
      },err=>this.obsErr(obs,err))
    })
  }

  guestUsers(queryOpts)
  {
    return Rx.Observable.create(obs=>{
      this.C('usersgroup').findNodes({}).subscribe($groups=>{
        let nonguests = []
        $groups.forEach($g=>{
          $g.field('users').all().forEach(uid=>nonguests.push(uid))
        })
        this.C('user').findNodes({_id:{$nin:nonguests}},queryOpts).subscribe($guests=>{
          this.obsEnd(obs,$guests)
        },err=>this.obsErr(obs,err))
      },err=>this.obsErr(obs,err))
    })
  }

  createIfaceSession(iface,user,codeSize=8)
  {
    return Rx.Observable.create(obs=>{
      if (iface=='dashboard')
        return this.obsEnd(obs,null)
      let code = crypto.randomBytes(codeSize).toString('hex')
      let token = crypto.randomBytes(32).toString('hex')
      this.C('session').initNode().subscribe($sess=>{
        $sess.f('user',user)
        $sess.f('code',code)
        $sess.f('token',token)
        $sess.f('iface',iface)
        $sess.save().subscribe(s=>{
          log.info('new session generated',code)
          this.obsEnd(obs,code)
        },err=>{
          log.err('saving new session FAILS')
          console.log(err)
          this.obsErr(obs,err)
        })
      },err=>{
        log.err('session node initialization FAILED')
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }

  openIfaceSession(iface,code)
  {
    return Rx.Observable.create(obs=>{
      this.C('session').findNode({code:code,iface:iface}).subscribe(sess=>{
        //console.log(['search sesss by code',iface,code,sess])
        if (!sess)
          return this.obsEnd(obs,null)
        if (sess.f('open') || sess.f('closed'))
        {
          obs.error({CODE:'SESSION_FAILED'})
          return this.obsEnd(obs,null)
        }
        this.C('user').findNode({_id:sess.f('user')}).subscribe(u=>{
          if (!u)
            return this.obsEnd(obs,null)
            sess.f('open',true)
            sess.save().subscribe(s=>{
              this.obsEnd(obs,{
                user: u,
                sess: s
              })
            },err=>{
              log.err('search session user failed',sess.tgid)
              console.log(err)
              this.obsEnd(obs,null)
            })
        },err=>{
          log.err('search session user failed',sess.fp('user').toObject())
          console.log(err)
          this.obsEnd(obs,null)
        })
      },err=>{
        log.err('search session with code failed',code)
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }

  findIfaceSession(iface,token=null)
  {
    if (typeof token != 'string')
      return Rx.of(null)

    return this.C('session').findNode({token:token,iface:iface}).pipe(
      RxO.mergeMap(sess=>Rx.iif(_=>!sess||!sess.f('open')||sess.f('closed'),Rx.of(null),this.C('user').findNode({_id:sess.f('user')}).pipe(
        RxO.mergeMap(u=>Rx.iif(_=>!u,Rx.of(null),u.populateAccess()))
      )))
    )
  }

}

module.exports = Access
