/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('UserCanAccessTest')
let g18n = require('../../../g18n')

const AbstractTest = require('../../logic/abstract/test')
const User = require('../user/user')

class UserCanAccessTest extends AbstractTest
{

  static info()
  {
    return {
      code: 'usercan',
      invert: true,
      name: g18n.t('User can (has role)'),
      iname: g18n.t('User can not (has not role)'),
      types: ['relation'],
      xtypes: ['relation:user'],
      ops: [{
        type: 'string',
        name: g18n.t('full role code'),
        manual: true
      }],
      scope: 'context'
    }
  }

  test(ctx,dataTarget)
  {
    return Rx.Observable.create(obs=>{
      let fin = false
      //let inf = this.constructor.info()
      dataTarget.data().pipe(
        RxO.mergeMap(u=>{
          if (!(u instanceof User))
            return Rx.of(false)
          return u.populateAccess().pipe(RxO.map(uu=>u.can(this.op(0))))
        }),
        RxO.map(tr=>{
          if (!!this._conf.invert)
            tr = !tr
          return tr
        })
      ).subscribe(tr=>{
        dataTarget.accumulate(tr,ar=>{
          fin = ar
        })
      },err=>{
        log.err('abstract test err')
        console.log(err)
        this.obsEnd(obs,!!this._conf.invert)
      },()=>{
        //log.warn('abstract test result',[inf.code,this.op(),fin])
        //let ret = !!this._conf.invert?!fin:fin
        this.obsEnd(obs,fin)
      })
    })
  }


}

module.exports = UserCanAccessTest

