/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('AuthModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')

const User = require('./user/user')
const Role = require('./role/role')
const UsersGroup = require('./usersgroup/usersgroup')

class AuthModule extends CoreModule
{

  userByIfaceSid(iface,sid)
  {
    return Rx.Observable.create(obs=>{
      let ifaceAuth = iface.field('auth').get()
      let ifaceConf = iface.field('conf').get()
      this._genGuestUser().subscribe(guest=>{
        obs.next(guest)
        obs.complete()
      })
      /*
      let isGuest = this._genGuestUser()
      let q = {}
      q['ifaces.'+iface.field('code').get()] = sid
      this._sc.findNode(q).subscribe(sess=>{
        let next = Rx.of(null)
        if (!sess)
        {
          //new session required
          next = this._createIfaceSession(iface,sid)
        } else {
          //check session and load session user
          next = this._loadIfaceSession(iface,sid)
        }
        next.subscribe(u=>{
          console.log(sid,ifaceConf,ifaceAuth)
          obs.next(u)
          obs.complete()
        },e=>{
          log.err('failed open session')
          console.log(e)
          obs.error({code:'SESSION_OPEN_FAILED'})
        })
      },e=>{
        log.err('failed search session')
        console.log(e)
        obs.error({code:'SESSION_SEARCH_FAILED'})
      })
      /**/
    })
  }

  _genGuestUser()
  {
    return Rx.Observable.create(obs=>{
      this.C('user').initNode().subscribe(guest=>{
        guest.isGuest = true
        obs.next(guest)
        obs.complete()
      })
    })
  }

  _createIfaceSession(iface,sid)
  {
    return Rx.Observable.create(obs=>{
      obs.next(null)
      obs.complete()
    })
  }

  _loadIfaceSession(iface,sid)
  {
    return Rx.Observable.create(obs=>{
      obs.next(null)
      obs.complete()
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{
      let rtProfile = {
        url: '/api/profile',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            if (!user)
              return this.obsEnd(obs,null)
            let ret = {
              uid: user.id(),
              tgid: user.f('tgid'),
              name: user.f('firstName'),
              username: user.f('username'),
              locale: user.f('locale'),
              modules: [],
              acl: []
            }
            ret.acl = user._roles.map(r=>r.role())
            this.obsEnd(obs,ret)
          })
        }
      }

      let rtLogin = {
        url: '/api/login/:code',
        access: null,
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            this.M('auth/access').dashboardCodeUser(p.code).subscribe(res=>{
              if (!res)
                return this.obsEnd(obs,{success:false})
              ctx.sfield('token',res.sess.f('token'))
              this.obsEnd(obs,{success:true})
            },err=>{
              log.err('login with code error!')
              console.log(err)
              this.obsEnd(obs,{success:false})
            })
          })
        }
      }

      let rtLogout = {
        url: '/logout',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return this.M('auth/access').closeDashboardSessions(ctx.user()).pipe(
            RxOps.map(res=>{
              ctx.sdrop('token')
              return {success:true}
            })
          )
        }
      }


      obs.next([rtProfile,rtLogin,rtLogout])
      obs.complete()
    })
  }


  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing auth data')

      let $t = []

      log.info('installing roles')
      $t.push(this.M('auth/role').roleSave('global','root','Absolute administrator'))
      $t.push(this.M('auth/role').roleSave('auth','root','Users administrator'))
      $t.push(this.M('auth/role').roleSave('auth','dashboard','Allows login to dashboard'))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing roles error')
        this.obsErr(obs,err)
      },()=>{
        this.C('role').findNodes().subscribe(roles=>{
          let $tt = []
          log.info('installing users groups')

          let grpAdmins = this.C('usersgroup').newNode()
          grpAdmins.f('code','admins')
          grpAdmins.f('name','Administrators')
          grpAdmins.f('description','System administrators')
          grpAdmins.f('isSystem',true)
          grpAdmins.field('roles').all(roles)
          $tt.push(grpAdmins.save())

          let grpGuests = this.C('usersgroup').newNode()
          grpGuests.f('code','guests')
          grpGuests.f('name','Guests')
          grpGuests.f('isSystem',true)
          grpGuests.f('description','System guests')
          $tt.push(grpGuests.save())

          $tt.push(this.M('auth/access').updateAdminsAccess())

          Rx.concat(...$tt).subscribe(()=>{},err=>{
            this.obsErr(obs,err)
          },()=>{
            this.obsEnd(obs,'auth installed')
          })
        },err=>{
          this.obsErr(obs,err)
        })
      })
    })
  }



}

module.exports = AuthModule
