/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('RolesManager')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')
const Role = require('./role')

class RolesManager extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._roles = {}
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.C('role').setNodeClass(Role)
      this.obsEnd(obs)
    })
  }


  moduleInstall()
  {
    return Rx.Observable.create(obs=>{

      let $t = []
      //role bundle
      let module = { name:g18n.t('Module'), code: 'module', type: 'string',flags: ['system','required','logic','template','title'] }
      let code = { name:g18n.t('Code'), code: 'code', type: 'string', flags: ['system','required','logic','template','title'] }
      let desc = { name:g18n.t('Description'), code: 'description', type: 'string', flags: ['system','logic','template']}

      let bundle = this._core.module('bundle').bundle('role',true)
      bundle.name(g18n.t('Role'))
      bundle.desc(g18n.t('System roles'))
      bundle.addFlag('system')
      bundle.addFlag('locked')
      bundle.addField(module)
      bundle.addField(code)
      bundle.addField(desc)

      log.log('installing bundle',bundle.code())
      $t.push(bundle.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        this._core.module('bundle').collection('role').setNodeClass(Role)
        log.success('bundle installed')
        obs.next()
        obs.complete()
      })
    })
  }

  roleExists(role,code=null)
  {
    if (code) role += '.'+code
    if (this._roles[role]) return true
    return false
  }

  roleSave(mdl,code,desc)
  {
    return Rx.Observable.create(obs=>{
      log.log('saving role',[mdl,code])
      this.C('role').findNode({$and:[{
        module: mdl
      },{
        code: code
      }]}).subscribe(role=>{
        if (!role)
          role = this.C('role').newNode()
        role.field('module').set(mdl)
        role.field('code').set(code)
        if (typeof desc!='undefined')
          role.field('description').set(desc)
        role.save().subscribe(()=>{
          this._roles[role.role()] = role
          log.success('role saved',role.role())
          obs.next()
          obs.complete()
        },obs.error)
      },obs.error)
    })
  }

  role(mdl,code=null)
  {
    if (code) mdl += '.'+code
    if (!this.roleExists(mdl)) return null
    return this._roles[mdl]
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{
      let rtList = {
        url: '/api/permissions/list',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            this.C('role').findNodes({}).subscribe($list=>{
              let ret = $list.map(o=>o.toObject())
              this.obsEnd(obs,ret)
            },err=>{
              log.err('fail loading roles')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }

      obs.next([rtList])
      obs.complete()
    })
  }



}

module.exports = RolesManager
