/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('User')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const Node = require('../../node/node')
const User = require('./user')
const Context = require('../../context/context')

const CurrentUserGenerator = require('./currentUserGenerator')

class UsersModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this.M('field').registerGenerator(CurrentUserGenerator)
  }

  userFieldUsers(data,ctx,getUsers=false)
  {
    return Rx.Observable.create(obs=>{
      let ret = []
      if (!data)
        return this.obsEnd(obs,ret)
      //let pop = getUsers?0:1
      if (data.fn=='user')
      {
        this.C('user').findNodes({tgid:data.utgid},{populate:1}).subscribe(list=>{
          if (getUsers)
            return this.obsEnd(obs,list)
          let chats = list.map(x=>x&&x.fp('privateChat')?x.fp('privateChat'):null).filter(x=>!!x)
          this.obsEnd(obs,chats)
        },err=>{
          log.err('search by tgid error')
          console.log(err)
          this.obsEnd(obs,ret)
        })
        return
      }
      if (data.fn=='field')
      {
        //let q = {}
        //q[data.field] = data.arg
        let vctx = new Context(this,'view',true)
        let coll = this.C('user')
        let nd = coll.newNode()
        vctx.addNode(nd,'user')
        //console.log(data)
        this.M('logic').queryFor('user','user',data.arg,ctx,vctx).subscribe(q=>{
          //console.log(['query',q])
          //this.obsEnd(obs,ret)

          this.C('user').findNodes(q,{populate:1}).subscribe(list=>{
            if (getUsers)
              return this.obsEnd(obs,list)
            let chats = list.map(x=>x&&x.fp('privateChat')?x.fp('privateChat'):null).filter(x=>!!x)
            this.obsEnd(obs,chats)
          },err=>{
            log.err('search by tgid error')
            console.log(err)
            this.obsEnd(obs,ret)
          })
        },err=>{
          log.err('prepare query error')
          console.log(err)
          this.obsEnd(obs,ret)
        })
        return
      }
      if (data.fn=='group')
      {
        //console.log(['GROUP!',data])
        let groupCode = data.field
        if (!groupCode||!groupCode.length)
        {
          log.err('no group code!',data)
          return this.obsEnd(obs,ret)
        }
        if (groupCode!='_all'&&groupCode!='guests')
        {
          this.C('usersgroup').findNode({code:data.field},{populate:2}).subscribe(grp=>{
            let chats = []
            let ulist = []
            grp.field('users').forEach(u=>{
              if (!u||!u.f('privateChat')) return
              ulist.push(u)
              chats.push(u.f('privateChat'))
            })
            if (getUsers)
              return this.obsEnd(obs,ulist)
            this.C('tgchat').findNodes({_id:{$in:chats}}).subscribe(chlist=>{
              this.obsEnd(obs,chlist)
            },err=>{
              log.err('search by chatsid error')
              console.log(err)
              this.obsEnd(obs,ret)
            })
          },err=>{
            log.err('search by tgid error')
            console.log(err)
            this.obsEnd(obs,ret)
          })
        }
        if (groupCode=='_all')
        {
          this.C('user').findNodes({},{populate:1}).subscribe(list=>{
            if (getUsers)
              return this.obsEnd(obs,list)
            let chats = list.map(x=>x&&x.fp('privateChat')?x.fp('privateChat'):null).filter(x=>!!x)
            this.obsEnd(obs,chats)
          },err=>{
            log.err('search by tgid error')
            console.log(err)
            this.obsEnd(obs,ret)
          })
        }
        if (groupCode=='guests')
        {
          this.C('usersgroup').findNode({code:data.field},{populate:0}).subscribe(grs=>{
            let uids = []
            grps.forEach(grp=>{
              grp.field('users').all().forEach(uid=>{
                if (!uid) return
                if (uids.includes(uid)) return
                uids.push(uid)
              })
            })
            this.C('user').findNodes({_id:{$nin:uids}},{populate:1}).subscribe(list=>{
              if (getUsers)
                return this.obsEnd(obs,list)
              let chats = list.map(x=>x&&x.fp('privateChat')?x.fp('privateChat'):null).filter(x=>!!x)
              this.obsEnd(obs,chats)
            },err=>{
              log.err('search by tgid error')
              console.log(err)
              this.obsEnd(obs,ret)
            })
          },err=>{
            log.err('search by tgid error')
            console.log(err)
            this.obsEnd(obs,ret)
          })
        }
        return
      }
      if (data.fn=='context')
      {
        let f = ctx.logicFieldByPath(data.arg)
        if (!f||!f.values[0])
        {
          return this.obsEnd(obs,ret)
        }
        let lst = f.values[0]
        if (!Array.isArray(lst))
          lst = f.values
        //console.log(['push reaction from context',f.values])
        let gids = []
        let uids = []
        let cids = []
        //if (Array.isArray(lst))
          lst.forEach(obj=>{
            let bnd = null
            if (!obj||!obj.bundle) return
            if (obj instanceof Node)
            {
              if (obj.bundle()=='usersgroup')
                gids.push(obj.id())
              if (obj.bundle()=='user')
                uids.push(obj.id())
              if (obj.bundle()=='tgchat')
                cids.push(obj.id())
              return
            }
            if (obj.bundle=='usersgroup')
              gids.push(obj.id)
            if (obj.bundle=='user')
              uids.push(obj.id)
            if (obj.bundle=='tgchat')
              cids.push(obj.id)
          })
        console.log([gids,uids,cids])
        if (gids.length)
        {
          this.C('usersgroup').findNodes({_id:{$in:gids}},{populate:2}).subscribe(list=>{
            let chats = []
            let ulist = []
            list.forEach(grp=>{
              grp.field('users').forEach(u=>{
                if (!u||!u.fp('privateChat')) return
                ulist.push(u)
                chats.push(u.fp('privateChat'))
              })
            })
            if (getUsers)
              return this.obsEnd(obs,ulist)
            this.obsEnd(obs,chats)
          },err=>{
            log.err('search usersgroups by usergroupids error!')
            console.log(err)
            this.obsEnd(obs,ret)
          })
          return
        }
        if (uids.length)
        {
          this.C('user').findNodes({_id:{$in:uids}},{populate:1}).subscribe(list=>{
            if (getUsers)
              return this.obsEnd(obs,list.map(x=>x&&x.fp('privateChat')?x:null).filter(x=>!!x))
            let chats = list.map(x=>x&&x.fp('privateChat')?x.fp('privateChat'):null).filter(x=>!!x)
            this.obsEnd(obs,chats)
          },err=>{
            log.err('search users by usergroup error!')
            console.log(err)
            this.obsEnd(obs,ret)
          })
          return
        }
        if (cids.length)
        {
          this.C('tgchat').findNodes({_id:{$in:cids}}).subscribe(chats=>{
            this.obsEnd(obs,chats)
          },err=>{
            log.err('search chats by cids error!')
            console.log(err)
            this.obsEnd(obs,ret)
          })
          return
        }
        return
      }
      //console.log(['recievers extract from',data])
      this.obsEnd(obs,ret)
    })
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.C('user').setNodeClass(User)
      this.obsEnd(obs)
    })
  }


  moduleInstall()
  {
    return Rx.Observable.create(obs=>{

      let $t = []
 
      //user bundle
      let username = { name:g18n.t('Username'), code: 'username', type: 'string', flags: ['system','locked','logic','template','title']}
      let passwd = { name: g18n.t('Password'), code: 'password', type: 'password', flags:['system','locked']}
      let uuid = { name: g18n.t('UUID'), code: 'uuid', type: 'string', flags:['system','unique','required','logic','locked','template'], generator:'uuid'}
      let locale = { name: g18n.t('Language'), code: 'locale', type: 'string', flags:['system','required','logic','template'], def: this._core.locale }
      let timezone = { name: g18n.t('Time zone'), code: 'timezone', type: 'timezone', flags:['system','required','logic','template'], def: this._core.timezone }
      let created = { name: g18n.t('Created'), code: 'created', type: 'datetime', flags:['system','logic','locked','template'], generator: 'now', conf:{ops:[{source:'timezone',manual:''},{source:'manual',manual:['second','minute','hour','day','month','year']}]} }

      let b = this._core.module('bundle').bundle('user',true)
      b.name(g18n.t('User'))
      b.desc(g18n.t('System users'))
      b.addFlag('system')
      b.addFlag('routed')
      b.addField(username)
      b.addField(passwd)
      b.addField(uuid)
      b.addField(locale)
      b.addField(timezone)
      b.addField(created)

      log.log('installing bundle',b.code())
      $t.push(b.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        this._core.module('bundle').collection('user').setNodeClass(User)
        log.success('bundle installed')
        obs.next()
        obs.complete()
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let apiOne = {
        url: '/api/users/one/:uid',
        access: ['auth.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let uid = p.uid || null
            if (!uid)
              this.obsEnd(obs,null)
            this.C('user').findNode({_id:uid},{populate:1}).subscribe($u=>{
              let obj = $u.toObject()
              let info = $u.bundleFields()
              this.obsEnd(obs,{user:obj,info:info})
            },err=>{
              log.err('user route search one fail')
              console.log(err)
              this.obsEnd(obs,null)
            })
          })
        }
      }

      let apiSave = {
        url: '/api/users/save',
        access: ['auth.root'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('body')
            let groups = p.groups || null
            let uid = p.id || null
            if (!groups||!uid)
              this.obsEnd(obs,{success:false,err:'Op!',error:'OpOpOp!'})
            groups = groups.filter(g=>!!g)
            this.C('user').findNode({_id:uid}).subscribe($user=>{
              if (!$user)
              {
                log.err('user save route user search fail')
                console.log(err)
                this.obsEnd(obs,{success:false})
              }
              this.C('usersgroup').findNodes({}).subscribe($groups=>{
                let $t = []
                $groups.forEach($g=>{
                  let all = $g.field('users').all().map(e=>e)
                  let exIn = all.reduce((acc,uid)=>{
                    return $user.eq(uid)?true:acc
                  },false)
                  let exG = groups.reduce((acc,grp)=>{
                    return $g.eq(grp.id)?true:acc
                  },false)
                  if (exG==exIn)
                    return
                  let dir = 'none'
                  if (exG&&!exIn)
                  {
                    dir = 'in'
                    $g.field('users').push($user)
                  }
                  if (!exG&&exIn)
                  {
                    dir = 'out'
                    $g.field('users').clear()
                    all.forEach(uid=>{
                      if (!$user.eq(uid))
                        $g.field('users').push(uid)
                    })
                  }
                  if (dir!='none')
                  {
                    log.info('updating usergroup',[$user.titles(),dir,$g.f('code')])
                    $t.push($g.save())
                  }
                })
                Rx.concat(...$t).subscribe(()=>{},err=>{
                  log.err('failed group update')
                  console.log(err)
                  this.obsEnd(obs,{success:false})
                },()=>{
                  this.obsEnd(obs,{success:true})
                })
              },err=>{
                log.err('user save route groups load fail')
                console.log(err)
                this.obsEnd(obs,{success:false})
              })
            },err=>{
              log.err('user save route user load fail')
              console.log(err)
              this.obsEnd(obs,{success:false})
            })
          })
        }
      }


      let apiSearch = {
        url: '/api/users/search/:srch',
        access: ['auth.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            if (!p||!p.srch||p.srch.length<2)
              return this.obsEnd(obs,[])
            let rx = new RegExp('^.*'+p.srch+'.*$','i')
            let q = {$or:[
              { username: { $regex: rx } },
              { lastName: { $regex: rx } },
              { firstName: { $regex: rx  } }
            ]}
            this.C('user').findNodes(q).subscribe($users=>{
              let ret = $users.map($u=>$u.toObject())
              this.obsEnd(obs,ret)
            },err=>{
              log.err('search users error')
              console.log(err)
              this.obsEnd(obs,[])
            })
          })
        }
      }

      let apiList = {
        url: '/api/users/list/:group?/:start?/:limit?',
        access: ['auth.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let group = p.group || '-'
            let start = Number.parseInt(p.start) || 0
            let limit = Number.parseInt(p.limit) || 10
            //console.log([group,start,limit])
            let ret = {
              users: [],
              total: 0,
              pos: 0,
              amount: 0
            }
            if (group!='guests'&&group!='-')
            {
              //some "standart group"
              this.C('usersgroup').findNode({code:group}).subscribe($grp=>{
                if (!$grp)
                  return this.obsEnd(obs,ret)
                let all = $grp.field('users').all()
                let uids = all.slice(start,start+limit)
                this.C('user').findNodes({_id:{$in:uids}}).subscribe($users=>{
                  ret.total = all.length
                  ret.pos = start
                  ret.amount = limit
                  ret.users = $users.map($u=>$u.toObject())
                  //console.log(ret)
                  this.obsEnd(obs,ret)
                },err=>{
                  log.err('users route users find error!',group)
                  this.obsEnd(obs,ret)
                })
              },err=>{
                log.err('users route group find error!',group)
                this.obsEnd(obs,ret)
              })
              return
            }
            if (group=='-') 
            {
              //all users
              this.C('user').findNodes({},{skip:start,limit:limit}).subscribe($users=>{
                this.C('user').countNodes({}).subscribe(total=>{
                  ret.total = total
                  ret.pos = start
                  ret.amount = limit
                  ret.users = $users.map($u=>$u.toObject())
                  //console.log(ret)
                  this.obsEnd(obs,ret)
                },err=>{
                  log.err('users route all users find error!')
                  this.obsEnd(obs,ret)
                })
              },err=>{
                log.err('users route all users find error!')
                this.obsEnd(obs,ret)
              })
              return
            }
            if (group=='guests')
            {
              this.M('auth/access').guestUsers({}).subscribe($guests=>{
                ret.total = $guests.length
                ret.pos = start
                ret.amount = limit
                ret.users = $guests.slice(start,start+limit).map($u=>$u.toObject())
                this.obsEnd(obs,ret)
              },err=>{
                log.err('users route guests users find error!')
                this.obsEnd(obs,ret)
              })
              return
            }
            
            this.obsEnd(obs,ret)
          })
        }
      }

 
      obs.next([apiOne,apiSave,apiSearch,apiList])
      obs.complete()
    })
  }


}

module.exports = UsersModule
