/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('User')
let g18n = require('../../../g18n')

const Node = require('../../node/node')

class User extends Node
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._type = 'user'
    this.isGuest = false
    this._roles = []
    this._gcodes = []
  }

  iisGuest()
  {
    if (this.isGuest)
      return true
    if (!this._gcodes.length)
      return true
    if (this._gcodes.includes('guests'))
      return true
    return false
  }

  locale()
  {
    return this.field('locale').get()
  }

  timezone()
  {
    return this.field('timezone').get()
  }

  setPassword(pwd)
  {
    let hash = this._core.module('auth/access').salt(pwd)
    this.field('password').set(hash)
  }

  verifyPassword(pwd)
  {
    let salt = this.field('password').get()
    return this._core.module('auth/access').saltVerify(pwd,salt)
  }

  titles()
  {
    let username = this.field('username').get() || ''
    let firstname = this.field('firstName').get() || ''
    return firstname+' @'+username
  }

  //beforeSave(obj)
  //{
    //return Rx.Observable.create(obs=>{
      //console.log(obj)
      //console.log(this._model)
      //this.obsNext(obs,this)
    //})
  //}

  saveUser()
  {
    return Rx.Observable.create(obs=>{
      let next = this.save()
      if (this.isGuest)
        next = Rx.of(1)
      next.subscribe(()=>{
        obs.next()
        obs.complete()
      })
    })
  }

  populateAccess()
  {
    return Rx.Observable.create(obs=>{
      this._roles = []
      this._gcodes = []
      this._core.module('auth/access').userAccess(this).subscribe(data=>{
        //console.log(['_access',roles])
        this._roles = data.roles
        this._gcodes = data.gcodes
        this.isGuest = true
        if (this._gcodes.length > 1)
          this.isGuest = false
        if (!this.isGuest)
          this._gcodes = data.gcodes.filter(x=>x!='guests')
        //console.log(this._gcodes,this.isGuest)
        obs.next(this)
        obs.complete()
      },err=>{
        log.err('populate access fail',this.titles())
        console.log(err)
        obs.error(err)
      })
    })
  }

  _can(role,doStrict=false)
  {
    let [mod,...xx] = role.split('.')
    if (!mod||!mod.length)
      mod = null
    return this._roles.reduce((acc,r)=>{
      //log.test('can?',[this.titles(),r.role()])
      if (r.role()=='global.root'&&!doStrict)
        return true
      if (mod&&r.f('module')==mod&&r.f('code')=='root'&&!doStrict)
        return true
      return r.role()==role?true:acc
    },false)
  }

  can(a,doStrict=false)
  {
    //console.log(['can?',this.titles(),this._roles])
    let list = [a]
    if (Array.isArray(a))
      list = a
    let ret = true
    list.forEach(rl=>{
      if (!this._can(rl,doStrict))
        ret = false
    })
    return ret
  }

}

module.exports = User
