/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('UsersGroup')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const UsersGroup = require('./usersgroup')

class UsersGroupModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtOne = {
        url: '/api/groups/one/:code',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let code = p.code || null
            if (!code)
              return this.obsEnd(obs,null)
            this.C('usersgroup').findNode({code:code},{
              populate: 1,
            }).subscribe($grp=>{
              this.obsEnd(obs,$grp.toObject())
            },err=>{
              log.err('fail loading group')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }

      let rtSave = {
        url: '/api/groups/save',
        access: 'auth.root',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('body')
            //console.log(p)
            let node = this.C('usersgroup').initNode()
            if (p.id)
              node = this.C('usersgroup').findNode({_id:p.id})
            node.subscribe($grp=>{
              if (!$grp)
              {
                log.err('fail search/init group')
                console.log(err)
                return this.obsErr(obs,{error:err})
              }
              if (!p.id)
                $grp.f('code',p.code)
              $grp.f('name',p.name)
              $grp.f('code',p.code)
              $grp.f('description',p.description)
              //let curr = $grp.field('roles').all()
              $grp.field('roles').clear()
              p.roles.forEach(r=>{
                if(!r) return
                $grp.field('roles').push(r.id)
              })
              //console.log($grp._model)
              $grp.save().subscribe($g=>{
                this.obsEnd(obs,{success:true})
              },err=>{
                log.err('fail saving group')
                console.log(err)
                this.obsEnd(obs,{success:false})
              })
            },err=>{
              log.err('fail loading group')
              console.log(err)
              this.obsErr(obs,{error:err})
            })
          })
        }
      }

      let rtDel = {
        url: '/api/groups/del/:code',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let code = p.code || null
            if (!code)
              return this.obsEnd(obs,{success:false})
            if (code=='admins'||code=='guests')
              return this.obsEnd(obs,{success:false})
            this.C('usersgroup').removeNodes({code:code}).subscribe($grp=>{
              this.obsEnd(obs,{success:true})
            },err=>{
              log.err('fail deleting group')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }


      let rtList = {
        url: '/api/groups/list',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            this.C('usersgroup').findNodes({},{
              populate: 1,
            }).subscribe($list=>{
              let ret = $list.map(o=>o.toObject())
              this.obsEnd(obs,ret)
            },err=>{
              log.err('fail loading screengroups')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }

      obs.next([rtOne,rtSave,rtDel,rtList])
      obs.complete()
    })
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.C('usersgroup').setNodeClass(UsersGroup)
      this.obsEnd(obs)
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      let $t = []
      //user groups bundle
      let name = { name:g18n.t('Name'), code: 'name', type: 'string', flags:['system','required','logic','template','title']}
      let code = { name:g18n.t('Code'), code: 'code', type: 'string', flags:['system','required','unique','logic','template','title']}
      let desc = { name:g18n.t('Description'), code: 'description', type: 'string', flags:['system','logic','template'], conf:{ops:[{source:'manual',manual:true}]} }
      let isSystem = { name:g18n.t('Is system'), code: 'isSystem', type: 'boolean', flags:['system','logic','locked'], def: false }
      let locked = { name:g18n.t('Is locked'), code: 'locked', type: 'boolean',  flags:['system','logic','locked'], def: false }
      let users = { name:g18n.t('Group users'), code: 'users', type: 'relation', conf: {ops:[{source:'manual',manual:'user'}]}, flags:['system','logic','template'], quantity: 0 }
      let roles = { name:g18n.t('Group roles'), code: 'roles', type: 'relation', conf: {ops:[{source:'manual',manual:'role'}]}, flags:['system','logic'], quantity: 0 }

      //let UserGroupBundle = { name:g18n.t('Users group'), code: 'usersgroup', desc:g18n.t('System users groups'), flags:['system'], fields: [ugfName,ugfCode,ugfDesc,ugfIsSystem,ugfLocked,ugfUsers,ugfRoles] }
      let bnd = this._core.module('bundle').bundle('usersgroup',true)
      bnd.name(g18n.t('Users Group'))
      bnd.desc(g18n.t('System users groups'))
      bnd.addFlag('system')
      bnd.addField(name)
      bnd.addField(code)
      bnd.addField(desc)
      bnd.addField(isSystem)
      //bnd.addField(locked)
      bnd.addField(users)
      bnd.addField(roles)

      log.log('installing bundle',bnd.code())
      $t.push(bnd.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        this._core.module('bundle').collection('usersgroup').setNodeClass(UsersGroup)
        log.success('bundle installed')
        obs.next()
        obs.complete()
      })
 
    })
  }

}

module.exports = UsersGroupModule
