/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('BotRouter')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

class BotRouter extends CoreModule
{
  constructor(parent)
  {
    super(parent)
  }

  /*
  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      log.success('initialized')
      this.obsEnd(obs)
    })
  }
  */

  nodeAfterSave(node)
  {
    if (node.bundle()=='botroute')
      return Rx.of(node)

    let fields = node.bundleFields()
    let routes = fields.filter(f=>f.type=='botroute')
    if (!routes.length)
      return Rx.of(node)
    return Rx.Observable.create(obs=>{
      this.C('botroute').removeNodes({nid:node.id()}).subscribe(()=>{
        let $t = []
        routes.forEach(rt=>{
          let isKey = node.field(rt.code).confOp(0,false)
          if (!isKey)
            isKey = false
          node.field(rt.code).all().forEach(path=>{
            let rtn = this.C('botroute').newNode()
            rtn.f('nid',node)
            rtn.f('target',node.bundle())
            rtn.f('route',path)
            rtn.f('iskey',isKey)
            $t.push(rtn.save())
          })
        })
        Rx.concat(...$t).subscribe($rt=>{
          log.info('node route created',[$rt.f('target'),$rt.f('route')])
        },err=>{
          log.err('node route update error!')
          console.log(err)
          this.obsEnd(obs,node)
        },()=>{
          this.obsEnd(obs,node)
        })
      },err=>{
        log.err('dropping node routes failed!')
        console.log(err)
        this.obsEnd(obs,node)
      })
    })
  }

  nodesAfterRemove(info)
  {
    if (info.bundle=='botroute')
      return Rx.of(info)
    if (!this.C('botroute'))
      return Rx.of(info)

    return Rx.Observable.create(obs=>{
      this.C('botroute').removeNodes({nid:{$in:info.nids}}).subscribe(()=>{
        this.obsEnd(obs,info)
      },err=>{
        log.err('dropping removed nodes routes failed!')
        console.log(err)
        this.obsEnd(obs,info)
      })
    })
  }



  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      //routes bundle
      let routes = this._core.module('bundle').bundle('botroute',true)
      routes.name(g18n.t('Bot Route'))
      routes.desc(g18n.t('Bot\'s routes'))
      routes.addFlag('system')
      //routes.addFlag('locked')

      routes.addField({
        name: g18n.t('Node ID'),
        desc: g18n.t('Node ID'),
        code: 'nid',
        type: 'relation',
        quantity: 1,
        flags: ['system','locked','logic','template']
      })

      routes.addField({
        name: g18n.t('Target bundle'),
        desc: g18n.t('Target node bundle'),
        code: 'target',
        type: 'string',
        quantity: 1,
        flags: ['system','locked','logic','template']
      })

      routes.addField({
        name: g18n.t('Route'),
        desc: g18n.t('Node route string'),
        code: 'route',
        type: 'string',
        quantity: 1,
        flags: ['system','locked','logic','template']
      })

      routes.addField({
        name: g18n.t('Is Key'),
        desc: g18n.t('Node route is key'),
        code: 'iskey',
        type: 'boolean',
        quantity: 1,
        def: false,
        flags: ['system','locked','logic','template']
      })

      log.info('saving bundle',routes.code())
      $t.push(routes.save(true,true))

      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing routes bundle failed')
        console.log(err)
        this.obsErr(obs,err)
      },()=>{
        log.success('installed')
        this.obsEnd(obs)
      })
    })
  }


}

module.exports = BotRouter
