/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('AbstractMessage')
let g18n = require('../../../g18n')

const Node = require('../../node/node')
const ScreenNode = require('../screen/screen')

class AbstractMessage extends Node
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._type = 'screenmessagenode'
    this._screen = null
    this._msgI = 0
  }

  screen(val)
  {
    if (val&&(val instanceof ScreenNode))
      this._screen = val
    return this._screen
  }

  msgI(val)
  {
    if (typeof val == 'number')
      this._msgI = val
    return this._msgI
  }

  title()
  {
    return this._screen.f('title')
  }

  execute(ctx) //zero last executing by default
  {
    return Rx.Observable.create(obs=>{
      this.M('logic').testLogicRx(this,ctx).subscribe(logicOk=>{
        if (!logicOk)
        {
          ctx.f('fail',ctx.f('fail')+1)
          return this.obsEnd(obs)
        }

        let $t = []
        let $msg = this.C('tgmessage').newNode()
        $msg.open()
        let mdl = this.M('telegram')
        if (ctx.vnode('interface'))
          mdl = ctx.vnode('interface')
        $msg.setIface(mdl)
        $msg.f('type',ctx.type())
        $msg.f('sender',mdl.me())
        $msg.f('chat',ctx.chat())
        $msg.f('screen',this.screen())
        this.field('reactions').forEach((rct,ri)=>{
          //console.log([[rct]])
          log.info('reaction',rct.category())
          rct.rI(ri)
          rct.screen(this.screen())
          rct.message(this)
          $t.push(rct.execute(ctx,$msg))
        })
        if ($t.length)
          $t.push($t.shift())

        Rx.concat(...$t).subscribe(res=>{
          //console.log(['reaction executed'])
        },err=>{
          ctx.f('error',ctx.f('error')+1)
          //ctx._stats.error++
          log.err('ERROR')
          console.log(err)
          this.obsEnd(obs)
        },()=>{
          ctx.f('success',ctx.f('success')+1)
          //ctx._stats.success++
          if (!$msg.sendable())
            return this.obsEnd(obs)

          $msg.onSended.subscribe(msgRes=>{
            log.info('message sended',msgRes)
            if (msgRes)
            {
              if(msgRes.success)
                ctx.f('sended',ctx.f('sended')+1)
                //ctx._stats.sended++
              if(msgRes.error)
                ctx.f('error',ctx.f('error')+1)
                //ctx._stats.error++
            }
            this.obsEnd(obs)
          },err=>{
            log.err('MESSAGE SEND ERROR')
            console.log(err)
            this.obsEnd(obs)
          })
          $msg.send()
        })
      },err=>{
        log.err('testing message logic error happens!')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }

}

module.exports = AbstractMessage
