/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('LoopMessage')
let g18n = require('../../../../g18n')

const AbstractMessage = require('../message')
const VNode = require('../../../node/vnode')
//const ScreenNode = require('../screen/screen')

class LoopMessage extends AbstractMessage
{

  loop(ctx,type,code,data,dataI=0)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['looping',code,data,dataI])

      let vinfo = {
        type: type,
        code: code,
        name: 'Loop data',
      }
      let ctxVNode = new VNode(this)
      ctxVNode.addField(vinfo)
      ctxVNode.f(code,data)
      ctx.addNode(ctxVNode,'loop')

      let $t = []

      let $msg = this.C('tgmessage').newNode()
      $msg.open()
      let mdl = this.M('telegram')
      if (ctx.vnode('interface'))
        mdl = ctx.vnode('interface')
      $msg.setIface(mdl)
      $msg.f('type',ctx.type())
      $msg.f('sender',mdl.me())
      $msg.f('chat',ctx.chat())
      $msg.f('screen',this.screen())
      this.field('reactions').forEach((rct,ri)=>{
        if (!ri) return //skip first reaction

        log.log('reaction',rct.category())
        rct.rI(ri-1)
        rct.screen(this.screen())
        rct.message(this)
        $t.push(rct.execute(ctx,$msg))
      })
      if ($t.length)
        $t.push($t.shift())

      Rx.concat(...$t).subscribe(res=>{
        //console.log(['reaction executed'])
      },err=>{
        ctx.f('error',ctx.f('error')+1)
        //ctx._stats.error++
        log.err('ERROR')
        console.log(err)
        this.obsEnd(obs)
      },()=>{
        ctx.f('success',ctx.f('success')+1)
        //ctx._stats.success++
        if (!$msg.sendable())
          return this.obsEnd(obs)

        $msg.onSended.subscribe(msgRes=>{
          log.info('message sended',msgRes)
          if (msgRes)
          {
            if(msgRes.success)
              ctx.f('sended',ctx.f('sended')+1)
              //ctx._stats.sended++
            if(msgRes.error)
              ctx.f('error',ctx.f('error')+1)
              //ctx._stats.error++
          }
          this.obsEnd(obs)
        },err=>{
          log.err('MESSAGE SEND ERROR')
          console.log(err)
          this.obsEnd(obs)
        })
        $msg.send()
      })

      //this.obsEnd(obs)
    })
  }

  execute(ctx) //loop message executing
  {
    return Rx.Observable.create(obs=>{
      this.M('logic').testLogicRx(this,ctx).subscribe(logicOk=>{
        if (!logicOk)
        {
          ctx.f('fail',ctx.f('fail')+1)
          return this.obsEnd(obs)
        }


        let loopReaction = this.field('reactions').getp(0)
        let loopData = loopReaction.f('data')
        if (!loopData||!loopData.loop)
          return this.obsErr(obs,{code:'LOOP_CONFIGURATION_NOT_EXISTS'})
        loopData = loopData.loop

        let ctxCode = loopData.code
        let ctxType = loopData.type || 'any'
        let src = ctx.fieldSelector(loopData.source)
        src.init(ctx).subscribe(res=>{
          //console.log(['selector init ret',res,src])
          if (!res)
            return this.obsErr(obs,{code:'LOOP_DATA_SOURCE_ERROR'})

          log.log('executing')
          let srcData = src.allModified()
          if (loopData.fn=='times'&&loopData.source&&loopData.source.manual&&!srcData)
          {
            srcData = loopData.source.target
            ctxType = 'number'
          }

          //console.log([loopData,srcData])

          let $t = []
          if (loopData.fn == 'context')
          {
            if (!Array.isArray(srcData))
            {
              $t.push(this.loop(ctx,ctxType,ctxCode,srcData))
            } else {
              srcData.forEach((dt,dti)=>{
                $t.push(this.loop(ctx,ctxType,ctxCode,dt,dti))
              })
            }
          }

          if (loopData.fn == 'times')
          {
            ctxType = 'number'
            let times = parseInt(srcData)
            if (times<1)
            {
              this.obsEnd(obs)
            } else {
              for(let i=0;i<times;i++)
                $t.push(this.loop(ctx,ctxType,ctxCode,i,i))
            }
          }

          Rx.concat(...$t).subscribe(oneRes=>{
            //console.log(['oneres',oneRes])
          },err=>{
            log.err('error!')
            console.log(err)
            this.obsErr(obs,err)
          },()=>{
            log.log('done')
            this.obsEnd(obs)
          })

        },err=>{
          log.err('loop source error')
          console.log(err)
          this.obsErr(obs,{code:'LOOP_DATA_INIT_ERROR'})
        })
        /*
        let $t = []
        let $msg = this.C('tgmessage').newNode()
        $msg.open()
        $msg.f('type',ctx.type())
        $msg.f('sender',this.M('telegram').me())
        $msg.f('chat',ctx.chat())
        $msg.f('screen',this.screen())
        this.field('reactions').forEach((rct,ri)=>{
          log.info('reaction',rct.category())
          rct.rI(ri)
          rct.screen(this.screen())
          rct.message(this)
          $t.push(rct.execute(ctx,$msg))
        })
        if ($t.length)
          $t.push($t.shift())

        Rx.concat(...$t).subscribe(res=>{
          //console.log(['reaction executed'])
        },err=>{
          ctx.f('error',ctx.f('error')+1)
          //ctx._stats.error++
          log.err('ERROR')
          console.log(err)
          this.obsEnd(obs)
        },()=>{
          ctx.f('success',ctx.f('success')+1)
          //ctx._stats.success++
          if (!$msg.sendable())
            return this.obsEnd(obs)

          $msg.onSended.subscribe(msgRes=>{
            log.info('message sended',msgRes)
            if (msgRes)
            {
              if(msgRes.success)
                ctx.f('sended',ctx.f('sended')+1)
                //ctx._stats.sended++
              if(msgRes.error)
                ctx.f('error',ctx.f('error')+1)
                //ctx._stats.error++
            }
            this.obsEnd(obs)
          },err=>{
            log.err('MESSAGE SEND ERROR')
            console.log(err)
            this.obsEnd(obs)
          })
          $msg.send()
        })
        /**/
      },err=>{
        log.err('testing loop message logic error happens!')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }

}

module.exports = LoopMessage
