/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('MessagesModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const AbstractMessage = require('./message')
const LoopMessage = require('./messages/loop')

class MessagesModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._messageClasses = {}

    this.registerMessageType('loop',LoopMessage)
  }

  registerMessageType(code,Cls)
  {
    if (!(AbstractMessage.isPrototypeOf(Cls)))
      return
    log.log('registring message type',code)
      this._messageClasses[code] = Cls
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')

      this.C('screenmessage').setNodeClass(AbstractMessage)
      this.C('screenmessage').setNodeClassFn(mdl=>{
        if (!mdl.type) return AbstractMessage
        if(this._messageClasses[mdl.type]) return this._messageClasses[mdl.type]
        return AbstractMessage
      })

      this.obsEnd(obs)

      /*
      let $t = []

      Rx.concat(...$t).subscribe(()=>{},err=>{obs.error(err)},()=>{
        log.success('initialized')
        obs.next()
        obs.complete()
      })
      */
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlScreenMessage = this._core.module('bundle').bundle('screenmessage',true)
      bndlScreenMessage.name(g18n.t('Screen Messages'))
      bndlScreenMessage.desc(g18n.t('Telegram screen messages'))
      bndlScreenMessage.flags(['system','locked'])
      bndlScreenMessage.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','required','logic']})
      bndlScreenMessage.addField({name:g18n.t('Kbd Type'),code:'kbdType',type:'string',flags:['system','logic'],def:'none'})
      bndlScreenMessage.addField({name:g18n.t('Global name'),code:'globalName',type:'string',flags:['system','logic']})
      bndlScreenMessage.addField({name:g18n.t('Logic'),code:'logic',type:'any',flags:['system','logic']})
      bndlScreenMessage.addField({name:g18n.t('Reactions'),code:'reactions',type:'relation',conf:{ops:[{source:'manual',manual:'screenreaction'},{source:'manual',manual:'globalName, category'}]}, flags:['system','logic'],quantity:0})
      
      log.info('saving bundle',bndlScreenMessage.code())
      $t.push(bndlScreenMessage.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing messages failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

}

module.exports = MessagesModule
