/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('BotModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')
const ScreenGroupNode = require('./screen/group/node')

class BotModule extends CoreModule
{

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing screens data')

      let $rt = []

      log.info('installing basic reactions')
      //login
      let rctLoginLogin = this.C('screenreaction').newNode()
      rctLoginLogin.f('category','user_login')
      rctLoginLogin.f('data',{})
      $rt.push(rctLoginLogin.save())

      let rctLoginText = this.C('screenreaction').newNode()
      rctLoginText.f('category','text')
      rctLoginText.f('data',{text:"{{ virtual.otlCode }}"})
      $rt.push(rctLoginText.save())

      //make me admin
      let rctMMAGroup = this.C('screenreaction').newNode()
      rctMMAGroup.f('category','auth_group')
      rctMMAGroup.f('data',{auth_group:{
        includes: ['admins'],
        excludes: []
      }})
      $rt.push(rctMMAGroup.save())

      let rctMMAText = this.C('screenreaction').newNode()
      rctMMAText.f('category','text')
      rctMMAText.f('data',{text:"ok! use /login command to login"})
      $rt.push(rctMMAText.save())

      let rctStartText = this.C('screenreaction').newNode()
      rctStartText.f('category','text')
      rctStartText.f('data',{text:"Hello {{user.firstName}}!"})
      $rt.push(rctStartText.save())

      Rx.concat(...$rt).subscribe(()=>{},err=>{
        log.err('saving reactions error!',err)
        this.obsErr(obs,err)
      },()=>{
        log.info('installing screens messages')

        let $mt = []
        let msgLogin = this.C('screenmessage').newNode()
        msgLogin.f('type','text')
        msgLogin.f('kbdType','none')
        msgLogin.f('globalName','')
        msgLogin.f('logic',{ conditions: { op: 'and', tests: [], groups: [] } })
        msgLogin.field('reactions').push(rctLoginText)
        msgLogin.field('reactions').push(rctLoginLogin)
        $mt.push(msgLogin.save())

        let msgMMA = this.C('screenmessage').newNode()
        msgMMA.f('type','text')
        msgMMA.f('kbdType','none')
        msgMMA.f('globalName','')
        msgMMA.f('logic',{ conditions: { op: 'and', tests: [], groups: [] } })
        msgMMA.field('reactions').push(rctMMAText)
        msgMMA.field('reactions').push(rctMMAGroup)
        $mt.push(msgMMA.save())

        let msgStart = this.C('screenmessage').newNode()
        msgStart.f('type','text')
        msgStart.f('kbdType','none')
        msgStart.f('globalName','')
        msgStart.f('logic',{ conditions: { op: 'and', tests: [], groups: [] } })
        msgStart.field('reactions').push(rctStartText)
        $mt.push(msgStart.save())


        Rx.concat(...$mt).subscribe(()=>{},err=>{
          log.err('installing screens error',err)
          this.obsErr(obs,err)
        },()=>{
          log.info('installing screens')

          let $st = []
          let scrLogin = this.C('screen').newNode()
          scrLogin.f('title','Login screen')
          scrLogin.f('enabled',true)
          scrLogin.f('action','text')
          scrLogin.f('actionData',{
            text: {
              hasKey: false,
              key: '',
              hasCommand: true,
              command: 'login',
              hasRx: false,
              rx: ''
            }
          })
          scrLogin.f('logic',{ conditions: { op: 'and', tests: [], groups: [] } })
          scrLogin.field('messages').push(msgLogin)
          $st.push(scrLogin.save())

          let scrMMA = this.C('screen').newNode()
          scrMMA.f('title','Make me admin screen')
          scrMMA.f('enabled',true)
          scrMMA.f('action','text')
          scrMMA.f('actionData',{
            text: {
              hasKey: false,
              key: '',
              hasCommand: true,
              command: 'mma',
              hasRx: false,
              rx: ''
            }
          })
          scrMMA.f('logic',{
            conditions: {
              op: 'and',
              tests: [],
              groups: []
            }
          })
          scrMMA.field('messages').push(msgMMA)
          $st.push(scrMMA.save())

          let scrStart = this.C('screen').newNode()
          scrStart.f('title','Start screen')
          scrStart.f('enabled',true)
          scrStart.f('action','text')
          scrStart.f('actionData',{
            text: {
              hasKey: false,
              key: '',
              hasCommand: true,
              command: 'start',
              hasRx: false,
              rx: ''
            }
          })
          scrStart.f('logic',{
            conditions: {
              op: 'and',
              tests: [],
              groups: []
            }
          })
          scrStart.field('messages').push(msgStart)
          $st.push(scrStart.save())

          Rx.concat(...$st).subscribe(()=>{},err=>{
            log.err('installing screens error',err)
            this.obsErr(obs,err)
          },()=>{
            log.info('installing screengroups')

            let $sgt = []
            this.C('screengroup').setNodeClass(ScreenGroupNode)

            let scgPrivate = this.C('screengroup').newNode()
            scgPrivate.f('name','Private chats')
            scgPrivate.f('code','privatechats')
            scgPrivate.f('router','tgdefault')
            scgPrivate.f('priority',0)
            scgPrivate.f('description','Screens for private chats')
            scgPrivate.field('screens').push(scrMMA)
            scgPrivate.field('screens').push(scrStart)
            $sgt.push(scgPrivate.save())

            let scgRegs = this.C('screengroup').newNode()
            scgRegs.f('name','Admins commands')
            scgRegs.f('code','adminscmd')
            scgRegs.f('router','tgdefault')
            scgRegs.f('priority',10)
            scgRegs.f('description','Screens for administrators')
            scgRegs.field('screens').push(scrLogin)
            $sgt.push(scgRegs.save())

            Rx.concat(...$sgt).subscribe(()=>{},err=>{
              log.err('installing screengroups error',err)
              this.obsErr(obs,err)
            },()=>{
              log.info('apllying screengroups auth')

              Rx.forkJoin(
                this.C('role').findNode({code:'privatechats:private'}),
                this.C('usersgroup').findNode({code:'guests'})
              ).subscribe(lst=>{
                lst[1].field('roles').push(lst[0])
                lst[1].save().subscribe(()=>{
                  this.obsEnd(obs,'screens installed')
                })
              })
            })
          })
        })
      })

    })
  }
}

module.exports = BotModule
