/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('ButtonsReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class ButtonsReaction extends AbstractReaction
{


  static info()
  {
    return {
      code: 'buttons',
      name: g18n.t('Inline buttons bot reaction'),
      shortname: g18n.t('Inline'),
      faicon: 'fas fa-table',
      uicolor: 'text-success',
      uigroup: 'tgmessage',
      routersLimit: ['telegrambotrouter'],
      mode: 'reaction', //message/reaction/any
      greedy: true,
      conflicts: ['keyboard'],
      requireSend: true,
      doSend: false
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['exec kbd'])
      let data = this.f('data')
      //console.log('buttons',data)
      if (!data||!data.buttons)
        return this.obsEnd(obs)
      let kbd = data.buttons
      let $kbd = []
      kbd.forEach((row,rowi)=>{
        let $row = []
        row.forEach((btn,btni)=>{
          $row.push(this._buildButton(ctx,btn,rowi,btni))
        })
        let $build = Rx.concat(...$row).pipe( RxO.map(res=>res) )
        $kbd.push($build)
      })
      let ret = []
      let rowShift = 0
      Rx.concat(...$kbd).subscribe(br=>{

        //console.log('key builded',br)
        if (!br.obj)
          return

        if (this._isButton(br.obj))
        {
        //console.log('key builded is btn',br.obj)
          if (!ret[br.row+rowShift])
            ret[br.row+rowShift] = []
          ret[br.row+rowShift].push(br.obj)
        }
        if (this._isRow(br.obj))
        {
          if (!ret[br.row+rowShift])
            ret[br.row+rowShift] = br.obj
          else
            br.obj.forEach(x=>ret[br.row+rowShift].push(x))
        }
        if (this._isMenu(br.obj))
        {
          br.obj.forEach(mr=>ret.push(mr))
          rowShift+=br.obj.length
        }
        //if (!ret[br.row])
          //ret[br.row] = []
        //ret[br.row].push(br.obj)
      },err=>{
        console.log(err)
      },()=>{
        $msg.f('inline',ret)
        //console.log('build done',ret)
        this.obsEnd(obs)
      })
    })
  }

  _buildButton(ctx,btn,rowi,btni)
  {
    return Rx.Observable.create(obs=>{
      let txt = btn.caption
      if (txt.startsWith('{{virtual.')&&txt.endsWith('}}'))
      {
        let path = txt.slice(2,-2)
        ctx.dataByPath(path).subscribe(ndres=>{
          if (!ndres)
            return this.obsEnd(obs,{
              row: rowi,
              col: btni,
              obj: null
            })
          this.obsEnd(obs,{
            row: rowi,
            col: btni,
            obj: ndres
          })
        })
        return
      } else {
        this.M('template').compile(txt,ctx).subscribe(res=>{
          //console.log(['buildInlineButton',res,rowi,btni,btn])
          res = this.M('template').sanitizeHtml(res,true)
          let ret = {
            row: rowi,
            col: btni,
            obj: null
          }
          if (btn.type=='screen')
          {
            ret.obj = {
              text: res,
              callback_data: 'sid:'+btn.sid
            }
            return this.obsEnd(obs,ret)
          }
          this.M('template').compile(btn.url,ctx).subscribe(burl=>{
            burl = this.M('template').sanitizeHtml(burl,true)
            if (!burl.length)
              return this.obsEnd(obs,ret)
            if (btn.type=='url')
              ret.obj = {
                text: res,
                url: burl,
              }
            if (btn.type=='data')
              ret.obj = {
                text: res,
                callback_data: burl
              }
            return this.obsEnd(obs,ret)
          })
        })
      }
    })
  }

  _isButton(el)
  {
    return (el&&el.text&&(el.callback_data||el.url))
  }

  _isRow(arr)
  {
    if (!Array.isArray(arr)) return false
    let ok = true
    arr.forEach(el=>{
      if (!this._isButton(el))
        ok = false
    })
    return ok
  }

  _isMenu(arr)
  {
    if (!Array.isArray(arr)) return false
    let ok = true
    arr.forEach(el=>{
      if (!this._isRow(el))
        ok = false
    })
    return ok
  }
 

}

module.exports = ButtonsReaction
