/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('FieldReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')
const LogicOperation = require('../../../logic/abstract/operation')

class FieldReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'field',
      name: g18n.t('Manipulating context data fields'),
      shortname: g18n.t('Field'),
      faicon: 'fas fa-tags',
      uicolor: 'text-info',
      uigroup: 'helpers',
      routersLimit: [],
      mode: 'any', //message/reaction/any
      greedy: false,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }




  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['exec kbd'])
      let data = this.f('data')
      if (!data||!data.field)
        return this.obsErr(obs,{code:'NO_DATA'})
      data = data.field
      //console.log(['FIELD REACTION',data])
      if (!data.target||!data.setter)
        return this.obsErr(obs,{code:'DATA_ERROR'})
      let op = new LogicOperation(this,'setter',data)
      //log.info('executing setter',data.setter.target)
      op.init(ctx).subscribe(tres=>{
        if (!tres)
          return this.obsErr(obs,{code:'OPERATION_INIT_FAIL'})
        op.execute(ctx).subscribe(xres=>{
          log.info('setter executed')
          //console.log(ctx.view())
          this.obsEnd(obs,xres)
        },err=>{
          this.obsErr(obs,err)
        })
      },err=>this.obsErr(obs,err))
    })
  }

}

module.exports = FieldReaction
