/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('FileReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')
const FieldSelector = require('../../../logic/abstract/context_field')

const Node = require('../../../node/node')

class FileReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'file',
      name: g18n.t('Sending files bot reaction'),
      shortname: g18n.t('File/Media'),
      faicon: 'fas fa-images',
      uicolor: 'text-primary',
      uigroup: 'tgmessage',
      routersLimit: ['telegrambotrouter'],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: ['text'],
      requireSend: false,
      doSend: true
    }
  }


  genQR(text,ctx)
  {
    return Rx.Observable.create(obs=>{
      if (!text||text=='')
        return this.obsEnd(obs,null)
      this.M('template').compile(text,ctx).subscribe(tres=>{
        this.obsEnd(obs,null)
      },err=>{
        log.err('compile caption error')
        console.log(err)
        this.obsEnd(obs,null)
      })
    })
  }

  _ctxFile(fdata)
  {
    return Rx.Observable.create(obs=>{
      if (fdata instanceof Node&&fdata.bundle()=='file')
        fdata = fdata.toObject()
      /*
      {
        let fname = fdata.f('localPath').split('/').pop()
        let ret = {
          obj: fdata,
          source: fdata.f('localPath'),
          filename: fname,
          caption: fdata.f('name'),
        }
        return this.obsEnd(obs,ret)
      }
      /**/
      //console.log(['file reaction','fdata'],fdata)
      if (typeof fdata == 'object'&&fdata.bundle == 'file'&&fdata.id)
      {
        //can be file bundle node only
        this.C('file').findId(fdata.id).subscribe(fnode=>{
          if (!fnode)
            return this.obsEnd(obs,null)
          let fname = fnode.f('localPath').split('/').pop()
          let ret = {
            obj: fnode,
            source: fnode.f('localPath'),
            filename: fname,
            caption: fnode.f('name'),
          }
          this.obsEnd(obs,ret)
        },err=>{
          //console.log
          this.obsErr(obs,err)
        })
        return
      }
      this.obsEnd(obs,null)
    })
  }

  ctxFile(f,ctx)
  {
    return Rx.Observable.create(obs=>{
      log.warn('CTX')
      //console.log(f.path)
      let selector = ctx.fieldSelector(f.path)
      if (!selector)
        return
      selector.init(ctx).subscribe(x=>{
        if (!x)
          this.obsEnd(obs,null)
        let fdata = selector.oneModified()
        //console.log({fdata:fdata,selector:selector})
        let $t = []
        if (Array.isArray(fdata))
        {
          fdata.forEach(ffd=>{
            $t.push(this._ctxFile(ffd))
          })
        } else {
          $t.push(this._ctxFile(fdata))
        }
        Rx.concat(...$t).subscribe(x=>{
          //console.log([x])
          obs.next(x)
        },err=>{
          log.err('ctx file error')
          obs.complete()
        },()=>{
          obs.complete()
        })
      })
    })
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['exec text'])
      let data = this.f('data')
      //console.log(['file reaction',data])
      if (!data||!data.files||!data.files.length)
        return this.obsEnd(obs)
      let $t = []
      data.files.forEach(f=>{
        if (f.extension=='{CTX}')
        {
          $t.push(this.ctxFile(f,ctx))
          return
        }
        if (f.extension=='{QR}')
        {
          $t.push(this.genQR(f.caption,ctx))
          return
        }
        //console.log(['send file ???',f])
        $t.push(Rx.forkJoin(
          this.M('template').compile(f.filename,ctx),
          this.M('template').compile(f.caption,ctx),
          this.C('file').findNode({localPath:f.path})
        ).pipe(RxOps.map(compiles=>{
          if (!compiles)
            return null
          //console.log(['send file compiles ???',compiles])
          let fn = this.M('template').sanitizeHtml(compiles[0],true)
          let cp = this.M('template').sanitizeHtml(compiles[1])
          let ret = {
            obj: compiles[2],
            source: f.path,
            filename: fn,
            caption: cp
          }
          //console.log(['send file',ret])
          return ret
        })))
      })
      Rx.concat(...$t).subscribe(f=>{
        if (!f)
          return
        //console.log(f)
        $msg.addFile(f)
      },err=>{
        log.err('prepare files to send error')
        console.log(err)
        this.obsErr(obs,err)
      },()=>{
        this.obsEnd(obs)
      })
    })
  }

}

module.exports = FileReaction
