/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('GAReaction')
let g18n = require('../../../../g18n')

let uuid = require('uuid-v4')
let UA = require('universal-analytics')

const AbstractReaction = require('../reaction')

class GAReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'ga',
      name: g18n.t('Sending data to Google Analytics'),
      shortname: g18n.t('GA'),
      faicon: 'fas fa-chart-area',
      uicolor: 'text-warning',
      uigroup: 'helpers',
      routersLimit: ['telegrambotrouter'],
      mode: 'any', //message/reaction/any
      greedy: false,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.ga||!data.ga.length)
        return this.obsEnd(obs)

      let gaID = this.M('config').conf('ga.id')
      let gaAPP = this.M('config').conf('ga.app')
      if (!gaID||!gaAPP)
        return this.obsEnd(obs)

      let gUuid = ctx.user().f('uuid')||uuid()

      this.M('template').compile(data.ga,ctx).subscribe(res=>{
        if (!res||!res.length)
          return this.obsEnd(obs)
        let GA = UA(gaID,gUuid)
        GA.screenview(res,gaAPP).send()
        log.success('screenview sended',res)
        this.obsEnd(obs)
      })
    })
  }

}

module.exports = GAReaction
