/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let request = require('request')

let log = require('../../../../log').m('GroupReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')
const User = require('../../../auth/user/user')

class GroupReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'auth_group',
      name: g18n.t("Change user's group"),
      shortname: g18n.t('Auth Group'),
      help: g18n.t('Groups changed for context user. REQUIRES USER AT CONTEXT!'),
      faicon: 'fab fa-keycdn',
      uicolor: 'text-danger',
      uigroup: 'auth',
      routersLimit: [],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.auth_group)
        return this.obsEnd(obs)
      data = data.auth_group
      //console.log(['exec group',data])
      let $inc = []
      let $exc = []
      if (data.includes.length)
      {
        if (data.includes[0]=='guests')
        {
          //exclude from all
          $exc.push(this.C('usersgroup').findNodes({},{list:false}))
        } else {
          $inc.push(this.C('usersgroup').findNodes({code:{$in:data.includes}},{list:false}))
        }
      }
      if (data.excludes.length)
        $exc.push(this.C('usersgroup').findNodes({code:{$in:data.excludes}},{list:false}))

      ctx.selectorResult(data.user,{type:'relation',quantity:1}).pipe(RxO.catchError(_=>Rx.of(null))).subscribe(targetUser=>{
        //console.log(['targetUser',targetUser])
        if(targetUser && !(targetUser instanceof User))
        {
          log.err('target user defined but is not a user!')
          return this.obsErr(obs,{code:'CHANGE_USER_GROUP_REACTION_ERROR_TARGET_USER_IS_NOT_USER'})
        }
        
        if (!targetUser)
        {
          //костыль для совместимости со старыми версиями данных, в которых это поле не указано вообще
          //но должно работать в таком случае как будто "надо именно у сендера поменять"
          if (typeof data.senderIsDefault == 'undefined')
            data.senderIsDefault = true
          if (!data.senderIsDefault)
          {
            log.err('target user not found!')
            return this.obsErr(obs,{code:'CHANGE_USER_GROUP_REACTION_ERROR_TARGET_USER_NOT_FOUND'})
          }
          targetUser = ctx.user()
        }

        let $t = []
        $exc.forEach(exc=>{
          $t.push(exc.pipe(RxO.mergeMap(scg=>{ 
            if (!scg) return Rx.of(null)
            //console.log(scg)
            scg.delUser(targetUser)
            return scg.save()
          })))
        })
        $inc.forEach(inc=>{
          $t.push(inc.pipe(RxO.mergeMap(scg=>{
            if (!scg) return Rx.of(null)
            scg.addUser(targetUser)
            return scg.save()
          })))
        })
        Rx.concat(...$t).subscribe(scgs=>{
          log.info('group updated',scgs.f('code'))
        },err=>{
          log.err('error updating groups')
          console.log(err)
          this.obsEnd(obs)
        },()=>{
          log.success('user groups updated')
          this.obsEnd(obs)
        })
      })
    })
  }


}

module.exports = GroupReaction
