/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('IsolateReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class IsolateReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'isolate',
      name: g18n.t('Provide isolation reaction for bots'),
      shortname: g18n.t('Isolate'),
      faicon: 'fas fa-user-lock',
      uicolor: 'text-danger',
      uigroup: 'auth',
      routersLimit: ['telegrambotrouter'],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }




  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['exec kbd'])
      let data = this.f('data')
      if (!data||!data.isolate)
        return this.obsEnd(obs)
      data = data.isolate
      //console.log('isolate',data)
      let u = ctx.user()
      if (!data.mode)
      {
        u.field('isolateScope').clear()
        u.field('isolateScreen').clear()
        u.field('isolateScreengroup').clear()
        u.save().subscribe(su=>{
          ctx.user(0,su)
          this.obsEnd(obs)
        },err=>{
          log.err('unisolate user failed!')
          console.log(err)
          this.obsErr(obs)
        })
        return
      }

      let where = 'curr'
      if (typeof data.target != 'undefined' && data.target != '')
        where = data.target
      if (where=='curr')
        where = ctx.chat().f('type')

      this.C('screengroup').findNode({code:data.screengroup},{populate:1}).subscribe(scg=>{
        if (!scg)
          return this.obsEnd(obs)
        if (data.scope=='screengroup')
        {
          u.f('isolateScope',where)
          u.f('isolateScreengroup',scg)
          u.save().subscribe(su=>{
            this.obsEnd(obs)
          },err=>{
            log.err('isolate user failed!')
            console.log(err)
            this.obsErr(obs)
          })
          return
        }
        let sc = scg.field('screens').reduce((acc,x)=>x.eq(data.screen)?x:acc,null)
        if (!sc)
        {
          log.err('search screen to isolate failed')
          this.obsErr(obs)
          return
        }
        u.f('isolateScope',where)
        u.f('isolateScreen',sc)
        u.f('isolateScreengroup',scg)
        u.save().subscribe(su=>{
          this.obsEnd(obs)
        },err=>{
          log.err('isolate user failed!')
          console.log(err)
          this.obsErr(obs)
        })
      },err=>{
        log.err('search screengroup to isolate failed')
        console.log(err)
        this.obsErr(obs)
      })


    })
  }
 

}

module.exports = IsolateReaction
