/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('IsolateReactionModule')
let g18n = require('../../../../g18n')

const CoreModule = require('../../../core/abstract/module')

const IsolateReaction = require('./isolate')

class IsolateReactionModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

    this._reactionClasses = {}

    this.M('bot/reactions').registerReactionType(IsolateReaction)
    this.M('actions').registerAction(IsolateReaction)
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      log.info('modifying user bundle')
      let bndUser = this._core.module('bundle').bundle('user')
      bndUser.addField({name:g18n.t('Isolate scope'),code:'isolateScope',type:'string', quantity:1, flags:['system','logic','locked']})
      bndUser.addField({name:g18n.t('Isolate screen'),code:'isolateScreen',type:'relation', quantity:1, conf:{ops:[{source:'manual',manual:'screen'}]}, flags:['system','logic','locked']})
      bndUser.addField({name:g18n.t('Isolate screengroup'),code:'isolateScreengroup',type:'relation',conf:{ops:[{source:'manual',manual:'screengroup'}]},quantity:1,flags:['system','logic','locked']})
      $t.push(bndUser.save())
      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing screens failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }



}

module.exports = IsolateReactionModule
