/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')
let request = require('request')

let log = require('../../../../log').m('JsonReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class JsonReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'json',
      name: g18n.t('Grabs JSON data from url'),
      shortname: g18n.t('JSON'),
      faicon: 'fas fa-parachute-box',
      uicolor: 'text-info',
      uigroup: 'helpers',
      routersLimit: [],
      mode: 'message', //message/reaction/any
      greedy: true,
      conflicts: ['*'],
      requireSend: false,
      doSend: false
    }
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      //console.log(['exec json',data,this])
      if (!data||!data.json)
        return this.obsEnd(obs)
      data = data.json
      this.M('template').compile(data.url,ctx).subscribe(url=>{
        log.info('requesting',url)
        this.requestJson(url).subscribe(obj=>{
          ctx.addField({
            name: g18n.t('JSON reaction result'),
            code: data.rootNode,
            type: 'any',
            quantity: data.isarray?0:1,
            flags: ['readonly','virtual','logic','template']
          })
          ctx.f(data.rootNode,obj)
          this.obsEnd(obs)
        },err=>{
          this.obsEnd(obs)
        })
      },err=>{
        log.err('url template error!')
        this.obsEnd(obs)
      })
    })
  }

  requestJson(url)
  {
    return Rx.Observable.create(obs=>{
      request(encodeURI(url),(err,resp,body)=>{
        if (err)
          return this.obsErr(obs)
        let ret = null
        try {
          ret = JSON.parse(body)
        } catch(e){
          return this.obsErr(obs)
        }
        this.obsEnd(obs,ret)
      })
    })
  }


}

module.exports = JsonReaction
