/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('KeyboardReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class KeyboardReaction extends AbstractReaction
{


  static info()
  {
    return {
      code: 'keyboard',
      name: g18n.t('Fast reply buttons bot reaction'),
      shortname: g18n.t('Keyboard'),
      faicon: 'fas fa-keyboard',
      uicolor: 'text-success',
      uigroup: 'tgmessage',
      routersLimit: ['telegrambotrouter'],
      mode: 'reaction', //message/reaction/any
      greedy: true,
      conflicts: ['buttons'],
      requireSend: true,
      doSend: false
    }
  }




  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      //console.log(['exec kbd'])
      let data = this.f('data')
      if (!data||!data.keyboard)
        return this.obsEnd(obs)
      let kbd = data.keyboard
      //console.log('keyboard',kbd)
      let $kbd = []
      kbd.forEach((row,rowi)=>{
        let $row = []
        row.forEach((btn,btni)=>{
          $row.push(this._buildButton(ctx,btn,rowi,btni))
        })
        let $build = Rx.concat(...$row).pipe( RxOps.map(res=>res) )
        $kbd.push($build)
      })
      let ret = []
      let rowShift = 0
      Rx.concat(...$kbd).subscribe(br=>{
        if (this._isButton(br.obj))
        {
          if (!ret[br.row+rowShift])
            ret[br.row+rowShift] = []
          ret[br.row+rowShift].push(br.obj)
        }
        if (this._isRow(br.obj))
        {
          if (!ret[br.row+rowShift])
            ret[br.row+rowShift] = br.obj
          else
            br.obj.forEach(x=>ret[br.row+rowShift].push(x))
        }
        if (this._isMenu(br.obj))
        {
          br.obj.forEach(mr=>ret.push(mr))
          rowShift+=br.obj.length
        }
      },err=>{
        console.log(err)
      },()=>{
        ret = ret.filter(x=>this._isRow(x))
        $msg.f('keyboard',ret)
        //console.log(['build done',ret])
        this.obsEnd(obs)
      })
    })
  }

  _buildButton(ctx,btn,rowi,btni)
  {
    return Rx.Observable.create(obs=>{
      let txt = btn.key
      if (txt.startsWith('{{virtual.')&&txt.endsWith('}}'))
      {
        let path = txt.slice(2,-2)
        ctx.dataByPath(path).subscribe(ndres=>{
          if (!ndres)
            return this.obsEnd(obs,{
              row: rowi,
              col: btni,
              obj: ''
            })
          this.obsEnd(obs,{
            row: rowi,
            col: btni,
            obj: ndres
          })
        })
        return
      }
      this.M('template').compile(txt,ctx).subscribe(res=>{
        this.obsEnd(obs,{
          row: rowi,
          col: btni,
          obj: res
        })
      })
    })
  }

  _isButton(el)
  {
    
    if( typeof el == 'string'&&el.length)
      return true
    if (typeof el =='number')
      return true
    return false
  }

  _isRow(arr)
  {
    if (!Array.isArray(arr)) return false
    let ok = true
    arr.forEach(el=>{
      if (!this._isButton(el))
        ok = false
    })
    return ok
  }

  _isMenu(arr)
  {
    if (!Array.isArray(arr)) return false
    let ok = true
    arr.forEach(el=>{
      if (!this._isRow(el))
        ok = false
    })
    return ok
  }
 

}

module.exports = KeyboardReaction
