/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('KickReaction')
let g18n = require('../../../../g18n')

let uuid = require('uuid-v4')
let UA = require('universal-analytics')

const AbstractReaction = require('../reaction')

class KickReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'kick',
      name: g18n.t('Kick user from chat bot reaction'),
      shortname: g18n.t('Kick/Ban'),
      faicon: 'fas fa-ban',
      uicolor: 'text-danger',
      uigroup: 'chatadmin',
      routersLimit: ['telegrambotrouter'],
      mode: 'any', //message/reaction/any
      greedy: false,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.kick||!data.kick.user||!data.kick.chat)
        return this.obsEnd(obs)
      data = data.kick
      log.warn('kick!',data)
      Rx.forkJoin(
        ctx.nodeByPath(data.user,true),
        ctx.nodeByPath(data.chat,true)
      ).subscribe(cmpls=>{
        let user = cmpls[0]
        let chat = cmpls[1]
        if (!user||!chat)
          return this.obsErr(obs,{code:'NO_USER_OR_CHAT_TO_BAN'})
        if (!user.bundle||user.bundle()!='user')
          return this.obsErr(obs,{code:'NO_USER_TO_BAN'})
        if (!chat.bundle||chat.bundle()!='tgchat')
          return this.obsErr(obs,{code:'NO_CHAT_TO_BAN'})
        //console.log([[user,chat]])
        let mdl = this.M('telegram')
        if (ctx.vnode('interface'))
          mdl = ctx.vnode('interface')
        mdl.kickUser(user,chat).subscribe(()=>{
          this.obsEnd(obs)
        },err=>{
          log.err('ban not success!')
          console.log(err)
          this.obsErr(obs,err)
        })
      },err=>{
        log.err('extracting data from context failed')
        console.log(err)
        this.obsErr(obs,err)
      })
    })
  }

}

module.exports = KickReaction
