/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('ReactionsModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const AbstractReaction = require('./reaction')
const AbstractAction = require('../../actions/abstract')


class ReactionsModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

    this._reactionClasses = {
      'abstract': AbstractReaction
    }
  }

  registerReactionType(Cls)
  {
    /*
    if (!(AbstractReaction.isPrototypeOf(Cls)||AbstractAction.isPrototypeOf(Cls)))
    {
      log.err('registring reaction type failed',Cls)
      return
    }
    /**/
    let inf = Cls.info()
    log.log('registring reaction type',inf.code)
    if (typeof this._reactionClasses[inf.code] != 'undefined')
      log.warn('already registered! overwriting',inf.code)
    this._reactionClasses[inf.code] = Cls
  }

  reactionClassesInfo()
  {
    return Object.keys(this._reactionClasses).reduce((acc,x)=>{
      if (x=='abstract') return acc
      acc[x] = this._reactionClasses[x].info()
      return acc
    },{})
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      this.C('screenreaction').setNodeClass(AbstractReaction)
      this.C('screenreaction').setNodeClassFn(mdl=>{
        if (!mdl||!mdl.category) return AbstractReaction
        //console.log(['asking',mdl.category,this._reactionClasses[mdl.category]])
        if(this._reactionClasses[mdl.category]) return this._reactionClasses[mdl.category]
        //console.log(['asking but return abstract'])
        return AbstractReaction
      })

      //let $t = []
      //let $t = [this.M('screen/group').moduleInit()]

      //Rx.concat(...$t).subscribe(()=>{},err=>{obs.error(err)},()=>{
        log.log('initialized')
        this.obsEnd(obs)
        //obs.next()
        //obs.complete()
      //})
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlReaction = this._core.module('bundle').bundle('screenreaction',true)
      bndlReaction.name(g18n.t('Screen reactions'))
      bndlReaction.desc(g18n.t('Telegram screen messages reactions'))
      bndlReaction.flags(['system','locked'])
      bndlReaction.addField({name:g18n.t('Category'),code:'category',type:'string',flags:['system','required','logic']})
      bndlReaction.addField({name:g18n.t('Global name'),code:'globalName',type:'string',flags:['system','logic']})
      bndlReaction.addField({name:g18n.t('Data'),code:'data',type:'any',flags:['system','logic']})
      
      log.info('saving bundle',bndlReaction.code())
      $t.push(bndlReaction.save(true,true))

      //$t.push(this.M('reactions/isolate').moduleInstall())

      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      $t.push(this.M('auth/role').roleSave('reaction','root','Allows use any reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','text','Allows use text reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','keyboard','Allows use keyboard reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','buttons','Allows use buttons reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','file','Allows use file reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','wallet_recognition','Allows use wallet_recognition reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','push','Allows use push reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','user_login','Allows use user_login reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','user_logout','Allows use user_logout reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','auth_group','Allows use auth_group reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','field','Allows use field reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','exec','Allows use exec reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','json','Allows use json reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','isolate','Allows use isolate reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','library','Allows use library reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','loop','Allows use loop reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','kick','Allows use kick reaction'))
      $t.push(this.M('auth/role').roleSave('reaction','ga','Allows use ga reaction'))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  _update02to011()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>this.M('module_manager').installAndStart('bot/reactions/loop'))
    )
  }



  _update01to02()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>Rx.concat(
        this.M('auth/role').roleSave('reaction','root',g18n.t('Bot reactions root')),
      ).pipe(RxO.last()))
    )
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1'&&to=='0.2')
          return this._update01to02()
        if (from=='0.2'&&to=='0.1.1')
          return this._update02to011()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }



}

module.exports = ReactionsModule
