/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('NodeReaction')
let g18n = require('../../../../g18n')

//let uuid = require('uuid-v4')
//let UA = require('universal-analytics')

const AbstractReaction = require('../reaction')
const LogicOperation = require('../../../logic/abstract/operation')

class NodeReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'node',
      name: g18n.t('Create new node'),
      shortname: g18n.t('Node'),
      uigroup: 'library',
      routersLimit: [],
      faicon: 'fas fa-plus-square',
      uicolor: 'text-warning',
      mode: 'message', //message/reaction/any
      greedy: true,
      conflicts: ['*'],
      requireSend: false,
      doSend: false,
      confs: []
      /**
      confs: [{
        name: g18n.t('Target bundle'),
        code: 'target',
        desc: g18n.t('Bandle of new node'),
        allowManual: true,
        allowGenerators: false,
        type: 'relation'
      },{
        name: g18n.t('Context code'),
        desc: g18n.t('Context code at virtuals. to save set new node'),
        allowManual: true,
        allowGenerators: false,
        code: 'ctxCode',
        type: 'string'
      }]
      /**/
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.node)
        return this.obsErr(obs,{code:'NO_DATA'})

      data = data.node
      let C = this.C(data.target)
      if (!C)
        return this.obsErr(obs,{code:'NO_DATA_TARGET_COLLECTION'})

      C.initNode(ctx).subscribe(nd=>{
        let $s = C.bundle().fields().map(f=>{
          let cfg = data.fields[f.code]
          let op = new LogicOperation(this,'setter',{setter:cfg})
          //let sel = ctx.fieldSelector(cfg,f)
          return op.initOnNodeField(nd,f,ctx).pipe(
            RxO.mergeMap(initRes=>{
              //if (initRes)
                return op.execute(ctx)
              //return Rx.of(null)
            })
          )
        })
        Rx.concat(...$s).subscribe(_=>{},err=>this.obsErr(obs,err),_=>{
            //console.log(['node!',nd.toObject()])
            //this.obsEnd(obs)
          /**/
          nd.save().subscribe(snd=>{
            ctx.addField({
              name: g18n.t('Created node'),
              code: data.code,
              type: 'relation',
              xtypes: ['relation','relation:'+data.target],
              conf: {ops:[{source:data.target,manual:''},{source:'',manual:''}]},
              quantity: 1,
              flags: ['virtual','logic','template']
            })
            ctx.f(data.code,snd)
            ctx.addNode(snd,data.code)
            this.obsEnd(obs)
          },err=>this.obsErr(obs,err))
          /**/
        })
      },err=>this.obsErr(obs,err))
    })
  }

}

module.exports = NodeReaction
