/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('RemoveNodeReaction')
let g18n = require('../../../../g18n')

//let uuid = require('uuid-v4')
//let UA = require('universal-analytics')

const AbstractReaction = require('../reaction')
const Node = require('../../../node/node')

class RemoveNodeReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'delnode',
      name: g18n.t('Remove node/s reaction'),
      shortname: g18n.t('Remove Node/s'),
      faicon: 'fas fa-minus-square',
      uicolor: 'text-warning',
      uigroup: 'library',
      routersLimit: [],
      mode: 'message', //message/reaction/any
      greedy: true,
      conflicts: ['*'],
      requireSend: false,
      doSend: false,
      confs: [{
        name: g18n.t('Node to delete'),
        code: 'oneNode',
        desc: g18n.t('Single node to delete'),
        allowManual: false,
        allowGenerators: false,
        type: 'relation'
      },{
        name: g18n.t('Nodes to delete'),
        code: 'listNodes',
        desc: g18n.t('List of nodes to delete'),
        allowManual: false,
        allowGenerators: false,
        type: '[relation]'
      }]
      /**/
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.delnode)
        return this.obsErr(obs,{code:'NO_DATA'})

      data = data.delnode
      //console.log(['!!DEL',data])
      this.obsEnd(obs)

      let dellist = []
      /**/

      ctx.nodeByPath(data.oneNode.target).subscribe(resOne=>{
        //console.log(['resOne',resOne])
        if (resOne&&resOne.node&&typeof resOne.node.f == 'function')
          dellist.push(resOne.node.f(resOne.fcode))

        ctx.nodeByPath(data.listNodes.target).subscribe(resList=>{
          //console.log(['resList',resList])
          if (resList&&resList.node && typeof resList.node.f == 'function')
          {
            let lst = resList.node.field(resList.fcode).all()
            if (Array.isArray(lst))
              dellist = [...dellist,...lst]
          }
          let dels = {}
          dellist.forEach(x=>{
            if(!(x instanceof Node)) return
            if(x.bundle()=='virtual') return
            let C = this.C(x.bundle())
            if (!C) return
            //console.log(C.bundle())
            if (C.bundle().hasFlag('locked')) return
            if (!dels[x.bundle()])
              dels[x.bundle()] = []
            dels[x.bundle()].push(x.id())
            return true
          })
          //console.log(['dels',dels])
          let $t = Object.keys(dels).map(bndl=>{
            return this.C(bndl).removeNodes({_id:{$in:dels[bndl]}}).pipe(RxO.catchError(_=>Rx.of(_)))
          })
          if (!$t.length)
            return this.obsEnd(obs)
          Rx.concat(...$t).subscribe(_=>{},err=>this.obsErr(obs,err),_=>{
            this.obsEnd(obs)
          })
        },err=>this.obsErr(obs,err))
        /*
        if (!ndd||!ndd.node||typeof ndd.node.f != 'function')
          return this.obsEnd(obs)

        let $nd = ndd.node.f(ndd.fcode)
        if (!$nd || !($nd instanceof Node) || $nd.bundle() != data.target)
          return this.obsErr(obs,{code:'TARGET_NODE_IS_ANOTHER_BUNDLE'})

        if (!Array.isArray(data.selected)||!data.selected.length)
          return this.obsEnd(obs)

        let $nx = C.bundle().fields().filter(f=>data.selected.includes(f.code)).map(f=>{
          let cfg = data.fields[f.code]
          let sel = ctx.fieldSelector(cfg,f)
          return sel.init(ctx).pipe(RxO.map(x=>{
            let v = sel.oneModified()
            $nd.f(f.code,v)
            return v
          }))
        })
        Rx.concat(...$nx).subscribe(_=>{},err=>this.obsErr(obs,err),_=>{
          $nd.save().subscribe(snd=>{
            this.obsEnd(obs)
              
          },err=>this.obsErr(obs,err))
        })
        //console.log(['data!updateNode',data,[$nd.toObject()]])
        /**/

      },err=>this.obsErr(obs,err))
    })
  }

}

module.exports = RemoveNodeReaction
