/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('UpdateNodeReaction')
let g18n = require('../../../../g18n')

//let uuid = require('uuid-v4')
//let UA = require('universal-analytics')

const AbstractReaction = require('../reaction')
const Node = require('../../../node/node')
const LogicOperation = require('../../../logic/abstract/operation')

class UpdateNodeReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'updnode',
      name: g18n.t('Update node reaction'),
      shortname: g18n.t('Update Node'),
      faicon: 'fas fa-pen-square',
      uicolor: 'text-warning',
      uigroup: 'library',
      routersLimit: [],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: [],
      requireSend: false,
      doSend: false,
      confs: []
      /**
      confs: [{
        name: g18n.t('Target bundle'),
        code: 'target',
        desc: g18n.t('Bandle of new node'),
        allowManual: true,
        allowGenerators: false,
        type: 'relation'
      },{
        name: g18n.t('Context code'),
        desc: g18n.t('Context code at virtuals. to save set new node'),
        allowManual: true,
        allowGenerators: false,
        code: 'ctxCode',
        type: 'string'
      }]
      /**/
    }
  }



  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.updnode)
        return this.obsErr(obs,{code:'NO_DATA'})

      data = data.updnode
      let C = this.C(data.target)
      if (!C)
        return this.obsErr(obs,{code:'NO_DATA_TARGET_COLLECTION'})

      ctx.nodeByPath(data.node.target).subscribe(ndd=>{
        if (!ndd||!ndd.node||typeof ndd.node.f != 'function')
          return this.obsEnd(obs)

        let $nd = ndd.node.f(ndd.fcode)
        if (!$nd || !($nd instanceof Node) || $nd.bundle() != data.target)
          return this.obsErr(obs,{code:'TARGET_NODE_IS_ANOTHER_BUNDLE'})

        if (!Array.isArray(data.selected)||!data.selected.length)
          return this.obsEnd(obs)

        let $nx = C.bundle().fields().filter(f=>data.selected.includes(f.code)).map(f=>{
          let cfg = data.fields[f.code]
          let sel = ctx.fieldSelector(cfg,f)
          let op = new LogicOperation(this,'setter',{setter:cfg})
          return op.initOnNodeField($nd,f,ctx).pipe(
            RxO.mergeMap(initRes=>{
              //if (initRes)
                return op.execute(ctx)
              //return Rx.of(null)
            })
          )
 
          //return sel.init(ctx).pipe(RxO.map(x=>{
            //let v = sel.oneModified()
            //$nd.f(f.code,v)
            //return v
          //}))
        })
        Rx.concat(...$nx).subscribe(_=>{},err=>this.obsErr(obs,err),_=>{
          $nd.save().subscribe(snd=>{
            this.obsEnd(obs)
              
          },err=>this.obsErr(obs,err))
        })
        //console.log(['data!updateNode',data,[$nd.toObject()]])

      },err=>this.obsErr(obs,err))
      /*
      C.initNode(ctx).subscribe(nd=>{
        let $s = C.bundle().fields().map(f=>{
          let cfg = data.fields[f.code]
          let sel = ctx.fieldSelector(cfg,f)
          return sel.init(ctx).pipe(RxO.map(x=>{
            let v = sel.oneModified()
            nd.f(f.code,v)
            return v
          }))
        })
        Rx.concat(...$s).subscribe(_=>{},err=>this.obsErr(obs,err),_=>{
          nd.save().subscribe(snd=>{
            ctx.addField({
              name: g18n.t('Created node'),
              code: data.code,
              type: 'relation',
              xtypes: ['relation','relation:'+data.target],
              conf: {ops:[{source:data.target,manual:''},{source:'',manual:''}]},
              quantity: 1,
              flags: ['virtual','logic','template']
            })
            ctx.f(data.code,snd)
            this.obsEnd(obs)
          },err=>this.obsErr(obs,err))
        })
      },err=>this.obsErr(obs,err))
      /**/
    })
  }

}

module.exports = UpdateNodeReaction
