/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('PushReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')
const Context = require('../../../context/context')

class PushReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'push',
      name: g18n.t('Push message/s'),
      shortname: g18n.t('Push'),
      desc: g18n.t('Push message/s to defined user/users'),
      help: g18n.t('When used as main action - message to send builded from subreacrions and reply text sends to main context user on push complete. When used as subreaction - push only text message from config text.'),
      uigroup: 'tgmessage',
      routersLimit: ['telegrambotrouter'],
      faicon: 'fas fa-bullhorn',
      uicolor: 'text-warning',
      mode: 'any', //message/reaction/any
      greedy: false,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }




  _finalize(data,ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      if (this.rI()>0||this._alreadyFinalized)
        return this.obsEnd(obs)
      this._areadyFinalized = true
      $msg.f('keyboard',[])
      $msg.f('inline',[])
      $msg.f('files',[])
      $msg.field('sticker').clear()
      $msg.field('location').clear()
      $msg.field('contact').clear()
      this.M('template').compile(data.text,ctx).subscribe(text=>{
        $msg.f('text',text)
        this.obsEnd(obs)
      },err=>{
        log.err('result push text templating error')
      })
    })
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.push)
      {
        this._finalize(data,ctx,$msg).subscribe(()=>{
          this.obsEnd(obs)
        })
        return
      }
      data = data.push
      //log.test('search recievers for push',data)
      //this._recievers(data,ctx,$msg).subscribe(list=>{
      this.M('auth/user').userFieldUsers(data.to,ctx).subscribe(list=>{
        log.test('search recievers for push',[list.length])
        if (!list||!list.length)
          return this.obsEnd(obs)
        this.M('template').compile(data.text,ctx).subscribe(b4text=>{
          let $t = []
          let msgs = []
          let tosend = list.length
          this._areadyFinalized = false
          list.forEach(chat=>{
            let msg = this.C('tgmessage').newNode()
            msg.open()
            let mdl = this.M('telegram')
            if (ctx.vnode('interface'))
              mdl = ctx.vnode('interface')
            msg.setIface(mdl)
            msg.f('chat',chat)
            msg.f('type',ctx.type())
            msg.f('sender',mdl.me())
            msg.f('screen',$msg.f('screen'))
            if (this.rI()>0)
            {
              msg.f('text',b4text)
            } else {
              msg.f('text',$msg.f('text'))
              msg.f('keyboard',$msg.f('keyboard'))
              msg.f('inline',$msg.f('inline'))
              msg.f('files',$msg.f('files'))
              msg.f('sticker',$msg.f('sticker'))
              msg.f('location',$msg.f('location'))
              msg.f('contact',$msg.f('contact'))
            }
            let s = msg.onSended.pipe(RxOps.take(1))
            s.subscribe(sr=>{
              if (sr)
              {
                if (sr.success)
                  ctx.f('pushed',ctx.f('pushed')+1)
                  //ctx._stats.pushed++
                if (sr.error)
                  ctx.f('error',ctx.f('error')+1)
                  //ctx._stats.error++
              }
              tosend--
              //console.log(['srsbs',sr,tosend])
              if (tosend<1)
              {
                log.success('push done!')
                this._finalize(data,ctx,$msg).subscribe(()=>{
                  this.obsEnd(obs)
                })
              }
            },err=>{
              log.warn('push error!')
              this._finalize(data,ctx,$msg).subscribe(()=>{
                this.obsEnd(obs)
              })
            })
            //log.test('push msg',msg.toObject())
            msg.send()
          })
        },err=>{
          log.err('templateing b4 error')
          console.log(err)
          this.obsEnd(obs)
        })
      },err=>{
        log.err('recivers search error')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }

  /*
  pushText(data,ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      this._recievers(data,ctx,$msg).subscribe(list=>{
        if (!list||!list.length)
          return this.obsEnd(obs)
        this.M('template').compile(data.text,ctx).subscribe(text=>{
          
          console.log(['pushing TEXT only',data,text])
          let $t = []
          let msgs = []
          list.forEach(chat=>{
            let msg = this.C('tgmessage').newNode()
            msg.open()
            msg.f('chat',chat)
            msg.f('text',text)
            let s = msg.onSended.pipe(RxOps.take(1))
            $t.push(s)
            msgs.push(msg)
          })
          Rx.concat(...$t).subscribe(()=>{
            ctx._stats.pushed++
          },err=>{
            this.obsEnd(obs)
          },()=>{
            this.obsEnd(obs)
          })
          msgs.forEach(msg=>msg.send())
        },err=>{
          log.err('compiling pish text error')
          console.log(err)
          this.obsEnd(obs)
        })
      },err=>{
        log.err('recivers search error')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }

  pushMsg(data,ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      this._recievers(data,ctx,$msg).subscribe(list=>{

        let $tt = []
        let msgs = []
        let done = list.length
        list.forEach(chat=>{
          //console.log(['push to chat',[chat.toObject()]])
          let msg = this.C('tgmessage').newNode()
          msg.open()
          msg.f('chat',chat)
          msg.f('text',$msg.f('text'))
          msg.f('keyboard',$msg.f('keyboard'))
          msg.f('inline',$msg.f('inline'))
          msg.f('files',$msg.f('files'))
          msg.f('sticker',$msg.f('sticker'))
          msg.f('location',$msg.f('location'))
          msg.f('contact',$msg.f('contact'))
          let s = msg.onSended.pipe(RxOps.take(1))
          $tt.push(s)
          msgs.push(msg)
        })
        //console.log($t)
        Rx.concat(...$tt).subscribe((rr)=>{
          console.log(['??',rr])
        },err=>{
          log.err('error on push!')
          this.M('template').compile(data.text,ctx).subscribe(text=>{
            $msg.f('text',text)
            $msg.f('keyboard',[])
            $msg.f('inline',[])
            $msg.f('files',[])
            $msg.field('sticker').clear()
            $msg.field('location').clear()
            $msg.field('contact').clear()
            this.obsEnd(obs)
          })
        },()=>{
          log.success('push completed!')
          this.M('template').compile(data.text,ctx).subscribe(text=>{
            $msg.f('text',text)
            $msg.f('keyboard',[])
            $msg.f('inline',[])
            $msg.f('files',[])
            $msg.field('sticker').clear()
            $msg.field('location').clear()
            $msg.field('contact').clear()
            this.obsEnd(obs)
          })
        })
        msgs.forEach(msg=>msg.send())
      },err=>{
        log.err('recivers search error')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }
/**/

}

module.exports = PushReaction
