/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('AbstractReaction')
let g18n = require('../../../g18n')

const Node = require('../../node/node')
const ScreenNode = require('../screen/screen')
const AbstractMessage = require('../messages/message')

class AbstractReaction extends Node
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._type = 'screenmessagereactionnode'
    this._screen = null
    this._message = null
    this._rI = 0
  }

  static info()
  {
    return {
      code: 'abstract',
      name: g18n.t('Abstract bot reaction'),
      faicon: 'fas fa-question',
      uicolor: 'text-warning',
      //message = as main reaction only
      //reaction = as subreaction only
      //any = can be any
      mode: 'message', 
      //can be only one in action/message block
      greedy: true,
      //codes of reactions, can't use together in same action/message block
      conflicts: [],
      //can be used only when other reaction with 'doSend:true' exists
      requireSend: false,
      //this reaction do some real whatever sending via bot
      doSend: false
    }
  }

  loadConfs(ctx)
  {
    return Rx.Observable.create(obs=>{
      let inf = this.constructor.info()
      let ret = {}
      if (!inf||!Array.isArray(inf.confs)||!inf.confs.length)
        return this.obsEnd(obs,ret)
          //ctx.selectorResult(data.user),
      let data = this.f('data')
      if (!data||!data[inf.code])
        return this.obsEnd(obs,ret)
      data = data[inf.code]
      let $t = inf.confs.map(cfg=>{
        return ctx.selectorResult(data[cfg.code],cfg).pipe(RxO.map(v=>{
          ret[cfg.code] = v
          return v
        }))
      })
      Rx.concat(...$t).subscribe(_=>{},err=>{
        this.obsEnd(obs,ret)
      },_=>{
        this.obsEnd(obs,ret)
      })
    })
  }

  screen(val)
  {
    if (val&&(val instanceof ScreenNode))
      this._screen = val
    return this._screen
  }

  message(val)
  {
    if (val&&(val instanceof AbstractMessage))
      this._message = val
    return this._message
  }

  rI(val)
  {
    if (typeof val != 'undefined')
      this._rI = val
    return this._rI
  }
  
  category() { return this.f('category') }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      log.err('abstract reaction executing!',this.category())
      this.obsEnd(obs)
    })
  }

  _update01to011()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        return this.M('module_manager').installAndStart('bot/reactions/node')
      })
    )
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1'&&to=='0.1.1')
          return this._update01to011()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }
 


}

module.exports = AbstractReaction
