/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('TGMsgDelReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

const TGMsg = require('../../../telegram/nodes/message')
const TGChat = require('../../../telegram/nodes/chat')
const User = require('../../../auth/user/user')

class TGMsgDelReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'tgmsgdel',
      name: g18n.t('TG Message Delete'),
      shortname: g18n.t('TGM Delete'),
      desc: g18n.t('Removes message from telegram chat history'),
      help: g18n.t('Message to remove should exists in context'),
      faicon: 'fas fa-eraser',
      uicolor: 'text-danger',
      uigroup: 'chatadmin',
      routersLimit: ['telegrambotrouter'],
      mode: 'message', //message/reaction/any
      greedy: false,
      conflicts: ['*'],
      requireSend: false,
      doSend: false,
      confs: []
      /*
      confs: [{
        name: g18n.t('message'),
        code: 'target',
        desc: g18n.t('Telegram message to update'),
        allowManual: false,
        allowGenerators: false,
        type: 'relation',
        xtype: 'relation:tgmessage'
      }]
      /**/
    }
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.tgmsgdel)
        return this.obsErr(obs,{code:'NO_DATA'})
      data = data.tgmsgdel
      let dels = []
      let $load = null
      //console.log(['DEL data',data])
      if (data.type=='msg')
        $load = ctx.selectorResult(data.msg).pipe(RxO.map(x=>{
          return { msg: x || null }
        }))
      if (data.type=='user')
        $load = Rx.forkJoin(
          ctx.selectorResult(data.user),
          ctx.selectorResult(data.chat)
          //ctx.selectorResult(data.amount,{type:'number',code:'amount'})
        ).pipe(RxO.map(x=>{
          return {
            user: x[0] || null,
            chat: x[1] || null
            //amount: parseInt(x[2]) || 0
          }
        }))
      /*
      if (data.type=='chat')
        $load = ctx.selectorResult(data.chat).pipe(RxO.map(x=>{
          return {
            chat: x || null
          }
        }))
      /**/

      if (!$load)
        return this.obsEnd(obs)
      $load.subscribe(x=>{
        //console.log(['data loaded',x])
        if (!x)
          return this.obsEnd(obs)
        let $msgsload = null
        if (data.type=='msg')
        {
          if (!x.msg||!(x.msg instanceof TGMsg))
            return this.obsEnd(obs)
          $msgsload = this.C('tgmessage').findNodes({_id:x.msg.id()},{populate:1})
        }
        if (data.type=='user')
        {
          if (!x.user||!(x.user instanceof User))
            return this.obsEnd(obs)
          let q = {
            sender: x.user.id()
          }
          let $cload = Rx.of(null)
          if (x.chat&&x.chat instanceof TGChat)
            q.chat = x.chat.id()
          $msgsload = this.C('tgmessage').findNodes(q,{populate:1})
        }
        if (!$msgsload)
          return this.obsEnd(obs)
        $msgsload.subscribe(list=>{
          //console.log(['messages to delete!',list])
          let mdl = this.M('telegram')
          if (ctx.vnode('interface'))
            mdl = ctx.vnode('interface')
          mdl.deleteMessages(list).subscribe(res=>this.obsEnd(obs),err=>this.obsErr(obs,err))
        },err=>this.obsErr(obs,err))
      })
    })
  }

}

module.exports = TGMsgDelReaction
