/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('TGMEditMessage')
let g18n = require('../../../../g18n')

const AbstractMessage = require('../../messages/message')
const VNode = require('../../../node/vnode')

class TGMEditMessage extends AbstractMessage
{

  execute(ctx) //loop message executing
  {
    return Rx.Observable.create(obs=>{
      this.M('logic').testLogicRx(this,ctx).subscribe(logicOk=>{
        if (!logicOk)
        {
          ctx.f('fail',ctx.f('fail')+1)
          return this.obsEnd(obs)
        }


        let rct = this.field('reactions').getp(0)
        let data = rct.f('data')
        if (!data||!data.tgmsgedit)
          return this.obsErr(obs,{code:'NO_DATA_TO_GET_UPDATE_MESSAGE'})
        data = data.tgmsgedit

        if (!data||!data.target||!data.target.target)
          return this.obsErr(obs,{code:'NO_DATA_TO_GET_UPDATE_MESSAGE'})

        ctx.fieldByPath(data.target.target).subscribe(emsgf=>{
          if (!emsgf)
            return this.obsErr(obs,{code:'NO_CONTEXT_MESSAGE_FIELD_TO_UPDATE'})
          let $msg = emsgf.getp()
          if (!$msg)
            return this.obsErr(obs,{code:'NO_MESSAGE_TO_UPDATE'})

          $msg._update = true
          $msg.setIface(ctx.vnode('interface'))
          $msg.fp('chat',ctx.chat())
          $msg.open()
          let $t = []
          this.field('reactions').forEach((rct,ri)=>{
            if (!ri) return //skip first reaction

            log.log('reaction',rct.category())
            rct.rI(ri-1)
            rct.screen(this.screen())
            rct.message(this)
            $t.push(rct.execute(ctx,$msg))
          })
          if ($t.length)
            $t.push($t.shift())


          Rx.concat(...$t).subscribe(res=>{
            //console.log(['reaction executed'])
          },err=>{
            ctx.f('error',ctx.f('error')+1)
            log.err('ERROR')
            console.log(err)
            this.obsEnd(obs)
          },()=>{
            ctx.f('success',ctx.f('success')+1)
            if (!$msg.sendable())
            {
              console.log(['ops'])
              return this.obsEnd(obs)
            }

            $msg.onSended.subscribe(msgRes=>{
              log.info('message sended',msgRes)
              if (msgRes)
              {
                if(msgRes.success)
                  ctx.f('sended',ctx.f('sended')+1)
                if(msgRes.error)
                  ctx.f('error',ctx.f('error')+1)
              }
              this.obsEnd(obs)
            },err=>{
              log.err('MESSAGE SEND ERROR')
              console.log(err)
              this.obsEnd(obs)
            })
            $msg.send()
          })

        },err=>this.obsErr(obs,err))

      },err=>{
        log.err('testing tgedit message logic error happens!')
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }

}

module.exports = TGMEditMessage
