/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('TGMIAReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

//const TGMsg = require('../../../telegram/nodes/message')
//const TGChat = require('../../../telegram/nodes/chat')
//const User = require('../../../auth/user/user')
const Node = require('../../../node/node')
const VNode = require('../../../node/vnode')

class TGMIAReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'tginlineanswer',
      name: g18n.t('TG Inline Answer'),
      shortname: g18n.t('TGM IA'),
      desc: g18n.t('Answer for inline query'),
      help: g18n.t('Answer for inline query event'),
      faicon: 'fas fa-pen-square',
      uicolor: 'text-primary',
      uigroup: 'chatanswers',
      routersLimit: ['telegrambotrouter'],
      mode: 'message', //message/reaction/any
      greedy: false,
      conflicts: ['*'],
      requireSend: false,
      doSend: true,
      confs: []
      /*
      confs: [{
        name: g18n.t('message'),
        code: 'target',
        desc: g18n.t('Telegram message to update'),
        allowManual: false,
        allowGenerators: false,
        type: 'relation',
        xtype: 'relation:tgmessage'
      }]
      /**/
    }
  }

  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      if (!data||!data.tginlineanswer)
        return this.obsErr(obs,{code:'NO_DATA'})
      data = data.tginlineanswer
      //console.log(data)
      ctx.selectorResult(data.source,{type:'relation',quantity:0}).subscribe(src=>{
        let ret = {
          inline_query_id: '',
          results: [],
          cache_time: data.cachetime,
          is_personal: data.cacheuser
        }
        if (data.type=='direct')
        {
          ret.results = src
          return this.obsEnd(obs,ret)
        }
        if (!Array.isArray(src)||!src.length)
          return this.obsEnd(obs,ret)
        let $load = src.map(nd=>{
          if (!(nd instanceof Node))
            return null
          return Rx.of(nd).pipe(RxO.mergeMap(nnd=>{

            ctx.addNode(nnd,nnd.bundle())
            //console.log(nnd.toObject())
            if (data.maptype=='article')
              return Rx.forkJoin(
                ctx.selectorResult(data.maps.id),
                ctx.selectorResult(data.maps.title),
                ctx.selectorResult(data.maps.description),
                ctx.selectorResult(data.maps.url),
                ctx.selectorResult(data.maps.hide_url),
                ctx.selectorResult(data.maps.thumb_url),
                ctx.selectorResult(data.maps.input_message_content)
              ).pipe(RxO.map(x=>{
                //console.log(['ctx res',x])
                x = x.map(xx=>{
                  if (Array.isArray(xx))
                    return xx[0]
                  return xx
                })
                return {
                  type: 'article',
                  id: !!x[0]?x[0]:nnd.id(),
                  title: x[1],
                  description: x[2],
                  url: x[3],
                  hide_url: x[4],
                  thumb_url: x[5],
                  input_message_content: {
                    message_text: x[6]
                  }
                }
              }))
            if (data.maptype=='photo')
              return Rx.forkJoin(
                ctx.selectorResult(data.maps.id),
                ctx.selectorResult(data.maps.title),
                ctx.selectorResult(data.maps.description),
                ctx.selectorResult(data.maps.caption),
                ctx.selectorResult(data.maps.photo_url),
                ctx.selectorResult(data.maps.thumb_url),
                ctx.selectorResult(data.maps.input_message_content)
              ).pipe(RxO.map(x=>{
                //console.log(['ctx res',x])
                x = x.map(xx=>{
                  if (Array.isArray(xx))
                    return xx[0]
                  return xx
                })
                return {
                  type: 'photo',
                  id: !!x[0]?x[0]:nnd.id(),
                  title: x[1],
                  description: x[2],
                  caption: x[3],
                  photo_url: x[4],
                  thumb_url: x[5],
                  input_message_content: {
                    message_text: x[6]
                  }
                }
              }))
          }))
        }).filter(x=>!!x)
        if (!$load.length)
          return this.obsEnd(obs,ret)
        //console.log(['tgmia source',data.type,src])
        Rx.concat(...$load).subscribe(r=>{
          //console.log(['one more map res',r])
          ret.results.push(r)
        },err=>{
          log.err('error happened on mapping',err)
          console.log(err)
          this.obsEnd(obs,ret)
        },_=>{
          this.obsEnd(obs,ret)
        })
      })
      /*
      let dels = []
      let $load = null
      //console.log(['DEL data',data])
      if (data.type=='msg')
        $load = ctx.selectorResult(data.msg).pipe(RxO.map(x=>{
          return { msg: x || null }
        }))
      if (data.type=='user')
        $load = Rx.forkJoin(
          ctx.selectorResult(data.user),
          ctx.selectorResult(data.chat)
          //ctx.selectorResult(data.amount,{type:'number',code:'amount'})
        ).pipe(RxO.map(x=>{
          return {
            user: x[0] || null,
            chat: x[1] || null
            //amount: parseInt(x[2]) || 0
          }
        }))
      /*
      if (data.type=='chat')
        $load = ctx.selectorResult(data.chat).pipe(RxO.map(x=>{
          return {
            chat: x || null
          }
        }))
      /**/
      /*

      if (!$load)
        return this.obsEnd(obs)
      $load.subscribe(x=>{
        //console.log(['data loaded',x])
        if (!x)
          return this.obsEnd(obs)
        let $msgsload = null
        if (data.type=='msg')
        {
          if (!x.msg||!(x.msg instanceof TGMsg))
            return this.obsEnd(obs)
          $msgsload = this.C('tgmessage').findNodes({_id:x.msg.id()},{populate:1})
        }
        if (data.type=='user')
        {
          if (!x.user||!(x.user instanceof User))
            return this.obsEnd(obs)
          let q = {
            sender: x.user.id()
          }
          let $cload = Rx.of(null)
          if (x.chat&&x.chat instanceof TGChat)
            q.chat = x.chat.id()
          $msgsload = this.C('tgmessage').findNodes(q,{populate:1})
        }
        if (!$msgsload)
          return this.obsEnd(obs)
        $msgsload.subscribe(list=>{
          //console.log(['messages to delete!',list])
          this.M('telegram').deleteMessages(list).subscribe(res=>this.obsEnd(obs),err=>this.obsErr(obs,err))
        },err=>this.obsErr(obs,err))
      })
      /**/
    })
  }

}

module.exports = TGMIAReaction
