/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('IfaceLoginReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class IfaceLoginReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'iflogin',
      name: g18n.t('Interface otl code'),
      desc: g18n.t('Interface one time login code generator. REQUIRES USER AT CONTEXT!'),
      help: g18n.t('Generates interface one time code login and provide it as {{virtual.otlCode}}. REQUIRES USER AT CONTEXT!'),
      shortname: g18n.t('Interface OTL'),
      faicon: 'fas fa-user-plus',
      uicolor: 'text-info',
      uigroup: 'auth',
      routersLimit: [],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: [],
      requireSend: false,
      confs: [{
        name: g18n.t('Interface code'),
        code: 'iface',
        desc: g18n.t('Interface code where user should authorize'),
        allowManual: true,
        allowGenerators: false,
        type: 'string'
      },{
        name: g18n.t('Code size'),
        code: 'csize',
        desc: g18n.t('Number of symbols (8 default)'),
        allowManual: true,
        allowGenerators: false,
        type: 'number',
        def: 8
      }],
      doSend: false
    }
  }


  execute(ctx,$msg)
  {
    return this.loadConfs(ctx).pipe(
      RxO.mergeMap(cfg=>{
        //console.log(['cfg',cfg])
        let csize = typeof cfg.csize == 'number' && cfg.csize > 0 ? cfg.csize : 8
        log.test('creating session for iface',[ctx.user().titles(),cfg.iface])
        return this.M('auth/access').createIfaceSession(cfg.iface,ctx.user(),csize).pipe(RxO.catchError(err=>{
          console.log('iflogin.js reaction, createIfaceSession',err)
          return Rx.of(null)
        }))
      }),
      RxO.map(code=>{
        if (!code)
          return
        log.success('session generated for iface',[ctx.user().titles(),code])
        ctx.addField({
          name: g18n.t('One Time Login Code'),
          code: 'otlCode',
          type: 'string',
          quantity: 1,
          flags: ['readonly','virtual']
        })
        ctx.f('otlCode',code)
      })
    )
  }


}

module.exports = IfaceLoginReaction
