/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('UserLoginReaction')
let g18n = require('../../../../g18n')

const AbstractReaction = require('../reaction')

class UserLoginReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'user_login',
      name: g18n.t('Dashboard otl code'),
      desc: g18n.t('Dashboard one time code login generator. REQUIRES USER AT CONTEXT!'),
      help: g18n.t('Generates dashboard one time code login and provide it as {{virtual.otlCode}}. REQUIRES USER AT CONTEXT!'),
      shortname: g18n.t('Dashboard OTL'),
      faicon: 'fas fa-user-plus',
      uicolor: 'text-danger',
      uigroup: 'auth',
      routersLimit: [],
      mode: 'any', //message/reaction/any
      greedy: true,
      conflicts: [],
      requireSend: false,
      doSend: false
    }
  }


  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      this.M('auth/access').generateDashboardSession(ctx.user()).subscribe(code=>{
        if (!code)
          return this.obsEnd(obs)
        ctx.addField({
          name: g18n.t('One Time Login Code'),
          code: 'otlCode',
          type: 'string',
          quantity: 1,
          flags: ['readonly','virtual']
        })
        ctx.f('otlCode',code)
        this.obsEnd(obs)
      },err=>{
        console.log(err)
        this.obsEnd(obs)
      })
    })
  }


}

module.exports = UserLoginReaction
