/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../../log').m('WalletReaction')
let g18n = require('../../../../g18n')

let BE = require('blockchain.info/blockexplorer')
let BUC = require('bitcoin-convert')
let EUC = require('ethereumjs-units')
let WAV = require('wallet-address-validator')

const AbstractReaction = require('../reaction')
const VirtualNode = require('../../../node/vnode')

class WalletReaction extends AbstractReaction
{

  static info()
  {
    return {
      code: 'wallet_recognition',
      name: g18n.t('Search BTC/ETH wallet'),
      desc: g18n.t('Checks if data is BTC/ETH wallet and load data about it.'),
      help: g18n.t('Checks if data is BTC/ETH wallet and load data about it.'),
      shortname: g18n.t('Wallet'),
      faicon: 'fas fa-id-card',
      uicolor: 'text-warning',
      uigroup: 'crypto',
      routersLimit: ['telegrambotrouter'],
      mode: 'message', //message/reaction/any
      greedy: true,
      conflicts: ['*'],
      requireSend: false,
      doSend: false
    }
  }


  execute(ctx,$msg)
  {
    return Rx.Observable.create(obs=>{
      let data = this.f('data')
      //console.log(['exec wallet',data])
      let cmd = ctx.cmd()
      let args = []
      if (cmd.command().length)
      {
        let lst = cmd.args()
        if (lst.length)
          lst.shift()
        args = lst
      } else {
        args = cmd.raw().split(' ').filter(l=>!!l.length)
      }
      if (!args.length)
        return this.obsErr(obs,{code:"NO_DATA_TO_SEARCH"})
      let symbol = this.whoisWallet(args[0])
      if (!symbol)
        return this.obsErr(obs,{code:"WALLET NOT RECOGNIZED"})
      let nx = this.checkBTC(args)
      if (symbol=='ETH')
        nx = this.checkETH(args)
      nx.subscribe(wlt=>{
        if (!wlt)
          return this.obsErr(obs,{code:"WALLET NOT RECOGNIZED"})
        let wobj = new VirtualNode(ctx)
        wobj.addField({
          name: g18n.t('Wallet coin symbol'),
          code: 'coin',
          type: 'string'
        })
        wobj.addField({
          name: g18n.t('Wallet balance'),
          code: 'balance',
          type: 'number',
          conf: {ops:[
            { source: 'manual', manual: true },
            { source: 'manual', manual: 8 }
          ]}
        })
        wobj.f('coin',symbol)
        wobj.f('balance',wlt.balance)
        /*
        if (symbol=='ETH'&&Array.isArray(wlt.erc20))
        {
          wobj.addField({
            name: g18n.t('Wallet ERC20 coins'),
            code: 'erc20',
            type: 'any',
            quantity: 0
          })
          wobj.field('erc20').all(wlt.erc20)
        }
        /**/
        ctx.addNode(wobj,'wallet')
        this.obsEnd(obs)
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

  whoisWallet(str)
  {
    if (WAV.validate(str,'BTC'))
      return 'BTC'
    if (WAV.validate(str,'ETH'))
      return 'ETH'
    return null
  }

  checkBTC(addrs)
  {
    return Rx.Observable.create(obs=>{
      Rx.from(BE.getBalance(addrs[0])).subscribe(wlt=>{
        if (!wlt||typeof wlt[addrs[0]].final_balance=='undefined')
          return this.obsErr(obs,{code:'BTC_BALANCE_ERROR'})
        let blnc = BUC(wlt[addrs[0]].final_balance,'Satoshi','BTC')
        //console.log(['BTC',blnc])
        this.obsEnd(obs,{symbol:addrs[0],balance:blnc})
      },err=>{
        this.obsErr(obs,{code:'BTC_SEARCH_ERROR',error:err})
      })
    })
  }

  checkETH(addrs)
  {
    return Rx.Observable.create(obs=>{
      let key = this.M('config').conf('etherscan.key','')
      //console.log(['check eth',key])
      let ESA = require('etherscan-api').init(key)
      Rx.from(ESA.account.balance(addrs[0])).subscribe(wlt=>{
        if(!wlt||wlt.message!='OK')
          return this.obsErr(obs,{code:'ETH_BALANCE_ERROR'})
        let blnc = EUC.convert(wlt.result,'wei','eth')
        //console.log(['check eth2',wlt,blnc])
        //if (addrs.lenth == 1)
        return this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:[]})
        //TODO REPAIR ERC20 SEARCH
        /*
        let srchs = []
        for(let i=1;i<addrs.length;i++)
        {
          let ln = addrs[i].trim()
          let lns = [ln]
          if(ln.includes(','))
            lns = ln.split(',')
          lns.forEach(xx=>{
            if (xx&&xx.length&&xx.trim().length)
              srchs.push(xx.trim())
          })
        }
        let $q = {}
        if (!srchs.includes('💰')&&!srchs.includes('🗝'))
        {
          $q = {$or:[]}
          srchs.forEach(tkn=>{
            $q['$or'].push({
              symbol: { $regex: new RegExp('^'+tkn+'$','i') }
            })
          })
          if (!$q['$or'].length)
            return this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:[]})
        }
        //console.log('search',$q)
        this.C('ethtokens').findNodes($q).subscribe(list=>{
          //console.log($tkn)
          if (!list||!list.length)
            return this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:[]})
          let $t = list.map($tkn=>{
            log.warn('search erc20',$tkn.f('symbol'))
            return Rx.from(ESA.account.tokenbalance(addrs[0],null,$tkn.f('address'))).pipe(RxOps.catchError(err=>{
              //TODO broken here! request returns 403
              log.err('ESA ERC20 error!')
              console.log(err)
              return Rx.of(null)
            }),RxOps.map(xxc=>{
              log.test('searchres erc20',[$tkn.f('symbol'),xxc])
              if (!xxc||!xxc.message||!xxc.message=='OK')
                return null
              let tblnc = EUC.convert(xxc.result,'wei','eth')
              return {
                symbol: $tkn.f('symbol'),
                balance: tblnc
              }
            }))
          })
          let ercs = []
          Rx.concat(...$t).subscribe(erc=>{
            if (erc)
              ercs.push(erc)
          },err=>{
            this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:ercs})
          },()=>{
            this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:ercs})
          })
        },err=>{
          this.obsEnd(obs,{symbol:addrs[0],balance:blnc,erc20:[]})
        })
        /**/
      },err=>{
        this.obsErr(obs,{code:'ETH_SEARCH_ERROR',error:err})
      })
    })
  }

}

module.exports = WalletReaction
