/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../../log').m('ScreenGroupModule')
let g18n = require('../../../../g18n')

const CoreModule = require('../../../core/abstract/module')

const ScreenGroupNode = require('./node')

class ScreenGroupModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{
      let rtScreenGroupsList = {
        url: '/api/screengroups/list',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let ret = []
            this.C('screengroup').findNodes({},{
              populate: 3,
              sort: { priority: 1 },
              list: false
            }).subscribe(scg=>{
              let obj = scg.toObject()
              obj.canRead = user.can('screengroup.'+obj.code+':read')
              obj.canWrite = user.can('screengroup.'+obj.code+':write')
              obj.canExecute = user.can('screengroup.'+obj.code+':private')
              if (user.can('screengroup.root'))
              {
                obj.canRead = true
                obj.canWrite = true
                obj.canExecute = true
              }
              if (!obj.canRead&&!obj.canWrite)
                return
              obj.screens = obj.screens.filter(x=>!!x)
              ret.push(obj)
            },err=>{
              log.err('fail loading screengroups')
              console.log(err)
              this.obsErr(obs,err)
            },()=>{
              this.obsEnd(obs,ret)
            })
          })
        }
      }

      let rtScreenGroupOne = {
        url: '/api/screengroups/one/:rcode/:code',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let p = ctx.f('params')
            if (!p.code)
              return this.obsEnd(obs,null)
            if(!user.can('screengroup.'+p.code+':read')||
               !user.can('screengroup.'+p.code+':write')||
               !user.can('screengroup.root'))
              return this.obsErr(obs,401)
            this.C('screengroup').findNode({router:p.rcode,code:p.code},{populate:1}).subscribe($scg=>{
              this.obsEnd(obs,$scg.toObject())
            },err=>this.obsEnd(obs,null))
          })
        }
      }

      let rtScreenGroupCheck = {
        url: '/api/screengroups/check',
        access: 'screengroup.root',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let opts = ctx.f('body')
            let ret = {
              code: true,
              priority: true
            }
            let q = {router:opts.router,$or:[]}
            if (opts.code)
              q.$or.push({code:opts.code})
            if (opts.priority)
              q.$or.push({priority:opts.priority})
            this.C('screengroup').findNodes(q,{list:false}).subscribe($scg=>{
              if ($scg.f('code')==opts.code&&opts.isNew)
                ret.code = false
              if ($scg.f('priority')==opts.priority&&$scg.f('code')!=opts.code)
                ret.priority = false
            },err=>{
              log.err('check failed')
              console.log(err)
              this.obsEnd(obs,ret)
            },()=>{
              this.obsEnd(obs,ret)
            })
          })
        }
      }

      let rtScreenGroupSave = {
        url: '/api/screengroups/save',
        access: 'screengroup.root',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let opts = ctx.f('body')
            let grp = opts.group
            let access = opts.perms
            if (!grp||!access)
              return this.obsEnd(obs,{err:'Form Invalid'})
            //console.log(['route screengroup save',opts,access])
            this.C('screengroup').findNode({_id:grp.id}).subscribe($grp=>{
              if (!$grp&&!grp.isNew)
                return this.obsEnd(obs,{err:'Search Invalid'})
              if (!$grp&&grp.isNew)
                $grp = this.C('screengroup').newNode()
              if (!$grp)
                return this.obsEnd(obs,{err:'Group Invalid'})
              $grp.f('router',grp.router)
              $grp.f('code',grp.code)
              $grp.f('name',grp.name)
              $grp.f('description',grp.description)
              $grp.f('priority',grp.priority)

              log.info('creating screengroup',$grp.f('code'))
              $grp.save().subscribe($g=>{
                log.info('screengroup saved, building access')
                this.C('usersgroup').findNodes({}).subscribe(list=>{
                  this.C('role').findNodes({module:'screengroup'}).subscribe(roles=>{
                    let $t = list.filter(ug=>ug.f('code')!='admins').map(ug=>this._updateUserGroupAccess(access[ug.f('code')],$g,ug,roles))
                    Rx.concat(...$t).subscribe(()=>{},err=>{
                      log.err('updating access error')
                      console.log(err)
                      this.obsEnd(obs,{err:'access update failed'})
                    },()=>{
                      log.success('screengroup saved')
                      this.obsEnd(obs,{success:true})
                    })
                  },err=>this.obsEnd(obs,{err:'roles search failed'}))
                },err=>this.obsEnd(obs,{err:'usersgroup search failed'}))
                //
              },err=>this.obsEnd(obs,{err:'not saved'}))
            },err=>this.obsEnd(obs,{err:'scgreengroup search failed'}))
          })
        }
      }

      let rtScreenGroupDel = {
        url: '/api/screengroups/del/:rcode/:code',
        access: 'screengroup.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let p = ctx.f('params')
            if (!p.code)
              return this.obsEnd(obs,{success:false})
            this.C('screengroup').findNode({router:p.rcode,code:p.code},{populate:3}).subscribe($scg=>{
              let $t = []
              let $r = []
              let $m = []
              let $s = []
              if (!!$scg)
                $scg.field('screens').forEach($sc=>{
                  $s.push($sc.id())
                  $sc.field('messages').forEach($msg=>{
                    let mgn = $msg.f('globalName')
                    if (!mgn||!mgn.length)
                      $m.push($msg.id())
                    $msg.field('reactions').forEach($rct=>{
                      let rgn = $rct.f('globalName')
                      if (!rgn||!rgn.length)
                        $r.push($rct.id())
                    })
                  })
                })
              $t.push(this.C('screenreaction').removeNodes({_id:$r}))
              $t.push(this.C('screenmessage').removeNodes({_id:$m}))
              $t.push(this.C('screen').removeNodes({_id:$s}))
              $t.push(this.C('screengroup').removeNodes({_id:$scg.id()}))

              let rx = new RegExp('$'+$scg.f('code')+':.*')
              $t.push(this.C('role').removeNodes({code:{$regex:rx}}))
              $t.push(this.M('auth/access').updateAdminsAccess())
              log.warn('removing screengroup',[$scg.f('router'),$scg.f('code')])
              Rx.concat(...$t).subscribe(()=>{
                log.info('...')
              },err=>{
                log.err('fail remove screengroup tasks')
                console.log(err)
                this.obsEnd(obs,{success:false})
              },()=>{
                log.success('screengroup removed')
                this.obsEnd(obs,{success:true})
              })
            },err=>this.obsEnd(obs,null))
          })
        }
      }

      obs.next([
        rtScreenGroupsList,
        rtScreenGroupOne,
        rtScreenGroupCheck,
        rtScreenGroupSave,
        rtScreenGroupDel
      ])
      obs.complete()
    })
  }

  _screenGroupsExectutableForScreenGroup(code)
  {
    return Rx.Observable.create(obs=>{
        /**
      this.C('screengroup').findNodes({},{populate:true}).subscribe(list=>{
          return this.obsEnd(obs,list
        this.C('role').findNodes({module:'screengroup'}).subscribe(list=>{
          let xp = list.reduce((acc,r)=>r.code==code+':private'?r:acc,null)
          let xg = list.reduce((acc,r)=>r.code==code+':group'?r:acc,null)
          let xs = list.reduce((acc,r)=>r.code==code+':supergroup'?r:acc,null)
          if (!xp||!xg||!xs)
            return this.obsEnd(obs,[])
          let ugsx = []
          this.C('usersgroup').findNodes({},{list:false}).pipe(
            RxO.filter($ug=>{
              if ($ug.field('roles').has(xp)||
                  $ug.field('roles').has(xp)||
                  $ug.field('roles').has(xp))
                return true
              return false
            })
          ).subscribe(ug=>{
            console.log(ug.view())
            ugsx.push(ug)
          },err=>{
            log.err('fail search usersgroups')
            console.log(err)
            this.obsEnd(obs,[])
          },()=>{
            
          })
        },err=>{
          log.err('fail search roles')
          console.log(err)
          this.obsEnd(obs,[])
        })
      },err=>{
        log.err('fail search main group')
        console.log(err)
        this.obsEnd(obs,[])
      })


    let ownerGroup = await ScreenGroup.findOne({code:screenGroupCode})
    if (!ownerGroup)
      return []
    let permXP = await Permission.findOne({scope:'screengroup',code:ownerGroup.code+':private'})
    let permXG = await Permission.findOne({scope:'screengroup',code:ownerGroup.code+':group'})
    let permXX = await Permission.findOne({scope:'screengroup',code:ownerGroup.code+':supergroup'})
    if (!permXP&&!permXg&&!permXX)
      return []
    let authGroups = await AuthGroup.find().populate('permissions')
    if (!authGroups.length)
      return []
    let screenGroups = await ScreenGroup.find().populate('screens')
    if (!screenGroups.length)
      return []
    let authGroupsX = []
    let screenGroupsX = []
    //filter auth groups, able execute owner groups
    for(let i=0;i<authGroups.length;i++)
    {
      if (authGroups[i].code == 'admins') //we don't collect what admins can
        continue
      if (authGroups[i].code == 'guests') //but we collect guests. what guest can - everybody can
      {
        authGroupsX.push(authGroups[i])
        continue
      }
      let hasX = false
      for(let j=0;j<authGroups[i].permissions.length;j++)
      {
        let p = authGroups[i].permissions[j]
        if (p.scope+'.'+p.code == permXP.scope+'.'+permXP.code)
          hasX = true
        if (p.scope+'.'+p.code == permXG.scope+'.'+permXG.code)
          hasX = true
        if (p.scope+'.'+p.code == permXX.scope+'.'+permXX.code)
          hasX = true
      }
      if(hasX)
        authGroupsX.push(authGroups[i])
    }
    //log.test('SCGS','AuthGroupsX',authGroupsX)
    //filter screen groups, can be executed by filterd auth groups
    for(let i=0;i<screenGroups.length;i++)
    {
      let groupRoleXP = 'screengroup.'+screenGroups[i].code+':private'
      let groupRoleXG = 'screengroup.'+screenGroups[i].code+':group'
      let groupRoleXX = 'screengroup.'+screenGroups[i].code+':supergroup'
      let ok = false
      for(let j=0;j<authGroupsX.length;j++)
      {
        for(let k=0;k<authGroupsX[j].permissions.length;k++)
        {
          let permRole = authGroupsX[j].permissions[k].scope+'.'+authGroupsX[j].permissions[k].code
          if (permRole==groupRoleXP||
              permRole==groupRoleXG||
              permRole==groupRoleXX)
            ok = true
        }
      }
      if(ok)
        screenGroupsX.push(screenGroups[i])
    }
    return screenGroupsX
        /**/
    })
  }

  _updateUserGroupAccess(access,$scg,$ug,$roles)
  {
    return Rx.Observable.create(obs=>{
      let scgCode = $scg.f('code')
      let $rr = $roles.reduce((acc,$r)=>($r.f('code')==scgCode+':read')?$r:acc,null)
      let $rw = $roles.reduce((acc,$r)=>($r.f('code')==scgCode+':write')?$r:acc,null)
      let $rxp = $roles.reduce((acc,$r)=>($r.f('code')==scgCode+':private')?$r:acc,null)
      let $rxg = $roles.reduce((acc,$r)=>($r.f('code')==scgCode+':group')?$r:acc,null)
      let $rxx = $roles.reduce((acc,$r)=>($r.f('code')==scgCode+':supergroup')?$r:acc,null)
      if (!$rr||!$rw||!$rxp||!$rxg||!$rxx)
      {
        console.log($rr,$rw)
        return this.obsErr(obs,{err:'not all roles found!'})
      }
      //console.log(['change access',access,$scg.f('code'),$ug.f('code')])
      let next = []
      let done = {
        r: false,
        w: false,
        xp: false,
        xg: false,
        xx: false
      }
      $ug.field('roles').all().forEach(rid=>{
        if ($rr.eq(rid))
        {
          done.r = true
          if (!access.r) return
        }
        if ($rw.eq(rid))
        {
          done.w = true
          if (!access.w) return
        }
        if ($rxp.eq(rid))
        {
          done.xp = true
          if (!access.xp) return
        }
        if ($rxg.eq(rid))
        {
          done.xg = true
          if (!access.xg) return
        }
        if ($rxx.eq(rid))
        {
          done.xx = true
          if (!access.xx) return
        }
        next.push(rid)
      })
      if (!done.r&&access.r) next.push($rr)
      if (!done.w&&access.w) next.push($rw)
      if (!done.xp&&access.xp) next.push($rxp)
      if (!done.xg&&access.xg) next.push($rxg)
      if (!done.xx&&access.xx) next.push($rxx)

      $ug.field('roles').all(next)
      $ug.save().subscribe(ug=>{
        this.obsEnd(obs)
      },err=>{
        this.obsErr(obs,{error:'update users group role failed'})
      })

    })
  }


  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      this.C('screengroup').setNodeClass(ScreenGroupNode)
      this.obsEnd(obs)
      /*
      let $t = []

      Rx.concat(...$t).subscribe(()=>{},err=>{obs.error(err)},()=>{
        log.success('initialized')
        obs.next()
        obs.complete()
      })
      /**/
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlScreengroup = this._core.module('bundle').bundle('screengroup',true)
      bndlScreengroup.name(g18n.t('Screengroup'))
      bndlScreengroup.desc(g18n.t('Telegram screengroups'))
      bndlScreengroup.flags(['system','locked'])
      bndlScreengroup.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','required','logic']})
      //bndlScreengroup.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system'],def:true})
      bndlScreengroup.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','required','logic']})
      bndlScreengroup.addField({name:g18n.t('Router'),code:'router',type:'string',flags:['system','logic']})
      bndlScreengroup.addField({name:g18n.t('Description'),code:'description',type:'string',flags:['system','logic']})
      bndlScreengroup.addField({name:g18n.t('Proirity'),code:'priority',type:'number',flags:['system','required','unique','logic'],def:0})
      bndlScreengroup.addField({name:g18n.t('Screens'),code:'screens',type:'relation',conf:{ops:[{source:'manual',manual:'screen'},{source:'manual',manual:'title'}]},flags:['system','logic'],quantity:0})
      
      log.info('saving bundle',bndlScreengroup.code())
      $t.push(bndlScreengroup.save(true,true))

      //log.info('modifying user bundle')
      //let bndUser = this._core.module('bundle').bundle('user')
      //bndUser.addField({name:g18n.t('Isolate screengroup'),code:'isolateScreengroup',type:'relation',conf:{ops:[{source:'manual',manual:['screengroup']},{source:'manual',manual:'name, code'}]},quantity:1,flags:['system','logic','locked']})
      //$t.push(bndUser.save())

      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      $t.push(this.M('auth/role').roleSave('screengroup','root','Full screens/screengroups administration'))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  _update01to011()
  {
    return Rx.Observable.create(obs=>{

      let $t = []

      let bnd = this.M('bundle').bundle('screengroup')
      if (!bnd.hasField('router'))
      {
        log.warn('adding router field to screengroups')
        bnd.addField({name:g18n.t('Router'),code:'router',type:'string',flags:['system','logic']})
        $t.push(bnd.save())
      }
      log.warn('updating screengroups with "tgdefault" router')
      $t.push(this.C('screengroup').findNodes().pipe(RxO.mergeMap(list=>{
        let $s = list.map(sg=>{
          sg.f('router','tgdefault')
          return sg.save()
        })
        return Rx.concat(...$s).pipe(RxO.last())
      })))

      if (!$t.length)
        return this.obsEnd(obs)

      Rx.concat(...$t).subscribe(_=>{},err=>this.obsErr(obs,err),_=>this.obsEnd(obs))
    })
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1'&&to=='0.1.1')
          return this._update01to011()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }


}

module.exports = ScreenGroupModule
