/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('ScreenModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const ScreenNode = require('./screen')

const Cmd = require('../../telegram/cmd')
const Context = require('../../context/context')
const Node = require('../../node/node')

class ScreenModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
  }

  _cronTick(t)
  {
    this.C('screen').findNodes({action:'cron'},{populate:2}).subscribe(scrs=>{
      if (!scrs||!scrs.length)
        return
      scrs.forEach(scr=>{
        if (!scr.f('enabled'))
          return
        let ad = scr.f('actionData')
        if (!this.M('cron').testMask(ad.cron.event))
          return
        //console.log(ad.cron)
        setTimeout(()=>{
          let ctxx = this.M('context').newContext(this,'telegram')
          this.M('auth/user').userFieldUsers(ad.cron.user,ctxx,true).subscribe(ulist=>{
            if (!Array.isArray(ulist))
              return
            ulist.forEach(u=>{
              if (!u)
                return
              if (!(u instanceof Node))
                return
              if (u.bundle()!='user')
                return
              //console.log(ulist.map(x=>x.toObject()))
              setTimeout(()=>{
                let cmd = new Cmd(this,'')
                let ctx = this.M('context').newContext(this,'telegram')
                //CTX.session(ctx.session)
                ctx.cmd(cmd)
                ctx.type('cron')
                ctx.users([u])
                ctx.chats([u.fp('privateChat')])
                //console.log(['cron screen',scr.f('title'),ctx.view()])
                log.info('executing screen by cron',scr.f('title'))
                scr.execute(ctx).subscribe(res=>{},err=>{
                  console.log(err)
                })
              },0)
            })
          },err=>{
            log.err('search user for cron execution error')
            console.log(err)
          })
        },0)
      })
    },err=>{
      log.err('search cron screens error')
      console.log(err)
    })
  }

  screensWithReaction(category)
  {
    return Rx.Observable.create(obs=>{
      this.C('screenreaction').findNodes({category:category},true).subscribe(rcts=>{
        let ids = rcts.map(r=>r.id())
        this.C('screenmessage').findNodes({reactions:{$in:ids}},true).subscribe(msgs=>{
          let msgsids = msgs.map(m=>m.id())
          this.C('screen').findNodes({messages:{$in:msgsids}},true).subscribe(scs=>{
            obs.next(scs)
            obs.complete()
          })
        })
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtScreenOne = {
        url: '/api/screen/one/:sid',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let p = ctx.f('params')
            if (!p.sid)
              return this.obsEnd(obs,null)

            //if(!user.can('screengroup.'+p.code+':read')||
               //!user.can('screengroup.'+p.code+':write')||
               //!user.can('screengroup.root'))
              //return this.obsErr(obs,401)
            this.C('screen').findNode({_id:p.sid},{populate:2}).subscribe($sc=>{
              if (!$sc)
                return this.obsEnd(obs,null)
              //console.log('screenOneRoute',$sc.toObject().messages[0].reactions[0].data.library.filter.tests[0])
              this.obsEnd(obs,$sc.toObject())
            },err=>this.obsEnd(obs,null))
          })
        }
      }

      let rtScreenSave = {
        url: '/api/screen/save',
        access: 'auth.dashboard',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let p = ctx.f('params')
            let b = ctx.f('body')
            let rcode = b.rcode || null
            let scgrp = b.gcode || null
            let scr = b.scr || null
            if (!rcode||!scgrp||!scr||!scr.messages||!scr.messages.length)
              return this.obsEnd(obs,{success:false})
            this.C('screengroup').findNode({router:rcode,code:scgrp}).subscribe($scg=>{
              if (!$scg)
                return this.obsEnd(obs,{success:false,error:'screengroup not found'})
              //check write premission
              let allow = false
              if (user.can('screengroup.root')||user.can('screengroup.'+$scg.f('code')+':write'))
                allow = true
              if (!allow)
                return this.obsEnd(obs,401)

              let getScreen = this.C('screen').initNode(ctx)
              if (scr.id)
                getScreen = this.C('screen').findNode({_id:scr.id},{populate:2})

              getScreen.subscribe($scr=>{
                //console.log(['saving screen',scr,'->',$scr.toObject()])
                $scr.f('title',scr.title)
                $scr.f('enabled',scr.enabled)
                $scr.f('logic',scr.logic)
                $scr.f('action',scr.action)
                $scr.f('actionData',scr.actionData)

                let $msgs = []

                let globalMessages = []
                let globalReactions = []

                let beforeMsgs = []
                let nextMsgs = []

                let beforeRcts = []
                let nextRcts = []

                $scr.field('messages').forEach($msg=>{
                  beforeMsgs.push($msg)
                  $msg.field('reactions').forEach($rct=>beforeRcts.push($rct))
                })

                scr.messages.forEach(msg=>{
                  let getMsg = this.C('screenmessage').initNode()
                  if (msg.id)
                    getMsg = this.C('screenmessage').findNode({_id:msg.id})
                  $msgs.push(getMsg)
                })

                Rx.concat(...$msgs).subscribe(m=>nextMsgs.push(m),err=>{
                  log.err('searchinig screenmessages to save/update failed!',scr.id)
                  this.obsEnd(obs,{success:false})
                },()=>{
                  let $rcts = []
                  let rcts = {}
                  scr.messages.forEach((msg,msgi)=>{
                    let $msg = nextMsgs[msgi]
                    //console.log([msg,$msg.toObject()])
                    $msg.f('type',msg.type)
                    $msg.f('kbdType',msg.kbdType)
                    $msg.f('globalName',msg.globalName)
                    $msg.f('logic',msg.logic)
                    rcts[$msg.id()] = []
                    msg.reactions.forEach(rct=>{
                      let getRct = this.C('screenreaction').initNode()
                      if (rct.id)
                        getRct = this.C('screenreaction').findNode({_id:rct.id})
                      $rcts.push(getRct.pipe(RxOps.map($rct=>{
                        rcts[$msg.id()].push($rct)
                        return $rct
                      })))
                    })
                  })
                  //log.info('looks like still fine')
                  //return this.obsEnd(obs,{success:false})
                  Rx.concat(...$rcts).subscribe($rct=>{},err=>{
                    log.err('searchinig reactions to save/update failed!',scr.id)
                    console.log(err)
                    this.obsEnd(obs,{success:false})
                  },()=>{
                    let $t = []
                    $scr.field('messages').clear()
                    scr.messages.forEach((msg,msgi)=>{
                      let $msg = nextMsgs[msgi]
                      $msg.field('reactions').clear()
                      msg.reactions.forEach((rct,rcti)=>{
                        let $rct = rcts[$msg.id()][rcti]
                        $rct.f('category',rct.category)
                        $rct.f('globalName',rct.globalName)
                        $rct.f('data',rct.data)
                        //console.log([$msg.toObject(),$rct.toObject(),rct])
                        $t.push($rct.save().pipe(RxOps.catchError(err=>{
                          //can be same global reaction
                          if (!(err&&err.code=='save'&&err.errors&&err.errors.name=='VersionError'))
                            return Rx.throwError({code:'SAVE_REACTION_ERROR',error:err})
                          //let's try reload and resave
                          return this.C('screenreaction').findNode({_id:$rct.id()}).pipe(RxOps.mergeMap($x=>{
                            if (!$x)
                              return Rx.throwError({code:'RESAVE_REACTION_ERROR'})
                            $x.f('category',rct.category)
                            $x.f('globalName',rct.globalName)
                            $x.f('data',rct.data)
                            return $x.save()
                          }))
                        })))
                        nextRcts.push($rct.id())
                        $msg.field('reactions').push($rct)
                      })
                      $t.push($msg.save().pipe(RxOps.catchError(err=>{
                        //can be same global message
                        if (!(err&&err.code=='save'&&err.errors&&err.errors.name=='VersionError'))
                          return Rx.throwError({code:'SAVE_MESSAGE_ERROR',error:err})
                        //let's try reload and resave
                        return this.C('screenmessage').findNode({_id:$msg.id()}).pipe(RxOps.mergeMap($x=>{
                          if (!$x)
                            return Rx.throwError({code:'RESAVE_MESSAGE_ERROR'})

                          $x.f('type',$msg.f('type'))
                          $x.f('kbdType',$msg.f('kbdType'))
                          $x.f('globalName',$msg.f('globalName'))
                          $x.f('logic',$msg.f('logic'))
                          $x.field('reactions').all(rcts[$x.id()])
                          return $x.save()
                        }))
                      })))
                      $scr.field('messages').push($msg)
                    })
                    $t.push($scr.save())
                    let ex = $scg.field('screens').reduce((acc,sc)=>{
                      return $scr.eq(sc)?true:acc
                    },false)
                    if (!ex)
                    {
                      $scg.field('screens').push($scr)
                      $t.push($scg.save())
                    }
                    //log.info('looks like still fine')
                    //console.log([$scr.toObject()])

                    //so, all saved, need to check for elements, required deleting
                    let delMsgs = []
                    beforeMsgs.forEach($m=>{
                      let ex = $scr.field('messages').reduce((acc,m)=>{
                        return m.eq($m)?true:acc
                      },false)
                      if (ex) return
                      if ($m.f('globalName')!='') return
                      delMsgs.push($m)
                    })
                    let delRcts = []
                    beforeRcts.forEach(br=>{
                      let ex = false
                      nextRcts.forEach(nr=>{
                        if (br.eq(nr))
                          ex = true
                      })
                      if (ex) return
                      if (br.f('globalName')!='') return
                      delRcts.push(br)
                    })
                    let delmids = delMsgs.map(x=>x.id())
                    let delrids = delRcts.map(x=>x.id())
                      
                    if (delrids.length)
                    {
                      log.warn('removing reactions',delrids.length)
                      $t.push(this.C('screenreaction').removeNodes({_id:{$in:delrids}}))
                    }
                    if (delmids.length)
                    {
                      log.warn('removing messages',delmids.length)
                      $t.push(this.C('screenmessage').removeNodes({_id:{$in:delmids}}))
                    }
                    //return this.obsEnd(obs,{success:false})
                  
                    /**/
                    Rx.concat(...$t).subscribe(node=>{},err=>{
                      log.err('saving/updating screengroup/screen/messages/reactions FAILED')
                      console.log(err)
                      this.obsEnd(obs,{success:false})
                    },()=>{
                      log.success('Screen saved',$scr.f('title'))
                      this.obsEnd(obs,{success:true})
                    })
                    /**/
                  })
                })

              },err=>{
                log.err('searchinig screen to save/update failed!',scr.id)
                this.obsEnd(obs,{success:false})
              })
            },err=>{
              log.err('saving screen to unfined screengroup!',scgrp)
              this.obsEnd(obs,{success:false})
            })

          })
        }
      }

      let rtScreenMove = {
        url: '/api/screen/move',
        access: 'auth.dashboard',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let b = ctx.f('body')
            let codeFrom = b.from||null
            let codeTo = b.to||null
            let routerAt = b.at||null
            let sid = b.sid||null
            //let $screen = null

            if (!codeFrom||!codeTo||!sid||!routerAt)
              return this.obsEnd(obs,{success:false})

            this.C('screengroup').findNode({router:routerAt,code:codeFrom,screens:sid}).subscribe($scgf=>{
              if (!$scgf)
              {
                log.err('no move from')
                return this.obsEnd(obs,{success:false})
              }
              //let nextf = []
              //let exf = $scgf.field('screens').reduce((acc,val)=>{
                //let ret = val.eq(sid)?true:acc
                //if (ret) $screen = val
                //if (!ret) nextf.push(val)
                //return ret
              //},false)
              let allowf = false
              if (user.can('screengroup.'+$scgf.f('code')+':write'))
                allowf = true
              if (user.can('screengroup.root'))
                allowf = true
              if (user.can('global.root'))
                allowf = true
              if (!allowf)
              {
                log.err('move not exists, not allowed or not found',[allowf])
                return this.obsEnd(obs,{success:false})
              }
              this.C('screengroup').findNode({router:routerAt,code:codeTo}).subscribe($scgt=>{
                if (!$scgt)
                {
                  log.err('no move to')
                  return this.obsEnd(obs,{success:false})
                }
                //let ext = $scgt.field('screens').reduce((acc,val)=>{
                  //return val.eq(sid)?true:acc
                //},false)
                let allowt = false
                if (user.can('screengroup.'+$scgt.f('code')+':write'))
                  allowt = true
                if (user.can('screengroup.root'))
                  allowt = true
                if (!allowt)
                {
                  log.err('not allowed move to')
                  return this.obsEnd(obs,{success:false})
                }
                let $t = []
                this.C('screen').findNode({_id:sid}).subscribe($screen=>{
                  if (!$screen)
                  {
                    log.err('no screen to move')
                    return this.obsEnd(obs,{success:false})
                  }
                  $scgf.field('screens').filter((x=>!$screen.eq(x)))
                  $t.push($scgf.save())
                  $scgt.field('screens').push($screen)
                  $t.push($scgt.save())
                  Rx.concat(...$t).subscribe(()=>{},err=>{
                    log.err('screen move error!')
                    console.log(err)
                    this.obsEnd(obs,{success:false})
                  },()=>{
                    log.success('screen moved',$screen.f('title'))
                    this.obsEnd(obs,{success:true})
                  })
                },err=>{
                  log.err('screen to move not found!')
                  this.obsEnd(obs,{success:false})
                })
              },err=>{
                log.err('search screengroup TO failed')
                console.log(err)
                this.obsEnd(obs,{success:false})
              })

            },err=>{
              log.err('search screengroup FROM failed')
              console.log(err)
              this.obsEnd(obs,{success:false})
            })
          })
        }
      }

      let rtScreenDel = {
        url: '/api/screen/del/:sid',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let user = ctx.user()
            let b = ctx.f('params')
            let sid = b.sid||null
            //let $screen = null

            if (!sid)
              return this.obsEnd(obs,{success:false})

            this.C('screengroup').findNode({screens:sid}).subscribe($scgf=>{
              if (!$scgf)
              {
                log.err('no delete from')
                return this.obsEnd(obs,{success:false})
              }
              let allowf = false
              if (user.can('screengroup.'+$scgf.f('code')+':write'))
                allowf = true
              if (user.can('screengroup.root'))
                allowf = true
              if (user.can('global.root'))
                allowf = true
              if (!allowf)
              {
                log.err('delete not not allowed or not found')
                console.log([allowf,$screen])
                return this.obsEnd(obs,{success:false})
              }
              this.C('screen').findNode({_id:sid},{populate:2}).subscribe($screen=>{
                if (!$screen)
                {
                  log.err('screen to delete not found')
                  return this.obsEnd(obs,{success:false})
                }
                let delm = []
                let delr = []
                $screen.field('messages').forEach($msg=>{
                  if ($msg.f('globalName').length) return
                  delm.push($msg)
                  $msg.field('reactions').forEach($rct=>{
                    if ($rct.f('globalName').length) return
                    delr.push($rct)
                  })
                })
                let $t = []
                if (delr.length)
                  $t.push(this.C('screenreaction').removeNodes({_id:{$in:delr.map(x=>x.id())}}))
                if (delm.length)
                  $t.push(this.C('screenmessage').removeNodes({_id:{$in:delm.map(x=>x.id())}}))
                $t.push(this.C('screen').removeNodes({_id:$screen.id()}))
                $scgf.field('screens').filter(x=>!$screen.eq(x))
                $t.push($scgf.save())
                Rx.concat(...$t).subscribe(()=>{},err=>{},()=>{
                  log.info('removed reactions',delr.length)
                  log.info('removed messages',delm.length)
                  log.success('screen removed',$screen.f('title'))
                  this.obsEnd(obs,{success:true})
                })
              },err=>{
                log.err('search screen failed')
                console.log(err)
                this.obsEnd(obs,{success:false})
              })
            },err=>{
              log.err('search screengroup FROM failed')
              console.log(err)
              this.obsEnd(obs,{success:false})
            })
          })
        }
      } 


      //global messages and reactions

      let rtGlobals = {
        url: '/api/reactions/globals',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            
            //  TODO

            //  currently - returns all, without checking access
            let q = {globalName:{$exists:true,$ne:''}}
            Rx.forkJoin(
              this.C('screenmessage').findNodes(q,{populate:1}),
              this.C('screenreaction').findNodes(q)
            ).subscribe(data=>{
              let r = {
                messages: data[0].filter(x=>x.f('globalName')&&x.f('globalName').length).map(x=>x.toObject()),
                reactions: data[1].filter(x=>x.f('globalName')&&x.f('globalName').length).map(x=>x.toObject())
              }
              let $t = []
              r.messages.forEach(msg=>{
                $t.push(this.C('screen').countNodes({messages:msg.id}).pipe(
                  RxOps.mergeMap(count=>{
                    msg._count = count
                    return Rx.of(msg)
                  })
                ))
              })
              r.reactions.forEach(rct=>{
                $t.push(this.C('screenmessage').countNodes({reactions:rct.id}).pipe(
                  RxOps.mergeMap(count=>{
                    rct._count = count
                    return Rx.of(rct)
                  })
                ))
              })
              let ret = {
                messages: [],
                reactions: [],
                //rtypes: this.M('bot/reactions').reactionClassesInfo()
                rtypes: this.M('actions').actionsTypes()
              }
              //console.log(this.M('actions').actionsTypes())
              Rx.concat(...$t).subscribe(obj=>{
                if (obj.bundle == 'screenmessage')
                  ret.messages.push(obj)
                else
                  ret.reactions.push(obj)
              },err=>{
                log.err('global story error!')
                console.log(err)
                this.obsEnd(obs,ret)
              },()=>{
                //console.log(['globals',ret.reactions[0]])
                this.obsEnd(obs,ret)
              })
            },err=>{
              log.err('error search globals')
              console.log(err)
              this.obsEnd(obs,[])
            })

          })
        }
      }

      obs.next([
        rtGlobals,

        rtScreenOne,
        rtScreenSave,
        rtScreenMove,
        rtScreenDel

      ])
      obs.complete()
    })
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      this.C('screen').setNodeClass(ScreenNode)

      this.M('cron').tick.subscribe(t=>{
        this._cronTick()
      })
      log.log('initialized')
      this.obsEnd(obs)
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlScreen = this._core.module('bundle').bundle('screen',true)
      bndlScreen.name(g18n.t('Screen'))
      bndlScreen.desc(g18n.t('Telegram screens'))
      bndlScreen.flags(['system','locked'])
      bndlScreen.addField({name:g18n.t('Title'),code:'title',type:'string',flags:['system','required','logic','template','title']})
      bndlScreen.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system','logic'],def:true})
      bndlScreen.addField({name:g18n.t('Action'),code:'action',type:'string',flags:['system','required','logic'],def:'none'})
      bndlScreen.addField({name:g18n.t('Action data'),code:'actionData',type:'any',flags:['system','logic']})
      bndlScreen.addField({name:g18n.t('Logic'),code:'logic',type:'any',flags:['system','logic']})
      bndlScreen.addField({name:g18n.t('Messages'),code:'messages',type:'relation', conf:{ops:[{source:'manual',manual:'screenmessage'}]}, flags:['system','logic'],quantity:0})
      
      log.info('saving bundle',bndlScreen.code())
      $t.push(bndlScreen.save(true,true))


      //$t.push(this.M('screen/group').moduleInstall())

      //log.info('modifying user bundle')
      //let bndUser = this._core.module('bundle').bundle('user')
      //bndUser.addField({name:g18n.t('Isolate screen'),code:'isolateScreen',type:'relation', quantity:1, conf:{ops:[{source:'manual',manual:['screen']},{source:'manual',manual:'title'}]}, flags:['system','logic','locked']})
      //$t.push(bndUser.save())
      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing screens failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

}

module.exports = ScreenModule
