/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('ConfigModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')
const Conf = require('./conf')

class ConfigModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

    this._confs = {}
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let apiGet = {
        url: '/api/config',
        access: ['global.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            this.C('config').findNodes({}).subscribe($confs=>{
              this.obsEnd(obs,$confs.map(c=>c.toObject()))
            },err=>{
              log.err('confs load error')
              console.log(err)
              this.obsEnd(obs,[])
            })
          })
        }
      }

      let apiSet = {
        url: '/api/config',
        access: ['global.root'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let b = ctx.f('body')
            let getc = (key)=>{
              return b.reduce((acc,v)=>v.key==key?v:acc,null)
            }
            let $t = []
            //easy confs
            let cnfCl = getc('cl.key')
            if (cnfCl) $t.push(this.setConf('cl.key',cnfCl.value))
            let cnfCmc = getc('cmc.key')
            if (cnfCmc) $t.push(this.setConf('cmc.key',cnfCmc.value))
            let cnfGaId = getc('ga.id')
            if (cnfGaId) $t.push(this.setConf('ga.id',cnfGaId.value))
            let cnfGaApp = getc('ga.app')
            if (cnfGaApp) $t.push(this.setConf('ga.app',cnfGaApp.value))
            let cnfPN = getc('project.name')
            if (cnfPN) $t.push(this.setConf('project.name',cnfPN.value))
            let cnfLD = getc('locale.default')
            if (cnfLD) $t.push(this.setConf('locale.default',cnfLD.value))
            let cnfBot = getc('telegram.token')
            $t.push(this.M('telegram').updateBotToken(cnfBot.value).pipe(
              RxOps.mergeMap(res=>{
                if (res)
                {
                  let me = this.M('telegram').me()
                  return Rx.concat(this.setConf('telegram.token',cnfBot.value),this.setConf('telegram.username',me.f('username')))
                }
                return Rx.of(true)
              })
            ))

            $t.push(this.moduleInit())
            //return this.obsEnd(obs,{success:false})

            Rx.concat(...$t).subscribe(()=>{
              //this.obsEnd(obs,$confs.map(c=>c.toObject()))
            },err=>{
              log.err('confs load error')
              console.log(err)
              this.obsEnd(obs,[])
            },()=>{
              this.obsEnd(obs,{success:true})
            })
            /**/
          })
        }
      }


      this.obsEnd(obs,[apiGet,apiSet])
    })
  }


  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      if (!this.C('config'))
        return this.obsErr({code:'CONFIG_COLLECTION_NOT_EXISTS'})
      this.C('config').setNodeClass(Conf)
      this.C('config').findNodes({}).subscribe(list=>{
        let tmp = {}
        list.forEach(conf=>{
          this._confs[conf.key()] = conf
          tmp[conf.key()] = conf.value()
        })
        log.success('initialized',tmp)
        this.obsEnd(obs)
      })
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      //confs bundle fields
      let key = { name: g18n.t('Key'), code: 'key', type: 'string', flags: ['system','required','unique','logic'] }
      let val = { name: g18n.t('Value'), code: 'value', type: 'any', flags: ['system','required','logic'] }
      let sys = { name: g18n.t('Flags'), code: 'flags', type: 'string', flags: ['system','logic'], quantity: 0 }
      let desc = { name: g18n.t('Description'), code: 'description', type: 'string', flags:['system','logic']}

      let confs = this._core.module('bundle').bundle('config',true)
      confs.name(g18n.t('Configs'))
      confs.desc(g18n.t('System configurations'))
      confs.addFlag('system')
      confs.addFlag('locked')

      confs.addField(key)
      confs.addField(val)
      confs.addField(sys)
      confs.addField(desc)

      log.info('saving bundle',confs.code())
      $t.push(confs.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        this.obsErr({code:'INSTALL_CONFIG_BUNDLE_ERROR',error:err})
      },()=>{
        log.success('installed')
        this.obsEnd(obs)
      })
    })
  }

  setConf(key,val,desc=null,flags=null)
  {
    return Rx.Observable.create(obs=>{
      if (!this.C('config'))
        return this.obsErr(obs,{code:'NO_CONFIG_COLLECTION'})
      this.C('config').findNode({key:key}).subscribe(conf=>{
        if (!conf)
        {
          //not exists
          conf = this.C('config').newNode()
          conf.field('key').set(key)
        }
        conf.field('value').set(val)
        if (typeof desc == 'string')
          conf.field('description').set(desc)
        if (Array.isArray(desc)&&!flags)
          flags = desc
        if (Array.isArray(flags))
          conf.field('flags').all(flags)
        conf.save().subscribe(()=>{
          this._confs[key] = conf
          log.success('set <'+key+'>',val)
          this.obsEnd(obs)
        },err=>{
          log.err('save conf error',err)
          this.obsErr(obs,err)
        })
      },err=>{
        log.err('search conf error',err)
        this.obsErr(obs,err)
      })
    })
  }

  conf(key,def=null)
  {
    if (!this._confs[key]) return def
    let ret = this._confs[key].f('value')
    return ret
  }

}

module.exports = ConfigModule
