/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
const Threads = require('worker_threads')

let log = require('../../../log').m('CoreThread')

const CoreObject = require('./object')

class CoreThread extends CoreObject
{
  constructor(parent=null)
  {
    super(parent)

    this._worker = null

    //used from parent thread
    this.onThreadStart = new Rx.Subject()
    this.onThreadStop = new Rx.Subject()
    this.onThreadError = new Rx.Subject()
    this.onThreadMessage = new Rx.Subject()
    //used from child thread
    this.onParentMessage = new Rx.Subject()
    if (!Threads.isMainThread&&!!Threads.parentPort)
      Threads.parentPort.on('message',msg=>this.onParentMessage.next(msg))
  }

  //used from parent thread
  runThread(data)
  {
    return Rx.Observable.create(obs=>{
      this._worker = new Threads.Worker(process.argv[1],{workerData:data})
        .on('message',msg=>{
          this.onThreadMessage.next(msg)
        })
        .on('error',err=>this.onThreadError.next(err))
        .on('exit',res=>{
          this._worker = null
          this.onThreadStop.next(res)
          this.onThreadMessage.complete()
          this.onThreadError.complete()
          this.onThreadStop.complete()
          this.onThread
        })
        .on('online',res=>{
          this.onThreadStart.next(res)
          this.onThreadStart.complete()
          this.obsEnd(obs,res)
        })
    })
  }

  stopThread()
  {
    return Rx.iif(_=>!!this._worker,Rx.from(this._worker.terminate()),Rx.throwError({code:'THREAD_NOT_STARTED'}))
  }

  postThreadMessage(msg)
  {
    return Rx.Observable.create(obs=>{
      if (!this._worker)
        return this.obsErr(obs,{code:'WORKER_NOT_EXISTS'})
      this._worker.postMessage(msg)
      this.obsEnd(obs)
    })
  }

  //used from child thread
  postParentMessage(msg)
  {
    return Rx.Observable.create(obs=>{
      if (!Threads.parentPort)
        return this.obsErr(obs,{code:'WORKER_NOT_EXISTS'})
      Threads.parentPort.postMessage(msg)
      this.obsEnd(obs)
    })
  }

}

module.exports = CoreThread
