/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


/**
@fileOverview Core class file
*/

let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let log = require('../../log').m('Core')
let g18n = require('../../g18n')

const fs = require('fs')
const ModuleManager = require('../module_manager/module')


/**
Core object - main system core managing main system functionality
@class Core
*/
class Core
{

  constructor(pkg)
  {
    this._pkg = pkg

    this.locale = 'en'
    this.timezone = 'Asia/Bangkok'

    g18n.configure({
      locale: 'en',
      languages: ['en','ru'],
      'google-translate': true,
      path: './locales2'
    })

    this._mm = new ModuleManager(this)

    this._classes = {}
  }

  start()
  {
    return this._mm.buildInfoTree().pipe(
      RxO.mergeMap(itree=>{
        return this._mm.buildWorkingTree(itree)
      }),
      RxO.mergeMap(order=>{
        return this._mm.loadAndStart(order)
      }),
      RxO.last(),
      RxO.tap(_=>{
        this.module('events').pushEvent({
          type: 'core:start',
          data: {}
        },true)
      })
    )
  }

  install()
  {
    return this._mm.buildInfoTree().pipe(
      RxO.mergeMap(itree=>{
        return this._mm.installCore(itree)
      })
    )
  }

  distUpgrade()
  {
    return this._mm.buildInfoTree().pipe(
      RxO.mergeMap(itree=>{
        return this._mm.distUpgrade(itree)
      })
    )
  }

  module(code)
  {
    return this._mm.moduleObject(code)
  }

  registerClass(code,cls)
  {
    this._classes[code] = cls
  }

  cls(code)
  {
    if (this._classes[code]) return this._classes[code]
    return null
  }

  hookMap(...a)
  {
    return this._mm.hookMap(...a)
  }

  hookCollect(...a)
  {
    return this._mm.hookCollect(...a)
  }

}

module.exports = Core
