/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


/**
* @module core
* @author abrakadobr@gmail.com
*/
let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('CoreModule')
let g18n = require('../../g18n')

const fs = require('fs')
const $P = require('promptly')
const AbstractModule = require('./abstract/module')


/**
* CoreModule class is module object for core. It impements some of parent methods.
* @class CoreModule
*/
class CoreModule extends AbstractModule
{

  _update012to013()
  {
    //return Rx.concat(
      return this.M('module_manager').installAndStart('events')
    //).pipe(RxO.last())
  }

  _update011to012()
  {
    return Rx.concat(
      this.M('module_manager').installAndStart('actions'),
      this.M('module_manager').installAndStart('forms'),
      this.M('module_manager').installAndStart('interface')
    ).pipe(RxO.last())
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1.1')
          return Rx.concat(this._update011to012(),this._update012to013())
        if (from=='0.1.2')
          return this._update012to013()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }

  version()
  {
    return this._core._pkg.version
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let install = {}
      let $t = []

      $t.push(this._pAsk(g18n.t('Project name [TakatanJS]:'),{default:'TakatanJS'}).pipe(RxO.map(val=>{install['project.name']=val})))
      $t.push(this._pAsk(g18n.t('Default language [en]:'),{default:'en'}).pipe(RxO.map(val=>{install['core.locale']=val})))
      $t.push(this._pAsk(g18n.t('Default timezone [Asia/Bangkok]:'),{default:'Asia/Bangkok'}).pipe(RxO.map(val=>{install['core.timezone']=val})))

      Rx.concat(...$t).subscribe(()=>{},err=>this.obsErr(obs,err),_=>{
        let $tt = []

        $tt.push(this.M('config').setConf('project.name',install['project.name'],'Project Name',['system']))
        $tt.push(this.M('config').setConf('locale.default',install['core.locale'],'Default locale',['system']))
        $tt.push(this.M('config').setConf('core.timezone',install['core.timezone'],'Default timezone',['system']))

        Rx.concat(...$tt).subscribe(()=>{},err=>this.obsErr(obs,err),__=>{
          this.obsEnd(obs)
        })
      })
 
    })
  }

  _pAsk(question,opts)
  {
    return Rx.Observable.create(obs=>{
      $P.prompt(question,opts).then(ans=>{
        this.obsEnd(obs,ans)
      })
    })
  }

  _pConfirm(question,opts)
  {
    return Rx.Observable.create(obs=>{
      $P.confirm(question,opts).then(ans=>{
        this.obsEnd(obs,ans)
      })
    })
  }

  /**
    Ask at console prompt using promptly
    @param {...any} args - promply.prompt() args
  */
  consolePrompt(...args)
  {
    return this._pAsk(...args)
  }

}

module.exports = CoreModule
