/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')
let Threads = require('worker_threads')

let log = require('../../log').m('CronModule')
let g18n = require('../../g18n')

let Mtz = require('moment-timezone')

const CoreModule = require('../core/abstract/module')

const CronrxTest = require('./tests/cronrx')

class CronModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

    this._run = false
    this._tasks = []

    this.tick = new Rx.Subject()
    this.M('logic').registerLogicTest(CronrxTest)
    Rx.interval(1000).subscribe(t=>{
      this._tick(t)
    },err=>{
      log.err('interval 1000 error!')
      console.log(err)
    })
  }

  _tick(t)
  {
    if (!this._run) return
    //this.tick.next(t)
    let ev = {
      type: 'cron',
      data: {}
    }
    this.logicFields().forEach(f=>{
      if (f.info.type == 'number')
        ev.data[f.info.code] = f.values[0]
    })
    if (this.M('events'))
      this.M('events').pushEvent(ev)
  }

  _testMask(mask,val)
  {
    if (mask=='*')
      return true
    let masks = mask.split(',')
    let ret = true
    masks.forEach(m=>{
      if (!ret)
        return
      if (m=='*')
        return
      if (m.includes('-'))
      {
        let dias = m.split('-')
        let min = parseInt(dias[0])
        let max = parseInt(dias[1])
        if (isNaN(min)||isNaN(max)||min>=max||val<min||val>max)
        {
          ret = false
          return
        }
        return
      }
      if (m.startsWith('*/'))
      {
        let del = parseInt(m.slice(2))
        if (isNaN(del)||del<1||val%del)
        {
          ret = false
          return
        }
        return
      }
      let mv = parseInt(m)
      if (isNaN(mv)||mv!=val)
        ret = false
    })
    return ret
  }

  testMask(mask,ts=null)
  {
    if (!ts)
      ts = Mtz.tz(this._core.timezone)
    let masks = mask.trim().split(' ')
    if (masks.length!=6)
      return false
    let ret = true
    ret = this._testMask(masks[0],ts.seconds())
    if (!ret) return ret
    ret = this._testMask(masks[1],ts.minutes())
    if (!ret) return ret
    ret = this._testMask(masks[2],ts.hours())
    if (!ret) return ret
    ret = this._testMask(masks[3],ts.date())
    if (!ret) return ret
    ret = this._testMask(masks[4],ts.month())
    if (!ret) return ret
    ret = this._testMask(masks[5],ts.isoWeekday())
    return ret
  }

  logicFields()
  {
    let ret = []
    let tz = this._core.timezone
    let m = Mtz().tz(tz)
    //console.log(['cron m',m])

    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Now'),code:'now',type:'datetime'},
      values: [m]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Second'),code:'second',type:'number'},
      values: [m.second()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Minute'),code:'minute',type:'number'},
      values: [m.minute()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Hour'),code:'hour',type:'number'},
      values: [m.hour()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Day of month'),code:'day',type:'number'},
      values: [m.date()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Month'),code:'month',type:'number'},
      values: [m.month()+1]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Year'),code:'year',type:'number'},
      values: [m.year()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Day of week'),code:'dow',type:'number'},
      values: [m.isoWeekday()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Week of year'),code:'woy',type:'number'},
      values: [m.week()]
    })
    ret.push({
      scope: 'cron',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Day of year'),code:'doy',type:'number'},
      values: [m.dayOfYear()]
    })
 
    return ret
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      //registring cron event
      let inf = {
        code: 'cron',
        name: g18n.t('On cron'),
        data: []
      }
      this.logicFields().forEach(f=>{
        if (f.info.type == 'number')
          inf.data.push(f.info)
      })
      log.success('cron event registred')
      if (this.M('events'))
        this.M('events').registerEventType(inf)
      this.obsEnd(obs)
    })
  }

  moduleStart()
  {
    return Rx.Observable.create(obs=>{
      log.log('starting')
      if (!Threads.isMainThread)
      {
        log.warn('not starting in threads!')
        return this.obsEnd(obs)
      }
      this._run = true
      log.info('started')
      this.obsEnd(obs)
    })
  }

  moduleStop()
  {
    return Rx.Observable.create(obs=>{
      log.log('stopping')
      this._run = false
      log.success('stoped')
      this.obsEnd(obs)
    })
  }

}

module.exports = CronModule
