/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('CronrxTest')
let g18n = require('../../../g18n')

const mtz = require('moment-timezone')

const AbstractDateTimeTest = require('../../field/datetime/tests/datetime')

class CronrxTest extends AbstractDateTimeTest
{

  static info()
  {
    return {
      code: 'cronrx',
      name: g18n.t('Cron test success'),
      iname: g18n.t('Cron test fail'),
      ops: [{
        //type: 'datetime',
        //name: g18n.t('Base'),
        //manual: true
      //},{
        type: 'string',
        name: g18n.t('test mask [s m h d m w] [!*0-9,]'),
        manual: true
      }],
      types: ['datetime']
    }
  }

  test(ctx,valI=0)
  {
    let target = this.value(valI)
    let mmb = mtz().tz(this._core.timezone)
    //console.log({t:target,b:base})

    let gran = this.fcOp(ctx,1,[])
    let targetFormat = this.granFormat(gran)
    let mmt = this.momentObj(ctx,target,true)
    //let mmb = this.momentObj(ctx,base)

    if (!mmt.isValid()||!mmb.isValid())
      return this._conf.invert?true:false

    let rxstr = this.op(ctx,1)||''
    let rxs = rxstr.split(' ')

    let ret = true

    //log.test('testing',[mmt,mmb,rxs])

    let crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('second'))
      if (!this.compOne(mmb.second(),mmt.second(),crx))
        ret = false

    if (!ret)
      return this._conf.invert?!ret:ret
    //log.test('sec ok, test minute')
    crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('minute'))
      if (!this.compOne(mmb.minute(),mmt.minute(),crx))
        ret = false

    if (!ret)
      return this._conf.invert?!ret:ret
    //log.test('min ok, test hour')
    crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('hour'))
      if (!this.compOne(mmb.hour(),mmt.hour(),crx))
        ret = false

    if (!ret)
      return this._conf.invert?!ret:ret
    //log.test('hour ok, test day')
    crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('day'))
      if (!this.compOne(mmb.date(),mmt.date(),crx))
        ret = false

    if (!ret)
      return this._conf.invert?!ret:ret
    //log.test('day ok, test month')
    crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('month'))
      if (!this.compOne(mmb.month()+1,mmt.month()+1,crx))
        ret = false

    if (!ret)
      return this._conf.invert?!ret:ret
    //log.test('month ok, test dow')
    crx = '*'
    if (rxs.length) crx = rxs.shift()
    if (gran.includes('month')&&gran.includes('day')&&gran.includes('year'))
      if (!this.compOne(mmb.day()+1,mmt.day()+1,crx))
        ret = false

    //log.test('all ok, tested!')
    return this._conf.invert?!ret:ret
  }

  compOne(base,val,rx)
  {
    //log.warn('testing piece!',[base,val,rx])
    if (rx=='*') return true
    if (rx=='!') return base==val
    let pieces = rx.split(',')
    let ret = true
    let done = false
    pieces.forEach(pc=>{
      if (!ret) return
      if (done) return

      if (pc.includes('-'))
      {
        //dia
        //log.test('dia piece test',[pc,val,base])
        let dia = pc.split('-')
        if (dia.length!=2)
        {
          ret = false
          return
        }
        let min = parseInt(dia[0])||-1
        let max = parseInt(dia[1])||-1
        if (dia[0]=='*') min = 1
        if (dia[0]=='!') min = base
        if (dia[1]=='*') max = val
        if (dia[1]=='!') max = base
        if (min>0&&min<=max&&val>=min&&val<=max)
        {
          ret = true
          done = true
          return
        }
      } else {
        //direct
        //log.test('direct piece test',[pc,val,base])
        if (pc=='*')
        {
          ret = true
          done = true
          return
        }
        if (pc=='!'&&val==base)
        {
          ret = true
          done = true
          return
        }
        let pnum = parseInt(pc)||-1
        if (pnum+''==pc&&pnum!=-1&&pnum==val)
        {
          ret = true
          done = true
          return
        }
      }
    })
    if (!done)
      ret = false
    return ret
  }
  

}

module.exports = CronrxTest
