/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')
let request = require('request')

let log = require('../../log').m('CurrencyLayer.js')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')

class CurrencyLayer extends CoreObject
{

  constructor(parent)
  {
    super(parent)
    this._inited = false
  }

  _cl(endpoint)
  {
    return Rx.Observable.create(obs=>{
      let apiKey = this.M('config').conf('cl.key','')
      if (!apiKey||apiKey=='')
        return this.obsErr(obs,{code:'NO_API_KEY'})
      const requestOptions = {
        method: 'GET',
        uri: 'http://www.apilayer.net/api/'+endpoint+'?access_key='+apiKey,
        json: true,
        gzip: true
      }
      request(requestOptions,(err,resp,body)=>{
        if (err)
          return this.obsErr(obs,{code:'REQUEST_ERROR',error:err})
        this.obsEnd(obs,body)
      })
    })
  }

  updateCoins()
  {
    return Rx.Observable.create(obs=>{
      log.log('updating currencylayer coins')
      this._cl('list').subscribe(res=>{
        let $t = []
        for(let symbol in res.currencies)
        {
          let name = res.currencies[symbol]
          $t.push(this.C('coin').findNode({symbol:symbol}).pipe(RxOps.mergeMap(coin=>{
            if (coin)
              return Rx.of(coin)
            coin = this.C('coin').newNode()
            coin.f('symbol',symbol)
            coin.f('name',name)
            coin.f('type','fiat')
            coin.f('decims',2)
            if (symbol=='BTC')
            {
              coin.f('type','crypto')
              coin.f('decims',8)
            }
            coin.f('lastusd',0)
            return coin.save()
          })))
        }
        Rx.concat(...$t).subscribe(cn=>{
          log.log('coin updated',cn.f('symbol'))
        },err=>{},()=>{
          this._inited = true
          log.success('coins list updated')
          this.obsEnd(obs,true)
        })
      },err=>{
        log.err('failed update coins list',err)
        this.obsEnd(obs,false)
      })
    })
  }

  update()
  {
    return Rx.Observable.create(obs=>{
      this._cl('live').subscribe(data=>{
        if (data.source!='USD')
          return this.obsErr(obs,'currencies base is node USD!',data.source)
        let $t = []
        for (let key in data.quotes)
        {
          if (!key.startsWith('USD'))
            continue
          let symbol = key.slice(3)
          if (!symbol.length)
            continue
          let price = data.quotes[key]
          if (price) price = 1/price
          $t.push(this.C('coin').findNode({symbol:symbol}).pipe(RxOps.mergeMap(cn=>{
            if (!cn)
              return Rx.of(cn)
            if (cn.f('type')!='fiat')
              return Rx.of(cn)
            cn.f('lastusd',price)
            return cn.save()
          })))
        }
        let cnt = 0
        Rx.concat(...$t).subscribe(coin=>{
          if (coin)
          {
            cnt++
            log.log('coin updated',coin.f('symbol'))
          }
        },err=>{
          log.err('updating coin error!')
          console.log(err)
          this.obsErr(obs,err)
        },()=>{
          this.obsEnd(obs,cnt)
        })
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

}

module.exports = CurrencyLayer
