/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')
let request = require('request')

let log = require('../../log').m('CoinMarketCap.js')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')

class CoinMarketCap extends CoreObject
{

  constructor(parent)
  {
    super(parent)
  }

  _cmc(endpoint)
  {
    return Rx.Observable.create(obs=>{
      let apiKey = this.M('config').conf('cmc.key','')
      if (!apiKey||apiKey=='')
        return this.obsErr(obs,{code:'NO_API_KEY'})
      const requestOptions = {
        method: 'GET',
        uri: 'https://pro-api.coinmarketcap.com/v1/'+endpoint,
        //qs: {
          //start: 1,
          //limit: 5000,
          //convert: 'USD'
        //},
        headers: {
          'X-CMC_PRO_API_KEY': apiKey
        },
        json: true,
        gzip: true
      }
      request(requestOptions,(err,resp,body)=>{
        if (err)
          return this.obsErr(obs,{code:'REQUEST_ERROR',error:err})
        this.obsEnd(obs,body)
      })
    })
  }


  update()
  {
    return Rx.Observable.create(obs=>{
      this._cmc('cryptocurrency/listings/latest').subscribe(data=>{
        let $t = []
        if (!data||!Array.isArray(data.data))
          return this.obsEnd(obs,0)
        data.data.forEach(el=>{
            
          let ret = {
            symbol: el.symbol,
            name: el.name,
            platform: null,
            cmcrank: el.cmc_rank,
            USD: el.quote.USD.price,
            c1h: el.quote.USD.percent_change_1h,
            c1d: el.quote.USD.percent_change_24h,
            c1w: el.quote.USD.percent_change_7d
          }
          ret.USD = Math.floor( ret.USD*100 )/100
          ret.c1h = Math.floor( ret.c1h*100 )/100
          ret.c1d = Math.floor( ret.c1d*100 )/100
          ret.c1w = Math.floor( ret.c1w*100 )/100
          if (el.platform&&el.platform.symbol)
            ret.platform = el.platform.symbol

          $t.push(this.C('coin').findNode({symbol:ret.symbol}).pipe(RxOps.mergeMap(nd=>{
            if (!nd)
            {
              nd = this.C('coin').newNode()
              nd.f('symbol',ret.symbol)
              nd.f('name',ret.name)
              nd.f('platform',ret.platform)
              nd.f('type','crypto')
              nd.f('decims',8)
            }
            nd.f('cmcrank',ret.cmcrank)
            nd.f('lastusd',ret.USD)
            nd.f('usdc1h',ret.c1h)
            nd.f('usdc1d',ret.c1d)
            nd.f('usdc1w',ret.c1w)
            return nd.save()
          })))
        })
        let cnt = 0
        Rx.concat(...$t).subscribe(coin=>{
          if (coin)
          {
            cnt++
            log.log('coin updated',coin.f('symbol'))
          }
        },err=>{
          log.err('updating coin error!')
          console.log(err)
          this.obsErr(obs,err)
        },()=>{
          //log.success('updated coins',cnt)
          this.obsEnd(obs,cnt)
        })
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

}

module.exports = CoinMarketCap
