/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let path = require('path')

let log = require('../../log').m('DashboardRouter')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')

class DashboardRouter extends CoreObject
{
  constructor(parent=null)
  {
    super(parent)

    this._app = null
    this._routes = {}
  }

  init()
  {
    return Rx.Observable.create(obs=>{
      log.log('initializing')
      this._app = this._parent._app
      this._core.hookCollect('dashboardRoutes',{app:this._app}).subscribe(list=>{
        //console.log(list)
        list.forEach(l=>{
          l.forEach(route=>{
            this.addRoute(route)
          })
        })
        //console.log(this._app._router)
        log.log('initialized')
        obs.next()
        obs.complete()
      })
      this._app.get('**',(req,res)=>{
        res.sendFile(path.join(__dirname+'../../../../public/index.html'))
      })
    })
  }

  addRoute(route,fn)
  {
    if (typeof this._app[route.method] != 'function')
      return
    log.log('registring route',route.method+' '+route.url)
    if (!this._routes[route.url])
      this._routes[route.url] = {}
    this._routes[route.url][route.method] = {
      access: route.access,
      fn: fn
    }
    this._app[route.method](route.url,(req,res)=>{
      setTimeout(()=>{
        let ctx = req._ctx
        ctx.addField({
          name: g18n.t('URL params'),
          code: 'params',
          type: 'any',
          quantity: 1,
          flags: ['readonly']
        })
        ctx.addField({
          name: g18n.t('Request query'),
          code: 'query',
          type: 'any',
          quantity: 1,
          flags: ['readonly']
        })
        ctx.addField({
          name: g18n.t('Request body'),
          code: 'body',
          type: 'any',
          quantity: 1,
          flags: ['readonly']
        })
        ctx.addField({
          name: g18n.t('Request files'),
          code: 'files',
          type: 'any',
          quantity: 1,
          flags: ['readonly']
        })
        ctx.f('params',req.params)
        ctx.f('body',req.body)
        ctx.f('query',req.query)
        if (req.files)
          ctx.f('files',req.files)
        let user = ctx.user()
        if (route.access&&(!user||!user.can(route.access)))
          return res.status(401).location('/logout').end()
        route.fn(ctx).subscribe(ret=>{
          if (typeof ret == 'string')
          {
            res.send(ret)
          } else {
            res.json(ret)
          }
        },err=>{
          log.err('route fn error')
          console.log(err)
          if (!err.code||err.code==401||err==401)
          {
            res.status(401).location('/logout').end()
            return
          }
          res.status(404).end()
        })
      },0)
    })
  }

}

module.exports = DashboardRouter
