/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('PostEventAction')
let g18n = require('../../g18n')

const AbstractAction = require('../actions/abstract')

class PostEventAction extends AbstractAction
{

  constructor(...args)
  {
    super(...args)
  }

  //common routines
  static info()
  {
    return {
      code: 'postevent',
      name: g18n.t('Post event'),   //used on big space
      shortname: g18n.t('Post event'),//used on small space
      desc: g18n.t('Generate system event'),
      help: g18n.t('Event can be catched by other interfaces or modules, to start special defined behaviour.'),
      confs: [{
        code: 'type',
        name: g18n.t('Code type'),
        type: 'string'
      },{
        code: 'data',
        name: g18n.t('Code data'),
        type: 'any'
      }],  //fields
      faicon: 'fas fa-bell',
      uicolor: 'text-black',  //old style
      uigroup: 'helpers',        //new style

      //for admin ui relations with other reactions

      //message = as main reaction only
      //reaction = as subreaction only
      //any = can be any
      mode: 'any', 
      //can be only one in action/message/page/etc block
      greedy: false,
      //codes of reactions, can't use together in same action/message/page/etc block
      conflicts: [],
      //can be used only when other reaction with 'doSend:true' exists
      requireSend: false,
      //this reaction do some real whatever sending via interface
      doSend: false,

      //for admin ui access by routers
      //by default will be enabled everythere.

      //should contain router type codes
      //action can be used only on limited types of routers
      routersLimit: [],
      //action can not be used at this types of routers
      routersConflict: []
    }
  }

  /**/
  loadConfs(ctx)
  {
    return Rx.Observable.create(obs=>{
      let inf = this.constructor.info()
      let ret = {}
      if (!inf||!Array.isArray(inf.confs)||!inf.confs.length)
        return this.obsEnd(obs,ret)
          //ctx.selectorResult(data.user),
      let data = this.f('data')
      if (!data||!data[inf.code])
        return this.obsEnd(obs,ret)
      data = data[inf.code]
      let einf = this.M('events').eventType(data.type)
      //console.log(['event info',einf])
      ret.type = data.type
      ret.data = {}
      let $t = einf.data.map(cfg=>{
        return ctx.selectorResult(data.data[cfg.code],cfg).pipe(RxO.map(v=>{
          ret.data[cfg.code] = v
          return v
        }))
      })
      Rx.concat(...$t).subscribe(_=>{},err=>{
        this.obsEnd(obs,ret)
      },_=>{
        this.obsEnd(obs,ret)
      })
    })
  }
  /**/

  execute(ctx,...data)
  {
    return this.loadConfs(ctx).pipe(RxO.map(cfg=>{
      log.info('pushing event',cfg)
      this.M('events').pushEvent(cfg)
      return true
    }))
  }

}

module.exports = PostEventAction
