/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('AllModifier')
let g18n = require('../../../../g18n')

const AbstractModifier = require('../../modifier')

class AllModifier extends AbstractModifier
{

  static info()
  {
    return {
      code: 'all',
      name: g18n.t('All'),
      iname: '',  //not exists
      invert: false, //cuz not invertable
      flat: true, //modifier result is single or array
      strict: '',
      ops: [],
      types: ['[]']
    }
  }

  setRange()
  {
    let q = this._dataTarget.quantity()
    if (q==1)
      return [0]
    let size = this._dataTarget.size()
    let max = q>1?q:size
    if (!max) max = 1
    let ret = []
    if (q===0)
      ret.push(-1)
    for(let i=0;i<max;i++)
      ret.push(i)
    return ret
  }

  getNext()
  {
    return this._dataTarget.one(this.pos())
  }

  reset()
  {
    this.pos(0)
  }

  accumulate(exRes)
  {
    let pos = this.pos()
    let size = this._dataTarget.size()
    let q = this._dataTarget.quantity()
    let max = q>1?q:size
    pos++
    let stop = false
    if (pos>=max)
      stop = true
    if (!exRes)
      stop = true
    if (!stop)
      this.pos(pos)
    return {
      stop: stop,
      res: exRes
    }
  }



  testValues(ctx)
  {
    return this._field.values
  }

  doNext(testRes)
  {
    let ret = {
      next: true,
      res: true
    }
    if (!testRes)
    {
      ret.res = false
      ret.next = false
    }
    //log.info('doNext?',{last:testRes,ret:ret})
    return ret
  }

  /*
  test(ctx=null,valI=0)
  {
    //let pos = this.op(ctx)
    let next = this._next
    let total = this._field.values.length
    if (this._field.info.quantity>1&&total<this._field.info.quantity)
      total = this._field.info.quantity
    let ret = true
    for(let vi=0;vi<total;vi++)
    {
      let t = next.test(ctx,vi)
      if (t)
        return this._conf.invert?!ret:ret
    }
    ret = false
    return this._conf.invert?!ret:ret
  }
  /**/

}

module.exports = AllModifier
