/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('DatetimeFieldModule')

const CoreModule = require('../../core/abstract/module')

const TimeZoneField = require('./timezone')
const DatetimeField = require('./datetime')
const NowGenerator = require('./generators/now')
const SysTZGenerator = require('./generators/systz')
//const ShiftGenerator = require('./generators/shift')
const EqTest = require('./tests/eq')
const OlderTest = require('./tests/older')
const YangerTest = require('./tests/yanger')

const TzFromStringSetter = require('./setters/tzfromstring')
const DTLFromStringSetter = require('./setters/dtlfromstring')

class DatetimeFieldModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this.M('field').registerFieldType(TimeZoneField)
    this.M('field').registerFieldType(DatetimeField)
    this.M('field').registerGenerator(NowGenerator)
    this.M('field').registerGenerator(SysTZGenerator)
    //this.M('field').registerGenerator(ShiftGenerator)

    this.M('logic').registerLogicTest(EqTest)
    this.M('logic').registerLogicTest(OlderTest)
    this.M('logic').registerLogicTest(YangerTest)

    this.M('field').registerSetter(TzFromStringSetter)
    this.M('field').registerSetter(DTLFromStringSetter)
  }

  granFormat(conf=[])
  {
    let ret = ''
    if (conf.includes('year'))
    {
      if (ret!='') ret+='-'
      ret += 'YYYY'
    }
    if (conf.includes('month'))
    {
      if (ret!='') ret+='-'
      ret += 'MM'
    }
    if (conf.includes('day'))
    {
      if (ret!='') ret+='-'
      ret += 'DD'
    }
    if (conf.includes('hour'))
    {
      if (ret!='') ret+='T'
      ret += 'HH'
    }
    if (conf.includes('minute'))
    {
      if (ret!='') ret+=':'
      ret += 'mm'
    }
    if (conf.includes('second'))
    {
      if (ret!='') ret+=':'
      ret += 'ss'
    }
    if (ret=='')
      ret = 'YYYY-MM-DDTHH:mm:ss'
    //ret += 'ZZ'
    return ret
  }


}

module.exports = DatetimeFieldModule
