/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('DateTimeListFromStringSetter<boolean>')
let g18n = require('../../../../g18n')

const AbstractSetter = require('../../../logic/setter')

let mtz = require('moment-timezone')

class DateTimeListFromStringSetter extends AbstractSetter
{

  static info()
  {
    return {
      code: 'dtlfromstring',
      invert: false,
      name: g18n.t('From string'),
      iname: '',
      ops: [{
        name: 'source',
        manual: true,
        type: 'string'
      },{
        name: 'delimeters',
        manual: true,
        type: 'string',
        value: ', ;/'
      },{
        name: 'format',
        helper: 'parsing format rewrite',
        manual: true,
        canNone: true,
        type: 'string'
      },{
        name: 'timezone',
        manual: true,
        type: 'timezone'
      }],
      types: ['[datetime]']
    }
  }

  setRange(range,target)
  {
    if (!range.length)
      return false  //can't work with arrays
    //let val = this._dataTarget.oneModified()
    let str = this.op()
    let dels = this.op(1,',')
    let fmt = this.op(2,'')
    let tz = this.op(3,this.M('config').conf('core.timezone'))
    //console.log(this.dcop(0))
    let ffmt = this.M('field/datetime').granFormat(this.dcop(1))
    let strs = str.split( new RegExp('['+dels+']') ).map(x=>x.trim()).filter(x=>x.length)
    if (!fmt.length)
      fmt = ffmt
    let parsed = strs.map(x=>mtz.tz(x,fmt,tz)).filter(x=>x.isValid())
    //console.log(parsed,tz,fmt,ffmt)
    if (!parsed.length)
      return false
    if (range[0]==-1)
    {
      //set all
      target.clear()
      parsed.forEach((p,pi)=>{
        target.setModified(p,pi)
      })
    } else {
      parsed.forEach(p=>{
        if (!range.length)
          return
        let ind = range.shift()
        target.setModified(p,ind)
      })
    }
    //console.log(parsed)
    //log.warn('set?',[strs,fmt,range])
    //return false
    //range.forEach(pos=>{
      //target.setModified(nxz,pos)
    //})
    return true
  }

}

module.exports = DateTimeListFromStringSetter
