/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../../log').m('TZFromStringSetter<boolean>')
let g18n = require('../../../../g18n')

const AbstractSetter = require('../../../logic/setter')

let mtz = require('moment-timezone')

class TZFromStringSetter extends AbstractSetter
{

  static info()
  {
    return {
      code: 'tzfromstring',
      invert: false,
      name: g18n.t('From string'),
      iname: '',
      ops: [{
        name: 'source',
        manual: true,
        type: 'string'
      }],
      types: ['timezone']
    }
  }

  setRange(range,target)
  {
    if (!range.length)
      return false  //can't work with arrays
    let val = this._dataTarget.oneModified()
    let nx = this.op()
    let names = mtz.tz.names()
    let nxi = parseInt(nx)
    let nxz = null
    if (!isNaN(nxi))
    {
      //parse from number offset
      let mins = nxi*60
      names.forEach(nm=>{
        let zn = mtz.tz(nm).format('z')
        if (!zn.startsWith('+')&&!zn.startsWith('-')) return
        if (zn.length==5)
          zn = zn.slice(0,3)
        let znh = parseInt(zn)
        if (znh == nxi)
        {
          if (nxz)
          {
            //prefer not indian
            if (!nm.startsWith('Etc/GMT'))
            {
              if (nxz.startsWith('Indian')&&!nm.startsWith('Indian'))
                nxz = nm
              if (!nxz.startsWith('Asia')&&nm.startsWith('Asia'))
                nxz = nm
              if (!nxz.startsWith('Europe')&&nm.startsWith('Europe'))
                nxz = nm
              //console.log(['zn',zn,znh,nxz,nm])
            }
          } else {
            nxz = nm
          }
        }
      })
    } else {
      //search from name
      let src = nx.toUpperCase()
      nxz = names.reduce((acc,nm)=>{
        return nm.toUpperCase().includes(src)?nm:acc
      },null)
    }
    if (!nxz)
      return false
    //log.warn('set?',[val,nx,nxz])
    range.forEach(pos=>{
      target.setModified(nxz,pos)
    })
    return true
  }

}

module.exports = TZFromStringSetter
