/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let moment = require('moment')
let mtz = require('moment-timezone')

let log = require('../../../../log').m('DatetimeEqTest')
let g18n = require('../../../../g18n')

//const AbstractTest = require('../../../logic/abstract/test')
const AbstractDateTimeTest = require('./datetime')

class EqTest extends AbstractDateTimeTest
{

  static info()
  {
    return {
      code: 'eq',
      invert: true,
      name: g18n.t('Equal to'),
      iname: g18n.t('Not equal to'),
      ops: [{
        type: 'datetime',
        name: g18n.t('comparing expression'),
        manual: true
      }],
      types: ['datetime']
    }
  }

  queryFor(ctx,vctx)
  {
    return Rx.Observable.create(obs=>{
      let tz = this.dcop(0,this.M('config').conf('core.timezone'))
      let gran = this.dcop(1,[])
      let targetFormat = this.granFormat(gran)
      let other = this.op()
      let tgt = this.dataTarget().targetCode()
      let ret = {}
      let vv = other.format(targetFormat)
      let sv = this.defGran(other).utc().format('YYYY-MM-DDTHH:mm:ssZZ')
      ret[tgt] = new Date(sv)
      if (this._conf.invert)
        ret = {$nin: [ret] }
      //console.log(ret)
      return this.obsEnd(obs,ret)
    })
  }


  _test(v)
  {
    let target = v
    let other = this.op()
    let mT = null
    let mO = null
    if (target.endsWith('Z'))
      mT = moment.tz(target,'YYYY-MM-DDTHH:mm:ss:sss')
    else
      mT = moment.tz(target,'YYYY-MM-DDTHH:mm:ssZZ')
    if (other.endsWith('Z'))
      mO = moment.tz(other,'YYYY-MM-DDTHH:mm:ss:sss')
    else
      mO = moment.tz(other,'YYYY-MM-DDTHH:mm:ssZZ')
    if (!mT.isValid()||!mO.isValid())
      return false
  
    let targetGran = this.dcop(1,['year','month','day','hour','monite','second'])
    //if (this._field&&this._field.info&&this._field.info.conf&&this._field.info.conf.ops&&this._field.info.conf.ops[1])
      //targetGran = this._field.info.conf.ops[1].manual

    let ret = true
    if (targetGran.includes('second')&&mT.utc().seconds()!=mO.utc().seconds())
      ret = false
    if (targetGran.includes('minute')&&mT.utc().minute()!=mO.utc().minute())
      ret = false
    if (targetGran.includes('hour')&&mT.utc().hour()!=mO.utc().hour())
      ret = false
    if (targetGran.includes('day')&&mT.utc().date()!=mO.utc().date())
      ret = false
    if (targetGran.includes('month')&&mT.utc().month()!=mO.utc().month())
      ret = false
    if (targetGran.includes('year')&&mT.utc().year()!=mO.utc().year())
      ret = false

    return ret
    //console.log(['EqDateTimeTest!!!!',mT,mO,targetGran,ret])
    //log.info('tested',[ret])
    //if (this._conf.invert) ret=!ret
    //this.obsEnd(obs,ret)
  }


}

module.exports = EqTest
