/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
//let moment = require('moment')
let moment = require('moment-timezone')

let log = require('../../../../log').m('YangerTest')
let g18n = require('../../../../g18n')

const AbstractDateTimeTest = require('./datetime')

class YangerTest extends AbstractDateTimeTest
{

  static info()
  {
    return {
      code: 'yanger',
      invert: true,
      name: g18n.t('Yanger than'),
      iname: g18n.t('Not yanger than'),
      ops: [{
        type: 'datetime',
        name: g18n.t('Moment'),
        manual: true
      },{
        type: 'string',
        name: g18n.t('Shift'),
        manual: true
      }],
      types: ['datetime']
    }
  }

  queryFor(ctx,vctx)
  {
    return Rx.Observable.create(obs=>{

      let ret = {}      
        
      let fmt = 'YYYY-MM-DDTHH:mm:ssZZ'
      let tgt = this.dataTarget().targetCode()
      let base = this.op(0)
      let shift = this.op(1)
     
      if (!base.isValid())
        return this.obsEnd(obs,ret)
      
      let mO = this.momentShift(base,shift)

      ret[tgt] = {$gte:mO.format(fmt) }
      if (this._conf.invert)
       ret[tgt] = {$lte: mO.format(fmt) }
   
      return this.obsEnd(obs,ret)
    })
  }


  _test(v)
  {
    let target = v
    let base = this.op(0)
    let shift = this.op(1)
    let gran = this.dcop(1,[])
    let targetFormat = this.granFormat(gran)
    if (!target||!base)
      return false
    let mT = this.momentObj(ctx,target,true)
    let mO = this.momentObj(ctx,base,false)
    if (!mT.isValid()||!mO.isValid())
      return false
    
    mO = this.momentShift(mO,shift)

    let m1 = moment( mT.utc().format(targetFormat), targetFormat, '')
    let m2 = moment( mO.utc().format(targetFormat), targetFormat, '')

    let ret = false
    if (m1.unix()<m2.unix())
      ret = true
    return ret
  }

}

module.exports = YangerTest
