/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')
let mg = require('mongoose')

let log = require('../../log').m('AbstractField')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')
const Node = require('../node/node')

class AbstractField extends CoreObject
{

  constructor(parent,info,ctx={})
  {
    super(parent)
    this._core = this._parent._core

    this._info = info
    this._ctx = ctx

    this.name = info.name
    this.code = info.code
    this.type = info.type
    this._xtypes = info.xtypes
    //if (this.code=='senderUser')
      //console.log('!',this._info)
    this.mapped = info.mapped
    this.quantity = info.quantity
    this.generator = info.generator
    this.def = info.def
    this.conf = info.conf
    this.flags = info.flags

    this._validators = []

    //if (info.code=='created')
    //{
      //console.log(['field constructor',this._parent._model,this.conf])
    //}
    this.flags.forEach(flag=>{
      let validator = this.M('field').genValidator(this,flag)
      if (validator&&validator.code!='nocare')
        this.addValidator(validator)
    })
  }

  static info()
  {
    return {
      code: 'abstract',
      name: g18n.t('Abstract field')
      /*
      conf: {
        generator: [ops]
        ops:[{
          type, name....
        }]
      }
      */
    }
  }

  static schema(info,schema={})
  {
    schema.type = mg.Schema.Types.Mixed
    return schema
  }

  static populateSchema(info)
  {
    //by default fields are 'data direct'
    return null
  }

  //dirty methods for frendship with setters
  dataType()
  {
    let tp = this.type
    if (this.quantity!=1)
      tp = '['+tp+']'
    return tp
  }

  setRange()
  {
    //if (this._modifier)
      //return this._modifier.setRange()
    let q = this.quantity
    if (q==1)
      return [0]
    if (q==0)
      return [-1]
    let ret = []
    for (let i=0;i<q;i++)
      ret.push(i)
    return ret
  }

  setModified(val,i=0)
  {
    this.set(val,i)
  }

  saveModified()
  {
    return Rx.of(1)
  }

  //can be redefined by children classes
  //for type detalization

  xtypes()
  {
    if (this._xtypes&&this._xtypes.length)
      return [this.dataType(),...this._xtypes]
    return [this.dataType()]
  }

  _acceptedValidators()
  {
    return []
  }

  acceptedValidators()
  {
    let ret = ['nocare','isnull','empty','required','unique']
    let childs = this._acceptedValidators()
    childs.forEach(c=>{
      ret.push(c)
    })
    return ret
  }

  hasFlag(flag)
  {
    return this.flags.reduce((acc,f)=>{
      return f==flag?true:acc
    },false)
  }

  validatorAccepted(code)
  {
    let list = this.acceptedValidators()
    return list.indexOf(code) != -1
  }

  ctx(next)
  {
    if (typeof next != 'undefined')
      this._ctx = next
    return this._ctx
  }

  addValidator(validator)
  {
    if(!this.validatorAccepted(validator.code)) return
    this._validators.push(validator)
  }

  validate()
  {
    return Rx.Observable.create(obs=>{
      //console.log(this.code,this._validators)
      let $t = this._validators.map(v=>{
        return v.validate()
      })
      let failed = []
      Rx.concat(...$t).subscribe(next=>{
        if (!next.valid)
          failed.push(next)
      },err=>{
        obs.error({valid:false,error:err,code:'valid'})
      },()=>{
        let ret = {valid:true}
        if (failed.length)
        {
          ret.code = this.code
          ret.valid = false
          ret.errors = failed
        }
        obs.next(ret)
        obs.complete()
      })
    })
  }

  init()
  {
    //by default field do nothing on init
  }
  _v2db(v,i) {
    if (this.hasFlag('virtual'))
      return v
    return JSON.stringify(v)
  }
  _db2v(db,i) {
    if (this.hasFlag('virtual'))
      return db
    //console.log(['ANY?',this.code,db])
    if(typeof db == 'undefined') return
    return JSON.parse(db)
  }

  serialize()
  {
    let ret = this.all().map(x=>this._serializev(x))
    if (this.quantity == 1)
      return ret[0]
    return ret
  }

  unserialize(v)
  {
    let lst = []
    if (Array.isArray(v))
      lst = v.map(x=>this._unserializev(x))
    else
      lst = [this._unserialize(v)]
    this.all(lst)
  }


  _serializev(v)
  {
    return v
  }

  _unserializev(v)
  {
    return v
  }

  size()
  {
    return this._parent._model[this.code].length
  }

  has(val)
  {
    let ret = false
    this.forEach(v=>{
      if (ret) return
      if (v instanceof Node&&v.eq(val))
      {
        ret = true
        return
      }
      if (val===ret)
        ret = true
    })
    return ret
  }

  populate(depth=0)
  {
    return Rx.of(this)
  }

  all(vals)
  {
    if (typeof(vals)!='undefined'&&Array.isArray(vals))
    {
      this.clear()
      vals.forEach((v,i)=>{
        this.set(v,i)
      })
    }
    if (!Array.isArray(this._parent._model[this.code]))
      return []
    return this._parent._model[this.code].map(d=>{
      return this._db2v(d)
    })
  }

  forEach(fn)
  {
    if (typeof fn != 'function')
      return this
    for(let i=0;i<this.size();i++)
      fn(this.getp(i),i)
    return this
  }

  filter(fn,self=true)
  {
    if (typeof fn != 'function')
      return this
    let next = []
    for(let i=0;i<this.size();i++)
      if(fn(this.getp(i),i))
        next.push(this.getp(i))
    if (!self)
      return next
    this.all(next)
    return this
  }

  reduce(fn,init)
  {
    let ret = init
    if (typeof fn != 'function')
      return ret
    for(let i=0;i<this.size();i++)
      ret = fn(ret,this.getp(i),i)
    return ret
  }

  get(index=0)
  {
    if (index<0) index = 0
    if (index>0&&this.quantity&&index>=this.quantity)
      index = this.quantity-1
    let ret
    if (this.hasFlag('virtual'))
    {
      ret = this._parent._model[this.code][index]
      //console.log(['field get',index,this.code,this._parent._model[this.code]])
      return ret
    }
    if (!this._parent._model||!Array.isArray(this._parent._model[this.code]))
      return ret
    return this._db2v(this._parent._model[this.code][index],index)
  }

  getp(index=0)
  {
    if (index<0) index = 0
    if (index>0&&this.quantity&&index>=this.quantity)
      index = this.quantity-1
    if (typeof(this._parent._populated[this.code][index])!='undefined')
      return this._parent._populated[this.code][index]
    return this.get(index)
  }


  set(v,index=0)
  {
    if (index<0) index = 0
    if (index>0&&this.quantity&&index>=this.quantity)
      index = this.quantity-1
    this._saved = false
    if (!Array.isArray( this._parent._model[this.code]))
      this._parent._model[this.code] = []
    if (!Array.isArray( this._parent._populated[this.code]))
      this._parent._model[this.code] = []
    if (this.hasFlag('virtual'))
    {
      this._parent._model[this.code][index] = v
      this._parent._populated[this.code][index] = v
    } else {
      this._parent._model[this.code][index] = this._v2db(v,index)
    }
    //this._parent._model.markModified(this.code)
    return
  }

  push(v,flags=null)
  {
    if (!flags)
      return this.set(v,this.size())
    if (flags.unique)
    {
      let dbv = this._v2db(v)
      if (!this._parent._model[this.code].reduce((acc,vv)=>{
        if(dbv===null||vv===null)
          return dbv===vv?true:acc
        if(typeof(dbv.toString)=='function'&&typeof(vv.toString)=='function')
          return dbv.toString()==vv.toString()?true:acc
        return dbv==vv?true:acc
      },false))
        this.set(v,this.size())
    }
  }

  genDefault(ctx=null)
  {
    return Rx.Observable.create(obs=>{
      log.log('generating default value',this.code)
      let $t = []
      let amount = 0
      if (this.conf&&parseInt(this.conf.genquantity))
        amount = parseInt(this.conf.genquantity)
      if (this.quantity<amount&&this.quantity>1)
        amount = this.quantity
      if (this.quantity==1)
        amount = 1

      //console.log(['generate values:',amount])
      for (let i=0;i<amount;i++)
      {
        let $n = Rx.of(undefined)
        if (this.generator)
        {
          let genConf = null
          if (this.conf&&this.conf.generator)
            genConf = this.conf.generator
          let gen = this.M('field').genGenerator(this,this.generator,genConf)
          //console.log(['generate gen:',this.generator,[gen]])
          $n = gen.generate(ctx)
        } else {
          $n = Rx.of(this.def)
        }
        $t.push($n)
      }
      let gi = 0
      Rx.concat(...$t).subscribe(v=>{
        //console.log(['generated value',v])
        if (typeof v != 'undefined')
          this.set(v,gi)
        gi++
      },err=>{
        log.err('field generating default failed',this.code)
        console.log(err)
        this.obsErr(obs,err)
      },()=>{
         this.obsEnd(obs,this) 
      })
    })
  }

  clear()
  {
    this._parent._model[this.code] = []
    this._parent._populated[this.code] = []
    //this._parent._model.markModified(this.code)
  }

  clearAt(i=0)
  {
    if(Array.isArray(this._parent._model[this.code]))
      this._parent._model[this.code] = this._parent._model[this.code].filter((x,ii)=>i!=ii)
    if(Array.isArray(this._parent._model[this.code]))
      this._parent._populated[this.code] = this._parent._populated[this.code].filter((x,ii)=>i!=ii)
    //this._parent._model.markModified(this.code)
  }

  save()
  {
    if(!(this._parent instanceof Node))
      return Rx.of(this._parent)
    if (this.hasFlag('virtual'))
      return Rx.of(this._parent)
    return this._parent.save()
  }

  confOp(opi,def)
  {
    let ret = def
    if(this.conf&&this.conf.ops&&this.conf.ops[opi])
    {
      let op = this.conf.ops[opi]
      if (op.source=='manual')
        ret = op.manual
      else
        ret = this._parent.f(op.source)
    }
    return ret
  }

  /*
  setMap(mp,index=0)
  {
    if (!this.mapped)
      return
    if (index<0) index = 0
    if (index>0&&this.quantity&&index>=this.quantity)
      index = this.quantity-1
    let obj = mp
    if (mp instanceof Map)
    {
      obj = {}
      for(let pair of mp)
        obj[pair[0]] = pair[1]
    }
    this._parent._model[this.code][index] = new Map()
    for(let key in obj)
    {
      let ctxVal = obj[key]
      this._parent._model[this.code][index].set(key,this._v2db(ctxVal))
    }

  }

  getMap(index=0)
  {
    if (!this.mapped)
      return null
    if (index<0) index = 0
    if (index>0&&this.quantity&&index>=this.quantity)
      index = this.quantity-1
    let mp = this._parent._model[this.code][index]
    let ret = new Map()
    for(let pair of mp)
      ret.set(pair[0],this._db2v(pair[1]))
    return ret
  }

  setAll(list)
  {
    this._parent[this.code] = list.map(e=>this._v2db(e))
  }

  getAll()
  {
    return this._parent[this.code].map(e=>this._db2v(e))
  }
  /**/

}

module.exports = AbstractField
