/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('AbstractModifier')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')

class AbstractModifier extends CoreObject
{

  constructor(parent,opsConf)
  {
    super(parent)
    this._dataTarget = parent
    this._conf = opsConf
    this._pos = 0
    this._ops = []
  }

  //SETTERS FUNCTIONS
  //'absctract modifier setRange always return [0]'
  setRange()
  {
    return [0]
  }

  //GETTERS FUNCTIONS
  //should be reimplemented
  getNext()
  {
    log.err('absctract modifier always return UNDEFINED data')
    return
  }
  //which requires for setter or getters
  modifyOne(val)
  {
    //by default - no modifications, direct return
    return val
  }

  modifyTarget(tgt)
  {
    return tgt
  }

  //should be reimplemented if required accumulation
  //default behavior - no accumulation
  //stop data stream
  //result of executor
  reset() {}
  accumulate(exRes)
  {
    return {
      stop: true,
      res: exRes
    }
  }

  static info()
  {
    return {
      code: 'abstract',
      name: 'Abstract modifier',
      iname: 'Not invertable',
      invert: false, //not invertable
      flat: false, //modifier return single element or array
      strict: '', //modifier keeps original type
      ops: [], //0 configurations
      types: ['[]'], //list of types, modifier can be applyed
    }
  }

  pos(nx)
  {
    if(typeof nx == 'number')
      this._pos = nx
    return this._pos
  }

  init(ctx)
  {
    return Rx.Observable.create(obs=>{
      this._ops = []
      let ok = true
      let inf = this.constructor.info()
      //log.info('initializing',inf.code)
      inf.ops.forEach((opInfo,opi)=>{
        let opCfg = null
        if (typeof this._conf[opi] != 'undefined')
          opCfg = this._conf[opi]
        if (!opCfg)
        {
          if (opInfo.required)
            ok = false
          return
        }
        let op = ctx.fieldSelector(opCfg,opInfo)
        if (!op)
        {
          if (opInfo.required)
            ok = false
          return
        }
        this._ops[opi] = op
      })
      if (!ok)
        return this.obsErr(obs,{code:'NO_OPS_ERROR'})
      let $t = this._ops.map(x=>x.init(ctx))
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('ops init error',err)
        this.obsErr(obs,err)
      },()=>{
        this.obsEnd(obs)
      })
    })
  }

  dataType()
  {
    let src = this._dataTarget.dataType(true)
    let inf = this.constructor.info()
    let dt = src
    if (src.startsWith('[')&&src.endsWith(']'))
      dt = src.slice(1,-1)
    let ret = dt
    if (inf.strict&&inf.strict.length)
      ret = inf.strict
    if (!inf.flat)
      ret = '['+ret+']'
    return ret
  }

  op(i=0,def)
  {
    if (typeof this._ops[i] == 'undefined')
      return def
    let op = this._ops[i]
    if (!op||typeof op.oneModified != 'function')
      return def
    let rr = op.oneModified()
    //console.log(['modifier.js MOD OP',this.constructor.info().code,rr])
    if (typeof rr != 'undefined')
      return rr
    return def
  }


}

module.exports = AbstractModifier
