/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('RelationField')

const CoreModule = require('../../core/abstract/module')
const RelationField = require('./relation')

const NumberFieldModifier = require('./modifiers/numfield')
const StringFieldModifier = require('./modifiers/strfield')
const BooleanFieldModifier = require('./modifiers/boolfield')
const DatetimeFieldModifier = require('./modifiers/dtfield')
const RelationFieldModifier = require('./modifiers/relfield')

const EqSetter = require('./setters/eq')

const EqTest = require('./tests/eq')

class RelationFieldModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this.M('field').registerFieldType(RelationField)
    this.M('field').registerSetter(EqSetter)
    this.M('field').registerModifier(NumberFieldModifier)
    this.M('field').registerModifier(StringFieldModifier)
    this.M('field').registerModifier(BooleanFieldModifier)
    this.M('field').registerModifier(DatetimeFieldModifier)
    this.M('field').registerModifier(RelationFieldModifier)

    this.M('logic').registerLogicTest(EqTest)
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtRelationData = {
        url: '/api/fields/relation/:bcode/:fcode',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let ret = []
            let p = ctx.f('params')
            let q = ctx.f('query')

            let bcode = p.bcode
            let fcode = p.fcode
            let fsrch = q.srch||null

            let bundle = this.M('bundle').bundle(bcode)
            if (!bundle)
              return this.obsEnd(obs,[])
            let field = bundle.fields().reduce((acc,x)=>x.code==fcode?x:acc,null)
            if (!field)
              return this.obsEnd(obs,[])
            if (field.type!='relation')
              return this.obsEnd(obs,[])
            let targetBundle = this.fcop(field,0,null)
            if (!targetBundle)
              return this.obsEnd(obs,[])
            let tb = this.M('bundle').bundle(targetBundle)
            if (!tb)
              return this.obsEnd(obs,[])
            let $searches = []
            let allTFs = []
            let targetFields = []
            tb.fields().forEach(f=>{
              if (f.flags.includes('title'))
              {
                if (!allTFs.includes(f.code))
                  allTFs.push(f.code)
                targetFields.push(f.code)
              }
            })
            //console.log(tb,targetFields)
            if (!fsrch)
            {
              $searches.push(tb.collection().findNodes({}))
            } else {
              let rx = new RegExp('^.*'+fsrch+'.*$','i')
              let $or = []
              targetFields.forEach(tfc=>{
                let tf = tb.getField(tfc)
                if (!tf) return
                //console.log(['tf',tf])
                if (tf.type != 'string') return
                let oor = {}
                oor[tf.code] = {$regex: rx}
                $or.push(oor)
              })
              //console.log($or)
              let c = tb.collection()
              if ($or.length)
                $searches.push(c.findNodes({$or:$or}))
              else
                $searches.push(c.findNodes())
            }
            let retlist = []
            Rx.concat(...$searches).subscribe(clist=>{
              clist.forEach(x=>retlist.push(x.toObject()))
            },err=>{},()=>{
              //console.log(retlist)
              this.obsEnd(obs,{nodes:retlist,titles:allTFs})
            })
          })
        }
      }


      let rtRelationData2 = {
        url: '/api/fields/relation2/:bcode',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let ret = []
            let p = ctx.f('params')
            let q = ctx.f('query')

            let bcode = p.bcode
            let fcode = p.fcode
            let fsrch = q.srch||null

            let tb = this.M('bundle').bundle(bcode)
            if (!tb)
              return this.obsEnd(obs,[])
            let $searches = []
            let allTFs = []
            let targetFields = []
            tb.fields().forEach(f=>{
              if (f.flags.includes('title'))
              {
                if (!allTFs.includes(f.code))
                  allTFs.push(f.code)
                targetFields.push(f.code)
              }
            })
            //console.log(tb,targetFields)
            if (!fsrch)
            {
              $searches.push(tb.collection().findNodes({}))
            } else {
              let rx = new RegExp('^.*'+fsrch+'.*$','i')
              let $or = []
              targetFields.forEach(tfc=>{
                let tf = tb.getField(tfc)
                if (!tf) return
                //console.log(['tf',tf])
                if (tf.type != 'string') return
                let oor = {}
                oor[tf.code] = {$regex: rx}
                $or.push(oor)
              })
              //console.log($or)
              let c = tb.collection()
              if ($or.length)
                $searches.push(c.findNodes({$or:$or}))
              else
                $searches.push(c.findNodes())
            }
            let retlist = []
            Rx.concat(...$searches).subscribe(clist=>{
              clist.forEach(x=>retlist.push(x.toObject()))
            },err=>{},()=>{
              //console.log(retlist)
              this.obsEnd(obs,{nodes:retlist,titles:allTFs})
            })
          })
        }
      }

      this.obsEnd(obs,[
        rtRelationData,
        rtRelationData2
      ])
    })

  }

  fcop(f,opi,def)
  {
    let ret = def
    if (f.conf&&f.conf.ops&&f.conf.ops[opi])
    {
      let op = f.conf.ops[opi]
      if (op.source=='manual')
        ret = op.manual
      if (op.source!='manual'&&op.source.length)
        ret = op.source
    }
    return ret
  }

}

module.exports = RelationFieldModule
