/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('StringField')

const CoreModule = require('../../core/abstract/module')
const StringField = require('./string')

const UUIDGenerator = require('./generators/uuid')
const HashGenerator = require('./generators/hash')

const RegexpTest = require('./tests/regexp')
const ContainsTest = require('./tests/contains')
const EqTest = require('./tests/eq')
const InListTest = require('./tests/inlist')
const ListIncludeTest = require('./tests/listinclude')

const EqSetter = require('./setters/eq')
const ListFromStrSetter = require('./setters/listfromstr')
const Push2ArrSetter = require('./setters/push2arr')
const RemoveFromArraySetter = require('./setters/remfromarr')

const AnyAsString = require('./modifiers/anyasstring')
const StringLength = require('./modifiers/length')

class StringFieldModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._core.module('field').registerFieldType(StringField)

    this._core.module('field').registerGenerator(UUIDGenerator)
    this._core.module('field').registerGenerator(HashGenerator)

    this._core.module('field').registerSetter(EqSetter)
    this._core.module('field').registerSetter(ListFromStrSetter)
    this._core.module('field').registerSetter(Push2ArrSetter)
    this._core.module('field').registerSetter(RemoveFromArraySetter)

    this._core.module('field').registerModifier(AnyAsString)
    this._core.module('field').registerModifier(StringLength)

    this._core.module('logic').registerLogicTest(RegexpTest)
    this._core.module('logic').registerLogicTest(EqTest)
    this._core.module('logic').registerLogicTest(ContainsTest)
    this._core.module('logic').registerLogicTest(InListTest)
    this._core.module('logic').registerLogicTest(ListIncludeTest)
  }

}

module.exports = StringFieldModule
