/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../../log').m('!StringField')
let g18n = require('../../../g18n')

const AbstractField = require('../field')

class StringField extends AbstractField
{

  static schema(info,sch={}) {
    sch.type = String
    return sch
  }

  static info()
  {
    return {
      code: 'string',
      name: g18n.t('Text/string'),
      conf: {
        ops: [{
          name: g18n.t('Big text'),
          type: 'boolean',
          manual: true
        },{
          name: g18n.t('Format'),
          type: '[string]',
          manual: false,
          desc: g18n.t('Text format'),
          list: [
            {label:'Liquid',value:'liquid'},
            {label:'Telegram html',value:'tg'},
            {label:'Full html',value:'html'},
            {label:'Markup',value:'markup'}
          ]
        }]
      }
    }
  }

  xtypes()
  {
    let isBigConf = this.confOp(0,false)
    let type = this.constructor.info().code
    let ret = []
    if (isBigConf)
      ret.push(type+':big')
    else
      ret.push(type+':small')
    return ret
  }

  _v2db(v) {
    if(typeof v == 'undefined'||v===null) return null
    if(typeof v != 'string') return v.toString()
    return v
  }
  _db2v(db) {
    if(typeof(db)=='undefinded'||db===null) return null
    return db
  }

  _serializev(val)
  {
    return val+''
  }

  _unserializev(val)
  {
    return val+''
  }


}

module.exports = StringField
