/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let QR = require('qr-image')
let moment = require('moment-timezone')
let FI = require('fileinfo')
let Threads = require('worker_threads')

const dirTree = require('directory-tree')
const path = require('path')
const fs = require('fs')
const util = require('util')
const crypto = require('crypto')

const exists = util.promisify(fs.exists)
const readdir = util.promisify(fs.readdir)
const lstat = util.promisify(fs.lstat)
const unlink = util.promisify(fs.unlink)
const rmdir = util.promisify(fs.rmdir)

let log = require('../../log').m('FileModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')

//const LogicGroup = require('./group')
//const AbstractTest = require('./abstract/test')

//const Cmd = require('../telegram/cmd')

class FileModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._allTests = []
    this._tests = {
      '*': {}
    }
  }

  _parseFile(tree)
  {
    return Rx.Observable.create(obs=>{
      log.log('checking file',tree.path)
      if (!tree.extension||tree.extension=='')
        return this.obsEnd(obs)
      this.C('file').findNode({localPath:tree.path}).subscribe(nd=>{
        if (nd)
        {
          log.log('file exists',nd.f('localPath'))
          return this.obsEnd(obs)
        }
        log.warn('file not exists',tree.path)
        nd = this.C('file').newNode()
        nd.f('localPath',tree.path)
        nd.f('created',moment())
        Rx.from(lstat(tree.path)).subscribe(finfo=>{
          Rx.from(FI(tree.path)).subscribe(fi=>{
            if (!finfo||!fi)
              return this.obsEnd(obs)
            nd.f('size',finfo.size)
            nd.f('info',fi)
            nd.save().subscribe(nds=>{
              log.success('file info updated',nds.f('localPath'))
              this.obsEnd(obs)
            },err=>{
              log.err('failed save file information')
              this.obsEnd(obs)
            })
          },err=>{
            log.err('failed get local file information',tree.path)
            this.obsEnd(obs)
          })
        },err=>{
          log.err('failed get local file info',tree.path)
          this.obsEnd(obs)
        })
      })
    })
  }

  _parseDir(tree)
  {
    return Rx.Observable.create(obs=>{
      log.log('parsing directory',tree.path)
      if (!tree.children||!tree.children.length)
        return this.obsEnd(obs)
      let $t = []
      tree.children.forEach(cld=>{
        if(cld.type=='file')
          $t.push(this._parseFile(cld))
        if(cld.type=='directory')
          $t.push(this._parseDir(cld))
      })
      Rx.concat(...$t).subscribe(()=>{},err=>{},()=>{
        log.log('done directory',tree.path)
        return this.obsEnd(obs)
      })
    })
  }

  fillLocal()
  {
    //return Rx.of(1)
    /**/
    return Rx.Observable.create(obs=>{
      let dir = dirTree(this._config.dir)
      log.test('check local directory for files')
      if (!dir||dir.path!=this._config.dir||!dir.children||!dir.children.length)
        return this.obsEnd(obs)
      this._parseDir(dir).subscribe(()=>{
        log.success('local storage syncronized')
        this.obsEnd(obs)
      })
    })
    /**/
  }

  moduleInit()
  {
    //return Rx.of(1)
    if (!Threads.isMainThread)
    {
      log.err('files sync on start only in main thread')
      return Rx.of(1)
    }
    return this.fillLocal()
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlFiles = this._core.module('bundle').bundle('file',true)
      bndlFiles.name(g18n.t('File'))
      bndlFiles.desc(g18n.t('Managemented file'))
      bndlFiles.flags(['system'])

      bndlFiles.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','logic','title']})
      bndlFiles.addField({name:g18n.t('Created at'),code:'creationDate',type:'datetime',flags:['system','logic']})
      bndlFiles.addField({name:g18n.t('Size'),code:'size',type:'number',flags:['system','required','logic']})
      bndlFiles.addField({name:g18n.t('Info'),code:'info',type:'any',flags:['system','required','logic']})
      bndlFiles.addField({name:g18n.t('Local path'),code:'localPath',type:'string',flags:['system','logic','title']})

      log.info('saving bundle',bndlFiles.code())
      $t.push(bndlFiles.save(true,true))
      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        this.obsErr(obs,err)
      },()=>{
        log.success('installed')
        this.obsEnd(obs,{
          dir: "./files/",
          qr: "./qrfiles/",
          maxSize: 10485760
        })
      })
    })
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      $t.push(this.M('auth/role').roleSave('file','root',g18n.t('Files root')))
      $t.push(this.M('auth/role').roleSave('file','tree',g18n.t('View files/directories tree')))
      $t.push(this.M('auth/role').roleSave('file','upload',g18n.t('Uploading files/creating directories')))
      $t.push(this.M('auth/role').roleSave('file','delete',g18n.t('Deleting files/directories')))

      $t.push(this.fillLocal())

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtFileUpload = {
        url: '/api/file/upload',
        access: ['auth.dashboard','file.tree','file.upload'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let path = ctx.f('body').uploadPath || '/'
            let files = ctx.f('files')
            if (!path||!files||!files.uploadFile)
              return this.obsEnd(obs,{error:'file not found - not uploaded?'})

            let dest = this._config.dir+path.slice(1)+files.uploadFile.name

            files.uploadFile.mv(dest,err=>{
              if (err)
                return this.obsEnd(obs,{code:'FILE_NOT_MOVED',error:err})
              this.fillLocal().subscribe(()=>{
                this.obsEnd(obs,{success:true})
              },errr=>{
                log.err('localfill failed!',errr)
                this.obsEnd(obs,{success:true})
              })
            })
          })
        }
      }

      let rtMkDir = {
        url: '/api/file/mkdir',
        access: ['auth.dashboard','file.tree','file.upload'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let root = ctx.f('body').root
            let name = ctx.f('body').name
            let rx = /^[a-zA-Z0-9_]{1,64}$/
            if (!root||!root.path||!root.path.length||root.type!='directory'||root.path.includes('..')||!name||!name.length||!rx.test(name))
              return this.obsEnd(obs,{error:'Some error happened. Refresh page better.'})

            fs.mkdirSync(root.path+'/'+name)
            this.obsEnd(obs,{success:true})
          })
        }
      }


      let rtDel = {
        url: '/api/file/del',
        access: ['auth.dashboard','file.tree','file.delete'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let data = ctx.f('body')
            if (!Array.isArray(data))
            //console.log(['data',data])
              return this.obsEnd(obs,{error:'Some error happened. Refresh page better.'})
            let $t = []
            let $td = []
            data.forEach(x=>{
              if (x.path.includes('..')||x.extension=='{QR}'||x.extension=='{CTX}')
                return
              if (x.type == 'file'&&fs.existsSync(x.path))
                $t.push(this.C('file').removeNodes({localPath:x.path}).pipe(
                  RxO.map(xx=>{
                    fs.unlinkSync(x.path)
                  })
                ))
              if (x.type == 'directory'&&fs.existsSync(x.path))
              {
                $td.push(Rx.of(x.path).pipe(RxO.map(p=>fs.rmdirSync(p))))
                this._traverseTree(x,cld=>{
                  if (cld.path.includes('..')||cld.extension=='{QR}'||cld.extension=='{CTX}')
                    return
                  $t.push(this.C('file').removeNodes({localPath:cld.path}).pipe(
                    RxO.map(xx=>{
                      fs.unlinkSync(cld.path)
                    })
                  ))
                },cldd=>{
                  $td.push(Rx.of(cldd.path).pipe(RxO.map(p=>fs.rmdirSync(p))))
                })
              }
            })
            if ($td.length)
              $t = [...$t,...$td.reverse()]
            Rx.concat(...$t).pipe(
              RxO.last(),
              RxO.mergeMap(_=>this.fillLocal())
            ).subscribe(()=>{
              this.obsEnd(obs,{success:true})
            },err=>{
              this.obsEnd(obs,{error:err})
            })
          })
        }
      }

      let rtFilesList = {
        url: '/api/file/tree',
        access: ['auth.dashboard','file.tree'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let dir = dirTree(this._config.dir)
            this.obsEnd(obs,dir)
          })
        }
      }

      this.obsEnd(obs,[
        rtFileUpload,
        rtMkDir,
        rtDel,
        rtFilesList
      ])
    })
  }

  _traverseTree(dir,fn,fnd)
  {
    dir.children.forEach(cld=>{
      if (!cld)
        return
      if (cld.type=='file'&&typeof fn == 'function')
        fn(cld)
      if (cld.type=='directory')
      {
        if (typeof fnd=='function')
          fnd(cld)
        this._traverseTree(cld,fn)
      }
    })
  }

  /*
  */

  _update01to02()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>Rx.concat(
        this.M('auth/role').roleSave('file','root',g18n.t('Files root')),
        this.M('auth/role').roleSave('file','tree',g18n.t('View files/directories tree')),
        this.M('auth/role').roleSave('file','upload',g18n.t('Uploading files/creating directories')),
        this.M('auth/role').roleSave('file','delete',g18n.t('Deleting files/directories'))
      ).pipe(RxO.last()))
    )
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1')
          return this._update01to02()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }



}

module.exports = FileModule
