/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('AbstractForm')
let g18n = require('../../g18n')

const Node = require('../node/node')
const VNode = require('../node/vnode')

class AbstractForm extends Node
{

  constructor(...args)
  {
    super(...args)
  }

  //common routines
  static info()
  {
    return {
      code: 'abstract',
      name: g18n.t('Abstract'),
      desc: g18n.t('Abstract form'),
      confs: []
    }
  }

  build()
  {
    return Rx.of(this.f('view')).pipe(
      RxO.map(view=>{
        //console.log(view)
        return view.map(x=>{
          return {
            code: x.field,
            type: x.widget
          }
        })
      })
    )
  }

  executeActions(data,ctx)
  {
    return Rx.of(this.f('fields')).pipe(
      RxO.mergeMap(flds=>{
        let fnode = new VNode(ctx)
        flds.forEach(fi=>{
          fnode.addField(fi)
          fnode.f(fi.code,data[fi.code])
        })
        ctx.addNode(fnode)
        return this.M('actions').executeActions(this.f('actions'),ctx)
      }),
      RxO.mergeMap(_=>this.execute(data,ctx))
    )
    //return this.execute(data,ctx)
  }

  execute(data,ctx)
  {
    return Rx.of(null)
  }

}

module.exports = AbstractForm
