/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('FormsModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')

const AbstractForm = require('./abstract')
const AbstractWidget = require('./widget')

const OTLForm = require('./otlcodelogin')
const StringWidget = require('./widgets/string')

class FormsModule extends CoreModule
{

  constructor(...args)
  {
    super(...args)

    this._forms = {}
    this._widgets = {}

    this.registerFormType(AbstractForm)
    this.registerFormType(OTLForm)
    this.registerWidgetType(AbstractWidget)
    this.registerWidgetType(StringWidget)
  }

  formClass(mdl)
  {
    if (typeof this._forms[mdl.type] != 'undefined')
      return this._forms[mdl.type]
    return AbstractForm
  }

  formsTypes()
  {
    return Object.keys(this._forms).map(x=>this._forms[x].info())
  }

  registerFormType(Form)
  {
    let inf = Form.info()
    this._forms[inf.code] = Form
  }

  widgetsTypes()
  {
    return Object.keys(this._widgets).map(x=>this._widgets[x].info())
  }

  registerWidgetType(Wgt)
  {
    let inf = Wgt.info()
    this._widgets[inf.code] = Wgt
  }

  fields2form(fields)
  {
    let ret = []
    if (!Array.isArray(fields))
      return ret
    for(let i=0;i<fields.length;i++)
    {
      let f = {
        code: fields[i].code,
        name: fields[i].name,
        desc: fields[i].desc,
        type: fields[i].type,
        xtype: undefined,
        quantity: 1,
        val: undefined,
        def: undefined,
        options: undefined,
        validadors: undefined
      }
      if (typeof fields[i].xtype == 'string')
        f.xtype = fields[i].xtype
      if (typeof fields[i].quantity == 'number')
        f.quantity = fields[i].quantity
      if (typeof fields[i].val != 'undefined')
        f.val = fields[i].val
      if (typeof fields[i].def != 'undefined')
        f.def = fields[i].def
      if (typeof fields[i].conf != 'undefined')
        f.conf = fields[i].conf
      if (Array.isArray(fields[i].options))
        f.options = fields[i].options
      if (Array.isArray(fields[i].validators))
        f.validators = fields[i].validators
      ret.push(f)
    }
    return ret
  }

}

module.exports = FormsModule
