/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('OTLForm')
let g18n = require('../../g18n')

const AbstractForm = require('./abstract')

class OTLForm extends AbstractForm
{

  //common routines
  static info()
  {
    return {
      code: 'otlcodelogin',
      name: g18n.t('OTL Form'),
      desc: g18n.t('One time code login form'),
      confs: []
    }
  }

  execute(data,ctx)
  {
    return Rx.of(null).pipe(
      RxO.mergeMap(_=>{
        let ifcode = ctx.vnode('interface').f('code')
        let scode = data.otlcode
        log.test('try open session',[ifcode,scode])
        return this.M('auth/access').openIfaceSession(ifcode,scode).pipe(
          RxO.catchError(err=>{
            log.err('open session failed',err)
            return Rx.of(null)
          })
        )
      }),
      RxO.map(sres=>{
        let ifcode = ctx.vnode('interface').f('code')
        if (!sres)
        {
          log.err('session not opened')
          return null
        }
        log.success('session opened',ifcode)
        ctx.sfield('token',sres.sess.f('token'))
        return { redirect: ctx.vnode('http').f('url') }
      })
    )
  }

}

module.exports = OTLForm
