/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let fs = require('fs')
//languages
const gt = require('google-translate-api')
//real translator
const gt2 = require('translate-google')

let log = require('../../log').m('G18N')

class G18N
{
  constructor()
  {
    this.conf = {
      locales: ['en','ru'],
      languages: {},
      locale: 'en',
      path: './locales2'
    }
    this.locale = ''
    //this.save = false
    this.translations = {}
    //hard integrate to google-translate-api
    for(let cd in gt.languages)
    {
      if(cd=='auto'||cd=='isSupported'||cd=='getCode') continue
      if(typeof gt.languages[cd] != 'string') continue
      this.conf.locales.push(cd)
      this.conf.languages[cd] = {
        locale: cd,
        name: gt.languages[cd],
        flag: '',
        localName: '',
        enabled: false
      }
      this.translations[cd] = {}
    }
    //console.log(this.conf)
  }

  configure(conf)
  {
    if (conf.locale && typeof this.conf.languages[conf.locale] != 'undefined')
    {
      this.locale = conf.locale
      this.conf.locale = conf.locale
      this.conf.languages[conf.locale].enabled = true
    }
    if (Array.isArray(conf.languages))
    {
      conf.languages.forEach(l=>{
        if (typeof this.conf.languages[l] == 'undefined') return
        this.conf.languages[l].enabled = true
      })
    }
    if (typeof conf.path == 'string' && conf.path.length) this.conf.path = conf.path
    //let's check directory existance
    //this.save = false
    if (!fs.existsSync(this.conf.path))
    {
      log.warn('directory not exists, creating',{path:this.conf.path})
      fs.mkdirSync(this.conf.path)
      if (!fs.existsSync(this.conf.path))
      {
        log.err('Can\'t create directory! Saving disabled!',{path:this.conf.path})
      } else {
        //this.save = true
      }
    } else {
      //this.save = true
    }
    this.conf.locales.forEach(l=>{
      if (!this.conf.languages[l].enabled) return
      let lfile = this._lfile(l)
      if (fs.existsSync(lfile))
        this.translations[l] = JSON.parse(fs.readFileSync(lfile))
    })
    //console.log(this.conf)
  }

  webpack()
  {
    //console.log(this)
    let ret = []
    for(let lc in this.conf.languages)
    {
      let lan = this.conf.languages[lc]
      if (!lan.enabled) continue
      let pack = []
      for(let src in this.translations[lc])
      {
        let s = this.translations[lc][src]
        let t = {
          locale: lc,
          source: src,
          translation: s.original
        }
        if (s.google) t.translation = s.google
        if (s.manual) t.translation = s.manual
        pack.push(t)
      }
      if (pack.length)
        ret.push({
          locale: lc,
          texts: pack
        })
    }
    return ret
  }

  t(text='',arg1=null,arg2=null)
  {
    //log.test('t',text)
    let to = this.conf.locale
    let data = {}
    if (arg1)
    {
      if (typeof arg1 == 'string')
        to = arg1
      if (typeof arg1 == 'object')
        data = arg1
    }
    if (arg2)
    {
      if (typeof arg2 == 'object')
        data = arg2
    }
    if(text==''||to==''||!this.translations[to])
      return this._build(text,data)
    let ex = false
    if (this.translations[to][text]&&this.translations[to][text].original)
      ex = true
    let t = {
      original: text,
      manual: null,
      google: null
    }
    if (ex)
      t = this.translations[to][text]
    else
      setTimeout(()=>{
        //let's translate
        //log.test('t',text)
        this._add(text)
      },0)
    //1st check manual translation
    if (t.manual)
      return this._build(t.manual,data)
    //now google
    if (t.google)
      return this._build(t.google,data)
    //so, original
    return this._build(t.original,data)
  }

  m(text='',arg1=null,arg2=null)
  {
    let to = this.conf.locale
    let data = {}
    if (arg1)
    {
      if (typeof arg1 == 'string')
        to = arg1
      if (typeof arg1 == 'object')
        data = arg1
    }
    if (arg2)
    {
      if (typeof arg2 == 'object')
        data = arg2
    }
    let def = this.t(text,arg1,arg2)
    let ret = new Map()
    ret['locale=='+to] = text
    ret['locale==*'] = def
    return ret
  }

  _lfile(locale)
  {
    return this.conf.path+'/'+locale+'.json'
  }

  _build(text='',data=null)
  {
    if (text==''||!data) return text

    let tpls = []
    let rx = /(#\[([^\]]+)\])/
    let m
    while((m=rx.exec(text))!==null)
    {
      if(m.index===rx.lastIndex) rx.lastIndex++
      let i = tpls.length
      tpls.push({
        i:i,
        src: m[0],
        tpl: m[2],
        rpl: '_g18n_'+i,
        res: ''
      })
      while(text.indexOf(m[0])!=-1)
        text = text.replace(m[0],'_g18n_'+i)
    }
    tpls.forEach(t=>{
      //simple?
      if(t.tpl.indexOf(':')==-1)
      {
        if(data[t.tpl]) t.res = data[t.tpl].toString()
      } else {
        let infos = t.tpl.split(':')
        let vr = infos[0]
        if (typeof data[vr] != 'undefined')
          t.res = data[vr].toString()
        let fn = infos[1]
        //if (fn!='p'&&fn!='s')
          //return
        infos.shift()
        infos.shift()
        if (fn == 'p'&&vr!='') t.res = this._buildPlural(vr,infos,data)
        if (fn == 's'&&vr!='') t.res = this._buildSelect(vr,infos,data)
      }
      while(text.indexOf(t.rpl)!=-1)
        text = text.replace( t.rpl,t.res)
    })

    return text
  }

  _buildSelect(code,plrs,data)
  {
    if(typeof data[code] == 'undefined') return ''
    let val = data[code]
    if (typeof val != 'string'||!val.length) return ''
    let tpl = null
    plrs.forEach(p=>{
      let pl = p.split('>',2)
      //console.log(pl)
      if (pl.length==2)
      {
        let cond = pl[0]
        if(cond==='='+val)
          tpl = pl[1]
        if (cond=='other'&&tpl==null)
          tpl = pl[1]
      }
    })
    if (tpl===null) return val.toString()
    for(let d in data)
    {
      while(tpl.indexOf('#'+d)!=-1)
        tpl = tpl.replace('#'+d,data[d].toString())
    }
    return tpl
  }

  _buildPlural(code,plrs,data)
  {
    if(typeof data[code] == 'undefined') return ''
    let val = parseInt(data[code])
    if (Number.isNaN(val)) return ''
    let ret = ''
    let tpl = null
    plrs.forEach(p=>{
      let pl = p.split('>',2)
      //console.log(pl)
      if (pl.length==2)
      {
        let cond = pl[0]
        if (cond=='zero')cond='=0'
        if (cond=='one')cond='=1'
        if (cond=='two')cond='=2'
        if (cond=='few')cond='=5'
        if (cond=='many')cond='=10'
        if (cond.startsWith('='))
        {
          let v = parseInt(cond.slice(1))
          if (v===val) tpl = pl[1]
        }
        if (cond=='other'&&tpl==null)
          tpl = pl[1]
      }
    })
    if (tpl===null) return val.toString()
    for(let d in data)
    {
      while(tpl.indexOf('#'+d)!=-1)
        tpl = tpl.replace('#'+d,data[d].toString())
    }
    return tpl
  }

  _add(text='')
  {
    log.log('_add text',text)
    let llist = []
    this.conf.locales.forEach(l=>{
      if (!llist.includes(l))
        llist.push(l)
    })
    llist.forEach(l=>{
      if (!this.conf.languages[l].enabled) return
      if (!this.translations[l]) this.tranlations[l] = {}
      let t = {
        original: text,
        manual: null,
        google: null
      }
      let lfile = this._lfile(l)
      //log.warn('_add locales loop',l)
      if (l==this.conf.locale)
      {
        t.manual = text
        this.translations[l][text] = t
        fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
      } else {
        //google translate
        log.warn('Require translation',{
          text: text, from: this.conf.locale, to: l
        })
        //without translate-google
        gt2(text,{from:this.conf.locale,to:l}).then(tt=>{
          //console.log('GT',tt)
          if (!tt||!tt.length) return
          t.google = tt
          this.translations[l][text] = t
          fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
        }).catch(e=>{
          log.err('Google translate error!',{error:e})
          this.translations[l][text] = t
          fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
        })
        /*
        //without google translator just save
        this.translations[l][text] = t
        fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
        /**/
        /*
        //for google-translate-api, but not working
        gt(text,{from:this.conf.locale,to:l}).then(tt=>{
          if (!tt||!tt.text||!tt.text.length) return
          t.google = tt.text
          this.translations[l][text] = t
          fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
        }).catch(e=>{
          log.err('G18N','Google translate error!',{error:e})
          this.translations[l][text] = t
          fs.writeFileSync(lfile,JSON.stringify(this.translations[l],null,2))
        })
        /**/
      }
    })
  }
}

let g18n = new G18N()

module.exports = g18n
