/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('HttpBlockModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const AbstractHttpBlock = require('./abstract')
const ContentHttpBlock = require('./content')
const TextHttpBlock = require('./text')
const LibraryHttpBlock = require('./library')
const VueHttpBlock = require('./vue')
const FormHttpBlock = require('./form')

class HttpBlockModule extends CoreModule
{

  constructor(parent)
  {
    super(parent)
    this._classes = {}

    this.registerBlockType(ContentHttpBlock)
    this.registerBlockType(TextHttpBlock)
    this.registerBlockType(LibraryHttpBlock)
    this.registerBlockType(VueHttpBlock)
    this.registerBlockType(FormHttpBlock)
  }

  registerBlockType(Block)
  {
    if (!AbstractHttpBlock.isPrototypeOf(Block))
    {
      log.err('registing block failed. Not a child of AbstractHttpBlock')
      return
    }
    let info = Block.info()
    log.log('registring block type',info.code)
    this._classes[info.code] = Block
  }

  /*
  hookHttpTemplatesPath()
  {
    return Rx.of({
      type: 'module',
      scope: 'core',
      path: 'http/block'
    })
  }
  */

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtBlocksTypes = {
        url: '/api/http/blocks/types',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let list = Object.keys(this._classes).map(code=>{
              let inf = this._classes[code].info()
              if (inf.code=='content'||inf.code=='abstract')
                return null
              if (Array.isArray(inf.confs))
                inf.confs = this.M('forms').fields2form(inf.confs)
              return inf
            }).filter(x=>!!x)
            this.obsEnd(obs,list)
          })
        }
      }

      this.obsEnd(obs,[
        rtBlocksTypes
      ])
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      //blocks data bundle
      let bndBlocks = this.M('bundle').bundle('http_blocks',true)
      bndBlocks.name(g18n.t('Web Blocks'))
      bndBlocks.desc(g18n.t('Web site themes blocks'))
      bndBlocks.addFlag('system')
      bndBlocks.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system','logic','title','template'],def:true})
      bndBlocks.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','required','unique','template','logic']})
      bndBlocks.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','required','template','logic']})
      bndBlocks.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','required','template','logic']})
      bndBlocks.addField({name:g18n.t('Flags'),code:'flags',type:'string',flags:['system','template','logic'],quantity:0})
      bndBlocks.addField({name:g18n.t('Logic'),code:'logic',type:'any',flags:['system','logic','template'],def:null})
      bndBlocks.addField({name:g18n.t('Title'),code:'title',type:'string',flags:['system','template','logic']})
      bndBlocks.addField({name:g18n.t('Data'),code:'data',type:'any',flags:['system','template','logic']})

      log.log('installing bundle',bndBlocks.code())
      $t.push(bndBlocks.save(true,true))

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        log.success('bundle installed')
        this.obsEnd(obs)
      })
 
    })
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.C('http_blocks').setNodeClassFn(mdl=>{
        let type = mdl.type[0]
        if (this._classes[type])
          return this._classes[type]
        return AbstractHttpBlock
      })
      this.obsEnd(obs)
    })
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      let contentBlock = this.C('http_blocks').newNode()
      contentBlock.f('enabled',true)
      contentBlock.f('code','content')
      contentBlock.f('name','Content')
      contentBlock.f('type','content')
      contentBlock.f('title','<none>')
      contentBlock.field('flags').all(['system'])
      $t.push(contentBlock.save())

      let tjsBlock = this.C('http_blocks').newNode()
      tjsBlock.f('enabled',true)
      tjsBlock.f('code','buildedwith')
      tjsBlock.f('name','Builded with')
      tjsBlock.f('type','text')
      tjsBlock.f('title','<none>')
      tjsBlock.f('data',{
        content: 'Builded with takatanJS'
      })
      $t.push(tjsBlock.save())

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        log.success('default blocks installed')
        this.obsEnd(obs)
      })
    })
  }


  _update02to011()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      //blocks data bundle
      let bndBlocks = this.M('bundle').bundle('http_blocks')
      bndBlocks.updField({name:g18n.t('Logic'),code:'logic',type:'any',flags:['system','logic','template'],def:null})

      log.log('updating bundle',bndBlocks.code())
      $t.push(bndBlocks.save())

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        log.success('bundle updated')
        this.obsEnd(obs)
      })
 
    })
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.2')
          return this._update02to011()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }




}

module.exports = HttpBlockModule
