/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let fs = require('fs')
let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('JsonHttpRenderer')
let g18n = require('../../../g18n')

const pug = require('pug')

const AbstractHttpRenderer = require('./renderer')

class JSONRenderer extends AbstractHttpRenderer
{

  static info()
  {
    return {
      code: 'json',
      name: g18n.t('Json renderer'),
      desc: g18n.t('Http json reply')
    }
  }

  render(ctx)
  {
    return Rx.of(ctx.vnode('page')).pipe(
      //building page data
      RxO.mergeMap(page=>{
        let data = page.variant().data
        //console.log(['data',data])
        if (!data||!data.json||data.json!='page')
          return page.compile(ctx)
        let thm = null
        if (data.theme)
          thm = data.theme
        let theme = this.M('http/theme').theme(thm)
        ctx.addNode(theme,'theme')
        //console.log(theme.toObject())
        return theme.compile(ctx)
      }),
      RxO.map(x=>{
        let nx = x
        //console.log(['jsonRenderer','x',nx])
        if(x.content&&typeof x.content.toObject == 'function')
          nx = x.content.toObject()
        //console.log(['jsonRenderer','x',nx])
        return {
          json: nx
        }
      })
    )
  }


}

module.exports = JSONRenderer
