/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let fs = require('fs')
let DT = require('directory-tree')

let log = require('../../../log').m('ThemeEnginesModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')

const DefaultThemeEngine = require('./default')

const AbstractHttpRenderer = require('./renderer')
const JSONHttpRenderer = require('./json')
const VueHttpRenderer = require('./vue')
const RedirectRenderer = require('./redirect')
const ThemeHttpRenderer = require('./theme')

class ThemeEnginesModule extends CoreModule
{

  constructor(parent)
  {
    super(parent)
    this._engines = {}
    this._renderers = {}
    this._rclasses = {}
    this._eclasses = {}
    this.registerEngine(DefaultThemeEngine)
    this.registerRenderer(JSONHttpRenderer)
    this.registerRenderer(ThemeHttpRenderer)
    this.registerRenderer(VueHttpRenderer)
    this.registerRenderer(RedirectRenderer)
  }

  registerEngine(Engine)
  {
    let info = Engine.info()
    log.log('registring theme engine',info.code)
    this._eclasses[info.code] = Engine
  }

  registerRenderer(Cls)
  {
    if (!AbstractHttpRenderer.isPrototypeOf(Cls))
      return
    let info = Cls.info()
    log.log('registring http renderer',info.code)
    this._rclasses[info.code] = Cls
  }

  initEngine(code)
  {
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      let $t = Object.keys(this._eclasses).map(ecode=>{
        this._engines[ecode] = new this._eclasses[ecode](this)
        return this._engines[ecode].init()
      })
      Object.keys(this._rclasses).map(rcode=>{
        this._renderers[rcode] = new this._rclasses[rcode](this)
        return this._renderers[rcode].init()
      })
      Rx.concat(...$t).subscribe(()=>{},err=>this.obsErr(obs,err),_=>this.obsEnd(obs))
    })
  }

  renderer(code)
  {
    //console.log(['ask renderer',code,this._renderers[code]])
    if (this._renderers[code])
      return this._renderers[code]
    return this._renderers.abstract
  }

  engine(code='default')
  {
    if (this._engines[code])
      return this._engines[code]
    if (this._engines['default'])
      return this._engines['default']
    return null
  }
}

module.exports = ThemeEnginesModule
