/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let fs = require('fs')
let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('ThemeHttpRenderer')
let g18n = require('../../../g18n')

const AbstractHttpRenderer = require('./renderer')

class ThemeRenderer extends AbstractHttpRenderer
{

  static info()
  {
    return {
      code: 'theme',
      name: g18n.t('Theme renderer'),
      desc: g18n.t('Http themed page reply')
    }
  }

  render(ctx)
  {
    return Rx.of(ctx.vnode('page')).pipe(
      //building page data
      RxO.mergeMap(page=>{
        let variant = page.variant()
        if (!variant)
          return Rx.throwError({code:'PAGE_VARIANT_NOT_FOUND'})
        //console.log(['rendering variant',variant])
        let thm = null
        if (variant&&variant.theme&&variant.theme.length)
          thm = variant.theme

        if (!thm)
        {
          let router = ctx.vnode('router')
          if (router&&typeof router.cfgTheme == 'function')
            thm = ctx.vnode('router').cfgTheme()
        }
        let theme = this.M('http/theme').theme(thm)
        //console.log(['rendering with theme',thm,theme.f('code'),variant])
        ctx.addNode(theme,'theme')
        return theme.render(ctx)
        //page.compile(ctx)
      }),
      RxO.map(x=>{
        //console.log(['theme renderer done',x])
        return {
          html: x.html
        }
      })
    )
  }


}

module.exports = ThemeRenderer
