/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let fs = require('fs')
const http = require('http')
const express = require('express')

let expressSession = require('express-session')
let RedisStore = require('connect-redis')(expressSession)
let expressFileUpload = require('express-fileupload')

let log = require('../../log').m('HttpModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')
const VNode = require('../node/vnode')

const HttpInterface = require('./http.interface')

class HttpModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

  }

  interfaceRegister()
  {
    return Rx.Observable.create(obs=>{
      this.M('interface').registerInterfaceType(HttpInterface)
      this.obsEnd(obs)
    })
  }

  /**
  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      let $t = []
      //$t.push(this.moduleInstall())

        http.f('body',req.body)

        ctx.session(req.session)
        ctx.addNode(http,'http')

        log.info(req.method,req.url)
        let localFile = this.M('http/theme').currentTheme().hasLocalFile(req.url)
        if (localFile&&localFile.length)
        {
          let dir = __dirname.split('/').slice(0,-3).join('/')
          if (localFile.endsWith('.vue'))
          {
            log.success('vue file')
            res.header("Content-Type", "application/javascript; charset=utf-8");
            res.status(200);
            res.send(fs.readFileSync(localFile).toString())
            return
          }
          log.success('local file',localFile)
          res.sendFile(localFile,{root:dir})
          return
        }
        this.M('http/router').findRoute(ctx).subscribe(route=>{
          if (!route)
          {
            log.err('no route found!')
            res.status(500).send('Error: NULL_ROUTE!')
            return
          }
          this.M('http/page').findPage(ctx).subscribe(page=>{
            if (!page)
            {
              log.err('no page found!')
              res.status(500).send('Error: NULL_PAGE!')
              return
            }
            log.info('page found',page.f('code'))
            page.execute(ctx).subscribe(data=>{
              if (!data)
              {
                res.status(500).send('Data error: '+JSON.stringify(data))
                return
              }
              let code = data.code || 200
              if (ctx.vnode('route')&&ctx.vnode('route').f('code'))
                code = ctx.vnode('route').f('code')
              if (typeof data.html == 'string')
              {
                if (req.url.endsWith('.js'))
                {
                  log.success('js result!')
                  res.header("Content-Type", "application/javascript; charset=utf-8");
                } else {
                  log.success('html')
                }
                res.status(code).send(data.html)
                return
              }
              if (typeof data.js == 'string')
              {
                log.success('js result!')
                res.header("Content-Type", "application/javascript; charset=utf-8");
                res.status(code).send(data.js)
                return
              }
              if (data.json)
              {
                res.status(code).json(data.json)
                return
              }
              res.status(500).send('Data format unknown: '+JSON.stringify(data))
            },err=>{
              log.err('page executing error!',err)
              res.status(500).send('Error: '+JSON.stringify(err))
            })
          },err=>{
            log.err('page search error!',err)
            res.status(500).send('Error: '+JSON.stringify(err))
          })
        },err=>{
          log.err('router error!',err)
          res.status(500).send('Error: '+JSON.stringify(err))
        })
      })
      //this._app.use(express.static('public'))
      //this._app.use('/files',express.static('files'))
      //this._app.use('/docs',express.static('docs'))
      //this._app.use('/emoji',express.static('public/assets/emoji/imgs'))
      //this._app.use(expressFileUpload({ limits: { fileSize: this._conf.files.maxFileSize } }))
      /////
      this._app.use((req,res,next)=>{
        let ctx = this.M('context').newContext(this,'dashboard_http')
        ctx.session(req.session)
        ctx.addField({
          name: g18n.t('URL'),
          code: 'url',
          type: 'string',
          quantity: 1,
          flags: ['virtual','readonly']
        })
        ctx.addField({
          name: g18n.t('Request method'),
          code: 'method',
          type: 'string',
          quantity: 1,
          flags: ['readonly']
        })
        ctx.f('url',req.url)
        ctx.f('method',req.method)
        let token = ctx.sfield('token')
        this._core.module('auth/access').dashboardTokenUser(token).subscribe(u=>{
          if (u) ctx.users([u])
          if (u)
            log.info('dashbord user',[req.url,u.titles()])
          else
            log.warn('dashboard guest',req.url)
          req._ctx = ctx
          next()
        },err=>{
          log.err('search user failed')
          console.log(err)
          res.status(401).location('/logout').end()
        })
      })
      /////*****
      this._server.on('listening',()=>{
        let ip = this._config.host
        let port = this._config.port
        log.success('Website listen at',ip+':'+port)
      })

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('init error')
        console.log(err)
        this.obsErr(obs,err)
      },()=>{
        log.log('initialized')
        this.obsEnd(obs)    
      })
    })
  }

  moduleStart()
  {
    return Rx.Observable.create(obs=>{
      let host = this._config.host
      let port = this._config.port
      this._server.listen({
        port: port,
        host: host
      })
      this.obsEnd(obs)
    })
  }
  /**/

  _update03to031()
  {
    return this.C('ifrouters').findNodes({type:'httpthemerouter'}).pipe(
      RxO.mergeMap(rlist=>{
        if (!rlist||!rlist.length)
          return Rx.of(null)
        let $t = rlist.map(rt=>{
          let pb = this.M('bundle').bundle(rt._pcc())
          let fb = this.M('bundle').bundle(rt._fcc())

          let $tt = []

          if (fb&&!fb.hasField('actions'))
          {
            fb.addField({name:g18n.t('Actions'),code:'actions',type:'relation', conf:{ops:[{source:'manual',manual:'screenmessage'}]}, flags:['system','logic'],quantity:0})
            log.info('updating forms bundle for http router',rt.f('code'))
            $tt.push(fb.save())
          }

          if (pb&&!pb.hasField('actions'))
          {
            pb.addField({name:g18n.t('Actions'),code:'actions',type:'relation', conf:{ops:[{source:'manual',manual:'screenmessage'}]}, flags:['system','logic'],quantity:0})
            log.info('updating pages bundle for http router',rt.f('code'))
            $tt.push(pb.save())
          }
          if (!$tt.length)
            return Rx.of(null)
          return Rx.concat(...$tt)

        })
        if (!$t.length)
          return Rx.of(null)
        return Rx.concat(...$t)
      })
    )
  }

  _update02to03()
  {
    let wwwr = null
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>this.M('interface').moduleInit()),
      RxO.mergeMap(_=>{
        //return this.C('ifrouters').findNodes({}).pipe(RxO.mergeMap(lst=>{
          //let $l = lst.map(x=>x.install())
          //return Rx.forkJoin(...$l)
        //}))
      //}),
      //RxO.mergeMap(_=>this.M('interface').moduleInit()),
      //RxO.mergeMap(_=>{
        //transfering pages to public router
        log.warn('creating public www router')
        let $wr = this.C('ifrouters').newNode()
        $wr.f('code','website')
        $wr.f('name','Website http')
        $wr.f('type','httpthemerouter')
        $wr.f('config',{theme:'blocks',formsmethod:'POST'})
        $wr.field('flags').push('system')
        return $wr.save()
      }),
      RxO.mergeMap(_=>this.C('ifrouters').findNode({code:'website'}).pipe(
        RxO.mergeMap(x=>{
          wwwr = x
          return x.install()
        })
      )),
      RxO.mergeMap(_=>wwwr._rc().removeNodes({})),
      RxO.mergeMap(_=>wwwr._pc().removeNodes({})),
      RxO.mergeMap(_=>{
        log.info('transfering routes to new router')
        return this.C('http_routes').findNodes({}).pipe(RxO.mergeMap(plist=>{
          let $t = plist.map(rt=>{
            if (rt.f('target')=='http_pages')
              return null
            if (rt.f('target').startsWith('httppages_'))
              return null
            log.info('route',[rt.f('url'),rt.f('target')])
            if (!this.C(rt.f('target')))
              return null
            let $rt = wwwr._rc().newNode()
            $rt.f('target',rt.f('target'))
            $rt.f('nid',rt.f('nid'))
            $rt.f('rescode',rt.f('code'))
            $rt.f('type',rt.f('type'))
            $rt.f('url',rt.f('url'))
            $rt.f('method','GET')
            return $rt.save()
          }).filter(x=>!!x)
          return Rx.concat(...$t).pipe(RxO.last())
          //return Rx.of(null)

        }))
      }),

      RxO.mergeMap(_=>{
        log.info('transfering pages to new router')
        return this.C('http_pages').findNodes({}).pipe(RxO.mergeMap(plist=>{
          let $t = plist.map(pg=>{
            let $pg = wwwr._pc().newNode()
            //console.log([pg.toObject()])
            log.info('page',pg.f('code'))
            $pg.f('enabled',pg.f('enabled'))
            $pg.f('code',pg.f('code'))
            $pg.f('name',pg.f('name'))
            $pg.f('type',pg.f('type'))
            let pgd = pg.f('data')
            let $pgd = {}
            if (pgd.url&&pgd.url.length)
              $pgd['url'] = pgd.url
            $pg.f('data',$pgd)
            let $pgv = pgd.variants.map(v=>{
              let nv = {
                name: v.name,
                title: v.title,
                logic: v.logic,
                format: v.format,
                libs: [],
                data: {}
              }
              Object.keys(v.data).forEach(dkey=>{
                if (dkey == 'libs')
                  nv.libs = v.data[dkey]
                else
                  nv.data[dkey] = v.data[dkey]
              })
              //console.log(['!!!!!!!!!',v,nv])
              return nv
            })
            $pg.field('variants').all($pgv)
            return $pg.save()
          })
          return Rx.concat(...$t).pipe(RxO.last())
          //return Rx.of(null)
        }))
      }),

      RxO.mergeMap(_=>{
        log.info('creating public www interface')
        let iface = this.C('interfaces').newNode()
        iface.f('enabled',true)
        iface.f('name',g18n.t('Public website'))
        iface.f('code','publicwww')
        iface.f('type','http')
        iface.f('router','website')
        iface.f('config',{
          host: this._config.host,
          port: this._config.port,
          redisHost: this.M('db')._config.httpredis.host,
          redisPort: this.M('db')._config.httpredis.port,
          redisDB: this.M('db')._config.httpredis.db,
          secret: this._config.sessionSecret,
          publicDirs: ['files'],
          uploadDir: 'files',
          uploadMaxSize: 52428800,
          jsonMaxSize: '50mb'
        })
        iface.f('status','unknown')

        return iface.save()
      })
    )
  }

  _update01to02()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>Rx.concat(
        this.M('auth/role').roleSave('http','root',g18n.t('Web Site root')),
        this.M('auth/role').roleSave('http','theme',g18n.t('Web Site themes admin access')),
        this.M('auth/role').roleSave('http','page',g18n.t('Web Site pages admin access')),
        this.M('auth/role').roleSave('http','block',g18n.t('Web Site blocks admin access'))
      ).pipe(RxO.last()))
    )
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1')
          return this._update01to02()
        if (from=='0.2')
          return this._update02to03()
        if (from=='0.3')
          return this._update03to031()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }

  /**
  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      let install = {
        host: '0.0.0.0',
        port: '8080'
      }
      let $t = []
      $t.push(this.M('core').consolePrompt(g18n.t('Website host [0.0.0.0]:'),{default:'0.0.0.0'}).pipe(RxO.map(val=>{install.host=val})))
      $t.push(this.M('core').consolePrompt(g18n.t('Website port [8080]:'),{default:'8080'}).pipe(RxO.map(val=>{install.port=val})))


      Rx.concat(...$t).subscribe(()=>{},err=>this.obsErr(obs,err),_=>{
        let cfg = {
          host: install.host,
          port: install.port,
          sessionSecret: 'asdqweasdzxcasdqweasdzxcsdlkhjsdflkgjh'
        }

        this.obsEnd(obs,cfg)
      })

    })
  }


  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      $t.push(this.M('auth/role').roleSave('http','root',g18n.t('Web Site root')))
      $t.push(this.M('auth/role').roleSave('http','theme',g18n.t('Web Site themes admin access')))
      $t.push(this.M('auth/role').roleSave('http','page',g18n.t('Web Site pages admin access')))
      $t.push(this.M('auth/role').roleSave('http','block',g18n.t('Web Site blocks admin access')))

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }
  /**/



}

module.exports = HttpModule
