/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('HttpPageModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')
const VNode = require('../../node/vnode')

const AbstractHttpPage = require('./page')
const ContentPage = require('./contentpage')
//const BlockPage = require('./blockpage')
const LibraryPage = require('./librarypage')
const ModulePage = require('./modulepage')

class HttpPageModule extends CoreModule
{

  constructor(parent)
  {
    super(parent)

    this._pages = {}

    this.registerPageType('content',ContentPage)
    //this.registerPageType('block',BlockPage)
    this.registerPageType('library',LibraryPage)
    this.registerPageType('module',ModulePage)
  }

  registerPageType(code,Cls)
  {
    if (!(AbstractHttpPage.isPrototypeOf(Cls)))
      return
    this._pages[code] = Cls
  }

  findPage(ctx,coll)
  {
    return Rx.Observable.create(obs=>{
      let rt = ctx.vnode('route')
      if (rt&&this.M(rt.f('module')&&typeof this.M(rt.f('module')[rt.f('fn')] != 'function')))
      {
        coll.findNode({type:'module'}).subscribe(page=>{
          ctx.addNode(page,'page')
          this.obsEnd(obs,page)
        },err=>{
          this.obsErr(obs,err)
        })
        return
      }
      let contentNode = ctx.vnode('content')
      if (contentNode instanceof AbstractHttpPage)
      {
        ctx.addNode(contentNode,'page')
        return this.obsEnd(obs,contentNode)
      }

      coll.findNode({type:'content'}).subscribe(page=>{
        ctx.addNode(page,'page')
        this.obsEnd(obs,page)
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtPages = {
        url: '/api/http/pages/:rcode',
        access: ['auth.dashboard','http.page'],
        method: 'get',
        fn: (ctx)=>{
          return this.C('ifrouters').findNode({code:ctx.f('params').rcode}).pipe(
          //return Rx.of(this.M('interface').ifRouter(ctx.f('params').rcode)).pipe(
            //RxO.tap(x=>console.log(x)),
            RxO.mergeMap(rt=>rt._pc().findNodes({},{list:false})),
            //RxO.map(list=>list.map(x=>x.toObject()))
            /**/
            RxO.filter(x=>!!x),
            RxO.map(x=>x.toObject()),
            RxO.reduce((acc,x)=>{
              acc.push(x)
              return acc
            },[])
            /**/
          )
        }
      }

      let rtPage = {
        url: '/api/http/page/:rcode/:nid',
        access: ['auth.dashboard','http.page'],
        method: 'get',
        fn: (ctx)=>{
          return this.C('ifrouters').findNode({code:ctx.f('params').rcode}).pipe(
          //return this.C('http_pages').findId(ctx.f('params').nid).pipe(
            RxO.mergeMap(rt=>rt._pc().findId(ctx.f('params').nid)),
            RxO.filter(x=>!!x),
            RxO.map(x=>x.toObject())
            //RxO.tap(x=>console.log(x))
          )
        }
      }
 
      let rtPageSave = {
        url: '/api/http/page/save',
        access: ['auth.dashboard','http.page'],
        method: 'post',
        fn: (ctx)=>{
          //return this.C('ifrouters').findNode({code:ctx.f('body').router}).pipe(
          return Rx.of(ctx.f('body')).pipe(
            RxO.mergeMap(bd=>{
              //console.log(['saving body',bd])
              return this.C('ifrouters').findNode({code:bd.router})
            }),
          //return this.C('http_pages').findId(ctx.f('params').nid).pipe(
            //RxO.filter(x=>!!x),
            RxO.mergeMap(rt=>{
              //console.log(rt)
              if (!rt)
                return {error:'oooops'}
              let bd = ctx.f('body')
              if (bd.id)
                return rt._pc().findId(bd.id)
              return rt._pc().initNode(ctx)
            }),
            RxO.mergeMap(pg=>{
              let bd = ctx.f('body')
              pg.f('enabled',!!bd.enabled)
              pg.f('code',bd.code)
              pg.f('name',bd.name)
              pg.f('type',bd.type)
              pg.field('variants').all(bd.variants)
              pg.f('data',bd.data)
              return pg.save()
            }),
            RxO.mergeMap(pg=>{
              let url = pg.f('data').url || null
              if (!url)
                return Rx.of(pg)
              let bd = ctx.f('body')
              return this.C('ifrouters').findNode({code:bd.router}).pipe(
                RxO.mergeMap(rt=>rt.updatePageRoute(pg))
              )
            }),
            RxO.map(pg=>{
              if (pg)
                return {success:true}
              else
                return {success:false,error:'Unknown page error'}
            })
          )
        }
      }

      let rtContext = {
        url: '/api/http/pagecontext/:ptype/:rcode',
        access: ['auth.dashboard','http.page'],
        method: 'get',
        fn: (cctx)=>{
          return Rx.Observable.create(obs=>{
            let ctx = this.M('context').newContext(this,'http',true)

            let user = this.C('user').newNode(ctx)
            ctx.users([user])

            let http = new VNode(ctx)
            http.addField({code:'url',type:'string'})
            http.addField({code:'method',type:'string'})
            http.addField({code:'params',type:'any'})
            http.addField({code:'query',type:'any'})
            http.addField({code:'body',type:'any'})
            http.addField({code:'user',name:g18n.t('User'),type:'relation',xtype:'relation:user'})
            ctx.addNode(http,'http')

            let route = new VNode(ctx)
            route.addField({code:'target',type:'string'})
            ctx.addNode(route,'route')

            let ifrouter = this.C('ifrouters').newNode(ctx)
            ctx.addNode(ifrouter,'router')
            let iface = this.C('interfaces').newNode(ctx)
            ctx.addNode(iface,'interface')

            let content = new VNode(ctx)
            ctx.addNode(content,'content')

            this.obsEnd(obs,ctx.logicFields())

          })
        }
      }

      this.obsEnd(obs,[
        rtPages,
        rtPage,
        rtPageSave,
        rtContext
      ])
    })
  }

  pageClass(mdl)
  {
    if (!mdl.type) return ContentPage
    if(this._pages[mdl.type]) return this._pages[mdl.type]
    return ContentPage
  }

  _update011to012()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>this.C('ifrouters').findNodes({type:'httpthemerouter'})),
      RxO.mergeMap(rlist=>{
        let $t = rlist.map(rt=>{
          if (!rt._pc()) return null
          log.info('creating module page for',rt.f('code'))
          let pg = rt._pc().newNode()
          pg.f('enabled',true)
          pg.f('code','module')
          pg.f('name','Module defined page')
          pg.f('type','module')
          return pg.save()
        }).filter(x=>!!x)
        if (!$t.length)
          return Rx.of(1)
        return Rx.concat(...$t).pipe(RxO.last())
      }),
      RxO.map(_=>'0.1.2')
    )
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from == '0.1.1')
          return this._update011to012()
        return Rx.of(to)
      })
    )
  }

}

module.exports = HttpPageModule
