/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('AbstractHttpPage')
let g18n = require('../../../g18n')

const Node = require('../../node/node')

class AbstractHttpPage extends Node
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._type = 'http_page'
    this._variant = null
  }

  variant()
  {
    return this._variant
  }

  title()
  {
    return this.variant().title
  }

  pageTitle(ctx)
  {
    /**
    return Rx.of(this.f('title')).pipe(
      RxO.mergeMap(t=>{
        //console.log(['title',t,ctx.view()])
        return this.M('template').liquify(t,ctx.view())
      }),
      RxO.map(t=>{
        console.log(['tt',t])
        return t
      })
    )
    /**/
    return this.M('template').liquify(this.variant().title,ctx.view())
  }

  executeActions(ctx)
  {
    return this.execute(ctx)
  }

  execute(ctx)
  {
    return Rx.Observable.create(obs=>{
      //let cfg = this.f('conf')
      let data = this.f('data')
      let variants = this.field('variants').all()
      variants = Array.isArray(variants) ? variants.reverse() : []

      let variant = null
      let $t = []
      if (!variants.length)
        return this.obsErr(obs,{code:'PAGE_VARIANTS_EMPTY'})

      if (variants.length==1)
      {
        variant = variants[0]
        $t = [Rx.of(0)]
      } else {
        $t = variants.map(v=>this.M('logic').testLogicDirectRx(v.logic,ctx).pipe(RxO.map(ok=>{
          if (variant) return
          if (ok) variant = v
        })))
      }
      //console.log($t)
      Rx.concat(...$t).pipe(RxO.last(),RxO.mergeMap(v=>{
        if (!variant)
          variant = variants.pop()
        return this.M('actions').executeActions(variant.actions,ctx).pipe(RxO.map(_=>variant))
      })).subscribe(()=>{
        if (!variant)
          return this.obsErr(obs,{code:'PAGE_VARIANT_NOT_FOUND'})
        this._variant = variant
        //console.log(['page.js','rendering variant',variant])
        log.info('rendering variant',variant.name)
        //executing varian actions
        let fmt = variant.format
        let renderer = this.M('http/engine').renderer(fmt)
        if (!renderer)
          return this.obsErr(obs,{code:'PAGE_RENDERER_NOT_FOUND',format:fmt})
        //console.log(['use renderer',renderer])
        //ctx.addNode(renderer,'renderer')
        let nx = Rx.of(1)
        if (Array.isArray(variant.libs)&&variant.libs.length)
        {
          let $libs = variant.libs.map(l=>this.M('logic').library(ctx,l))
          nx = Rx.concat(...$libs).pipe(RxO.last())
        }
        //console.log(['page.js','start rendering variant'])
        nx.pipe(RxO.mergeMap(_=>renderer.render(ctx))).subscribe(resHtml=>{
          this.obsEnd(obs,resHtml)
        },err=>this.obsErr(obs,err))
      },err=>this.obsErr(obs,{code:'VARIANTS_TEST_ERROR',error:err}))
      /**/
    })
  }

  buildIndex(pageHtml,ctx)
  {
    return this.pageTitle(ctx).pipe(
      RxO.map(pTitle=>{
        //console.log(['page buildindex',pageHtml])
        let bld = {
          title: '',
          meta: [],
          css: [],
          js: [],
          body: pageHtml.body
        }
        let theme = ctx.vnode('theme')
        bld.css = theme.cssFiles()
        bld.js = theme.jsFiles()
        bld.title = theme.pageTitle(pTitle)
        return bld
      }),
      RxO.mergeMap(bld=>{
        let cfg = this.variant().data
        //console.log(['checking meta',cfg])
        if (!cfg||!Array.isArray(cfg.meta)||!cfg.meta.length)
          return Rx.of(bld)
        let $t = []
        cfg.meta.forEach(tg=>{
          if (!Array.isArray(tg)||!tg.length)
            return
          let $tt = []
          tg.forEach(el=>{
            //console.log(['building meta',el])
            $tt.push(Rx.forkJoin(
              this.M('template').liquify(el.key,ctx.view()),
              this.M('template').liquify(el.val,ctx.view())
            ).pipe(RxO.map(cmpl=>{
              return {
                key: cmpl[0],
                val: cmpl[1]
              }
            })))
          })
          $t.push(Rx.forkJoin(...$tt))
        })
        return Rx.forkJoin(...$t).pipe(RxO.map(metas=>{
          //console.log(['builded meta',metas])
          bld.meta = metas
          return bld
        }))
      })
    )
  }

  compile(ctx)
  {
    return Rx.throwError({code:'ABSTRACT_PAGE_COMPILE_ALWAYS_FAIL'})
  }

  build(regionsHtml,ctx)
  {
    return Rx.of({
      bodyClass: '',
      regions: regionsHtml
    })
  }

  buildContentBlock(ctx)
  {
    return Rx.Observable.create(obs=>{
      this.obsErr(obs,{code:'ABSTRACT_PAGE_CONTENT_BLOCK_ALWAYS_FAIL'})
      /*
      this.obsEnd(obs,{
        id: content.id(),
        type: content.bundle(),
        content: html,
        title: null
      })
      */
    })
  }

  /**/
  beforeSave()
  {
    return Rx.Observable.create(obs=>{
      let url = null
      let data = this.f('data')
      if (data&&data.url&&data.url.length)
        url = data.url
      if (!url)
        return this.obsEnd(obs,this)
      let rcode = this.bundle().slice(10)
      this.C('ifrouters').findNode({code:rcode}).subscribe(router=>{
        //let router = this.M('interface').ifRouter(rcode)
        //console.log([rcode,router])
        if (!router)
          return this.obsEnd(obs,this)
        let coll = router._rc()
        if (!coll)
          return this.obsEnd(obs,this)
        coll.findNode({url:url}).subscribe(rt=>{
          if (!rt)
            return this.obsEnd(obs,this)
          if (rt&&rt.f('target')==this.bundle()&&this.eq(rt.f('nid')))
            return this.obsEnd(obs,this)
          this.obsErr(obs,{code:'URL_ALLREADY_EXISTS',url:url,owner:rt.f('target')})
        },err=>this.obsErr(obs,err))
      },err=>this.obsErr(obs,err))
    })
  }

  afterSave()
  {
    return Rx.Observable.create(obs=>{
      let url = null
      let data = this.f('data')
      if (data&&data.url&&data.url.length)
        url = data.url
      if (!url)
        return this.obsEnd(obs,this)
      let rcode = this.bundle().slice(10)
      //let router = this.M('interface').ifRouter(rcode)
      //if (!router)
        //return this.obsEnd(obs,this)
      //router.addNodeRoute(
      //let coll = router._rc()
      //if (!coll)
        //return this.obsEnd(obs,this)
      this.M('http/router').addNodeRoute(rcode,url,this).subscribe(rt=>{
        this.obsEnd(obs,this)
      },err=>this.obsErr(obs,err))
    })
  }
  /**/

}

module.exports = AbstractHttpPage
