/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let PTR = require('path-to-regexp')

let log = require('../../../log').m('HttpRouterModule')
let g18n = require('../../../g18n')

const CoreModule = require('../../core/abstract/module')
const VNode = require('../../node/vnode')
const HTTPRoute = require('./route')
const HTTPThemeRouter = require('./router.theme')
const HttpRouteField = require('./urlfield')
const R2S = require('./urlfield.2string')

class HttpRouterModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this.M('field').registerFieldType(HttpRouteField)
    this.M('field').registerModifier(R2S)
  }

  interfaceRegister()
  {
    return Rx.Observable.create(obs=>{
      this.M('interface').registerRouterType(HTTPThemeRouter)
      this.obsEnd(obs)
    })
  }


  findRoute(ctx)
  {
    return Rx.Observable.create(obs=>{
      let http = ctx.vnode('http')
      if (!http)
        return this.obsErr(obs,{code:'NO_HTTP_CONTEXT'})
      let urls = http.f('url').split('?')
      let url = urls[0].slice(1)
      if (url=='') url = '<front>'
      //log.test('searching route for',url)
      Rx.concat(
        this._staticRoute(http),
        this._dynamicRoute(ctx)
      ).pipe(
        RxO.filter(x=>!!x),
        RxO.first(),
        RxO.mergeMap(x=>x.prepare(ctx))
      ).subscribe(rt=>{
        this.obsEnd(obs,rt)
      },err=>{
        log.err('oooops error',err)
        this.obsErr(obs,err)
      })
    })
  }

  _staticRoute(http)
  {
    return Rx.Observable.create(obs=>{
      let urls = http.f('url').split('?')
      let url = urls[0].slice(1)
      if (url=='') url = '<front>'
      //log.warn('search static',[url])
      this.C('http_routes').findNode({type:'static',url:url}).subscribe(rt=>{
        //log.info('res')
        //console.log(rt)
        this.obsEnd(obs,rt)
      },err=>{
        this.obsErr(obs,{code:'STATIC_ROUTE_SEARCH_ERROR',err:err})
      })
    })
  }

  _dynamicRoute(ctx)
  {
    return Rx.Observable.create(obs=>{
      let http = ctx.vnode('http')
      let urls = http.f('url').split('?')
      let url = urls[0].slice(1)
      if (url=='') url = '<front>'
      log.warn('search dynamic',[url])
      let urlNode = null
      this.C('http_routes').findNodes({type:'dynamic'}).subscribe(rts=>{
        let starRoute = null
        let okRoute = null
        rts.forEach(rt=>{
          if (okRoute) return
          let rurl = rt.f('url')
          //console.log(['test',rurl])
          if (rurl=='*')
          {
            starRoute = rt
            return
          }
          let keys = []
          let rx = PTR(rurl,keys)
          if (!rx)
            return
          let ex = rx.exec(url)
          if (!ex)
            return
          urlNode = new VNode(ctx)
          keys.forEach((key,ki)=>{
            urlNode.addField({
              code: key.name,
              type: 'string'
            })
            let val = ex[ki+1]
            if (typeof val != 'undefined')
              urlNode.f(key.name,val)
          })
          okRoute = rt
        })
        //console.log(['dres',!!okRoute,!!starRoute])
        if (urlNode)
          ctx.addNode(urlNode,'url')
        if (okRoute)
          return this.obsEnd(obs,okRoute)
        this.obsEnd(obs,starRoute)
      },err=>{
        this.obsErr(obs,{code:'DYNAMIC_ROUTE_SEARCH_ERROR',err:err})
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtWidgetsTypes = {
        url: '/api/http/widgetstypes',
        access: ['auth.dashboard','http.form'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.of(this.M('forms').widgetsTypes())
        }
      }


      let rtFormsTypes = {
        url: '/api/http/formstypes',
        access: ['auth.dashboard','http.form'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.of(this.M('forms').formsTypes())
        }
      }


      let rtPages = {
        url: '/api/http/forms/:rcode',
        access: ['auth.dashboard','http.form'],
        method: 'get',
        fn: (ctx)=>{
          //return this.C('ifrouters').findNode({code:ctx.f('params').rcode}).pipe(
          return Rx.of(this.M('interface').ifRouter(ctx.f('params').rcode)).pipe(
            //RxO.tap(x=>console.log(x)),
            RxO.mergeMap(rt=>rt._fc().findNodes({},{list:false})),
            //RxO.map(list=>list.map(x=>x.toObject()))
            /**/
            RxO.filter(x=>!!x),
            RxO.map(x=>x.toObject()),
            //RxO.tap(x=>console.log(x)),
            RxO.reduce((acc,x)=>{
              acc.push(x)
              return acc
            },[])
            /**/
          )
        }
      }

      let rtPage = {
        url: '/api/http/form/:rcode/:nid',
        access: ['auth.dashboard','http.form'],
        method: 'get',
        fn: (ctx)=>{
          return this.C('ifrouters').findNode({code:ctx.f('params').rcode}).pipe(
          //return this.C('http_pages').findId(ctx.f('params').nid).pipe(
            RxO.mergeMap(rt=>rt._fc().findId(ctx.f('params').nid)),
            //RxO.tap(x=>console.log(x)),
            //RxO.filter(x=>!!x),
            RxO.map(x=>x.toObject())
          )
        }
      }
 
      let rtPageSave = {
        url: '/api/http/form/save',
        access: ['auth.dashboard','http.form'],
        method: 'post',
        fn: (ctx)=>{
          //return this.C('ifrouters').findNode({code:ctx.f('body').router}).pipe(
          return Rx.of(ctx.f('body')).pipe(
            RxO.mergeMap(bd=>{
              //console.log(['saving body',bd])
              return this.C('ifrouters').findNode({code:bd.router})
            }),
          //return this.C('http_pages').findId(ctx.f('params').nid).pipe(
            //RxO.filter(x=>!!x),
            RxO.mergeMap(rt=>{
              //console.log(rt)
              if (!rt)
                return {error:'oooops'}
              let bd = ctx.f('body')
              if (bd.id)
                return rt._fc().findId(bd.id)
              return rt._fc().initNode(ctx)
            }),
            RxO.mergeMap(pg=>{
              let bd = ctx.f('body')
              //console.log(bd)
              //pg.f('enabled',!!bd.enabled)
              pg.f('code',bd.code)
              pg.f('name',bd.name)
              pg.f('type',bd.type)
              pg.f('url',bd.url)
              pg.f('fields',bd.fields)
              pg.f('view',bd.view)
              pg.f('actions',bd.actions)
              return pg.save()
            }),

            /*
            RxO.mergeMap(pg=>{
              let url = pg.f('data').url || null
              if (!url)
                return Rx.of(pg)
              let bd = ctx.f('body')
              return this.C('ifrouters').findNode({code:bd.router}).pipe(
                RxO.mergeMap(rt=>rt.updatePageRoute(pg))
              )
            }),
            /**/
            RxO.map(pg=>{
              if (pg)
                return {success:true}
              else
                return {success:false,error:'Unknown form error'}
            })
          )
        }
      }

      /*
      let rtContext = {
        url: '/api/http/pagecontext/:ptype/:rcode',
        access: ['auth.dashboard','http.page'],
        method: 'get',
        fn: (cctx)=>{
          return Rx.Observable.create(obs=>{
            let ctx = this.M('context').newContext(this,'http',true)

            let user = this.C('user').newNode(ctx)
            ctx.users([user])

            let http = new VNode(ctx)
            http.addField({code:'url',type:'string'})
            http.addField({code:'method',type:'string'})
            http.addField({code:'params',type:'any'})
            http.addField({code:'query',type:'any'})
            http.addField({code:'body',type:'any'})
            http.addField({code:'user',name:g18n.t('User'),type:'relation',xtype:'relation:user'})
            ctx.addNode(http,'http')

            let route = new VNode(ctx)
            route.addField({code:'target',type:'string'})
            //let route = 
            //let route = this.C('http_routes').newNode(ctx)
            ctx.addNode(route,'route')

            let ifrouter = this.C('ifrouters').newNode(ctx)
            ctx.addNode(ifrouter,'router')
            let iface = this.C('interfaces').newNode(ctx)
            ctx.addNode(iface,'interface')
            //let page = this.C('http_pages').newNode(ctx)
            //ctx.addNode(page,'page')

            //let theme = this.M('http/theme').currentTheme()
            //ctx.addNode(theme,'theme')


            let content = new VNode(ctx)
            ctx.addNode(content,'content')

            this.obsEnd(obs,ctx.logicFields())

          })
        }
      }
      /**/

      this.obsEnd(obs,[
        rtWidgetsTypes,
        rtFormsTypes,
        rtPages,
        rtPage,
        rtPageSave
        //rtContext
      ])
    })
  }


  nodeAfterSave(node)
  {
    if (node.bundle().startsWith('httproutes_'))
      return Rx.of(node)

    let fields = node.bundleFields()
    let routes = fields.filter(f=>f.type=='httproute')
    if (!routes.length)
      return Rx.of(node)
    let $t = routes.map(rf=>{
      let rcode = rf.conf.ops[0].manual
      let $rt = this.M('interface').ifRouter(rcode)
      if (!$rt)
        return null
      let rc = $rt._rc()
      return rc.removeNodes({nid:node.id()}).pipe(
        RxO.mergeMap(_=>{
          let urls = []
          node.field(rf.code).all().forEach(furl=>{
            if (furl&&furl.length)
              urls.push(furl)
          })
          let $tt = urls.map(url=>{
            let $rt = rc.newNode()
            $rt.f('target',node.bundle())
            $rt.f('nid',node.id())
            $rt.f('rescode',200)
            $rt.f('method','GET')
            let keys = []
            let rx = PTR(url,keys)
            if (keys.length)
              $rt.f('type','dynamic')
            else
              $rt.f('type','static')
            $rt.f('url',url)
            return $rt.save()
          })
          if (!$tt.length)
            return Rx.of(1)
          return Rx.concat(...$tt).pipe(RxO.last())
        })
      )
    }).filter(x=>!!x)

    if (!$t.length)
      return Rx.of(node)
    return Rx.concat(...$t).pipe(
      RxO.last(),
      RxO.map(_=>node)
    )
  }

  nodesAfterRemove(info)
  {
    /**/
    if (info.bundle.startsWith('httproutes_'))
      return Rx.of(info)

    let bndl = this.M('bundle').bundle(info.bundle)
    if (!bndl)
      return Rx.of(info)
    let fields = bndl.fields()

    let routes = fields.filter(f=>f.type=='httproute')
    if (!routes.length)
      return Rx.of(info)

    let $t = routes.map(rf=>{
      let rcode = rf.conf.ops[0].manual
      let $rt = this.M('interface').ifRouter(rcode)
      if (!$rt)
        return null
      let rc = $rt._rc()
      return rc.removeNodes({nid:{$in:info.nids}})
    }).filter(x=>!!x)
 
    if (!$t.length)
      return Rx.of(info)
    return Rx.concat(...$t).pipe(
      RxO.last(),
      RxO.map(_=>info)
    )
  }

  routeExists(url)
  {
    return this.C('http_routes').findNode({url:url})
  }

  addNodeRoute(rcode,url,node,code=200)
  {
    if (typeof url != 'string'||!url.length)
      return Rx.of(null)
    return this.C('ifrouters').findNode({code:rcode}).pipe(RxO.mergeMap($rt=>{
      //let $rt = this.M('interface').ifRouter(rcode)
      if (!$rt)
        return Rx.of(null)
      let rc = $rt._rc()
      return rc.removeNodes({nid:node.id()}).pipe(
        RxO.mergeMap(_=>{
          let $rt = rc.newNode()
          $rt.f('target',node.bundle())
          $rt.f('nid',node.id())
          $rt.f('rescode',code)
          $rt.f('method','GET')
          let keys = []
          let rx = PTR(url,keys)
          if (keys.length)
            $rt.f('type','dynamic')
          else
            $rt.f('type','static')
          $rt.f('url',url)
          return $rt.save()
        })
      )
    }))
  }

  moduleUpdate(from,to)
  {
    return Rx.of(to)
  }

}

module.exports = HttpRouterModule
