/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let PTR = require('path-to-regexp')

let log = require('../../../log').m('HttpThemeRouter')
let g18n = require('../../../g18n')

const AbstractRouter = require('../../interface/router')
const VNode = require('../../node/vnode')

class HttpThemeRouter extends AbstractRouter
{

  init(ctx)
  {
    return Rx.Observable.create(obs=>{
      if (this._pc())
        this._pc().setNodeClassFn(mdl=>this.M('http/page').pageClass(mdl))
      if (this._fc())
        this._fc().setNodeClassFn(mdl=>this.M('forms').formClass(mdl))
      this.obsEnd(obs,this)
    })
  }

  cfgTheme()
  {
    return this.f('config').theme
  }

  executeContext(ctx)
  {
    return this.M('http/page').findPage(ctx,this._pc()).pipe(
      RxO.mergeMap(page=>{
        //console.log(page)
        if (!page)
          return Rx.throwError({code:'NO_PAGE_FOUND'})
        log.info('page found',page.f('code'))
        return page.executeActions(ctx)
      })
    )
  }

  //common routines
  searchRoute(ctx)
  {
    //console.log(['searching route',ctx.view()])
    //return Rx.of(null)

    /**/
    return Rx.Observable.create(obs=>{
      let http = ctx.vnode('http')
      if (!http)
        return this.obsErr(obs,{code:'NO_CONTEXT_INTERFACE'})
      let method = http.f('method')
      let frx = Rx.of(null)
      if (method=='POST')
      {
        //search form
        let fdata = http.f('body')
        let fid = null
        if (fdata&&typeof fdata.fid != 'undefined')
          fid = fdata.fid
        if (fid)
        {
          frx = this._fc().findId(fid).pipe(
            RxO.mergeMap(form=>{
              if (!form)
                return Rx.of(null)
              return form.executeActions(fdata,ctx)
            })
          )
        }
      }
      let urls = http.f('url').split('?')
      let url = urls[0]//.slice(1)
      if (url=='/') url = '<front>'
      log.test('searching route for',url)
      frx.subscribe(fres=>{
        if (fres)
          return this.obsEnd(obs,fres)
        Rx.concat(
          this._staticRoute(http),
          this._dynamicRoute(ctx)
        ).pipe(
          RxO.filter(x=>!!x),
          RxO.first(),
          RxO.mergeMap(rt=>{
            let tbundle = rt.f('target')
            //console.log(['!!!',tbundle])
            if (!tbundle||!this.C(tbundle))
            {
              if (!this.M(rt.f('module')||typeof this.M(rt.f('module')[rt.f('fn')] != 'function')))
                return Rx.throwError({code:'ROUTE_TARGET_BUNDLE_NOT_FOUND',bundle:tbundle})
              ctx.addNode(rt,'route')
              return Rx.of(rt)
            }
            return this.C(tbundle).findId(rt.f('nid')).pipe(
              RxO.mergeMap(cnode=>{
                if (!cnode)
                  return Rx.throwError({code:'ROUTE_CONTENT_NODE_NOT_FOUND'})
                ctx.addNode(rt,'route')
                ctx.addNode(cnode,'content')
                return Rx.of(rt)
              })
            )
          }),
          RxO.mergeMap(_=>this.executeContext(ctx))
        ).subscribe(rt=>{
          log.log('search done')
          this.obsEnd(obs,rt)
        },err=>{
          log.err('oooops error',err)
          this.obsErr(obs,err)
        })
      },err=>{
        console.log(['search route error',err])
        this.obsErr(obs,err)
      })
    })
    /**/
  }

  _staticRoute(http)
  {
    return Rx.Observable.create(obs=>{
      let urls = http.f('url').split('?')
      let url = urls[0]//.slice(1)
      if (url=='/') url = '<front>'
      log.test('search static',[url])
      this._rc().findNode({type:'static',url:url}).subscribe(rt=>{
        //log.info('res')
        //console.log(rt)
        this.obsEnd(obs,rt)
      },err=>{
        this.obsErr(obs,{code:'STATIC_ROUTE_SEARCH_ERROR',err:err})
      })
    })
  }

  _dynamicRoute(ctx)
  {
    return Rx.Observable.create(obs=>{
      let http = ctx.vnode('http')
      let urls = http.f('url').split('?')
      let url = urls[0]//.slice(1)
      if (url=='/') url = '<front>'
      log.test('search dynamic',[url])
      let urlNode = null
      this._rc().findNodes({type:'dynamic'}).subscribe(rts=>{
        let starRoute = null
        let okRoute = null
        rts.forEach(rt=>{
          //console.log(rt)
          if (okRoute) return
          let rurl = rt.f('url')
          //console.log(['test',rurl])
          if (rurl=='*')
          {
            starRoute = rt
            return
          }
          //console.log(['test2',rurl])
          let keys = []
          let rx = PTR(rurl,keys)
          //console.log(['test3',rx])
          if (!rx)
            return
          let ex = rx.exec(url)
          if (!ex)
            return
          urlNode = new VNode(ctx)
          keys.forEach((key,ki)=>{
            urlNode.addField({
              code: key.name,
              type: 'string'
            })
            let val = ex[ki+1]
            if (typeof val != 'undefined')
              urlNode.f(key.name,val)
          })
          okRoute = rt
        })
        //console.log(['dres',!!okRoute,!!starRoute])
        if (urlNode)
          ctx.addNode(urlNode,'url')
        if (okRoute)
          return this.obsEnd(obs,okRoute)
        this.obsEnd(obs,starRoute)
      },err=>{
        console.log(err)
        this.obsErr(obs,{code:'DYNAMIC_ROUTE_SEARCH_ERROR',err:err})
      })
    })
  }

  updatePageRoute(page)
  {
    return this._rc().removeNodes({nid:page.id()}).pipe(
      RxO.mergeMap(_=>{
        let url = page.f('data').url
        let $rt = this._rc().newNode()
        $rt.f('target',page.bundle())
        $rt.f('nid',page.id())
        $rt.f('rescode',200)
        $rt.f('method','GET')
        let keys = []
        let rx = PTR(url,keys)
        if (keys.length)
          $rt.f('type','dynamic')
        else
          $rt.f('type','static')
        $rt.f('url',url)
        return $rt.save()
      }),
      RxO.map(_=>page)
    )
  }


  static info()
  {
    return {
      code: 'httpthemerouter',
      name: g18n.t('Http Theme'),
      desc: g18n.t('Http router with themed result.'),
      scope: 'http',
      confs: [{
        code: 'theme',
        name: g18n.t('Theme'),
        type: 'string'
      },{
        code: 'formsmethod',
        name: g18n.t('Form method'),
        type: 'string',
        def: 'POST'
      }]
    }
  }

  checkInstall()
  {
    return Rx.concat(
      Rx.iif(_=>!!this._rc(),Rx.of(true),this._createRouterCollection()),
      Rx.iif(_=>!!this._pc(),Rx.of(true),Rx.concat(
        this._createPagesCollection(),
        this.init(),
        this._createDefaultPages()
      ).pipe(RxO.last())),
      Rx.iif(_=>!!this._fc(),Rx.of(true),Rx.concat(
        this._createFormsCollection(),
        this.init(),
        this._createDefaultForms()
      ).pipe(RxO.last()))
    ).pipe(RxO.last(),RxO.map(_=>true))
  }


  install()
  {
    log.info('installing!')
    return Rx.concat(
      this._createRouterCollection(),
      this._createPagesCollection(),
      this._createFormsCollection(),
      this.init(),
      this._createDefaultForms(),
      this._createDefaultPages()
    ).pipe(RxO.last())
  }

  _pcc()
  {
    return 'httppages_'+this.f('code')
  }

  _pc()
  {
    return this.C(this._pcc())
  }

  _createPagesCollection()
  {
    return Rx.of(this.M('bundle').bundle(this._pcc(),true)).pipe(RxO.mergeMap(bndl=>{
          
      bndl.name(g18n.t('HTTP Pages for ')+this.f('name'))
      bndl.desc(g18n.t('HTTP Pages for ')+this.f('name'))
      bndl.flags(['system','locked'])

      bndl.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system','logic','title','template']})
      bndl.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','logic','unique','title','template']})
      bndl.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','logic','title','template']})
      bndl.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','logic','title','template']})
      bndl.addField({name:g18n.t('Variants'),code:'variants',type:'any',flags:['system','logic'],quantity:0})
      bndl.addField({name:g18n.t('Data'),code:'data',type:'any',flags:['system','logic']})

      log.info('creating bundle',bndl.code())
      return bndl.save(true,true)
    }),RxO.map(_=>this))
  }

  _rcc()
  {
    return 'httproutes_'+this.f('code')
  }

  _rc()
  {
    return this.C(this._rcc())
  }

  _createRouterCollection()
  {
    return Rx.of(this.M('bundle').bundle(this._rcc(),true)).pipe(RxO.mergeMap(bndl=>{
          
      bndl.name(g18n.t('HTTP Routes for ')+this.f('name'))
      bndl.desc(g18n.t('HTTP Routes for ')+this.f('name'))
      bndl.flags(['system','locked'])

      bndl.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','logic','template']})  //dynamic/static
      bndl.addField({name:g18n.t('Method'),code:'method',type:'string',flags:['system','logic','template'],def:'GET'}) //GET/POST/PUT/...
      bndl.addField({name:g18n.t('Url'),code:'url',type:'string',flags:['system','logic','template','title']})
      bndl.addField({name:g18n.t('Result Code'),code:'rescode',type:'number',flags:['system','logic','template'],def:200})  //default result code
      bndl.addField({name:g18n.t('Module'),code:'module',type:'string',flags:['system','logic','template','title']})
      bndl.addField({name:g18n.t('Function'),code:'fn',type:'string',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Data'),code:'data',type:'any',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Target'),code:'target',type:'string',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Node ID'),code:'nid',type:'relation',flags:['system','logic']})

      log.info('creating bundle',bndl.code())
      return bndl.save(true,true)
    }),
    RxO.map(_=>this))
  }

  _fcc()
  {
    return 'httpforms_'+this.f('code')
  }

  _fc()
  {
    return this.C(this._fcc())
  }

  _createFormsCollection()
  {
    return Rx.of(this.M('bundle').bundle(this._fcc(),true)).pipe(RxO.mergeMap(bndl=>{
          
      bndl.name(g18n.t('HTTP Forms for ')+this.f('name'))
      bndl.desc(g18n.t('HTTP Forms for ')+this.f('name'))
      bndl.flags(['system','locked'])

      bndl.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Url'),code:'url',type:'string',flags:['system','logic','template']})
      //bndl.addField({name:g18n.t('Fields'),code:'fields',type:'formfield',flags:['system','logic','template']})
      //bndl.addField({name:g18n.t('Actions'),code:'actions',type:'ctxaction',flags:['system','logic','template'],conf:{ops:[source:'manual',manual:'http']}})
      bndl.addField({name:g18n.t('Fields'),code:'fields',type:'any',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('View'),code:'view',type:'any',flags:['system','logic','template']})
      bndl.addField({name:g18n.t('Actions'),code:'actions',type:'any',flags:['system','logic','template']})

      log.info('creating bundle',bndl.code())
      return bndl.save(true,true)
    }),
    RxO.map(_=>{
      log.success('forms bundle created')
      //console.log(_)
      return this
    }))
  }

  _createDefaultPages()
  {
    return Rx.of(1).pipe(
      RxO.map(_=>this._pc()),
      RxO.mergeMap(coll=>{

        let $t = []

        log.info('creating default pages for',this.f('code'))
        let pgHome = coll.newNode()
        pgHome.f('enabled',true)
        pgHome.f('code','home')
        pgHome.f('name','Home page')
        pgHome.f('type','library')
        pgHome.f('data',{ url: '<front>' })
        pgHome.field('variants').all([{
          name: 'Default',
          title: 'Welcome! | {{project.name}}',
          format: 'theme',
          data: {
            content: 'This is home page',
            meta: []
          },
          libs: [],
          logic: null
        }])
        $t.push(pgHome.save())

        let pg404 = coll.newNode()
        pg404.f('enabled',true)
        pg404.f('code','notfoundpage')
        pg404.f('name','404 page')
        pg404.f('type','library')
        pg404.f('data',{url: '*'})
        pg404.field('variants').all([{
          name: 'Default',
          title: '404 | {{project.name}}',
          format: 'theme',
          logic: null,
          libs: [],
          data: {
            content: '404 - not found',
            meta: []
          }
        }])
        $t.push(pg404.save())

        let pgContent = coll.newNode()
        pgContent.f('enabled',true)
        pgContent.f('code','contentpages')
        pgContent.f('name','Content pages')
        pgContent.f('type','content')
        pgContent.f('data',{})
        pgContent.field('variants').all([{
            name: 'Default',
            logic: null,
            format: 'redirect',
            libs: [],
            data: {
              url: '/404',
            },
            title: 'Content pages'
        }])
        //pgContent.f('title','{{content.title}}')
        $t.push(pgContent.save())


        let pgModule = coll.newNode()
        pgModule.f('enabled',true)
        pgModule.f('code','module')
        pgModule.f('name','Modules defined page')
        pgModule.f('type','module')
        $t.push(pgContent.save())

        return Rx.concat(...$t).pipe(RxO.last())

      })
    )
  }

  _createDefaultForms()
  {
    return Rx.of(1).pipe(
      RxO.map(_=>this._fc()),
      RxO.mergeMap(coll=>{
        let $t = []

        log.info('creating default forms for',this.f('code'))

        let fLogin = coll.newNode()
        fLogin.f('name',g18n.t('User OTL form'))
        fLogin.f('type','otlcodelogin')
        fLogin.f('code','otlform')
        fLogin.f('url','*')
        fLogin.f('fields',[{
          code: 'otlcode',
          name: g18n.t('Login code'),
          desc: g18n.t('One time login code field'),
          type: 'string'
        }])
        fLogin.f('view',[{
          field: 'otlcode',
          widget: 'string',
          confs: [],
          required: true
        }])
        $t.push(fLogin.save())



        return Rx.concat(...$t).pipe(RxO.catchError(err=>{
          return Rx.whrowError(err)
        }),RxO.last())
      })
    )
  }



}

module.exports = HttpThemeRouter
