/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let fs = require('fs')
let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('Theme')
let g18n = require('../../../g18n')


const Node = require('../../node/node')

class Theme extends Node
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._engine = null
  }

  tplPaths()
  {
    let ret = []
    ret.push({
      type: 'theme',
      scope: 'user',
      path: this.f('code')
    })
    let base = this.M('http/theme').baseTheme(this.f('code'))
    if (!base)
      return ret
    let basePaths = base.tplPaths()
    basePaths.forEach(bp=>{
      ret.push(bp)
    })
    return ret
  }

  cssFiles()
  {
    let ret = this.f('conf').css
    let base = this.M('http/theme').baseTheme(this.f('code'))
    if (!base)
      return ret
    base.cssFiles().forEach(f=>{
      if (ret.includes(f))
        return
      ret.push(f)
    })
    return ret
  }

  jsFiles()
  {
    let ret = this.f('conf').js
    let base = this.M('http/theme').baseTheme(this.f('code'))
    if (!base)
      return ret
    base.jsFiles().forEach(f=>{
      if (ret.includes(f))
        return
      ret.push(f)
    })
    return ret
  }

  hasLocalFile(path)
  {
    let curr = './themes/'+this.f('code')+'/public'+path
    let stat = null
    //log.test('hash local?',[path,curr])
    try {
      stat = fs.statSync(curr)
    } catch(e) {
      stat = null
    }
    if (stat&&typeof(stat.isFile)=='function'&&stat.isFile())
      return curr
    let base = this.M('http/theme').baseTheme(this.f('code'))
    if (!base)
      return null
    return base.hasLocalFile(path)
  }

  pageTitle(title)
  {
    return title
  }

  init()
  {
    return Rx.Observable.create(obs=>{
      let engineCode = this.f('engine')
      this._engine = this.M('http/engine').engine(engineCode)
      //console.log(this._engine)
      log.log('engine initialized')
      this.obsEnd(obs,this)
    })
  }

  render(ctx)
  {
    return this._engine.build(ctx).pipe(
      RxO.mergeMap(build=>{
        //console.log(['theme render build',build])
        return this._engine.render(build,ctx)
      })
    )
  }

  build(ctx)
  {
    return this._engine.build(ctx)
  }


  compile(ctx)
  {
    return this._engine.build(ctx).pipe(
      RxO.mergeMap(build=>{
        //console.log(['theme render build',build])
        return this._engine.compile(build,ctx)
      })
    )
  }
}

module.exports = Theme
