/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('InterfacesModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')
const AbstractInterface = require('./abstract')
const AbstractRouter = require('./router')

class InterfacesModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)

    this._ifaceClasses = {}
    this._ifaces = {}
    this._routers = {}
    this._ifcodes = []
    this._routerClasses = {}
  }

  registerRouterType(Router)
  {
    if (!AbstractRouter.isPrototypeOf(Router))
      return false
    let inf = Router.info()
    log.log('registring router type',inf.code)
    this._routerClasses[inf.code] = Router
    return true
  }

  registerInterfaceType(IFace)
  {
    if (!AbstractInterface.isPrototypeOf(IFace))
      return false
    let inf = IFace.ifInfo()
    log.log('registring interface type',inf.code)
    this._ifaceClasses[inf.code] = IFace
    return true
  }

  interfaceClass(code)
  {
    if (typeof this._ifaceClasses[code]!='undefined')
      return null
    return this._ifaceClasses[code]
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let apiGet = {
        url: '/api/interface/list',
        access: ['auth.dashboard'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let list = []
            Object.keys(this._ifaces).forEach(code=>{
              list.push(this._ifaces[code].toObject())
            })
            let itypes = []
            Object.keys(this._ifaceClasses).forEach(code=>{
              let inf = this._ifaceClasses[code].ifInfo()
              if (Array.isArray(inf.confs))
                inf.confs = this.M('forms').fields2form(inf.confs)
              itypes.push(inf)
            })
            let rtypes = []
            Object.keys(this._routerClasses).forEach(code=>{
              let inf = this._routerClasses[code].info()
              if (Array.isArray(inf.confs))
                inf.confs = this.M('forms').fields2form(inf.confs)
              rtypes.push(inf)
            })
            this.C('ifrouters').findNodes({}).subscribe(rlist=>{
              this.obsEnd(obs,{list:list,types:itypes,rtypes:rtypes,rlist:rlist.map(x=>x.toObject())})
            },err=>{
              log.err('loading interface routers error!',err)
              this.obsEnd(obs,{list:list,types:itypes,rtypes:rtypes,rlist:[]})
            })
          })
        }
      }

      let apiSet = {
        url: '/api/interface/save',
        access: ['global.root'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let b = ctx.f('body')
            //console.log('saving interface',b)
            let $iface = null
            if (typeof b.id!='undefined')
            {
              //update
              if (!this._ifaces[b.code]||!this._ifaces[b.code].eq(b.id))
                return this.obsEnd(obs,{success:false,error:'INTERFACE_ID_AND_CODE_NOT_FOUND'})
              this._ifaces[b.code].f('enabled',b.enabled)
              this._ifaces[b.code].f('name',b.name)
              this._ifaces[b.code].f('router',b.router)
              let oldConf = this._ifaces[b.code].f('config')
              this._ifaces[b.code].f('config',b.config)
              if (!b.enabled)
                this._ifaces[b.code].f('status','off')
              this._ifaces[b.code].save().subscribe(_=>{
                this._ifaces[b.code].ifReconfigure(oldConf).subscribe(__=>{
                  this.obsEnd(obs,{success:true})
                },ierr=>{
                  console.log('err',ierr)
                  this._ifaces[b.code].f('enabled',false)
                  this._ifaces[b.code].f('status','fail')
                  this._ifaces[b.code].save().subscribe(_=>{
                    this.obsEnd(obs,{success:true})
                  },err=>{
                    this.obsEnd(obs,{success:false,error:'INTERFACE_SAVE_ERROR',err:err})
                  })
                })
              },err=>{
                this.obsEnd(obs,{success:false,error:'INTERFACE_SAVE_ERROR',err:err})
              })
              return
            }
            //new
            //code exists, return error
            if (this._ifaces[b.code])
              return this.obsEnd(obs,{success:false,error:'CODE_EXISTS'})
            let ifn = this.C('interfaces').newNode()
            ifn.f('enabled',b.enabled)
            ifn.f('name',b.name)
            ifn.f('code',b.code)
            ifn.f('type',b.type)
            ifn.f('router',b.router)
            ifn.f('config',b.config)
            ifn.f('status','unknown')
            if (!b.enabled)
              ifn.f('status','off')
            ifn.save().pipe(RxO.mergeMap(_=>this.C('interfaces').findNode({code:b.code}))).subscribe(iface=>{
              console.log(['NEW IFAFCE!!!',iface])
              this._ifaces[iface.f('code')] = iface
              if (!iface.f('enabled'))
                return this.obsEnd(obs,{success:true})
              this._ifaces[iface.f('code')].startIface().subscribe(_=>this.obsEnd(obs,{success:true}),err=>this.obsEnd(obs,err))
            },err=>{
              log.err('saving new interface error!',err)
              this.obsEnd(obs,{error:err})
            })
          })
        }
      }

      let apiSetR = {
        url: '/api/interface/rsave',
        access: ['global.root'],
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let b = ctx.f('body')
            //console.log('saving router',b)
            let $iface = null
            if (typeof b.id!='undefined')
            {
              //update
              if (!this._routers[b.code]||!this._routers[b.code].eq(b.id))
                return this.obsEnd(obs,{success:false,error:'ROUTER_ID_AND_CODE_NOT_FOUND'})
              this._routers[b.code].f('name',b.name)
              this._routers[b.code].f('config',b.config)
              this._routers[b.code].save().subscribe(_=>{
                this.obsEnd(obs,{success:true})
              },err=>{
                this.obsEnd(obs,{success:false,error:'ROUTER_SAVE_ERROR',err:err})
              })
              return
            }
            /**/
            //new
            //code exists, return error
            if (this._routers[b.code])
              return this.obsEnd(obs,{success:false,error:'CODE_EXISTS'})
            let rtn = this.C('ifrouters').newNode()
            rtn.f('name',b.name)
            rtn.f('code',b.code)
            rtn.f('type',b.type)
            rtn.f('config',b.config)
            rtn.save().pipe(RxO.mergeMap(_=>this.C('ifrouters').findNode({code:b.code}))).subscribe(rtn=>{
              //console.log(['NEW ROUTER!!!',rtn])
              this._routers[rtn.f('code')] = rtn
              this._routers[rtn.f('code')].install().subscribe(_=>this.obsEnd(obs,{success:true}),err=>{
                console.log(['!!!!',err])
                this.obsEnd(obs,err)
              })
            },err=>{
              log.err('saving new interface error!',err)
              this.obsEnd(obs,{error:err})
            })
            /**/
          })
        }
      }


      let apiIfDel = {
        url: '/api/interface/idel/:icode',
        access: ['global.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let code = ctx.f('params').icode
            console.log(code)
            this.obsEnd(obs,{success:false,err:'opopop'})
          })
        }
      }



      let apiRtDel = {
        url: '/api/interface/rdel/:rcode',
        access: ['global.root'],
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let code = ctx.f('params').rcode
            console.log('delr',code)
            if (!this._routers[code])
              this.obsEnd(obs,{success:false,error:'ROUTER_NOT_FOUND'})
            let $t = []
            Object.keys(this._ifaces).forEach(ifc=>{
              if (this._ifaces[ifc].f('router')==code)
              {
                $t.push(this._ifaces[ifc].stopIface())
                this._ifaces[ifc].field('router').clear()
                this._ifaces[ifc].f('enabled',false)
                $t.push(this._ifaces[ifc].save())
              }
            })
            $t.push(this._routers[code].uninstall())
            $t.push(this.C('ifrouters').removeNodes({code:code}))
            Rx.concat(...$t).pipe(RxO.last()).subscribe(_=>this.obsEnd(obs,{success:true}),err=>this.obsEnd(obs,{success:false,err:err}))
          })
        }
      }



      this.obsEnd(obs,[apiGet,apiSet,apiSetR,apiIfDel,apiRtDel])
    })
  }

  _registerClasses()
  {
      this.C('interfaces').setNodeClassFn(mdl=>{
        let type = mdl.type[0]
        if (this._ifaceClasses[type])
          return this._ifaceClasses[type]
        return AbstractInterface
      })
      this.C('ifrouters').setNodeClassFn(mdl=>{
        let type = mdl.type[0]
        //console.log(type,this._routerClasses)
        if (this._routerClasses[type])
          return this._routerClasses[type]
        return AbstractRouter
      })
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this._registerClasses()
      this.M('events').registerEventType({
        code: 'interface:online',
        name: g18n.t('Interface online'),
        data: [{
          type: 'string',
          code: 'iface',
          name: g18n.t('Interface code')
        }]
      })
      this.M('events').registerEventType({
        code: 'interface:offline',
        name: g18n.t('Interface offline'),
        data: [{
          type: 'string',
          code: 'iface',
          name: g18n.t('Interface code')
        }]
      })
      this._core.hookCollect('interfaceRegister',{}).subscribe(_xlist=>{
        this.C('interfaces').findNodes({}).subscribe(iflist=>{
          iflist.forEach(iff=>{
            this._ifcodes.push(iff.f('code'))
            this._ifaces[iff.f('code')] = iff
  
            iff.statusChange.subscribe(st=>{
              log.info('interface status',[iff.f('code'),st])
            })
            iff.runChange.subscribe(st=>{
              log.info('interface run',[iff.f('code'),st])
            })
          })
          this.C('ifrouters').findNodes().pipe(RxO.mergeMap(list=>{
            list.forEach(r=>{
              this._routers[r.f('code')] = r
            })
            let $l = list.map(x=>x.checkInstall())
            return Rx.forkJoin(...$l)
            //return Rx.of(true)
          })).subscribe(_=>{
            this.obsEnd(obs)
          },err=>this.obsErr(obs,err))
        },err=>{
          log.err('can not load interfaces!!')
          console.log(err)
          this.obsErr(obs,err)
        })
      })
    })
  }

  moduleStart()
  {
    return Rx.Observable.create(obs=>{
      let $t = []
      Object.keys(this._ifaces).forEach(ifcode=>{
        //console.log(ifcode)
        if (!this._ifaces[ifcode].f('enabled'))
          return
        //console.log(ifcode)
        log.info('starting interface',ifcode)
        $t.push(this._ifaces[ifcode].startIface(true).pipe(
          RxO.catchError(err=>{
            log.err('ifInit failed on interface',ifcode)
            console.log(err)
            this._ifaces[ifcode].f('status','fail')
            //this._ifaces[ifcode].f('enabled',false)
            return this._ifaces[ifcode].save()
          })
        ))
      })
      if (!$t.length)
        return this.obsEnd(obs)
      Rx.concat(...$t).subscribe(_=>{},err=>this.obsErr(obs,err),_=>this.obsEnd(obs))
    })
  }

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bIface = this._core.module('bundle').bundle('interfaces',true)
      bIface.name(g18n.t('Interfaces'))
      bIface.desc(g18n.t('System interfaces'))
      bIface.addFlag('system')
      bIface.addFlag('locked')

      bIface.addField({name: g18n.t('Enabled'), code: 'enabled', type: 'boolean', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Status'), code: 'status', type: 'string', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Name'), code: 'name', type: 'string', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Code'), code: 'code', type: 'string', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Type'), code: 'type', type: 'string', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Router'), code: 'router', type: 'string', flags: ['system','logic']})
      bIface.addField({name: g18n.t('Flags'), code: 'flags', type: 'string', flags: ['system','logic'],quantity:0})
      bIface.addField({name: g18n.t('Config'), code: 'config', type: 'any', flags: ['system','logic']})
      //bIface.addField({name: g18n.t('Routers'), code: 'routers', type:'relation', conf:{ops:[{source:'manual',manual:'ifrouters'}]}, flags:['system','logic'],quantity:0})

      log.info('saving bundle',bIface.code())
      $t.push(bIface.save(true,true))

      let bRouters = this._core.module('bundle').bundle('ifrouters',true)
      bRouters.name(g18n.t('Routers'))
      bRouters.desc(g18n.t('System interfaces routers'))
      bRouters.addFlag('system')
      bRouters.addFlag('locked')

      //bRouters.addField({name: g18n.t('Enabled'), code: 'enabled', type: 'boolean', flags: ['system','logic']})
      bRouters.addField({name: g18n.t('Name'), code: 'name', type: 'string', flags: ['system','logic']})
      bRouters.addField({name: g18n.t('Code'), code: 'code', type: 'string', flags: ['system','logic']})
      bRouters.addField({name: g18n.t('Type'), code: 'type', type: 'string', flags: ['system','logic']})
      bRouters.addField({name: g18n.t('Flags'), code: 'flags', type: 'string', flags: ['system','logic'],quantity:0})
      bRouters.addField({name: g18n.t('Config'), code: 'config', type: 'any', flags: ['system','logic']})

      log.info('saving bundle',bRouters.code())
      $t.push(bRouters.save(true,true))


      //$t.push(this.moduleDefaults())



      Rx.concat(...$t).subscribe(()=>{},err=>{
        this.obsErr({code:'INSTALL_INTERFACES_BUNDLE_ERROR',error:err})
      },()=>{
        log.success('installed')
        this.obsEnd(obs)
      })
    })
  }

      /*
  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      log.info('creating default routers')
      //default telegram bot router
      let $br = this.C('ifrouters').newNode()
      $br.f('code','tgdefault')
      $br.f('name','Telegram bot')
      $br.f('type','telegrambotrouter')
      $br.field('flags').push('system')
      $t.push($br.save())

      //default dashboard http router
      let $dr = this.C('ifrouters').newNode()
      $dr.f('code','dashboard')
      $dr.f('name','Dashboard http')
      $dr.f('type','httpthemerouter')
      $dr.f('config',{theme:'blocks'})
      $dr.field('flags').push('system')
      $t.push($dr.save())

      //default website http router
      let $wr = this.C('ifrouters').newNode()
      $wr.f('code','website')
      $wr.f('name','Website http')
      $wr.f('type','httpthemerouter')
      $wr.f('config',{theme:'blocks'})
      $wr.field('flags').push('system')
      $t.push($wr.save())

      Rx.concat(...$t).pipe(
        RxO.last(),
        RxO.map(_=>{
          this._registerClasses()
        }),
        RxO.mergeMap(_=>this.C('ifrouters').findNodes()),
        RxO.mergeMap(list=>{
          let $l = list.map(x=>x.install())
          return Rx.concat(...$l).pipe(RxO.last())
        })
      ).subscribe(_=>this.obsEnd(obs),err=>this.obsErr(obs,err))
    })
  }
      /**/

  /**
  loadAndStartIFace(code)
  {
    return Rx.of(this._ifaces[code]).pipe(
      RxO.mergeMap(iface=>{
        if (!iface||!(iface instanceof AbstractInterface))
          return Rx.throwError({code:'INTERFACE_IS_NOT_INTERFACE0_0'})
        
      })
    )
  }
  /**/

  iface(code)
  {
    if (this._ifaces[code])
      return this._ifaces[code]
    return null
  }

  ifRouter(code)
  {
    if (this._routers[code])
      return this._routers[code]
    return null
  }

  nodeAfterSave(node)
  {
    if (node.bundle()!='ifrouters')
      return Rx.of(node)
    this._routers[node.f('code')] = node
    return Rx.of(node)
  }



}

module.exports = InterfacesModule
