/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('ContextFieldSelector')
let g18n = require('../../../g18n')

const ContextSelector = require('./context_selector')
const VirtualNode = require('../../node/vnode')

class ContextFieldSelector extends ContextSelector
{

  setRange()
  {
    if (this._modifier)
      return this._modifier.setRange()
    let q = this.quantity()
    if (q==1)
      return [0]
    if (q==0)
      return [-1]
    let ret = []
    for (let i=0;i<q;i++)
      ret.push(i)
    return ret
  }

  data()
  {
    return Rx.Observable.create(obs=>{
      this._dataObs = obs
      if (this._modifier)
        this._modifier.reset()
      this._nextData()
    })
  }

  _nextData()
  {
    if (!this._dataObs)
      return
    if (this._modifier)
    {
      let nx = this._modifier.getNext()
      this._dataObs.next(nx)
      return
    }
    let q = this._target.quantity
    if (q==1)
      this._dataObs.next(this.oneModified())
    else
      this._dataObs.next(this.allModified())
  }

  one(i=0,def)
  {
    if (!this._target)
      return def
    //console.log(['contextFieldSelector.one',this._target])
    let ret = this._target.get(i)
    //console.log(['ONE',this._conf.target,i,def,'=>',ret])
    if (typeof ret != 'undefined')
      return ret
    return def
  }

  oneModified(i=0,def)
  {
    if (!this._target)
      return def
    if (this._modifier)
    {
      let ret = def
      ret = this._modifier.getNext()
      //console.log(['abs/ctx_fld.js oneModified with _modifier',this._conf.target,i,ret])
      return ret
    }
    return this.one(i,def)
  }

  allModified()
  {
    let ret = []
    //console.log(['allModified',this.size()])
    for(let i=0;i<this.size();i++)
      ret.push(this.oneModified(i))
    return ret
  }

  setModified(val,i=0)
  {
    if (val<0)
      return
    if (!this._target)
    {
      log.err('field selector has no _target at setModified!')
      return
    }
    if (this._modifier)
    {
      let mtgt = this._modifier.modifyTarget(this._target)
      val = this._modifier.modifyOne(val)
      mtgt.set(val,i)
    } else {
      this._target.set(val,i)
    }
  }

  saveModified()
  {
    return Rx.Observable.create(obs=>{
      let tgt = this._target
      if (this._modifier)
        tgt = this._modifier.modifyTarget(this._target)
      tgt.save().subscribe(()=>{
        this.obsEnd(obs)
      },err=>{
        log.err('save target (modified?) error')
        console.log(err)
        this.obsErr(obs,err)
      })
    })
  }

  size()
  {
    if (!this._target)
      return 0
    return this._target.size()
  }

  quantity()
  {
    if (!this._target)
      return 1
    return this._target.quantity
  }

  clear(pos)
  {
    if (typeof pos == 'number'&&pos>=0&&pos<this.size())
      this._target.clearAt(pos)
    else
      this._target.clear()
  }

  //fns = WTF???!!! CALLBACK??? WHAT??!!!! OMG!!!!
  //fucking dirty "return before return"
  accumulate(lastRes,fns)
  {
    if (!this._modifier) //no accumulation
    {
      fns(lastRes)
      this._dataObs.complete()
      this._dataObs = null
      return
    }
    let acc = this._modifier.accumulate(lastRes)
    fns(acc.res)
    if (acc.stop)
    {
      if (this._dataObs)
      {
        this._dataObs.complete()
        this._dataObs = null
      }
      return
    }
    this._nextData()
  }

  loadTarget(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      this._obsData = null
      let targetIsVirtual = false
      //console.log(['loadingFieldSelectorTarget',this._conf])
      //console.log(['loadingFieldSelectorTarget',this._conf,ctx.view()])
      //MANUAL
      if (this._conf.manual)
      {
        targetIsVirtual = true
        let tp = 'any'
        if (!this._info)
        {
          log.log('manual without config = context virtual',this._conf.target)
          this._targetNode = ctx
          this._targetCode = this._conf.target
        } else {
          log.log('manual with config = virtual node with single field',this._conf.target)
          this._targetNode = new VirtualNode(this)
          this._targetCode = 'manaulData'
          tp = this._info.type
        }
        let f = this._targetNode.field(this._targetCode)
        if (!f)
        {
          let q = tp.startsWith('[')&&tp.endsWith(']')?0:1
          this._targetNode.addField({
            type: tp,
            code: this._targetCode,
            name: this._targetCode,
            quantity: q,
            flags: ['virtual']
          })
        }
        this._target = this._targetNode.field(this._targetCode)
        //set manual value
        if (this._info)
          this._target.set(this._conf.target)
        //console.log([this._targetNode._model,this._target.code,'manual loaded'])
        //console.log(['LOADING MANUAL FOR FIELD SELECTOR',this._conf,this._info])
        return this.obsEnd(obs)
      }

      //GENERATOR
      if (!this._conf.manual&&this._conf.target.startsWith('generator.'))
      {
        targetIsVirtual = true
        let genCode = this._conf.target.split('.').pop()
        let GenCls = this.M('field').classGenerator(genCode)
        //console.log(['LOADING GENERATOR FOR FIELD SELECTOR',this._conf,this._info])
        if (!GenCls)
          return this.obsErr(obs,{code:'GENERATOR_NOT_FOUND!'})
        //console.log(['LOADING GENERATOR FOR FIELD SELECTOR',genCode,GenCls])
        this._targetNode = new VirtualNode(this)
        this._targetCode = 'generator'
        let tp = this._info.type
        this._targetNode.addField({
          type: tp,
          code: this._targetCode,
          name: this._targetCode,
          quantity: 1,
          flags: ['virtual','readonly']
        })
        this._target = this._targetNode.field(this._targetCode)
        let gen = new GenCls(this._target,{ops:this._conf.tops})
        gen.generate(ctx).subscribe(genv=>{
          //console.log(['VALUE GENERATOR FOR FIELD SELECTOR',genv])
          this._target.set(genv)
          this.obsEnd(obs,true)
        },err=>{
          console.log(err)
          this.obsErr(obs,{code:'GENERATOR_GENERATE_ERROR'})
        })
        return //this.obsErr(obs,{code:'GENERATOR_TODO!'})
      }

      //CONTEXTS
      if (!tctx)
      {
        //console.log(['conf',this._conf])
        ctx.nodeByPath(this._conf.target).subscribe(data=>{
          //console.log(['conf',this._conf,[data]])
          if (!data||!data.node||!data.fcode)
            return this.obsErr(obs,{code:'TARGET_NOT_FOUND'})
          this._targetNode = data.node
          this._targetCode = data.fcode
          this._target = this._targetNode.field(this._targetCode)
          log.log('field selector targeted',this._targetCode)
          this.obsEnd(obs,true)
        })
        return
      }

      tctx.nodeByPath(this._conf.target).subscribe(tdata=>{
        if (!tdata||!tdata.node||!tdata.fcode)
        {
          ctx.nodeByPath(this._conf.target).subscribe(rdata=>{
            if (!rdata||!rdata.node||!rdata.fcode)
              return this.obsErr(obs,{code:'TARGET_NOT_FOUND'})
            this._targetNode = rdata.node
            this._targetCode = rdata.fcode
            this._target = this._targetNode.field(this._targetCode)
            log.log('field selector targeted',this._targetCode)
            this.obsEnd(obs,true)
          })
          return// this.obsErr(obs,{code:'TARGET_NOT_FOUND'})
        }
        this._targetNode = tdata.node
        this._targetCode = tdata.fcode
        this._target = this._targetNode.field(this._targetCode)
        log.log('field selector targeted',this._targetCode)
        this.obsEnd(obs,true)
      })

    })
  }

  dataType(orig=false)
  {
    if (!this._target) return 'none'
    let dt = this._target.type
    if (this._target.quantity!=1)
      dt = '['+dt+']'
    if (orig)
      return dt
    if (this._modifier)
      return this._modifier.dataType()
    return dt
  }

  targetCode()
  {
    return this._targetCode
  }

  cop(i=0,def)
  {
    if (!this._target)
      return def
    let info = this._target.info
    if (!info)
      info = this._target._info
    //console.log(info)
    if (!info||!info.conf.ops||!info.conf.ops[i])
      return def
    let cop = info.conf.ops[i]
    if (cop.source == 'manual')
      return cop.manual
    return cop.source
  }

}

module.exports = ContextFieldSelector
