/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('ContextListSelector')
let g18n = require('../../../g18n')

const ContextSelector = require('./context_selector')
//const FieldSelector = require('./context_field')

class ContextListSelector extends ContextSelector
{

  setDataTarget(op)
  {
    this._dataTarget = op
  }

  loadTarget(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      if (this._conf.manual)
        return this.obsErr(obs,{code:'MANUAL_LIST!'})
      if (!this._dataTarget)
        return this.obsErr(obs,{code:'MO_DATA_TARGET'})

      let dt = this._dataTarget.dataType()
      //log.info('loading target for list selector',[this._conf,dt])
      //console.log('loading target for list selector',[this._conf,dt])
      let Cls = this.M('logic').classExecutor(dt,this._conf)
      if (!Cls)
        return this.obsErr(obs,{code:'EXECUTOR_NOT_FOUND'})

      this._target = new Cls(this,this._conf)
      this._target.dataTarget(this._dataTarget)

      this._target.init(ctx,tctx).subscribe(data=>{
        log.log('executor list targeted',this._conf.target)
        this.obsEnd(obs)
      },err=>{
        log.err('executor not initialized!')
        this.obsErr(obs,err)
      })
    })
  }

  loadModifier(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      log.log('list selector never has modifiers')
      this.obsEnd(obs)
    })
  }

  queryFor(ctx,vctx)
  {
    return this._target.queryFor(ctx,vctx)
  }

  execute(ctx,tctx=null)
  {
    return this._target.execute(ctx,tctx)
  }

}

module.exports = ContextListSelector
