/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('ContextSelector')
let g18n = require('../../../g18n')

const CoreObject = require('../../core/abstract/object')

class ContextSelector extends CoreObject
{

  constructor(parent,conf,info=null)
  {
    super(parent)
    this._conf = conf
    this._info = info
    this._target = null
    this._modifier = null

    this._ready = false
  }

  static info()
  {
    return {
      code: '',
      name: '',
      iname: '',
      ops: [],
      types: []
    }
  }

  ready(nx)
  {
    if (typeof nx=='boolean')
      this._ready = nx
    return this._ready
  }

  target()
  {
    return this._target()
  }

  loadTarget(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      log.warn('abstract context selector targeting!',this._conf.target)
      this.obsErr(obs,{code:'ABSTRACT_TARGETING'})
    })
  }

  loadModifier(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      //log.log('loading modifier')
      //console.log([this])
      if (!this._target||!this._target.type)
        return this.obsErr(obs,{code:'NO_TARGET_TYPE_TO_MODIFY'})

      let tp = this._target.type
      if(this._target.quantity!=1)
        tp = '['+tp+']'
      
      let Modifier = null
      if(this._conf.modifier!='')
        Modifier = this.M('field').classModifier(tp,this._conf.modifier)
      if (!Modifier)
        return this.obsEnd(obs)

      this._modifier = new Modifier(this,this._conf.mops)
      this._modifier.init(ctx,tctx).subscribe(()=>{
        this._modifier.reset()
        this.obsEnd(obs)
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

  init(ctx,tctx=null)
  {
    return Rx.Observable.create(obs=>{
      this.ready(false)
      Rx.concat(
        this.loadTarget(ctx,tctx),
        this.loadModifier(ctx,tctx)
      ).subscribe(()=>{},err=>{
        log.err('init error',err)
        this.obsErr(obs,err)
      },()=>{
        this.ready(true)
        this.obsEnd(obs,true)
      })
    })
  }



}

module.exports = ContextSelector
