/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../../log').m('AbstractExecutor')
let g18n = require('../../../g18n')

const CoreObject = require('../../core/abstract/object')

const FieldSelector = require('./context_field')
const ListSelector = require('./context_list')

class AbstractExecutor extends CoreObject
{

  constructor(parent,conf)
  {
    super(parent)
    this._conf = conf
    this._xtype = conf.xtype
    this._target = null
    this._dataTarget = null
    this._ops = []
  }

  xtype() { return this._xtype }
  dataTarget(dt)
  {
    if (typeof dt != 'undefined')
      this._dataTarget = dt
    return this._dataTarget
  }

  init(ctx)
  {
    return Rx.Observable.create(obs=>{
      if (!this.dataTarget())
        return this.obsErr(obs,{code:'NO_DATA_TARGET'})
      let inf = this.constructor.info()
      //console.log(['executor info',inf])
      this._ops = []
      let ok = true
      inf.ops.forEach((opInfo,opi)=>{
        if (typeof this._conf.tops[opi] =='undefined')
          return
        let opConf = this._conf.tops[opi]
        let op = ctx.fieldSelector(opConf,opInfo)
        if (!op)
        {
          ok = false
          return
        }
        this._ops[opi] = op
      })
      if (!ok)
        return this.obsErr(obs,{code:'NO_OPS_ERROR'})
      let $t = this._ops.map(x=>x.init(ctx))
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('ops init error',err)
        this.obsErr(obs,err)
      },()=>{
        this.obsEnd(obs)
      })

    })
  }

  queryFor(ctx,vctx)
  {
    return Rx.Observable.create(obs=>{
      let inf = this.constructor.info()
      log.warn('default queryFor returns null!',[inf.code,inf.types])
      this.obsEnd(obs,null)
    })
  }

  execute(ctx)
  {
    return Rx.Observable.create(obs=>{
      if (!this.dataTarget())
        return this.obsErr(obs,{code:'NO_DATA_TARGET_TO_EXECUTE'})
      let xtype = this._conf.xtype
      if (typeof this[xtype] != 'function')
        return this.obsErr(obs,{code:'NO_XTYPE_FN'})
      this[xtype](ctx,this.dataTarget()).subscribe(res=>this.obsEnd(obs,res),err=>this.obsErr(obs,err))
    })
  }

  op(i=0,def,all=false)
  {
    if (typeof this._ops[i] == 'undefined')
      return def
    let op = this._ops[i]
    if (!op||typeof op.oneModified != 'function')
      return def
    let rr = op.oneModified()
    if (all)
      rr = op.allModified()
    //console.log(['executor.js OP',this.constructor.info().code,i,rr])
    if (typeof rr != 'undefined')
      return rr
    return def
  }


  dcop(i=0,def)
  {
    //console.log(['dcop',!!this.dataTarget()])
    if (!this.dataTarget())
      return def
    return this.dataTarget().cop(i,def)
  }


}

module.exports = AbstractExecutor
