/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../../log').m('LogicOperation')
let g18n = require('../../../g18n')

const CoreObject = require('../../core/abstract/object')

const FieldSelector = require('./context_field')
const ListSelector = require('./context_list')

class LogicOperation extends CoreObject
{

  constructor(parent,op,conf)
  {
    super(parent)
    this._op = op
    this._conf = conf
    this._target = null
    this._operation = null
    this._inited = false
  }

  queryFor(coll,collAs,ctx,viewCtx)
  {
    return Rx.Observable.create(obs=>{
      this.init(ctx,viewCtx).subscribe(ires=>{
        if (!ires)
          return this.obsEnd(obs,null)
        this._operation.queryFor(ctx,viewCtx).subscribe(rq=>{
          this.obsEnd(obs,rq)
        },err=>{
          this.obsErr(obs,err)
        })
      },err=>{
        this.obsEnd(obs,null)
      })
      //this._operation = ctx.listSelector(this._conf[this._op])
      //if (!this._operation)
        //return this.obsErr(obs,{code:'OPERATION_LOAD_FAILS'})
      //this.
      //this.obsEnd(obs)
    })
  }

  initOnNodeField(node,field,ctx)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        this._target = node.field(field.code)
        this._operation = ctx.listSelector(this._conf[this._op])
        this._operation.setDataTarget(this._target)
        return this._operation.init(ctx)
      }),
      RxO.tap(_=>{
        this._inited = true
      })
    )
  }

  init(ctx,vctx=null,reinit=false)
  {
    return Rx.Observable.create(obs=>{
      if (this._inited&&!reinit)
        return this.obsEnd(obs,true)
      log.log('initializing logic operator',{x:this._op,on:this._conf.target.target,op:this._conf[this._op].target})
      if (!this._conf.target||!this._conf[this._op])
        return this.obsErr(obs,{code:'CONF_ERROR'})
      this._target = ctx.fieldSelector(this._conf.target)
      this._operation = ctx.listSelector(this._conf[this._op])
      if (!this._target||!this._operation)
        return this.obsErr(obs,{code:'SELECTORS_FAILS'})
      //console.log('field',data)
      Rx.concat(
        this._target.init(ctx,vctx).pipe(RxO.map(()=>{
          this._operation.setDataTarget(this._target)
        })),
        this._operation.init(ctx,vctx)
      ).subscribe(()=>{},err=>{
        log.err('init error',err)
        //console.log(err)
        this.obsEnd(obs,false)
      },()=>{
        //this.ready(true)
        this._inited = true
        this.obsEnd(obs,true)
      })
    })
  }

  execute(ctx)
  {
    return Rx.Observable.create(obs=>{
      //log.info('initializing',this._op)
      //console.log(this._conf)
      this.init(ctx).subscribe(initRes=>{
        log.log('initialized',[initRes])
        if (!initRes)
          return this.obsEnd(obs,false)

        this._operation.execute(ctx).subscribe(exRes=>{
          log.log('operation executed',[exRes])
          this.obsEnd(obs,exRes)
        },err=>{
          log.err('operation execute error',err)
          //console.log(err)
          this.obsErr(obs,err)
        })
      },err=>{
        log.err('init error',err)
        console.log(err)
        this.obsEnd(obs,false)
      })
    })
  }
}

module.exports = LogicOperation
