/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('LogicGroup')
let g18n = require('../../g18n')

const CoreObject = require('../core/abstract/object')
const LogicOperation = require('../logic/abstract/operation')

class LogicGroup extends CoreObject
{

  constructor(parent,ctx=null,grp=null,target=null)
  {
    super(parent)
    this._tctx = target
    this._g = []
    this._t = []
    this._tests = []
    this._groups = []
    this._fn = 'and'
    this._ctx = ctx
    if (grp)
      this._loadGroup(grp,ctx)
  }

  queryFor(coll,collAs)
  {
    return Rx.Observable.create(obs=>{
      let list = []
      let $t = this._groups.map(x=>x.queryFor(coll,collAs))
      this._tests.forEach(t=>{
        $t.push(t.queryFor(coll,collAs,this._ctx,this._tctx))
      })
      Rx.concat(...$t).subscribe(q=>{
        if (q)
          list.push(q)
      },err=>{
        this.obsErr(obs,err)
      },()=>{
        if (!list.length)
          return this.obsEnd(obs,null)
        if (list.length==1)
          return this.obsEnd(obs,list[0])
        if (this._fn == 'or')
          return this.obsEnd(obs,{$or:list})
        return this.obsEnd(obs,{$and:list})
      })
    })
  }

  _targetFieldByPath(path)
  {
    let fields = this._tctx.logicFields()
    console.log(fields)
    let paths = path.split('.')
    if (paths.length!=2)
    {
      log.err('logicFieldByPath error!',path)
      return null
    }
    return fields.reduce((acc,f)=>{
      return f.scope==paths[0]&&f.info.code==paths[1]?f:acc
    },null)
  }

  _loadGroup(grp,ctx)
  {
    //old
    if (grp.op)
      this._fn = grp.op
    if (grp.tests&&Array.isArray(grp.tests))
      this._t = grp.tests
    if (grp.groups&&Array.isArray(grp.groups))
      this._g = grp.groups
    this._tests = []
    this._t.forEach(t=>{
      this._tests.push(new LogicOperation(this,'test',t))
    })
    this._groups = []
    this._g.forEach(g=>{
      let grp = new LogicGroup(this._parent,this._ctx,g,this._tctx)
      this._groups.push(grp)
    })
  }

  isEmpty()
  {
    if (!this._tests.length&&!this._groups.length)
      return true
    return false
  }

  testRx(ctx)
  {
    return Rx.Observable.create(obs=>{
      let ret = true
      if (this._fn=='or')
        ret = false
      let done = new Rx.Subject()
      let $t = []
      this._tests.forEach(t=>$t.push(t.execute(ctx)))
      this._groups.forEach(t=>$t.push(t.testRx(ctx)))
      //log.test('testing',{tests:this._tests.length,groups:this._groups.length})
      Rx.concat(...$t).pipe(RxOps.takeUntil(done)).subscribe(oneRes=>{
        //log.warn('one more object tested',[this._fn,oneRes])
        if (oneRes&&this._fn=='or')
        {
          ret = true
          done.next(1)
        }
        if (!oneRes&&this._fn=='and')
        {
          ret = false
          done.next(1)
        }
      },err=>{
        log.err('testing failed')
        console.log(err)
        this.obsEnd(obs,false)
      },()=>{
        //log.info('all tests finished!',[ret])
        this.obsEnd(obs,ret)
      })
    })
  }

  test()
  {
    let ret = true
    if (this._fn=='or')
      ret = false
    let done = false
    this._tests.forEach(t=>{
      if (done) return
      let subtest = t._test(this._ctx)
      if (subtest&&this._fn=='or')
      {
        ret = true
        done = true
        return
      }
      if (!subtest&&this._fn=='and')
      {
        ret = false
        done = true
        return
      }
    })
    if (done)
      return ret
    this._groups.forEach(grp=>{
      if (done) return
      let subtest = grp.test()
      if (subtest&&this._fn=='or')
      {
        ret = true
        done = true
        return
      }
      if (!subtest&&this._fn=='and')
      {
        ret = false
        done = true
        return
      }
    })
   return ret
  }
}

module.exports = LogicGroup
