/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')

let log = require('../../log').m('Library')
let g18n = require('../../g18n')

const Context = require('../context/context')
const CoreObject = require('../core/abstract/object')

class Library extends CoreObject
{

  formatResult(view,res,ctx,viewCtx)
  {
    return Rx.Observable.create(obs=>{

      //console.log(view)
      let rndSort = Array.isArray(view.sorts)? view.sorts.reduce((acc,srt)=>srt.dir=='rnd'?true:acc,false):false
      let rxFrom = this.M('template').liquify(view.paging.skip,ctx.view()).pipe(
        RxO.map(v=>parseInt(v)||0)
      )
      let rxTo = this.M('template').liquify(view.paging.to,ctx.view()).pipe(
        RxO.map(v=>parseInt(v)||0)
      )
      Rx.forkJoin(rxFrom,rxTo).subscribe(cfgs=>{
          

        let from = cfgs[0]
        let to = cfgs[1]
        if (to) to += from

        //console.log(['exec library format result',viewCtx,res])
        let q = 0
        let tp = 'any'
        let xtps = []
        let cfg = undefined
  
        let ret = res.nodes
        if (rndSort)
        {
          ret = res.nodes.sort((a,b)=>0.49-Math.random())
          if (from||to)
          {
            if (!to)
              ret = ret.slice(from)
            else
              ret = ret.slice(from,to)
          }
        }
 
        let retData = []
        if (res.nodes.length==1)
          q = 1

        if (view.format.code=='nodes')
        {
          if(view.format.conf.flat)
            q = 1
          tp = 'relation'
          xtps = ['relation:'+view.target]
          cfg = {ops:[{source:'manual',manual:view.target}]},
          //retData = ret.map(x=>x.toObject())
          retData = ret
          ctx.addField({
            name: g18n.t('Library result'),
            code: view.result,
            type: tp,
            xtypes: xtps,
            conf: cfg,
            quantity: q,
            flags: ['virtual','logic','template']
          })
          //console.log(['view res',ret])
          if (retData.length)
          {
            if (q==1)
            {
              ctx.f(view.result,retData[0])
              if(view.format.conf.flat&&retData.length==1)
                ctx.addNode(retData[0],view.result)
            } else {
              ctx.field(view.result).all(retData)
            }
          }
          return this.obsEnd(obs)
        }
        if (view.format.code=='keyboard')
        {
          retData = ret
          let cols = view.format.conf.cols
          let tpl = view.format.conf.source
          //console.log(['retData',[retData]])
          let $compile = retData.map($nd=>{
            let bctx = ctx.cloneDepth()
            bctx.addNode($nd,view.targetCode)
            return this.M('template').liquify(tpl,bctx.view())
          })
          let compiled = []
          Rx.concat(...$compile).subscribe(btn=>compiled.push(btn),err=>{compiled.push(err)},()=>{
            //console.log(['compiled menu',compiled])
            let lst = compiled.filter(x=>!!x&&x.length)
            if (!lst.length)
              return this.obsEnd(obs)
            let finMenu = lst
            tp = 'string'
            q = 0
            if (lst.length>cols)
            {
              tp = 'any'
              q = 1
              finMenu = []
              let coli = 0
              let rowi = 0
              lst.forEach(btn=>{
                if (!Array.isArray(finMenu[rowi]))
                  finMenu[rowi] = []
                finMenu[rowi][coli] = btn
                coli++
                if (coli>=cols)
                {
                  rowi++
                  coli = 0
                }
              })
            }

            ctx.addField({
              name: g18n.t('Library keyboard result'),
              code: view.result,
              type: tp,
              xtypes: xtps,
              conf: cfg,
              quantity: q,
              flags: ['virtual','logic','template']
            })
            if (q==1)
              ctx.f(view.result,finMenu)
            else
              ctx.field(view.result).all(finMenu)

            this.obsEnd(obs)
          })
          return
        }

        if (view.format.code=='inline')
        {
          retData = ret
          let cols = view.format.conf.cols
          let tplc = view.format.conf.source
          let tpld = view.format.conf.data
          //console.log(['retData',[retData]])
          let $compile = retData.map($nd=>{
            let bctx = ctx.cloneDepth()
            bctx.addNode($nd,view.targetCode)
            return Rx.forkJoin(
              this.M('template').liquify(tplc,bctx.view()),
              this.M('template').liquify(tpld,bctx.view())
            ).pipe(RxO.map(x=>{
              let ret = {
                text: x[0]
              }
              if (x[1].startsWith('http://')||
                  x[1].startsWith('https://')||
                  x[1].startsWith('tg://'))
                ret.url = x[1]
              else
                ret.callback_data = x[1]
              return ret
            }))
          })
          let compiled = []
          Rx.concat(...$compile).subscribe(btn=>compiled.push(btn),err=>{compiled.push(err)},()=>{
            //console.log(['compiled menu',compiled])
            //let lst = compiled.filter(x=>!!x&&x.length)
            let lst = compiled
            if (!lst.length)
              return this.obsEnd(obs)
            let finMenu = lst
            tp = 'any'
            q = 1
            if (lst.length>cols)
            {
              finMenu = []
              let coli = 0
              let rowi = 0
              lst.forEach(btn=>{
                if (!Array.isArray(finMenu[rowi]))
                  finMenu[rowi] = []
                finMenu[rowi][coli] = btn
                coli++
                if (coli>=cols)
                {
                  rowi++
                  coli = 0
                }
              })
            }

            ctx.addField({
              name: g18n.t('Library keyboard result'),
              code: view.result,
              type: tp,
              xtypes: xtps,
              conf: cfg,
              quantity: 1,
              flags: ['virtual','logic','template']
            })
            ctx.f(view.result,finMenu)

            this.obsEnd(obs)
          })
          return
        }



        this.obsEnd(obs)
      })
    })
  }

  execute(ctx,data)
  {
    return Rx.Observable.create(obs=>{
      if (!data)
        return this.obsEnd(obs)
      //console.log('exec library',data)
      let from = parseInt(data.paging.skip)||0
      let to = parseInt(data.paging.limit)||0
      if (to) to += from

      let q = {}

      //let ctx2 = null
      let vctx = new Context(this,'view',true)
      //this.init(ctx,vctx).subscribe(()=>{
        let chain = []
        //only target now
        chain.push({
          coll: data.target,
          as: data.targetCode,
          dest: data.result,
          result: true,
          sorts: data.sorts,
          paging: data.paging,
          depth: data.relations.depth
        })
        let $chain = chain.map(chq=>{
          let coll = this.C(chq.coll)
          if (!coll)
            return Rx.of(null)
          let nd = coll.newNode()
          vctx.addNode(nd,chq.as)
          return this.M('logic').queryFor(coll,chq.as,data.filter,ctx,vctx).pipe(RxO.mergeMap(q=>{
            if (!q) q = {}
            let ops = {populate:0}
            if (chq.result)
            {
              if (chq.depth)
                ops.populate = chq.depth
              let rndSort = false
              if (chq.sorts)
              {
                let sortObj = {}
                chq.sorts.forEach(srt=>{
                  if (srt.dir=='down')
                    sortObj[srt.target] = -1
                  if (srt.dir=='up')
                    sortObj[srt.target] = 1
                  if (srt.dir=='rnd')
                    rndSort = true
                })
                if (!rndSort)
                ops.sort = sortObj
              }
              if (!rndSort)
              {
                let ffrom = 0
                if (chq.paging&&chq.paging.skip)
                  from = parseInt(chq.paging.skip)
                let fto = 0
                if (chq.paging&&chq.paging.limit)
                  fto = parseInt(chq.paging.limit)
                if (fto) fto += ffrom
                if (ffrom||fto)
                {
                  ops.skip = ffrom
                  ops.limit = fto
                }
              }
            }
            //console.log(['!!search nodes query'],q,ops)
            return coll.findNodes(q,ops).pipe(RxO.map(nodes=>{
              //console.log(['!!search nodes result'],nodes)
              return {
                chq: chq,
                nodes: nodes
              }
            }))
          }))
        })
        let ret = null
        Rx.concat(...$chain).subscribe(qret=>{
          if (!qret||!qret.chq)
          {
            log.err('chain queue returns something strange')
            console.log(qret)
            return
          }
          if (qret.chq.result)
          {
            ret = qret
            return
          }
          log.err('TODO. save to viewCtx')
        },err=>{
          log.err('error happens on query')
          console.log(err)
          this.obsErr(obs,{code:'QUERY_ERROR',error:err})
        },()=>{
          this.formatResult(data,ret,ctx,vctx).subscribe(()=>{
            log.info('library done')
            this.obsEnd(obs)
          },err=>{
            log.err('formatting result error')
            console.log(err)
            this.obsEnd(obs)
          })
        })
      //})//_init
    })
  }

}

module.exports = Library
