/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')

let log = require('../../log').m('LogicModule')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')

const LogicGroup = require('./group')
const Library = require('./library')
const AbstractTest = require('./abstract/test')

const Cmd = require('../telegram/cmd')

const VNode = require('../node/vnode')

const LogicFilterField = require('./logicfilterfield')

class LogicModule extends CoreModule
{
  constructor(parent)
  {
    super(parent)
    this._allTests = []
    this._tests = {
      '*': {}
    }
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.M('field').registerFieldType(LogicFilterField)
      this.obsEnd(obs)
    })
  }

  registerLogicTest(Cls,code)
  {
    if (typeof code != 'undefined') return
    this._allTests.push(Cls)
    let info = Cls.info()
    if (Array.isArray(info.types))
      info.types.forEach(tp=>{
        if (!this._tests[tp])
          this._tests[tp] = {}
        this._tests[tp][info.code] = Cls
        log.log('registered test',[tp,info.code])
      })
    if (!info.type)
      return
    if (!this._tests[info.type])
      this._tests[info.type] = {}
    this._tests[info.type][info.code] = Cls
    log.log('registered test',[info.type,info.code])
  }

  _testCls(scope,code)
  {
    let isArr = scope.startsWith('[')&&scope.endsWith(']')
    let cls = this._allTests.reduce((acc,t)=>{
      if (acc) return acc
      let inf = t.info()
      if (inf.code!=code) return acc
      if (inf.types.includes(scope)) return t
      if (!isArr&&inf.types.includes('*')) return t
      if (isArr&&inf.types.includes('[*]')) return t
      if (isArr&&inf.types.includes('[]')) return t
      return acc
    },null)
    //console.log(['_testCls',scope,code,cls])
    return cls
  }

  classTest(type,code)
  {
    return this._testCls(type,code)
  }

  modifierCls(code)
  {
    return this._testCls('[]',code)
  }

  generateTest(parent,field,test)
  {
    if (!field||!test)
      return null
    let scope = field.info.type
    let code = test.code
    let cls = this._testCls(scope,code)
    if (!cls)
      cls = this._testCls('*',code)
    if (!cls)
    {
      log.warn('test not found!',[scope,code])
      cls = AbstractTest
    }
    let ret = new cls(parent,field,test)
    return ret
  }

  classExecutor(dataType,conf)
  {
    if(conf.xtype=='test')
      return this.classTest(dataType,conf.target)
    if(conf.xtype=='setter')
      return this.M('field').classSetter(dataType,conf.target)
    return null
  }

  testLogicRx(obj,ctx,mapCode=null)
  {
    return Rx.Observable.create(obs=>{
      let logic = obj.f('logic')
      if (!logic)
        return this.obsEnd(obs,true)
      if (mapCode)
        logic = logic[mapCode]
      if (!logic)
        return this.obsEnd(obs,true)
      let mainGroup = new LogicGroup(obj,ctx,logic.conditions)
      if (mainGroup.isEmpty())
        return this.obsEnd(obs,true)
      mainGroup.testRx(ctx).subscribe(res=>{
        this.obsEnd(obs,res)
      },err=>{
        log.err('testing object logic error!')
        console.log(err)
        this.obsEnd(obs,false)
      })
    })
  }

  testLogicDirectRx(logic,ctx)
  {
    return Rx.Observable.create(obs=>{
      if (!logic)
        return this.obsEnd(obs,true)
      let mainGroup = new LogicGroup(ctx,ctx,logic.conditions)
      if (mainGroup.isEmpty())
        return this.obsEnd(obs,true)
      mainGroup.testRx(ctx).subscribe(res=>{
        this.obsEnd(obs,res)
      },err=>{
        log.err('testing object logic error!')
        console.log(err)
        this.obsEnd(obs,false)
      })
    })
  }

  library(ctx,data)
  {
    let lib = new Library(ctx)
    return lib.execute(ctx,data)
  }

  /*
  testViewFilterRx(node,filter,ctx,vctx=null)
  {
    return Rx.Observable.create(obs=>{
      let mainGroup = new LogicGroup(this,ctx,filter,node)
      if (mainGroup.isEmpty())
        return this.obsEnd(obs,true)
      mainGroup.testRx(ctx).subscribe(res=>{
        this.obsEnd(obs,res)
      },err=>{
        log.err('testing view filter error!')
        console.log(err)
        this.obsEnd(obs,false)
      })
    })
  }
  */

  queryFor(coll,collAs,filter,ctx,viewCtx)
  {
    return Rx.Observable.create(obs=>{
      let ret = {}
      let mainGroup = new LogicGroup(this,ctx,filter,viewCtx)
      mainGroup.queryFor(coll,collAs).subscribe(ret=>{
        this.obsEnd(obs,ret)
      },err=>{
        this.obsErr(obs,err)
      })
    })
  }

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtFieldsList = {
        url: '/api/logic/fields/list',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let vctx = this.M('context').newContext(this,'telegram')
            let vchat = this.C('tgchat').newNode()
            let vuser = this.C('user').newNode()
            let vmsg = this.C('tgmessage').newNode()
            let vscreen = this.C('screen').newNode()
            let vcmd = new Cmd(vctx,'/virtual virtual')
            vctx.users([vuser])
            vctx.chats([vchat])
            vctx.cmd(vcmd)
            vctx.message(vmsg)
            vctx.screen(vscreen)
            //reply message data
            vctx.addNode(vuser,'replyUser')
            vctx.addNode(vchat,'replyChat')
            vctx.addNode(vmsg,'replyMessage')
            
            let tgn = new VNode(vctx)
            tgn.addField({code:'senderUser',name:'Sender User',type:'relation',xtypes:['relation','relation:user'],conf:{ops:[{target:'manual',manual:'user'}]}})
            tgn.addField({code:'senderChat',name:'Sender User',type:'relation',xtypes:['relation','relation:tgchat'],conf:{ops:[{target:'manual',manual:'tgchat'}]}})
            tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',xtypes:['relation','relation:tgmessage'],conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

            tgn.addField({code:'replyUser',name:'Reply User',type:'relation',xtypes:['relation','relation:user'],conf:{ops:[{target:'manual',manual:'user'}]}})
            tgn.addField({code:'replyChat',name:'Reply User',type:'relation',xtypes:['relation','relation:tgchat'],conf:{ops:[{target:'manual',manual:'tgchat'}]}})
            tgn.addField({code:'replyMessage',name:'Reply Message',type:'relation',xtypes:['relation','relation:tgmessage'],conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

            //tgn.addField({code:'senderUser',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
            //tgn.addField({code:'senderChat',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
            //tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})
            vctx.addNode(tgn,'telegram')


            let photo = new VNode(vctx)
            photo.addField({name:g18n.t('TGID'),code:'tgid',type:'string'})
            photo.addField({name:g18n.t('Width'),code:'width',type:'number'})
            photo.addField({name:g18n.t('Height'),code:'height',type:'number'})
            photo.addField({name:g18n.t('Size (kb)'),code:'size',type:'number'})
            photo.addField({name:g18n.t('Caption'),code:'caption',type:'string'})
            photo.addField({name:g18n.t('Gallery ID'),code:'gid',type:'string'})
        
            vctx.addNode(photo,'photo')




            let fields = vctx.logicFields()
            this.obsEnd(obs,fields)
          })
        }
      }

      let rtFieldsType = {
        url: '/api/logic/fields/types',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let fields = this.M('field').fieldTypes()
            let lst = fields.map(x=>{
              x.xtype = 'type'
              return x
            })
            this.obsEnd(obs,lst)
          })
        }
      }

      let rtFieldsTest = {
        url: '/api/logic/fields/tests',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let fields = this._allTests.map(t=>{
              let x = t.info()
              x.xtype = 'test'
              return x
            })
            this.obsEnd(obs,fields)
          })
        }
      }

      this.obsEnd(obs,[rtFieldsList,rtFieldsType,rtFieldsTest])
    })
  }


}

module.exports = LogicModule
