/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxOps = require('rxjs/operators')

let log = require('../../log').m('NodeManager')
let g18n = require('../../g18n')

const CoreModule = require('../core/abstract/module')

//const Bundle = require('../bundle/bundle')
//const Collection = require('../bundle/collection')
const Node = require('./node')

const Context = require('../context/context')

class NodeManager extends CoreModule
{

  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtNodeNew = {
        url: '/api/node/new/:code',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let code = p.code || null
            if (!code)
              return this.obsEnd(obs,null)
            let bundle = this.M('bundle').bundle(code)
            if (!bundle)
              return this.obsEnd(obs,null)
            bundle.collection().initNode(ctx).subscribe(node=>{
              //console.log(node._model,node.toObject())
              let ret = node.toObject()
              delete ret.id
              this.obsEnd(obs,ret)
            },err=>{
              log.err('init node error')
              //console.log(err)
              this.obsEnd(obs,null)
            })
          })
        }
      }

      let rtNodeEdit = {
        url: '/api/node/edit/:code/:nid',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let code = p.code || null
            let nid = p.nid || null
            if (!code||!nid)
              return this.obsEnd(obs,null)
            let bundle = this.M('bundle').bundle(code)
            if (!bundle)
              return this.obsEnd(obs,null)
            bundle.collection().findId(nid).subscribe(node=>{
              //console.log(node.toObject())
              let ret = null
              if (node&&typeof(node.toObject)=='function')
                ret = node.toObject()
              this.obsEnd(obs,ret)
            },err=>{
              log.err('init node error')
              //console.log(err)
              this.obsEnd(obs,null)
            })
          })
        }
      }

      let rtNodeSave = {
        url: '/api/node/save/:bcode',
        access: 'auth.dashboard',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let b = ctx.f('params').bcode
            let bundle = this.M('bundle').bundle(b)
            let node = ctx.f('body')
            if (!node||!bundle)
              return this.obsEnd(obs,{success:false,errors:{},error:'No Node Found'})
            //console.log(['save node',bundle.code()],node)
            let $prep = bundle.collection().initNode()
            if (typeof node.id != 'undefined')
              $prep = bundle.collection().findId(node.id)
            $prep.subscribe($node=>{
              if (!$node)
              {
                log.err('prepare node NULL')
                console.log(err)
                return this.obsEnd(obs,{success:false,errors:{},error:'Node prepare NULL!'})
              }
              $node.bundleFields().forEach(f=>{
                let v = node[f.code]
                $node.field(f.code).clear()
                if (f.quantity!=1)
                {
                  if (Array.isArray(v))
                    v.forEach((v,vi)=>{
                      $node.field(f.code).set(v,vi)
                    })
                  else
                    $node.f(f.code,v)
                } else {
                  $node.f(f.code,v)
                }
              })
              $node.save().subscribe($saved=>{
                this.obsEnd(obs,{success:true,id:$saved.id()})
              },err=>{
                console.log(err)
                this.obsEnd(obs,{success:false,errors:{},error:'Not Valid!'})
              })
            },err=>{
              log.err('prepare node error')
              console.log(err)
              this.obsEnd(obs,{success:false,errors:{},error:'Node prepare failed!'})
            })
          })
        }
      }

      let rtNodeDel = {
        url: '/api/node/del/:code/:nid',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let code = p.code || null
            let nid = p.nid || null
            if (!code||!nid)
              return this.obsEnd(obs,null)
            let bundle = this.M('bundle').bundle(code)
            if (!bundle)
              return this.obsEnd(obs,null)
            bundle.collection().removeNodes({_id:nid}).subscribe(node=>{
              //console.log(node.toObject())
              //let ret = node.toObject()
              this.obsEnd(obs,{success:true})
            },err=>{
              log.err('delete node error')
              //console.log(err)
              this.obsEnd(obs,{success:false})
            })
          })
        }
      }

      let rtNodesList = {
        url: '/api/node/list/:bcode/:start?/:limit?',
        access: 'auth.dashboard',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let filters = ctx.f('body')
            let code = p.bcode || null
            let start = parseInt(p.start) || 0
            let limit = parseInt(p.limit) || 10
            let ret = {
              nodes: [],
              total: 0,
              pos: 0,
              amount: 0
            }
            if (!code)
              return this.obsEnd(obs,ret)
            let bundle = this.M('bundle').bundle(code)
            if (!bundle)
              return this.obsEnd(obs,ret)
            //console.log(['nodes search',code])
            let coll = bundle.collection()
            //console.log(['nodes search',filters])
            
            let cctx = new Context(this,'library',true)
            let vctx = new Context(this,'library',true)
            let ctxNode = coll.newNode()
            vctx.addNode(ctxNode,code)
            this.M('logic').queryFor(coll,code,filters,cctx,vctx).subscribe(q=>{
              //console.log(['nodes list Q',code,start,limit,q])
              //console.log(q)
              if (!q)
                q = {}
              coll.findNodes(q,{skip:start,limit:limit}).subscribe($nodes=>{
                bundle.collection().countNodes(q).subscribe(count=>{
                  ret.total = count
                  ret.pos = start
                  ret.amount = limit
                  ret.nodes = $nodes.map($x=>$x.toObject())
                  this.obsEnd(obs,ret)
                },err=>{
                  log.err('count nodes error')
                  console.log(err)
                  this.obsEnd(obs,ret)
                })
              },err=>{
                log.err('find node error')
                console.log(err)
                this.obsEnd(obs,ret)
              })
            },err=>{
              log.err('build query error')
              console.log(err)
              this.obsEnd(obs,ret)
            })
          })
        }
      }

      let rtGenField = {
        url: '/api/node/fgen/:bcode/:fcode',
        access: 'auth.dashboard',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let p = ctx.f('params')
            let bcode = p.bcode || null
            let fcode = p.fcode || null
            console.log(p)
            if (!bcode||!fcode)
              return this.obsEnd(obs,{success:false,error:'Something wrong'})
            let bundle = this.M('bundle').bundle(bcode)
            if (!bundle)
              return this.obsEnd(obs,{success:false,error:'Bundle wrong'})
            if (!bundle.hasField(fcode))
              return this.obsEnd(obs,{success:false,error:'Field wrong'})
            let ret = bundle.fields()
            log.info('regenerating field defaults',{bundle:bcode,field:fcode})
            bundle.collection().regenerateFields([fcode]).subscribe(rres=>{
              log.success('collection updated')
              this.obsEnd(obs,{success:true})
            },err=>{
              console.log(err)
              log.err('failed')
              this.obsEnd(obs,{success:false,error:'Regenerate failed'})
            })
          })
        }
      }

      this.obsEnd(obs,[
        rtNodeNew,
        rtNodeEdit,
        rtNodeSave,
        rtNodeDel,
        rtNodesList,
        rtGenField
      ])
    })
  }

  /*
  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      //nodes bundle
      let nodes = this._core.module('bundle').bundle('node',true)
      nodes.name(g18n.t('Nodes'))
      nodes.desc(g18n.t('System nodes'))
      nodes.addFlag('system')
      //nodes.addFlag('locked')

      nodes.addField({
        name: g18n.t('Node ID'),
        desc: g18n.t('Node ID'),
        code: 'nid',
        type: 'relation',
        quantity: 1,
        flags: ['system','locked']
      })

      nodes.addField({
        name: g18n.t('Bundle'),
        desc: g18n.t('Node bundle'),
        code: 'bundle',
        type: 'string',
        quantity: 1,
        flags: ['system','locked']
      })

      nodes.addField({
        name: g18n.t('Route'),
        desc: g18n.t('Node route string'),
        code: 'route',
        type: 'string',
        quantity: 1,
        flags: ['system','locked']
      })

      nodes.addField({
        name: g18n.t('Is Key'),
        desc: g18n.t('Node route is key'),
        code: 'iskey',
        type: 'boolean',
        quantity: 1,
        def: false,
        flags: ['system','locked']
      })

      log.info('saving bundle',nodes.code())
      $t.push(nodes.save(true,true))

      
      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing config bundle failed')
        console.log(err)
        this.obsErr(obs,err)
      },()=>{
        log.success('installed')
        this.obsEnd(obs)
      })
    })
  }
  */


}

module.exports = NodeManager
