/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let sanitizeHtml = require('sanitize-html')

let log = require('../../log').m('TelegramBot')
let g18n = require('../../g18n')

const AbstractInterface = require('../interface/abstract')
const VNode = require('../node/vnode')

const TelegramBot = require('./bot')

class TelegramBotInterface extends AbstractInterface
{

  constructor(parent,model,isNew=false)
  {
    super(parent,model,isNew)
    this._type = 'iface'

    this._bot = null
    this._router = null
  }


  static ifInfo()
  {
    return {
      code: 'telegrambot',
      name: g18n.t('Telegram Bot'),
      desc: g18n.t('Classic telegram bot'),
      scope: 'telegram',
      directions: 2,
      confs: [
        { code: 'username', name: g18n.t('Bot username'), desc: g18n.t('Telegram bot username'), type: 'string' },
        { code: 'token', name: g18n.t('API Token'), desc: g18n.t('Telegram bot API Token from @BotFather'), type: 'string' },
        { code: 'isWebhook', name: g18n.t('Web Hook'), desc: g18n.t('OFF (default) = Long polling, ON = webhook '), type: 'boolean' },
        { code: 'webhookDomain', name: g18n.t('Webhook domain'), desc: g18n.t('Domain for webhook mode. Telegram servers will POST to https://domain/telegraf'), type: 'string' },
        { code: 'webhookPort', name: g18n.t('Webhook port'), desc: g18n.t('Port for webhook mode. Local webhook server will be listnening here. Use nginx/other to proxy'), type: 'number' }
      ]
    }
  }


  ifContext(demo=false)
  {
    let ctx = this.M('context').newContext(this,'telegram')

    let tgn = new VNode(ctx)

    tgn.addField({name:g18n.t('Interface code'),code:'code',type:'string'})
    tgn.addField({name:g18n.t('Interface type'),code:'type',type:'string'})

    tgn.addField({name:g18n.t('User'),code:'user',type:'relation',xtype:'relation:user'})
    //tgn.addField({code:'files',type:'any'})
    tgn.addField({code:'senderUser',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
    tgn.addField({code:'senderChat',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
    tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

    tgn.addField({code:'replyUser',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
    tgn.addField({code:'replyChat',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
    tgn.addField({code:'replyMessage',name:'Reply Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

    tgn.addField({code:'forwardUser',name:'Forward User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
    tgn.addField({code:'forwardChat',name:'Forward User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
    tgn.addField({code:'forwardMessage',name:'Forward Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

    tgn.f('code',this.f('code'))
    tgn.f('type',this.f('type'))

    ctx.addNode(tgn,'telegram')
    ctx.addNode(this,'interface')

    let photo = new VNode(ctx)
    photo.addField({name:g18n.t('TGID'),code:'tgid',type:'string'})
    photo.addField({name:g18n.t('Width'),code:'width',type:'number'})
    photo.addField({name:g18n.t('Height'),code:'height',type:'number'})
    photo.addField({name:g18n.t('Size (kb)'),code:'size',type:'number'})
    photo.addField({name:g18n.t('Caption'),code:'caption',type:'string'})
    photo.addField({name:g18n.t('Gallery ID'),code:'gid',type:'string'})

    ctx.addNode(photo,'photo')

    if (!demo)
      return ctx

    let u = this.C('user').newNode()
    ctx.users([u])

    //let route = this._rc().newNode()

    return ctx
  }

  ifInit(start=false)
  {
    return Rx.of(this.f('config')).pipe(
      RxO.map(cfg=>{
        //console.log(cfg)
        /**/
        this.ifStatus('initializing')
        this._bot = new TelegramBot(this)
        this._router = this.M('interface').ifRouter(this.f('router'))

        this._bot.configure(cfg)
        this._bot.init()
        //user-chat-type middleware

        /**
        this._bot.use((ctx,next)=>{
          setTimeout(_=>{
            
            let upd = ctx.update[ctx.updateType]
            //console.log(ctx.update)
            log.info('>>telegram mw')
            let updSubTypes = ctx.updateSubTypes

            let srcUser = upd.from
            let srcChat = upd.chat
            ctx._type = null
            if (ctx.updateType=='callback_query')
            {
              //console.log(upd)
              ctx._type = 'callback_query'
              srcChat = upd.message.chat
            }

            ctx._users = []
            ctx._ucs = {}
            ctx._chat = null
            let $usersLoad = []
            if (ctx.updateType!='channel_post')
              $usersLoad.push(this.tgUser(srcUser))

            //update type
            if (upd.text&&upd.text.length)
              ctx._type = 'text'
            if (upd.photo&&upd.photo.length)
              ctx._type = 'photo'

            if (!ctx._type&&upd.left_chat_member)
            {
              $usersLoad.push(this.tgUser(upd.left_chat_member))
              if (upd.left_chat_member.is_bot&&upd.left_chat_member.username==this._bot._username) {
                ctx._type = 'leave_group_chat'
              } else {
                if (srcUser.id==upd.left_chat_member.id) ctx._type = 'group_chat_member_leave'
                else ctx._type = 'group_chat_member_kicked'
              }
              $usersLoad = $usersLoad.reverse()
            }

            if (!ctx._type&&upd.new_chat_member)
            {
              $usersLoad.push(this.tgUser(upd.new_chat_member))
              if (upd.new_chat_member.is_bot&&upd.new_chat_member.username==this._bot._username) {
                ctx._type = 'enter_group_chat'
              } else {
                if (srcUser.id==upd.new_chat_member.id) ctx._type = 'group_chat_member_enter'
                else ctx._type = 'group_chat_member_added'
              }
              $usersLoad = $usersLoad.reverse()
            }

            if (upd.reply_to_message)
            {
              this.tgUser(upd.reply_to_message.from)
            }


          },0)
        })
        /**/
        this._bot.use((ctx,next)=>{
          setTimeout(()=>{
            let upd = ctx.update[ctx.updateType]
            //console.log(ctx.update)
            log.info('>>telegram mw')
            let updSubTypes = ctx.updateSubTypes

            let srcUser = upd.from
            let srcChat = upd.chat
            ctx._type = null
            if (ctx.updateType=='callback_query')
            {
              //console.log(upd)
              ctx._type = 'callback_query'
              srcChat = upd.message.chat
            }

            ctx._users = []
            ctx._chat = null
            let $t = []
            if (ctx.updateType!='channel_post')
              $t.push(this.tgUser(srcUser))

            //update type
            if (upd.text&&upd.text.length)
              ctx._type = 'text'
            if (upd.photo&&upd.photo.length)
              ctx._type = 'photo'

            if (!ctx._type&&upd.left_chat_member)
            {
              $t.push(this.tgUser(upd.left_chat_member))
              if (upd.left_chat_member.is_bot&&upd.left_chat_member.username==this._bot._username) {
                ctx._type = 'leave_group_chat'
              } else {
                if (srcUser.id==upd.left_chat_member.id) ctx._type = 'group_chat_member_leave'
                else ctx._type = 'group_chat_member_kicked'
              }
              $t = $t.reverse()
            }

            if (!ctx._type&&upd.new_chat_member)
            {
              $t.push(this.tgUser(upd.new_chat_member))
              if (upd.new_chat_member.is_bot&&upd.new_chat_member.username==this._bot._username) {
                ctx._type = 'enter_group_chat'
              } else {
                if (srcUser.id==upd.new_chat_member.id) ctx._type = 'group_chat_member_enter'
                else ctx._type = 'group_chat_member_added'
              }
              $t = $t.reverse()
            }

            Rx.concat(...$t).subscribe(u=>ctx._users.push(u),err=>{
              log.err('ctx users fail')
              console.log(err)
            },()=>{
              //console.log([ctx.updateType,upd])
              if (ctx.updateType=='inline_query')
              {
                next()
                return 
              }
              this.tgChat(srcChat).subscribe(chat=>{
                ctx._chat = chat
                if (ctx._type=='leave_group_chat')
                  ctx._chat.field('open').set(false)
                if (ctx._type=='enter_group_chat')
                {
                  ctx._chat.field('open').set(true)
                  ctx._chat.field('counter').set(0)
                }
                //console.log(chat.toObject())
                let nx = chat.save()
                if (ctx.updateType!='channel_post')
                  nx = this.addChat2Users(ctx._chat,ctx._users)
                nx.subscribe(()=>{
                  this.tgMessage(upd,ctx,ctx._type=='callback_query'?'callback_query':'default').subscribe(msg=>{
                    ctx._msg = msg
                    //reply_to_message
                    if (!upd.reply_to_message)
                    {
                      next()
                      return
                    }
                    let rmsg = upd.reply_to_message
                    Rx.forkJoin(
                      this.tgUser(rmsg.from),
                      this.tgChat(rmsg.chat)
                      //this.tgMessage(rmsg,ctx,'reply')
                    ).subscribe(cmpls=>{
                      ctx._replyUser = cmpls[0]
                      ctx._replyChat = cmpls[1]
                      //ctx._replyMessage = cmpls[2]
                      this.tgMessage(rmsg,ctx,'reply').subscribe($rmsg=>{
                        ctx._replyMessage = $rmsg
                        next()
                      },err=>{
                        log.err('extract reply message error!')
                        console.log(err)
                        next()
                      })
                    },err=>{
                      log.err('extract reply error!')
                      console.log(err)
                      next()
                    })
                  },err=>{ //tgMessage
                    log.err('ctx message fail')
                    console.log(err)
                  })
                },err=>{ //addChat2Users
                  log.err('ctx addChat2Users fail')
                  console.log(err)
                }) 
              },err=>{ //tgChat
                log.err('ctx chat fail')
                console.log(err)
              })
            })
          },10)
        })
        /**/
        //events connect/disconnect
        this._bot.signals.connected.subscribe(state=>{
          log.info('bot connection state',[state])
          if (!state)
            setTimeout(()=>{
              this._bot.start()
            },1000*7) //autorestart in 7 sec. why 7? why not? =)
        })
        //events incomes
        this._bot.on('message',(ctx)=>{
          setTimeout(()=>{
            let txt = ''
            let tpok = false
            let CTX = this.ifContext()
            if (ctx._type == 'text')
            {
              tpok = true
              if (ctx.update&&ctx.update.message&&ctx.update.message.text)
                txt = ctx.update.message.text
              log.info('>>text message',txt)
            }
            if (ctx._type == 'photo')
            {
              tpok = true
              log.info('>>photo message',ctx.update.message.photo.length)
              let phn = CTX.vnode('photo')
              let phi = ctx.update.message.photo.pop()
              phn.f('tgid',phi.file_id)
              phn.f('width',phi.width)
              phn.f('height',phi.height)
              phn.f('size',phi.file_size)
              if (ctx.update.message.caption)
                phn.f('caption',ctx.update.message.caption)
              if (ctx.update.message.media_group_id)
                phn.f('gid',ctx.update.message.media_group_id)
              //console.log(phn.toObject())
            }
            if (!tpok)
            {
              log.err('unsupported type?')
              console.log(ctx.update)
            }
            //let CTX = this.M('context').newContext(this,'telegram')
            //console.log(['!!!',CTX.view()])
            CTX.session(ctx.session)
            //console.log(['!!!',CTX.view()])
            CTX.type(ctx._type)
            //console.log(['!!!',CTX.view()])
            CTX.users(ctx._users)
            //console.log(['!!!',CTX.view()])
            CTX.chats([ctx._chat])
            CTX.message(ctx._msg)
            CTX.raw(ctx.update)
            CTX.upd(ctx.update.message)
            if (ctx._replyUser)
              CTX.addNode(ctx._replyUser,'replyUser')
            if (ctx._replyChat)
              CTX.addNode(ctx._replyChat,'replyChat')
            if (ctx._replyMessage)
              CTX.addNode(ctx._replyMessage,'replyMessage')

            //log.warn('!!!',CTX.view())

            //let tgn = new VNode(CTX)
            let tgn = CTX.vnode('telegram')
            //tgn.addField({code:'senderUser',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
            //tgn.addField({code:'senderChat',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
            //tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})
  
            //tgn.addField({code:'replyUser',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
            //tgn.addField({code:'replyChat',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
            //tgn.addField({code:'replyMessage',name:'Reply Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})
  
            tgn.fp('senderUser',ctx._users[0])
            tgn.fp('senderChat',ctx._chat)
            tgn.fp('senderMessage',ctx._msg)
            if (ctx._replyUser)
              tgn.fp('replyUser',ctx._replyUser)
            if (ctx._replyChat)
              tgn.fp('replyChat',ctx._replyChat)
            if (ctx._replyMessage)
              tgn.fp('replyMessage',ctx._replyMessage)
            //CTX.addNode(tgn,'telegram')

            this._router.onMessage(CTX)
          },10)
        })

        this._bot.on('callback_query',(ctx)=>{
          setTimeout(()=>{
            let q = null
            if (ctx&&ctx.update&&ctx.update.callback_query)
              q = ctx.update.callback_query.data
            log.info('>>callback_query',q)
            let CTX = this.ifContext()
            //dirty hack
            CTX._rawctx = ctx
            CTX.session(ctx.session)
            CTX.type(ctx._type)
            CTX.users(ctx._users)
            CTX.chats([ctx._chat])
            CTX.message(ctx._msg)
            CTX.raw(ctx.update)
            CTX.upd(ctx.update.callback_query)

            let tgn = CTX.vnode('telegram')
  
            tgn.fp('senderUser',ctx._users[0])
            tgn.fp('senderChat',ctx._chat)
            tgn.fp('senderMessage',ctx._msg)

            //CTX.addNode(tgn,'telegram')
            this._router.onCallbackQuery(CTX)
          },10)
        })

        this._bot.on('inline_query',(ctx)=>{
          setTimeout(()=>{
            let q = null
            if (ctx&&ctx.update&&ctx.update.inline_query)
              q = ctx.update.inline_query.query
            log.info('>>inline_query',q)
            let CTX = this.ifContext()
            //dirty hack
            CTX._rawctx = ctx
            CTX.session(ctx.session)
            CTX.type('inline_query')
            CTX.users(ctx._users)
            //CTX.chats([ctx._chat])
            //CTX.message(ctx._msg)
            CTX.raw(ctx.update)
            CTX.upd(ctx.update.inline_query)

            let tgn = CTX.vnode('telegram')
  
            tgn.fp('senderUser',ctx._users[0])
            //tgn.fp('senderChat',ctx._chat)
            //tgn.fp('senderMessage',ctx._msg)

            //CTX.addNode(tgn,'telegram')
            this._router.onInlineQuery(CTX)
          },10)
        })

        //this._bot.on('photo',(ctx)=>{
          //console.log(['ON PHOTO!!!',ctx])
        //})


        //router
        this._router.init()
        log.log('initialized')
 
        this.M('events').onEvent.subscribe(e=>{
          //react on events only if runing
          if (!this._run)
            return
          let CTX = this.ifContext()
          CTX.type('event')
          let evn = new VNode(CTX)
          let evt = this.M('events').eventType(e.type)
          evt.data.forEach(f=>{
            evn.addField(f)
            evn.f(f.code,e.data[f.code])
          })
          CTX.addNode(evn,'event')
          this._router.onEvent(e,CTX)
        })

        return true
      }),
      RxO.mergeMap(initRes=>Rx.iif(_=>initRes&&start,this.ifStart(),Rx.of(true)))
    )
  }

  ifAttach(iface)
  {
    return Rx.throwError({code:'HTTP_IFACE_IS_NOT_ATTACHABLE'})
  }

  ifStart()
  {
    return Rx.of(this.f('config')).pipe(
      RxO.mergeMap(cfg=>{
        if (this.ifRun())
          return Rx.of(true)
        if (this.ifStatus()=='stopping')
          return Rx.of(false)

        log.log('starting')
        this.ifStatus('starting')
        this._bot.start()
        this.ifRun(true)
        this.ifStatus('running')
        return this._bot.loadProfile()
      }),
      RxO.map(_=>!!_)
    )
  }

  ifStop()
  {
    return Rx.of(this.f('config')).pipe(
      RxO.map(cfg=>{
        if (!this._run)
          return true
        this.ifRun(false)
        this.ifStatus('stopping')
        this._bot.stop()
        this.ifStatus('stopped')
        return true
      })
    )
  }

  ////////////////////

  me()
  {
    return this._bot.profile()
  }

  sendMessage($msg)
  {
    this._bot.sendMessage($msg)
  }

  kickUser(user,chat)
  {
    return this._bot.kickUser(user,chat)
  }

  deleteMessages(list)
  {
    return this._bot.deleteMessages(list)
  }

  inlineReply(iqid,rep,ext)
  {
    return this._bot.inlineReply(iqid,rep,ext)
  }

  tgUser(src)
  {
    if (!src||!src.id)
      return Rx.of(null)
    return this.C('user').findNode({tgid:src.id}).pipe(
      RxO.mergeMap(user=>Rx.iif(_=>!!user,Rx.of(user),this.C('user').initNode({tg:src}))),
      RxO.mergeMap(user=>user.populateAccess()),
      RxO.map(user=>{
        user.f('firstName',src.first_name)
        if (src.last_name)
          user.f('lastName',src.last_name)
        if (src.username)
          user.f('username',src.username)
        return user
      })
    )
    /*
    return Rx.Observable.create(obs=>{
      //console.log(['tguser',src])
      this.C('user').findNode({tgid:src.id}).subscribe(user=>{
        if (user)
        {
          user.f('firstName',src.first_name)
          if (src.last_name)
            user.f('lastName',src.last_name)
          if (src.username)
            user.f('username',src.username)
          return this.obsEnd(obs,user)
        }
        this.C('user').initNode({tg:src}).subscribe(u=>{
          if (src.username)
            u.f('username',src.username)
          this.obsEnd(obs,u)
        },err=>obs.error(err))
      },err=>obs.error(err))
    })
    /**/
  }

  tgChat(src)
  {
    if (!src||!src.id)
      return Rx.of(null)
    return this.C('tgchat').findNode({chatId:src.id}).pipe(
      RxO.map(chat=>{
        if (!chat)
        {
          chat =  this.C('tgchat').newNode()
          chat.field('chatId').set(src.id)
          chat.field('type').set(src.type)
          if (src.type=='supergroup')
            chat.field('username').set(src.username)
          if (src.type!='private')
            chat.field('priority').set(6)
        }
        let title = src.type=='private'?src.first_name:src.title
        chat.field('title').set(title)
        chat.field('open').set(true)
        return chat
      })
    )
    /*
    return Rx.Observable.create(obs=>{
      this.C('tgchat').findNode({chatId:src.id}).subscribe(chat=>{
        if (chat)
        {
          let title = src.type=='private'?src.first_name:src.title
          chat.field('title').set(title)
          chat.field('open').set(true)
          return this.obsEnd(obs,chat)
        }
        this.C('tgchat').initNode({tg:src}).subscribe(c=>{

          c.field('chatId').set(src.id)
          c.field('type').set(src.type)
          let title = src.type=='private'?src.first_name:src.title
          c.field('title').set(title)
          if (src.type=='supergroup')
            c.field('username').set(src.username)
          if (src.type!='private')
            c.field('priority').set(6)

          this.obsEnd(obs,c)
        },err=>obs.error(err))
      })
    })
    /**/
  }

  tgMessage(upd,ctx,mode='default')
  {
    return Rx.Observable.create(obs=>{
      let mid = upd.message_id || null
      if (mode=='callback_query')
      {
        mid = upd.message.message_id
        //console.log(upd,mid,mode)
        this.tgChat(upd.message.chat).subscribe($chat=>{
          if (!$chat)
            return this.obsEnd(obs,null)
          this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._chat.id()}]}).subscribe(m=>{
            if (m)
              return this.obsEnd(obs,m)
            this.C('tgmessage').initNode(ctx).subscribe($m=>{
              $m.f('messageId',mid)
              $m.fp('chat',$chat)
              $m.fp('sender',this.me())
              //console.log(['tgMessage$init.callback_query',$m.toObject()])
              this.obsEnd(obs,$m)
            },err=>{
              log.err('init msg fail')
              console.log(err)
              this.obsEnd(obs,null)
            })
          })
        },err=>this.obsEnd(obs,null))
        
      }
      if (mode=='default')
      {
        if (!mid)
          return this.obsEnd(obs,null)
        this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._chat.id()}]}).subscribe(m=>{
          if (m)
            return this.obsEnd(obs,m)
          this.C('tgmessage').initNode(ctx).subscribe($m=>{
            //console.log(['tgMessage$init.default',$m.toObject()])
            this.obsEnd(obs,$m)
          },err=>{
            log.err('init msg fail')
            console.log(err)
            this.obsEnd(obs,null)
          })
        })
      }
      if (mode=='reply')
      {
        //console.log(['updRply',upd,mid])
        //mid = upd.message.reply_to_message.message_id
        if (!mid)
          return this.obsEnd(obs,null)
        this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._replyChat.id()}]}).subscribe(m=>{
          if (m)
            return this.obsEnd(obs,m)
          ctx._replyMode = true
          this.C('tgmessage').initNode(ctx).subscribe($m=>{
            //console.log(['tgMessage$init.reply',$m.toObject()])
            this.obsEnd(obs,$m)
          },err=>{
            log.err('init msg fail')
            console.log(err)
            this.obsEnd(obs,null)
          })
        })
        //log.err('REPLY MSG MODE INIT!!!')
        //console.log(upd,mid,mode)
        //this.obsEnd(obs,null)
      }
    })
  }

  addChat2Users(chat,users)
  {
    return Rx.Observable.create(obs=>{
      let $t = []
      users.forEach(u=>{
        if (chat.f('type')=='private')
          u.field('privateChat').set(chat)
        else
          u.field('publicChats').push(chat,{unique:true})
        $t.push(u.save())
        chat.field('users').push(u,{unique:true})
      })
      $t.push(chat.save())
      Rx.concat(...$t).subscribe(()=>{
        
      },err=>{
        if (err.code=='save'&&err.name=='VersionError')
          return this.obsEnd(obs)
        this.obsErr(obs,err)
        //obs.error(err)
      },()=>{
        this.obsEnd(obs)
      })
    })
  }



}

module.exports = TelegramBotInterface
