/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let sanitizeHtml = require('sanitize-html')

let log = require('../../log').m('TelegramBotRouter')
let g18n = require('../../g18n')

const AbstractRouter = require('../interface/router')
const Cmd = require('./cmd')
const Screen = require('../bot/screen/screen')
const Node = require('../node/node')
const VNode = require('../node/vnode')

class TelegramBotRouter extends AbstractRouter
{
  constructor(...args)
  {
    super(...args)

    this._crons = []
    this._keywordsCache = {}
    this._lastCronMinute = -1
  }

  static info()
  {
    return {
      code: 'telegrambotrouter',
      name: g18n.t('Telegram Bot Router'),
      desc: g18n.t('Classic telegram bot router'),
      scope: 'telegram'
    }
  }

  _search3(ctx,testfn,withIsolate=true)
  {
    return Rx.Observable.create(obs=>{

      let sender = ctx.user()
      let chat = ctx.chat()
      let chatt = chat?chat.f('type'):'private'

      // TODO DIRTY HARDFIX
      if (chatt == 'channel')
        chatt = 'group'
          
      let isolationScope = sender.f('isolateScope')
      //dirty fix to make "super isolation" for inline mode
      //currently used for tutorials
      if (typeof isolationScope == 'string' && isolationScope.endsWith('+'))
      {
        isolationScope = isolationScope.slice(0,-1)
        withIsolate = true
      }
      let isolationScreen = sender.f('isolateScreen')
      let isolationScreengroup = sender.f('isolateScreengroup')

      let doIso = false
      if (withIsolate&&isolationScope)
      {
        //console.log([isolationScreen,isolationScreengroup])
        if(isolationScope=='priv'&&chatt=='private')
          doIso = true
        if(isolationScope=='private'&&chatt=='private')
          doIso = true
        if(isolationScope=='grp'&&chatt=='group')
          doIso = true
        if(isolationScope=='xgrp'&&chatt=='supergroup')
          doIso = true
        if(isolationScope=='pub'&&chatt!='private')
          doIso = true
        if(isolationScope=='all')
          doIso = true
      }
      log.warn('DO ISO?',{scope:isolationScope,ok:doIso})


      let ret = {
        main: null,
        after: [],
        before: []
      }

      let main = null
      let empty = null
      let $mains = []
      let $empties = []
      let before = []
      let after = []

      let searchComplete = new Rx.Subject()

      this.C('screengroup').findNodes({router:this.f('code')},{
        list: true,
        sort: {priority:-1},
        populate: 1
      }).subscribe(scgs=>{
        if (!scgs||!scgs.length)
          return this.obsEnd(obs,ret)
        let allowed = scgs.filter(scg=>{
          if (doIso&&isolationScreengroup)
          {
            //log.warn('has isolation scg')
            if (scg.eq(isolationScreengroup))
              return true
            return false
          }
          /**/
          //check chat allow overrides
          let groupAllowed = false
          let groupRole = 'screengroup.'+scg.f('code')+':'+chatt
          //log.test('can?',groupRole)
          if (sender.can(groupRole,true))
            groupAllowed = true
          if (chat&&chat.field('allow').has(scg.id()))
            groupAllowed = true
          if (chat&&chat.field('deny').has(scg.id()))
            groupAllowed = false
          return groupAllowed
        })
        if (!allowed||!allowed.length)
          return this.obsEnd(obs,ret)
        let $srchs = []
        //passing all allowed screengroups
        //collecting before/after and maybe main, if isolated
        for(let i=0;i<allowed.length;i++)
        {
          let scrs = allowed[i].field('screens').filter(sc=>{
            if (!sc) return false
            if (!sc.f('enabled')) return false
            if (doIso&&isolationScreen)
            {
              //log.warn('has isolation sc')
              if (sc.eq(isolationScreen))
              {
                main = sc
                return true
              }
              return false
            }
            sc.screengroup(allowed[i])
            let action = sc.f('action')
            //console.log(['action?',action,ctx.type()])
            if (action == 'before_'+ctx.type())
            {
              before.push(sc)
              return false
            }
            if (action == 'after_'+ctx.type())
            {
              after.push(sc)
              return false
            }
            if (action!=ctx.type()) return false
              return true
          },false)
          //all before/after collected
          //we have main? so done
          if (main)
          {
            ret.main = main
            ret.before = before
            ret.after = after
            return this.obsEnd(obs,ret)
          }
          //no isolation, we need to search main screen
          //collecting $srchs rxs
          scrs.forEach(sc=>{
            $srchs.push(testfn(sc).pipe(
              RxO.mergeMap(testFnRes=>{
                if (!testFnRes.basic&&!testFnRes.empty)
                  return Rx.of(null)
                return this.M('logic').testLogicRx(sc,ctx).pipe(
                  RxO.map(lr=>{
                    if (!lr)
                      return null
                    //so, some screen passed all required tests
                    //if it's main - all done
                    //if empty - need to store first one
                    if (testFnRes.basic)
                    {
                      main = sc
                      searchComplete.next(1)
                      return
                    } else {
                      if (!empty)
                        empty = sc
                    }
                  })
                )
              })
            ))
          })
        }
        if (!$srchs||!$srchs.length)
          return this.obsEnd(obs,ret)
        Rx.concat(...$srchs).pipe(RxO.takeUntil(searchComplete)).subscribe(_=>{},err=>{
          log.err('$srchs',err)
          this.obsEnd(obs,ret)
        },_=>{
          ret.before = before
          ret.after = after
          if (empty)
            ret.main = empty
          if (main)
            ret.main = main
          this.obsEnd(obs,ret)
        })
      },err=>{
        log.err('err',err)
        this.obsEnd(obs,ret)
      })

    })
  }

  _search2(ctxx,testfn,withIsolate=true)
  {
    return Rx.of(ctxx).pipe(
      RxO.mergeMap(ctx=>{
        let sender = ctx.user()
        let chat = ctx.chat()
        let chatt = chat?chat.f('type'):'private'

        // TODO DIRTY HARDFIX
        if (chatt == 'channel')
          chatt = 'group'
            
        let isolationScope = sender.f('isolateScope')
        //dirty fix to make "super isolation" for inline mode
        //currently used for tutorials
        if (typeof isolationScope == 'string' && isolationScope.endsWith('+'))
        {
          isolationScope = isolationScope.slice(0,-1)
          withIsolate = true
        }
        let isolationScreen = sender.f('isolateScreen')
        let isolationScreengroup = sender.f('isolateScreengroup')

        let doIso = false
        if (withIsolate&&isolationScope)
        {
          //console.log([isolationScreen,isolationScreengroup])
          if(isolationScope=='priv'&&chatt=='private')
            doIso = true
          if(isolationScope=='private'&&chatt=='private')
            doIso = true
          if(isolationScope=='grp'&&chatt=='group')
            doIso = true
          if(isolationScope=='xgrp'&&chatt=='supergroup')
            doIso = true
          if(isolationScope=='pub'&&chatt!='private')
            doIso = true
          if(isolationScope=='all')
            doIso = true
        }
        log.warn('DO ISO?',{scope:isolationScope,ok:doIso})
 


        let main = null
        let empty = null
        let $mains = []
        let $empties = []
        let before = []
        let after = []

        return this.C('screengroup').findNodes({router:this.f('code')},{
          list: true,
          sort: {priority:-1},
          populate: 2,
          list: false
        }).pipe(
          RxO.filter(scg=>{
            if (doIso&&isolationScreengroup)
            {
              //log.warn('has isolation scg')
              if (scg.eq(isolationScreengroup))
                return true
              return false
            }
            /**/
            //check chat allow overrides
            let groupAllowed = false
            let groupRole = 'screengroup.'+scg.f('code')+':'+chatt
            //log.test('can?',groupRole)
            if (sender.can(groupRole,true))
              groupAllowed = true
            if (chat&&chat.field('allow').has(scg.id()))
              groupAllowed = true
            if (chat&&chat.field('deny').has(scg.id()))
              groupAllowed = false
            return groupAllowed
          }),
          RxO.mergeMap(scg=>{
            let $t = scg.field('screens').filter(sc=>{
              if (!sc) return false
              if (!sc.f('enabled')) return false
              if (doIso&&isolationScreen)
              {
                //log.warn('has isolation sc')
                if (sc.eq(isolationScreen))
                  return true
                return false
              }
              sc.screengroup(scg)
              let action = sc.f('action')
              //console.log(['action?',action,ctx.type()])
              if (action == 'before_'+ctx.type())
              {
                before.push(sc)
                return false
              }
              if (action == 'after_'+ctx.type())
              {
                after.push(sc)
                return false
              }
              if (action!=ctx.type()) return false
                return true
            },false).map(sc=>{
              return testfn(sc).pipe(
                RxO.mergeMap(tr=>{
                  //console.log(['!!TR',tr])
                  if (!tr.basic&&!tr.empty)
                    return Rx.of(null)
                  return this.M('logic').testLogicRx(sc,ctx).pipe(
                    RxO.map(lr=>{
                      if (!lr)
                        return null
                      return tr
                    })
                  )
                })
              )
            })
            if (!$t.length)
              return Rx.of(null)
            return Rx.concat(...$t)
          }),
          //RxO.tap(_=>console.log(['tap'])),
          RxO.filter(x=>!!x),
          RxO.map(testFnRes=>{
            if (!main&&testFnRes.basic)
              main = testFnRes.screen
            if (!empty&&testFnRes.empty)
              empty = testFnRes.screen
            return testFnRes
          }),
          RxO.last(),
          RxO.map(_=>{
            //console.log([['main',main,'empty',empty]])
            return {
              main: main?main:empty,
              after: after,
              before: before
            }
          }),
          RxO.catchError(e=>{
            let ret = {
              main: null,
              after: [],
              before: []
            }
            return Rx.of(ret)
          })
        )
      })
    )
  }

  /*
  _search(ctx,testfn,withIsolate=true)
  {
    return Rx.Observable.create(obs=>{
      let sender = ctx.user()
      let chat = ctx.chat()
      let chatt = chat?chat.f('type'):'private'

      // TODO DIRTY HARDFIX
      if (chatt == 'channel')
        chatt = 'group'

      let main = null
      let empty = null
      let $mains = []
      let $empties = []
      let before = []
      let after = []

      this.C('screengroup').findNodes({router:this.f('code')},{
        list: true,
        sort: {priority:-1},
        populate: 2
      }).subscribe(scgs=>{
        let allowed = []
        let isolationScope = sender.f('isolateScope')
        //dirty fix to make "super isolation" for inline mode
        //currently used for tutorials
        if (typeof isolationScope == 'string' && isolationScope.endsWith('+'))
        {
          isolationScope = isolationScope.slice(0,-1)
          withIsolate = true
        }
        let isolationScreen = sender.f('isolateScreen')
        let isolationScreengroup = sender.f('isolateScreengroup')

        let doIso = false
        if (withIsolate&&isolationScope)
        {
          //console.log([isolationScreen,isolationScreengroup])
          if(isolationScope=='priv'&&chatt=='private')
            doIso = true
          if(isolationScope=='private'&&chatt=='private')
            doIso = true
          if(isolationScope=='grp'&&chatt=='group')
            doIso = true
          if(isolationScope=='xgrp'&&chatt=='supergroup')
            doIso = true
          if(isolationScope=='pub'&&chatt!='private')
            doIso = true
          if(isolationScope=='all')
            doIso = true
        }
        log.warn('DO ISO?',{scope:isolationScope,ok:doIso})
        scgs.forEach(scg=>{
          //console.log(['scg',scg.id(),scg.field('code').get(),scg.field('priority').get()])
          //check isolation
          if (doIso&&isolationScreengroup)
          {
            //log.warn('has isolation scg')
            if (scg.eq(isolationScreengroup))
              allowed.push(scg)
            return
          }
          //check chat allow overrides
          let groupAllowed = false
          let groupRole = 'screengroup.'+scg.f('code')+':'+chatt
          //log.test('can?',groupRole)
          if (sender.can(groupRole,true))
            groupAllowed = true
          if (chat&&chat.field('allow').has(scg.id()))
            groupAllowed = true
          if (chat&&chat.field('deny').has(scg.id()))
            groupAllowed = false
          if (groupAllowed)
            allowed.push(scg)
        })
        //let $maintests = []
        allowed.forEach(scg=>{
          scg.field('screens').forEach(sc=>{
            if (!sc) return
            if (!sc.f('enabled')) return
            if (doIso&&isolationScreen)
            {
              //log.warn('has isolation sc')
              if (sc.eq(isolationScreen))
                $main = sc
              return
            }
            sc.screengroup(scg)
            let action = sc.f('action')
            if (action == 'before_'+ctx.type())
            {
              before.push(sc)
              return
            }
            if (action == 'after_'+ctx.type())
            {
              after.push(sc)
              return
            }
            //console.log(['??',action,ctx.type()])
            if (action!=ctx.type()) return
            if (action.includes('group_chat')) //join/leave
            {
              $mains.push(sc)
              return
            }

            //TESTFN
            let basicOk = false
            let emptyOk = false
            let bt = testfn(sc)
            if (bt&&bt.basic) basicOk = true
            if (bt&&bt.empty) emptyOk = true
            if (basicOk)
              $mains.push(sc)
            if (emptyOk)
              $empties.push(sc)
          })
        })
        //ok, now we have lists main/empties/before/after. need to test them and get finals
        //let's get main 1st. if we don't have main - b4 and after we don't care
        let $srch = []
        let mainFound = new Rx.Subject()
        $mains.forEach(sc=>{
          $srch.push(this.M('logic').testLogicRx(sc,ctx).pipe(RxO.mergeMap(testRes=>{
            //console.log(['screen router, screen tested',sc.f('title'),testRes,ctx.toObject()])
            if (doIso&&isolationScreen)
            {
              if (sc.eq(isolationScreen))
              {
                main = sc
                mainFound.next(1)
              }
              return Rx.of(null)
            }
            //log.info('main screen tested',[sc.f('title'),testRes])
            if(!testRes)
              return Rx.of(null)
            if (main)
              return Rx.of(null)
            main = sc
            mainFound.next(1)
          })))
        })
        $empties.forEach(sc=>{
          $srch.push(this.M('logic').testLogicRx(sc,ctx).pipe(RxO.mergeMap(testRes=>{
            //log.info('empty screen tested',[sc.f('title'),testRes])
            if(!testRes)
              return Rx.of(null)
            if (main)
              return Rx.of(null)
            main = sc
            mainFound.next(1)
          })))
        })
        Rx.concat(...$srch).pipe(RxO.takeUntil(mainFound)).subscribe(osc=>{
          //log.test('one more screen tested')
          //console.log([main,osc,'one more screen test chain done'])
        },err=>{
          log.err('error happens on search!')
          console.log(err)
        },()=>{
          //log.warn('search finished!')
          //log.success('found screen to execute!',main.f('title'))
          this.obsEnd(obs,{
            main: main,
            before: before,
            after: after
          })
        })
      })
    })
  }
  /**/

  keywordsCache(bndl,kwfields)
  {
    if (!this.C(bndl))
      return Rx.of(null)
    if (this._keywordsCache[bndl])
    {
      let updfields = kwfields.filter(x=>!this._keywordsCache[bndl].fields[x])
      //console.log(['cache up fields',updfields,this._keywordsCache[bndl]])
      if (!updfields.length)
        return Rx.of(this._keywordsCache[bndl])
      return this._updateKeywordsCache(bndl,updfields)
    }
    return this._updateKeywordsCache(bndl,kwfields)
  }

  _updateKeywordsCache(bndl,kwfields)
  {
    if (!this.C(bndl))
      return Rx.of(null)
    return this.C(bndl).findNodes().pipe(
      RxO.map(list=>{
        if (!list||!list.length)
          return null
        log.info('updating keywords cache',bndl)
        if (this._keywordsCache[bndl])
          Object.keys(this._keywordsCache[bndl].fields).forEach(kwf=>{
            if (!kwfields.includes(kwf))
              kwfields.push(kwf)
          })
        let cache = {
          list: {},
          fields: {}
        }
        list.forEach(nd=>{
          if (!nd) return
          cache.list[nd.id()] = nd
          kwfields.forEach(kwf=>{
            if (!nd.field(kwf)) return
            if (!cache.fields[kwf])
              cache.fields[kwf] = {}
            nd.field(kwf).all().forEach(kw=>{
              if (!kw||!kw.length) return
              if (!cache.fields[kwf][kw])
                cache.fields[kwf][kw] = []
              if (!cache.fields[kwf][kw].includes(nd.id()))
                cache.fields[kwf][kw].push(nd.id())
            })
          })
        })
        this._keywordsCache[bndl] = cache
        return this._keywordsCache[bndl]
      })
    )
  }

  _updateAllKeywordsCaches()
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        let $t = Object.keys(this._keywordsCache).map(bndl=>this._updateKeywordsCache(bndl,[]))
        if (!$t.length)
          return Rx.of(true)
        return Rx.concat(...$t).pipe(
          RxO.last(),
          RxO.map(_=>true)
        )
      })
    )
  }


  onMessage(ctx)
  {
      //log.info('xxx',ctx.view())
    log.info('message',[ctx.type(),ctx.user().titles(),ctx.chat().f('title'),ctx.chat().f('type')])
      //console.log(ctx.raw())
    let sender = ctx.user()
    let chat = ctx.chat()

    let msgText = ctx.upd().text || ''
    let cmd = new Cmd(ctx,msgText)
    ctx.cmd(cmd)

    this._search3(ctx,sc=>{
      let ret = {
        basic: false,
        empty: false,
        screen: sc
      }
      if (ctx.upd().photo&&sc.f('action')=='photo')
      {
        ret.basic = true
        return Rx.of(ret)
      }
      if (sc.f('action').includes('group_chat')) //join leave
      {
        ret.basic = true
        return Rx.of(ret)
      }
      if (!msgText||!msgText.length)
        return Rx.of(ret)
      let actionData = sc.f('actionData')
      if (!actionData||!actionData[ctx.type()])
        return Rx.of(ret)
      let ad = actionData[ctx.type()]
      //basic
      let basicOk = false
      if (ad.hasCommand&&ad.command!='')
      {
        if(ad.command==cmd.command())
          basicOk = true
        else
          return Rx.of(ret)
      }
      if (!basicOk&&ad.hasKey&&ad.key!='')
      {
        if (ad.key==cmd.key())
          basicOk = true
        else
          return Rx.of(ret)
      }
      //new
      if (!basicOk&&ad.hasRx&&ad.rx&&ad.rx!='')
      {
        let rxline = ad.rx
        let rg = null
        try {
          rg = this.M('template').str2rx(ad.rx)
        } catch(e) {
          rg = null
        }
        if (!rg)
          return Rx.of(ret)
        let ok = false
        try {
          ok = rg.test(cmd.raw())
        } catch(e) {
          ok = false
        }
        if (ok)
          basicOk = true
        else
          return Rx.of(ret)
      }
      if (ad.hasKeywords)
      {
        //console.log(ad)
        return this.keywordsCache(ad.keywordsBundle,ad.keywords).pipe(
          RxO.map(cache=>{
            if (!cache)
              return ret
            //console.log(['got cache to search',cache])
            let okcache = {}
            ad.keywords.forEach((kwf,lvli)=>{
              Object.keys(cache.fields[kwf]).forEach(kw=>{
                let rx = null
                try {
                  rx = this.M('template').str2rx(kw)
                } catch(e) {
                  rx = null
                }
                if (!rx)
                  return
                let testRes = false
                try {
                  testRes = rx.test(cmd.raw())
                } catch(e) {
                  testRes = false
                }
                let nodes = cache.fields[kwf][kw].map(nid=>cache.list[nid])
                if (!testRes)
                {
                  //nodes.forEach(okn=>{
                    //if (!okcache[okn.id()])
                      //return
                    //okcache[okn.id()].levels[lvli] = 0
                  //})
                  return
                }
                nodes.forEach(okn=>{
                  if (!okcache[okn.id()])
                    okcache[okn.id()] = {
                      node: okn,
                      levels: []
                    }
                  //for(let ii=0;ii<=lvli;ii++)
                    //if(!okcache[okn.id()].levels[ii])
                      //okcache[okn.id()].levels[ii] = 0
                  if (!okcache[okn.id()].levels[lvli])
                    okcache[okn.id()].levels[lvli] = 0
                  okcache[okn.id()].levels[lvli]++
                })
              })
            })
            //console.log('okcache',okcache)

            let frcache = {}
            Object.keys(okcache).forEach(nid=>{
              let chc = okcache[nid]
              //if(!chc.levels.reduce((acc,x)=>acc+x,0))
                //return
              let lvlsok = []
              ad.keywordsRequired.forEach((lreq,li)=>{
                lvlsok[li] = false
                if (!lreq)
                {
                  if (ad.keywordsSortData=='_all')
                    lvlsok[li] = true
                  if (ad.keywordsSortData=='_nonzero')
                    lreq = true
                }
                if (lreq&&chc.levels[li]>0) lvlsok[li] = true
              })
              let allok = lvlsok.reduce((acc,x)=>!x?false:acc,true)
              if (allok)
                frcache[nid] = chc
            })
            //console.log('frcache',frcache)

            if (Object.keys(frcache).length)
            {
              ret.basic = true
              let ctxlist = []
              if (ad.keywordsSort == 'field')
              {
                ctxlist = Object.keys(frcache).map(nid=>frcache[nid].node).sort((a,b)=>{
                  let fa = a.f(ad.keywordsSortData)
                  let fb = b.f(ad.keywordsSortData)
                  if (fa==fb) return 0
                  if (ad.keywordsSortUp)
                    return fa<fb?1:-1
                  else
                    return fa>fb?1:-1
                })
              } 
              if (ad.keywordsSort == 'keywords')
              {
                ctxlist = Object.keys(frcache).map(nid=>frcache[nid]).sort((a,b)=>{
                  let fa = 0
                  let fb = 0
                  //console.log(a.levels,b.levels)
                  if (ad.keywordsSortData=='_all')
                  {
                    fa = a.levels.reduce((acc,x)=>acc+x,0)
                    fb = b.levels.reduce((acc,x)=>acc+x,0)
                  } else {
                  }
                  if (fa==fb) return 0
                  if (ad.keywordsSortUp)
                    return fa<fb?1:-1
                  else
                    return fa>fb?1:-1
                })
                .map(x=>x.node)
              }
              let skip = parseInt(ad.keywordsSortSkip)||0
              let limit = parseInt(ad.keywordsSortLimit)||0
              if (skip||limit)
              {
                if (limit)
                  ctxlist = ctxlist.slice(skip,skip+limit)
                else
                  ctxlist = ctxlist.slice(skip)
              }

              let vnode = new VNode(ctx)
              if (ad.keywordsSortFlat)
              {
                vnode = ctxlist[0]
                ctx.addNode(vnode,ad.keywordsRescode)
              } else {
                vnode.addField({
                  name: g18n.t('Keywords nodes'),
                  code: ad.keywordsRescode,
                  type: 'relation',
                  xtypes: ['relation','relation:'+ad.keywordsBundle],
                  conf: {ops:[{source:'manual',manual:ad.keywordsBundle},{source:'',manual:''}]},
                  quantity: 0,
                  flags: ['virtual','logic','template']
                })
                //console.log(ctxlist)
                vnode.field(ad.keywordsRescode).all(ctxlist)
                ctx.addNode(vnode,'keywords')
              }
            }
            return ret
          })
        )
        /*
        if (!this.C(ad.keywordsBundle))
          return Rx.of(ret)
        return this.C(ad.keywordsBundle).findNodes({}).pipe(
          RxO.map(list=>{
            let next = []
            list.forEach(nd=>{
              ad.keywords.forEach((lvlk,lvli)=>{
                if (!nd.field(lvlk)||!nd.field(lvlk).all()||!nd.field(lvlk).all().length)
                  return
                nd.field(lvlk).all().forEach(kwrx=>{
                  console.log(['kwrx',lvlk,lvli,kwrx,cmd.raw(),nd.f('title')])
                  let isRx=/^\/(.+)\/([a-z]*)/
                  let m = isRx.test(kwrx)
                  let 
                  console.log(m)

                })

              })
            })
            if (!next.length)
              return false
            return true
          })
        )
        /**/
      } else {
        if (basicOk)
          ret.basic = true
        if (!ad.hasCommand&&!ad.hasKey&&!ad.hasRx) //&&!ad.hasKeywords already tested
          ret.empty = true
        return Rx.of(ret)
      }
    }).subscribe(sres=>{
      //log.warn('search finished!')
      if (!sres.main)
        return
      log.success('found screen to execute!',sres.main.f('title'))
      let $exec = []
      /**/
      let omsg = ctx.message()
      if (omsg)
      {
        //console.log(['saving message',omsg.toObject()])
        omsg.f('screen',sres.main)
        //ctx.message().field('',main)
        $exec.push(omsg.saveToChatHistory())
      }
      /**/
      sres.before.forEach(xsc=>{
        $exec.push(this.M('logic').testLogicRx(xsc,ctx).pipe(RxO.mergeMap(x=>{
          if(!x) return Rx.of(null)
          return xsc.populate(3).pipe(RxO.mergeMap($xsc=>{
            return $xsc.execute(ctx).pipe(RxO.catchError(exe=>{
              return Rx.of({success:false,error:exe,code:'SCREEN_EXECUTION_ERROR'})
            }))
          }))
        })))
      })
      $exec.push(sres.main.populate(2).pipe(RxO.mergeMap($xsc=>{
        return $xsc.execute(ctx).pipe(RxO.catchError(exe=>{
          log.err('execution error!')
          console.log(exe)
          return Rx.of({success:false,error:exe,code:'SCREEN_EXECUTION_ERROR'})
        }))
      })))
      sres.after.forEach(xsc=>{
        $exec.push(this.M('logic').testLogicRx(xsc,ctx).pipe(RxO.mergeMap(x=>{
          if(!x) return Rx.of(null)
          return xsc.populate(2).pipe(RxO.mergeMap($xsc=>{
            return $xsc.execute(ctx).pipe(RxO.catchError(exe=>{
              log.err('execution error!')
              console.log(exe)
              return Rx.of({success:false,error:exe,code:'SCREEN_EXECUTION_ERROR'})
            }))
          }))
        })))
      })
      Rx.concat(...$exec).subscribe(exRes=>{
        if (!exRes) return
        log.info('screen executed',exRes)
      },err=>{
        //should never come here
        log.err('screens execution error!!')
        console.log(err)
      },()=>{
        log.success('all executions complete')
      })

    })

  }


  onCallbackQuery(ctx)
  {
    let data = ctx.upd().data||null
    log.info('callback_query',[ctx.user().titles(),ctx.chat().field('title').get(),data])
    if (!data)
      return
    let $srch = null
    let cmd = new Cmd(ctx,'')
    ctx.cmd(cmd)
    if (data.startsWith('sid:'))
    {
      let sid = data.slice(4)
      $srch = this.C('screen').findNode({_id:sid}).pipe(RxO.map(x=>{
        return { main: x, before: [], after: []}
      }))
    } else {
      cmd.fromCBQ(data)
      ctx.cmd(cmd)
      $srch = this._search3(ctx,sc=>{
        //console.log(sc.f('title'))
        let ret = {
          basic: false,
          empty: false,
          screen: sc
        }
        if (!data||!data.length)
          return Rx.of(ret)
        //let cmd = new Cmd(ctx,msgText)
        //ctx.cmd(cmd)
        let actionData = sc.f('actionData')
        if (!actionData||!actionData[ctx.type()])
          return Rx.of(ret)
        let ad = actionData[ctx.type()]
        //basic
        let basicOk = false
        if (ad.hasCommand&&ad.command!=''&&ad.command==cmd.command())
          basicOk = true
        //new
        if (ad.hasRx&&ad.rx&&ad.rx!='')
        {
          let rxline = ad.rx
          let rg = null
          if (rxline.startsWith('/')&&rxline.endsWith('/i'))
          {
            rxline = rxline.slice(1,-2)
            rg = new RegExp(rxline,'i')
          } else {
            rg = new RegExp(rxline)
          }
          let ok = rg.test(cmd.raw())
          if (ok)
            basicOk = true
        }
        if (basicOk)
          ret.basic = true
        if (!ad.hasCommand&&!ad.hasRx)
          ret.empty = true
        return Rx.of(ret)
      },false)
    }
      //this.C('screen').findNode({_id:sid},{populate:2}).subscribe(sc=>{
    $srch.subscribe(sc=>{
      //log.test('search result',[!!sc])
      if (!sc||!sc.main)
        return
      log.success('found screen to execute',sc.main.f('title'))
      let omsg = ctx.message()
      omsg.f('screen',sc.main)
      //omsg.save().pipe(
        //RxO.mergeMap(_=>sc.main.populate(2)),
      sc.main.populate(2).pipe(
        RxO.mergeMap($sc=>$sc.execute(ctx))
      ).subscribe(exRes=>{
        let ad = sc.main.f('actionData')
        let ans = ''
        if (ad&&ad.callback_query&&ad.callback_query.text)
          ans = ad.callback_query.text
        if (!ans||!ans.length)
        {
          ctx._rawctx.answerCbQuery('')
          log.success('executed with no answer')
          return
        }
        this.M('template').compile(ans,ctx).subscribe(ansc=>{
          ctx._rawctx.answerCbQuery(ansc)
          log.success('executed with answer',ansc)
        },err=>{
          ctx._rawctx.answerCbQuery('')
          log.err('compiling callback answer error',err)
        })
      },err=>{
        log.err('callbackscreen execution error!')
        console.log(err)
      })
    },err=>{
      log.err('screen not found!',sid)
      console.log(err)
    })
  }

  onInlineQuery(ctx)
  {
    //console.log(ctx.upd())

    let inq = ctx.upd().query || ''
    log.info('on inline query',inq)
    let cmd = new Cmd(ctx,inq)
    ctx.cmd(cmd)

    let $srch = this._search3(ctx,sc=>{
      //console.log(sc.f('title'))
      let ret = {
        basic: false,
        empty: false,
        screen: sc
      }
      if (!inq||!inq.length)
        return Rx.of(ret)
      //let cmd = new Cmd(ctx,msgText)
      //ctx.cmd(cmd)
      let actionData = sc.f('actionData')
      if (!actionData||!actionData[ctx.type()])
        return Rx.of(ret)
      let ad = actionData[ctx.type()]
      //basic
      let basicOk = false
      if (ad.rx=='')
      {
        basicOk = true
      } else {
        let rxline = ad.rx
        let rg = null
        if (rxline.startsWith('/')&&rxline.endsWith('/i'))
        {
          rxline = rxline.slice(1,-2)
          rg = new RegExp(rxline,'i')
        } else {
          rg = new RegExp(rxline)
        }
        let ok = rg.test(cmd.raw())
        if (ok)
          basicOk = true
      }
      if (basicOk)
        ret.basic = true
      //if (!ad.hasCommand&&!ad.hasRx)
        //ret.empty = true
      return Rx.of(ret)
    },false)

    $srch.subscribe(res=>{
      log.info('search complete')
      if (!res||!res.main)
      {
        log.info('no screen found')
        return
      }
      res.main.populate(3).pipe(RxO.mergeMap(x=>res.main.execute(ctx))).subscribe(exres=>{
        log.info('executed!')
      },exerr=>{
        log.err('search error!')
        console.log(err)
      })
    },err=>{
      log.err('search error!')
      console.log(err)
    })

  }

  onEvent(e,ctx)
  {
    if (e.type == 'cron')
      return this._onCronEvent(e,ctx)
    //console.log(['EVENT!!',e])
    this.C('screengroup').findNodes({router:this.f('code')}).pipe(
      RxO.mergeMap(grps=>{
        let ids = []
        grps.forEach(grp=>{
          grp.field('screens').all().forEach(sid=>ids.push(sid))
        })
        return this.C('screen').findNodes({
          enabled: true,
          action: e.type,
          _id: {$in:ids}
        },{populate:2})
      })
    ).subscribe(screens=>{
      if (!screens||!Array.isArray(screens)||!screens.length)
        return
      let okScreens = []
      let $t = screens.map(sc=>this.M('logic').testLogicRx(sc,ctx).pipe(
        RxO.map(tres=>{
          if (tres) return sc
          return null
        })
      ))
      if (!$t.length)
        return
      Rx.concat(...$t).subscribe(sc=>{
        if (sc)
          okScreens.push(sc)
      },err=>{
        log.warn('error happened on searching "onEvent" screen')
        console.log(err)
      },_=>{
        if (!okScreens.length)
          return
        let $tt = okScreens.map(sc=>{
          let ad = sc.f('actionData')
          if (!ad||!ad[e.type])
            return null
          ad = ad[e.type]
          //console.log(['ad',ad])
          let ret = null
          if (e.hasUser)
            ret = Rx.of(e.data.user)
          else
            ret = this.M('auth/user').userFieldUsers(ad.user,ctx,true)
          if (!ret) return null
          return ret.pipe(
            RxO.mergeMap(ulist=>{
              //console.log(['ulist',ulist])
              if (!ulist)
                return Rx.of(null)
              if (!Array.isArray(ulist))
                ulist = [ulist]
              let $ttt = ulist.map(u=>{
                if (!u)
                  return null
                if (!(u instanceof Node))
                  return null
                if (u.bundle()!='user')
                  return null
                let cmd = new Cmd(this,'')
                let cctx = ctx.cloneFull()
                cctx.cmd(cmd)
                cctx.users([u])
                cctx.chats([u.fp('privateChat')])
                log.info('executing screen for event',sc.f('title'))
                return sc.execute(cctx)
              })
              if (!$ttt.length)
                return Rx.of(null)
              return Rx.concat(...$ttt)
            })
          )
        }).filter(x=>!!x)
        if (!$tt.length)
          return
        setTimeout(()=>{
          Rx.concat(...$tt).pipe(RxO.last()).subscribe(_=>{},err=>{
            log.err('on event screens flow execution error!')
            console.log(err)
          })
        },0)
      })
    })

    /*
    let usersList = Rx.of(null)
    if (!e.hasUser)
    {
      console.log(['try load user/s!!',e])
      
    } else {
    
    }
    /**/
  }

  _onEachMinute()
  {
    return Rx.of(1).pipe(
      //RxO.tap(_=>log.warn('each minute'))
      RxO.mergeMap(_=>this._updateAllKeywordsCaches())
    )
  }

  _onCronEvent(e,ctx)
  {
    let nx = Rx.of(this._crons)
    if (this._lastCronMinute!=e.data.minute)
    {
      this._lastCronMinute = e.data.minute
      nx = this._onEachMinute().pipe(
        RxO.mergeMap(_=>this.C('screengroup').findNodes({router:this.f('code')})),
        RxO.mergeMap(grps=>{
          let ids = []
          grps.forEach(grp=>{
            grp.field('screens').all().forEach(sid=>ids.push(sid))
          })
          return this.C('screen').findNodes({
            enabled: true,
            action: 'cron',
            _id: {$in:ids}
          },{populate:2})
        }),
        RxO.map(list=>{
          this._crons = list
          return list
        })
      )
    }
    nx.subscribe(scrs=>{
      if (!scrs||!scrs.length)
        return
      scrs.forEach(scr=>{
        let ad = scr.f('actionData')
        if (!this.M('cron').testMask(ad.cron.event))
          return
        //console.log(ad.cron)
        setTimeout(()=>{
          let iface = ctx.vnode('interface')
          //let ctxx = iface.ifContext()
          this.M('auth/user').userFieldUsers(ad.cron.user,ctx,true).subscribe(ulist=>{
            if (!Array.isArray(ulist))
              return
            ulist.forEach(u=>{
              if (!u)
                return
              if (!(u instanceof Node))
                return
              if (u.bundle()!='user')
                return
              //console.log(ulist.map(x=>x.toObject()))
              setTimeout(()=>{
                let cmd = new Cmd(this,'')
                let cctx = iface.ifContext()
                //CTX.session(ctx.session)
                cctx.cmd(cmd)
                cctx.type('cron')
                cctx.users([u])
                cctx.chats([u.fp('privateChat')])
                //console.log(['cron screen',scr.f('title'),ctx.view()])
                log.info('executing screen by cron',scr.f('title'))
                scr.execute(cctx).subscribe(res=>{},err=>{
                  console.log(err)
                })
              },0)
            })
          },err=>{
            log.err('search user for cron execution error')
            console.log(err)
          })
        },0)
      })
    },err=>{
      log.err('search cron screens error')
      console.log(err)
    })
    /**/


  }



}

module.exports = TelegramBotRouter
