/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
//let minimist = require('minimist')

let log = require('../../log').m('TelegramCommand')
let g18n = require('../../g18n')

//const CoreObject = require('../../core/abstract/object')
const VirtualNode = require('../node/vnode')

class Cmd extends VirtualNode
{
  constructor(parent,text=null)
  {
    super(parent)

    this.addField({
      name:g18n.t('Raw'),
      code:'raw',
      type:'string'
    })
    this.addField({
      name:g18n.t('Command'),
      code:'command',
      type:'string'
    })
    this.addField({
      name:g18n.t('Key'),
      code:'key',
      type:'string'
    })
    this.addField({
      name:g18n.t('Arguments line'),
      code:'args',
      type:'string'
    })
    this.addField({
      name:g18n.t('Arguments list'),
      code:'arg',
      type:'string',
      quantity: 0
    })
    this.addField({
      name:g18n.t('Words list'),
      code:'words',
      type:'string',
      quantity: 0
    })
    //this._command=''
    //this._args=''
    //this._key=''
    //this._raw = text || ''
    if (text)
      this._parse(text)
  }

  fromCBQ(q,opts)
  {
    if (!q||!q.length)
      return
    q = q.replace(/:/g,' ')
    q = '/'+q
    this._parse(q)
  }

  _parse(text)
  {
    this.field('command').clear()
    this.field('key').clear()
    this.field('args').clear()
    this.field('arg').clear()
    this.field('words').clear()
    this.f('raw',text)
    if(text&&text.length)
      this.field('words').all(text.split(' '))
    //this._command=''
    //this._args=''
    //this._key=''
    //this.opts = {}
    //this._raw=text
    if (text&&text.startsWith('/'))
    {
      let str = text.slice(1)
      let args = str.split(' ')
      let command = args.shift()
      let argl = args.map(x=>x).join(' ').trim()
      args.unshift(argl)
      this.f('command',command)
      this.f('args',argl)
      this.field('arg').all(args)
    } else {
      this.f('key',text)
    }
  }

  raw() { return this.f('raw')}
  command(upd)
  {
    if(typeof upd == 'string')
    {
      let args = this.field('arg').all().slice(1)
      if (!this.f('command').length)
        args = this.f('raw')
      let next = '/'+upd
      if (next!='/'&&args.length) next+=' '
      next += args.join(' ')
      this._parse(next)
    }
    return this.f('command')
  }
  key() { return this.f('key') }
  args()
  {
    return this.field('arg').all()
  }
  arg(i=0)
  {
    return this.field('arg').get(i)
  }
  dropArg(i=1)
  {
    if (this.f('command')==''||!this.field('arg').all().length)
      return
    let list = this.f('arg').filter((v,ii)=>i!=ii,false).slice(1)
    let next = '/'+this.f('command')
    if (next!='/'&&list.length) next+=' '
    next += list.join(' ')
    this._parse(next)
  }

  /*
  logicFields(ret=[])
  {
    ret.push({
      scope: 'cmd',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Input raw'),code:'raw',type:'string'},
      values: [this.raw()]
    })
    ret.push({
      scope: 'cmd',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Input command'),code:'command',type:'string'},
      values: [this.command()]
    })
    ret.push({
      scope: 'cmd',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Input key'),code:'key',type:'string'},
      values: [this.key()]
    })
    ret.push({
      scope: 'cmd',
      info: {flags:['virtual','readonly'],quantity:1,name:g18n.t('Input arguments'),code:'args',type:'string'},
      values: [this._args]
    })
    ret.push({
      scope: 'cmd',
      info: {flags:['virtual','readonly'],quantity:0,name:g18n.t('Input argument'),code:'arg',type:'string'},
      values: this.args()
    })
    return ret
  } 
  /**/

  clone()
  {
    let ret = new Cmd(this._parent,this.f('raw'))
    return ret
  }

  rearg(s)
  {
    let rx = new RegExp(s,'g')
    let argl = this.f('args')
    argl = argl.replace(rx,' ')
    let next = '/'+this.f('command')
    if (next!='/'&&argl.length) next+=' '
    next+=argl
    this._parse(next)
  }

  /*
  fields(ret=[])
  {
    ret.push({
      code: 'cmd:command', typeCode: 'cmd:command',
      type: {type:'string',code:'cmd:command',behavior:'screen'},
      vall: this.command
    })
    ret.push({
      code: 'cmd:key', typeCode: 'cmd:key',
      type: {type:'string',code:'cmd:key',behavior:'screen'},
      vall: this.key
    })
    ret.push({
      code: 'cmd:args', typeCode: 'cmd:args',
      type: {type:'string',code:'cmd:args',behavior:'screen'},
      vall: this.args
    })
    ret.push({
      code: 'cmd:raw', typeCode: 'cmd:raw',
      type: {type:'string',code:'cmd:raw',behavior:'screen'},
      vall: this.raw
    })
    return ret
  }
  /**/


  /*
  setCommand(cmd)
  {
    this.command = cmd
    this.raw = ('/'+this.command+' '+this.args).trim()
  }

  setArgs(args)
  {
    this.args = args.trim()
    if (this.command)
      this.raw = ('/'+this.command+' '+this.args).trim()
    else
      this.raw = this.args
    if (!this.command)
      this.key = this.raw
  }

  drop(n)
  {
    if (n>0)
      this._drop(n-1)
  }

  _drop(n)
  {
    if (n<0)
      return
    let arr = this.argarr()
    let next = []
    for(let i=0;i<arr.length;i++)
      if(i!=n)
        next.push(arr[i])
    log.test('CMD','drop',{arr:arr,i:n,nx:next})
    this.args = next.join(' ').trim()
    this.key = this.args
    if (this.command)
      this.key = ('/'+this.command+' '+this.args).trim()
  }


  str()
  {
  }

  argi(i)
  {
    if(i<1) return null
    if(!this.args||this.args=='')
      return null
    if(this.args.indexOf(' ')==-1&&i!=1)
      return null
    if(this.args.indexOf(' ')==-1&&i==1)
      return this.args
    let arr = this.args.split(' ')
    if (i>arr.length)
      return null
    return arr[i-1]
  }

  argarr()
  {
    if(!this.args||this.args=='')
      return []
    if(this.args.indexOf(' ')==-1)
      return [this.args]
    return this.args.split(' ')
  }

  count()
  {
    return this.argarr().length
  }
  /**/
 
}

module.exports = Cmd
