/*
  This file is part of TakatanJS software.
  @author abrakadobr
  @version 2.5.58
  @website https://js.takatan.tk
*/


let Rx = require('rxjs')
let RxO = require('rxjs/operators')
let Threads = require('worker_threads')

let log = require('../../log').m('TelegramModule')
let g18n = require('../../g18n')

// TELEGRAM

const CoreModule = require('../core/abstract/module')

const TelegramBot = require('./bot')
const TelegramBotRouter = require('./router')

const VNode = require('../node/vnode')

const ChatNode = require('./nodes/chat')
const MessageNode = require('./nodes/message')

const TgidGenerator = require('./generators/tgid')
const IsBotGenerator = require('./generators/isbot')
const FirstNameGenerator = require('./generators/firstname')
const LastNameGenerator = require('./generators/lastname')

const TelegramBotInterface = require('./bot.interface')
const TelegramBotRouter2 = require('./bot.router')

class TelegramModule extends CoreModule
{

  constructor(parent)
  {
    super(parent)

    this._router = new TelegramBotRouter(this)
    this._bot = new TelegramBot(this)
    this._username = ''

    this.M('field').registerGenerator(TgidGenerator)
    this.M('field').registerGenerator(IsBotGenerator)
    this.M('field').registerGenerator(FirstNameGenerator)
    this.M('field').registerGenerator(LastNameGenerator)

  }

  interfaceRegister()
  {
    return Rx.Observable.create(obs=>{
      this.M('interface').registerInterfaceType(TelegramBotInterface)
      this.M('interface').registerRouterType(TelegramBotRouter2)
      //this.M('interface').registerInterfaceType(HttpInterface)
      this.obsEnd(obs)
    })
  }


  me()
  {
    return this._bot.profile()
  }

  sendMessage($msg)
  {
    this._bot.sendMessage($msg)
  }

  kickUser(user,chat)
  {
    return this._bot.kickUser(user,chat)
  }

  deleteMessages(list)
  {
    return this._bot.deleteMessages(list)
  }

  tgUser(src)
  {
    if (!src||!src.id)
      return Rx.of(null)
    return this.C('user').findNode({tgid:src.id}).pipe(
      RxO.mergeMap(user=>Rx.iif(_=>!!user,Rx.of(user),this.C('user').initNode({tg:src}))),
      RxO.mergeMap(user=>user.populateAccess()),
      RxO.map(user=>{
        user.f('firstName',src.first_name)
        if (src.last_name)
          user.f('lastName',src.last_name)
        if (src.username)
          user.f('username',src.username)
        return user
      })
    )
    /*
    return Rx.Observable.create(obs=>{
      //console.log(['tguser',src])
      this.C('user').findNode({tgid:src.id}).subscribe(user=>{
        if (user)
        {
          user.f('firstName',src.first_name)
          if (src.last_name)
            user.f('lastName',src.last_name)
          if (src.username)
            user.f('username',src.username)
          return this.obsEnd(obs,user)
        }
        this.C('user').initNode({tg:src}).subscribe(u=>{
          if (src.username)
            u.f('username',src.username)
          this.obsEnd(obs,u)
        },err=>obs.error(err))
      },err=>obs.error(err))
    })
    /**/
  }

  tgChat(src)
  {
    return Rx.Observable.create(obs=>{
      this.C('tgchat').findNode({chatId:src.id}).subscribe(chat=>{
        if (chat)
        {
          let title = src.type=='private'?src.first_name:src.title
          chat.field('title').set(title)
          chat.field('open').set(true)
          return this.obsEnd(obs,chat)
        }
        this.C('tgchat').initNode({tg:src}).subscribe(c=>{

          c.field('chatId').set(src.id)
          c.field('type').set(src.type)
          let title = src.type=='private'?src.first_name:src.title
          c.field('title').set(title)
          if (src.type=='supergroup')
            c.field('username').set(src.username)
          if (src.type!='private')
            c.field('priority').set(6)

          this.obsEnd(obs,c)
        },err=>obs.error(err))
      })
    })
  }

  tgMessage(upd,ctx,mode='default')
  {
    return Rx.Observable.create(obs=>{
      let mid = upd.message_id || null
      if (mode=='callback_query')
      {
        mid = upd.message.message_id
        //console.log(upd,mid,mode)
        this.tgChat(upd.message.chat).subscribe($chat=>{
          if (!$chat)
            return this.obsEnd(obs,null)
          this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._chat.id()}]}).subscribe(m=>{
            if (m)
              return this.obsEnd(obs,m)
            this.C('tgmessage').initNode(ctx).subscribe($m=>{
              $m.f('messageId',mid)
              $m.fp('chat',$chat)
              $m.fp('sender',this.me())
              //console.log(['tgMessage$init.callback_query',$m.toObject()])
              this.obsEnd(obs,$m)
            },err=>{
              log.err('init msg fail')
              console.log(err)
              this.obsEnd(obs,null)
            })
          })
        },err=>this.obsEnd(obs,null))
        
      }
      if (mode=='default')
      {
        if (!mid)
          return this.obsEnd(obs,null)
        this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._chat.id()}]}).subscribe(m=>{
          if (m)
            return this.obsEnd(obs,m)
          this.C('tgmessage').initNode(ctx).subscribe($m=>{
            //console.log(['tgMessage$init.default',$m.toObject()])
            this.obsEnd(obs,$m)
          },err=>{
            log.err('init msg fail')
            console.log(err)
            this.obsEnd(obs,null)
          })
        })
      }
      if (mode=='reply')
      {
        //console.log(['updRply',upd,mid])
        //mid = upd.message.reply_to_message.message_id
        if (!mid)
          return this.obsEnd(obs,null)
        this.C('tgmessage').findNode({$and:[{messageId:mid},{chat:ctx._replyChat.id()}]}).subscribe(m=>{
          if (m)
            return this.obsEnd(obs,m)
          ctx._replyMode = true
          this.C('tgmessage').initNode(ctx).subscribe($m=>{
            //console.log(['tgMessage$init.reply',$m.toObject()])
            this.obsEnd(obs,$m)
          },err=>{
            log.err('init msg fail')
            console.log(err)
            this.obsEnd(obs,null)
          })
        })
        //log.err('REPLY MSG MODE INIT!!!')
        //console.log(upd,mid,mode)
        //this.obsEnd(obs,null)
      }
    })
  }

  addChat2Users(chat,users)
  {
    return Rx.Observable.create(obs=>{
      let $t = []
      users.forEach(u=>{
        if (chat.field('type').get()=='private')
          u.field('privateChat').set(chat)
        else
          u.field('publicChats').push(chat,{unique:true})
        $t.push(u.save())
        chat.field('users').push(u,{unique:true})
      })
      $t.push(chat.save())
      Rx.concat(...$t).subscribe(()=>{
        
      },err=>{
        if (err.code=='save'&&err.name=='VersionError')
          return this.obsEnd(obs)
        this.obsErr(obs,err)
        //obs.error(err)
      },()=>{
        this.obsEnd(obs)
      })
    })
  }

  updateBotToken(token)
  {
    return Rx.of(true)
    /*
    return Rx.Observable.create(obs=>{
      let current = this.M('config').conf('telegram.token')
      if (token==current)
        return this.obsEnd(obs,false)
      this._bot.reconfigure(token).subscribe(res=>this.obsEnd(obs,res))
    })
    /**/
  }

  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      log.log('init')
      this.C('tgchat').setNodeClass(ChatNode)
      this.C('tgmessage').setNodeClass(MessageNode)

      /**
      this._bot.configure(this.M('config').conf('telegram.token'),this.M('config').conf('telegram.username'))
      this._bot.init()
      //user-chat-type middleware
      this._bot.use((ctx,next)=>{
        setTimeout(()=>{
          let upd = ctx.update[ctx.updateType]
          //console.log(ctx.update)
          log.info('>>telegram mw')
          let updSubTypes = ctx.updateSubTypes

          let srcUser = upd.from
          let srcChat = upd.chat
          ctx._type = null
          if (ctx.updateType=='callback_query')
          {
            //console.log(upd)
            ctx._type = 'callback_query'
            srcChat = upd.message.chat
          }

          ctx._users = []
          ctx._chat = null
          let $t = []
          if (ctx.updateType!='channel_post')
            $t.push(this.tgUser(srcUser))

          //update type
          if (upd.text&&upd.text.length)
            ctx._type = 'text'

          if (!ctx._type&&upd.left_chat_member)
          {
            $t.push(this.tgUser(upd.left_chat_member))
            if (upd.left_chat_member.is_bot&&upd.left_chat_member.username==this._bot._username) {
              ctx._type = 'leave_group_chat'
            } else {
              if (srcUser.id==upd.left_chat_member.id) ctx._type = 'group_chat_member_leave'
              else ctx._type = 'group_chat_member_kicked'
            }
            $t = $t.reverse()
          }

          if (!ctx._type&&upd.new_chat_member)
          {
            $t.push(this.tgUser(upd.new_chat_member))
            if (upd.new_chat_member.is_bot&&upd.new_chat_member.username==this._bot._username) {
              ctx._type = 'enter_group_chat'
            } else {
              if (srcUser.id==upd.new_chat_member.id) ctx._type = 'group_chat_member_enter'
              else ctx._type = 'group_chat_member_added'
            }
            $t = $t.reverse()
          }

          Rx.concat(...$t).subscribe(u=>ctx._users.push(u),err=>{
            log.err('ctx users fail')
            console.log(err)
          },()=>{
            this.tgChat(srcChat).subscribe(chat=>{
              ctx._chat = chat
              if (ctx._type=='leave_group_chat')
                ctx._chat.field('open').set(false)
              if (ctx._type=='enter_group_chat')
              {
                ctx._chat.field('open').set(true)
                ctx._chat.field('counter').set(0)
              }
              //console.log(chat.toObject())
              let nx = chat.save()
              if (ctx.updateType!='channel_post')
                nx = this.addChat2Users(ctx._chat,ctx._users)
              nx.subscribe(()=>{
                this.tgMessage(upd,ctx,ctx._type=='callback_query'?'callback_query':'default').subscribe(msg=>{
                  ctx._msg = msg
                  //reply_to_message
                  if (!upd.reply_to_message)
                  {
                    next()
                    return
                  }
                  let rmsg = upd.reply_to_message
                  Rx.forkJoin(
                    this.tgUser(rmsg.from),
                    this.tgChat(rmsg.chat)
                    //this.tgMessage(rmsg,ctx,'reply')
                  ).subscribe(cmpls=>{
                    ctx._replyUser = cmpls[0]
                    ctx._replyChat = cmpls[1]
                    //ctx._replyMessage = cmpls[2]
                    this.tgMessage(rmsg,ctx,'reply').subscribe($rmsg=>{
                      ctx._replyMessage = $rmsg
                      next()
                    },err=>{
                      log.err('extract reply message error!')
                      console.log(err)
                      next()
                    })
                  },err=>{
                    log.err('extract reply error!')
                    console.log(err)
                    next()
                  })
                },err=>{ //tgMessage
                  log.err('ctx message fail')
                  console.log(err)
                })
              },err=>{ //addChat2Users
                log.err('ctx addChat2Users fail')
                console.log(err)
              }) 
            },err=>{ //tgChat
              log.err('ctx chat fail')
              console.log(err)
            })
          })
        },0)
      })
      //events connect/disconnect
      this._bot.signals.connected.subscribe(state=>{
        log.info('bot connection state',[state])
        if (!state)
          setTimeout(()=>{
            this._bot.start()
          },1000*7) //autorestart in 7 sec. why 7? why not? =)
      })
      //events incomes
      this._bot.on('message',(ctx)=>{
        setTimeout(()=>{
          let txt = ''
          if (ctx.update&&ctx.update.message&&ctx.update.message.text)
            txt = ctx.update.message.text
          log.info('>>message',txt)
          let CTX = this.M('context').newContext(this,'telegram')
          CTX.session(ctx.session)
          CTX.type(ctx._type)
          CTX.users(ctx._users)
          CTX.chats([ctx._chat])
          CTX.message(ctx._msg)
          CTX.raw(ctx.update)
          CTX.upd(ctx.update.message)
          if (ctx._replyUser)
            CTX.addNode(ctx._replyUser,'replyUser')
          if (ctx._replyChat)
            CTX.addNode(ctx._replyChat,'replyChat')
          if (ctx._replyMessage)
            CTX.addNode(ctx._replyMessage,'replyMessage')
          let tgn = new VNode(CTX)
          tgn.addField({code:'senderUser',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
          tgn.addField({code:'senderChat',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
          tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

          tgn.addField({code:'replyUser',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
          tgn.addField({code:'replyChat',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
          tgn.addField({code:'replyMessage',name:'Reply Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

          tgn.fp('senderUser',ctx._users[0])
          tgn.fp('senderChat',ctx._chat)
          tgn.fp('senderMessage',ctx._msg)
          if (ctx._replyUser)
            tgn.fp('replyUser',ctx._replyUser)
          if (ctx._replyChat)
            tgn.fp('replyChat',ctx._replyChat)
          if (ctx._replyMessage)
            tgn.fp('replyMessage',ctx._replyMessage)
          CTX.addNode(tgn,'telegram')

          this._router.onMessage(CTX)
        },0)
      })
      this._bot.on('callback_query',(ctx)=>{
        setTimeout(()=>{
          let q = null
          if (ctx&&ctx.update&&ctx.update.callback_query)
            q = ctx.update.callback_query.data
          log.info('>>callback_query',q)
          //console.log(ctx._msg,ctx.update)
          //console.log(['_msg',ctx._msg])
          let CTX = this.M('context').newContext(this,'telegram')
          //dirty hack
          CTX._rawctx = ctx
          CTX.session(ctx.session)
          CTX.type(ctx._type)
          CTX.users(ctx._users)
          CTX.chats([ctx._chat])
          CTX.message(ctx._msg)
          CTX.raw(ctx.update)
          CTX.upd(ctx.update.callback_query)

          let tgn = new VNode(CTX)
          tgn.addField({code:'senderUser',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
          tgn.addField({code:'senderChat',name:'Sender User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
          tgn.addField({code:'senderMessage',name:'Sender Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

          tgn.addField({code:'replyUser',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'user'}]}})
          tgn.addField({code:'replyChat',name:'Reply User',type:'relation',conf:{ops:[{target:'manual',manual:'tgchat'}]}})
          tgn.addField({code:'replyMessage',name:'Reply Message',type:'relation',conf:{ops:[{target:'manual',manual:'tgmessage'}]}})

          tgn.fp('senderUser',ctx._users[0])
          tgn.fp('senderChat',ctx._chat)
          tgn.fp('senderMessage',ctx._msg)

          CTX.addNode(tgn,'telegram')
          this._router.onCallbackQuery(CTX)
        },0)
      })
      //router
      this._router.init()
      /**/
      log.log('initialized')
      obs.next()
      obs.complete()
    })
  }

  /**
  moduleStart()
  {
    return Rx.Observable.create(obs=>{
      if (!Threads.isMainThread)
      {
        log.err('telegram bot can be started only in main thread')
        return this.obsEnd(obs)
      }
      log.log('starting')
      this._bot.start()
      this._bot.loadProfile().subscribe(()=>{
        //console.log(this._bot.profile().toObject())
        log.log('started')
        obs.next()
        obs.complete()
      },err=>{
        log.err('start FAIL')
        this.obsErr(obs,err)
      })
    })
  }
  /**/

  moduleInstall()
  {
    return Rx.Observable.create(obs=>{
      log.info('installing')
      let $t = []

      let bndlChat = this._core.module('bundle').bundle('tgchat',true)
      bndlChat.name(g18n.t('Chat'))
      bndlChat.desc(g18n.t('Telegram chats'))
      bndlChat.flags(['system'])

      bndlChat.addField({name:g18n.t('Chat ID'),code:'chatId',type:'string',flags:['system','required','logic']})
      bndlChat.addField({name:g18n.t('Title'),code:'title',type:'string',flags:['system','required','logic','title']})
      bndlChat.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','required','logic'],def:'private'})
      bndlChat.addField({name:g18n.t('Username'),code:'username',type:'string',flags:['system','logic','title']})
      bndlChat.addField({name:g18n.t('Open'),code:'open',type:'boolean',flags:['system','logic'],def:true})
      bndlChat.addField({name:g18n.t('History'),code:'history',type:'boolean',flags:['system','logic'],def:true})
      bndlChat.addField({name:g18n.t('Created'),code:'created',type:'datetime',flags:['system','logic'],generator:'now'})
      bndlChat.addField({name:g18n.t('Last message'),code:'lastMessage',type:'datetime',flags:['system','logic'],def:null})
      bndlChat.addField({name:g18n.t('Last close'),code:'lastClose',type:'datetime',flags:['system','logic'],def:null})
      bndlChat.addField({name:g18n.t('Memory'),code:'memory',type:'number',flags:['system','logic'],def:10})
      bndlChat.addField({name:g18n.t('Counter'),code:'counter',type:'number',flags:['system','logic'],def:0})
      bndlChat.addField({name:g18n.t('Total counter'),code:'totalCounter',type:'number',flags:['system','logic'],def:0})
      bndlChat.addField({name:g18n.t('Priority'),code:'priority',type:'number',flags:['system','logic'],def:4})
      bndlChat.addField({name:g18n.t('Conf'),code:'conf',type:'any',flags:['system','logic'],def:{
        'ga:on': true,
        'ga:prefix': '',
        'ga:suffix': '',
        'request:any': true,
        'request:username': false,
        'request:reply': true,
        'answer:keyboards': true,
        'answer:reply': '',
        'answer:max': 4000,
        'answer:mixx': '',
        'answer:delay': 1
      }})
      bndlChat.addField({name:g18n.t('Messages'),code:'messages',type:'relation',conf:{ops:[{source:'manual',manual:'tgmessage'}]},flags:['system','logic'],quantity:0})
      bndlChat.addField({name:g18n.t('Allow'),code:'allow',type:'relation',conf:{ops:[{source:'manual',manual:'screengroup'}]},flags:['system','logic'],quantity:0})
      bndlChat.addField({name:g18n.t('Deny'),code:'deny',type:'relation',conf:{ops:[{source:'manual',manual:'screengroup'}]},flags:['system','logic'],quantity:0})
      bndlChat.addField({name:g18n.t('Users'),code:'users',type:'relation',conf:{ops:[{source:'manual',manual:'user'}]},flags:['system','logic'],quantity:0})
      bndlChat.addField({name:g18n.t('Administrators'),code:'admins',type:'relation',conf:{ops:[{source:'manual',manual:'user'}]},flags:['system','logic'],quantity:0})

      log.info('saving bundle',bndlChat.code())
      $t.push(bndlChat.save(true,true))


      /*
      let bndlFiles = this._core.module('bundle').bundle('file',true)
      bndlFiles.name(g18n.t('Telegram file'))
      bndlFiles.desc(g18n.t('Telegram files'))
      bndlFiles.flags(['system'])

      let bndlFiles = this._core.module('bundle').bundle('file',true)
      bndlFiles.addField({name:g18n.t('File ID'),code:'fileId',type:'string',flags:['system','required','logic']})
      bndlFiles.addField({name:g18n.t('File type'),code:'type',type:'string',flags:['system','required','logic']})
      bndlFiles.addField({name:g18n.t('Local path'),code:'localPath',type:'string',flags:['system','logic']})

      log.info('saving bundle',bndlFiles.code())
      $t.push(bndlFiles.save(true,true))
      /**/

      let bndlMessages = this._core.module('bundle').bundle('tgmessage',true)
      bndlMessages.name(g18n.t('Telegram messages'))
      bndlMessages.desc(g18n.t('Telegram messages'))
      bndlMessages.flags(['system','locked'])

      bndlMessages.addField({name:g18n.t('Message ID'),code:'messageId',type:'string',flags:['system','required','logic','title']})
      bndlMessages.addField({name:g18n.t('Sender'),code:'sender',type:'relation',conf:{ops:[{source:'manual',manual:'user'}]},flags:['system','logic','template']})
      bndlMessages.addField({name:g18n.t('Submessage'),code:'submessage',type:'relation',conf:{ops:[{source:'manual',manual:'tgmessage'}]},flags:['system','logic','template'],def:null})
      bndlMessages.addField({name:g18n.t('Chat'),code:'chat',type:'relation',conf:{ops:[{source:'manual',manual:'tgchat'}]},flags:['system','required','logic']})
      bndlMessages.addField({name:g18n.t('Flags'),code:'flags',type:'string',flags:['system','logic'],quantity:0})

      bndlMessages.addField({name:g18n.t('Users'),code:'users',type:'relation',conf:{ops:[{source:'manual',manual:'user'}]},flags:['system','logic'],quantity:0})
      bndlMessages.addField({name:g18n.t('Type'),code:'type',type:'string',flags:['system','required','logic']})
      bndlMessages.addField({name:g18n.t('Date'),code:'date',type:'datetime',flags:['system','logic']})
      bndlMessages.addField({name:g18n.t('Raw'),code:'raw',type:'any',flags:['system']})
      bndlMessages.addField({name:g18n.t('Screen'),code:'screen',type:'relation',conf:{ops:[{source:'manual',manual:'screen'}]},flags:['system'],def:null})
      bndlMessages.addField({name:g18n.t('Hashes'),code:'hashes',type:'string',flags:['system','logic'],quantity:0})

      //message data 
      bndlMessages.addField({name:g18n.t('Text'),code:'text',type:'string',flags:['system','logic'],def:null})
      bndlMessages.addField({name:g18n.t('Keyboard'),code:'keyboard',type:'any',flags:['system'],def:null})
      bndlMessages.addField({name:g18n.t('Inline'),code:'inline',type:'any',flags:['system'],def:null})
      bndlMessages.addField({name:g18n.t('Media'),code:'media',type:'relation',conf:{ops:[{source:'manual',manual:'file'}]},flags:['system','logic'],quantity:0})
      bndlMessages.addField({name:g18n.t('Files'),code:'files',type:'relation',conf:{ops:[{source:'manual',manual:'file'}]},flags:['system','logic'],quantity:0})
      bndlMessages.addField({name:g18n.t('Sticker'),code:'sticker',type:'any',flags:['system'],def:null})
      bndlMessages.addField({name:g18n.t('Location'),code:'location',type:'any',flags:['system','logic'],def:null})
      bndlMessages.addField({name:g18n.t('Contact'),code:'contact',type:'any',flags:['system','logic'],def:null})

      log.info('saving bundle',bndlMessages.code())
      $t.push(bndlMessages.save(true,true))


      log.info('modifying user bundle')
      let bndUser = this._core.module('bundle').bundle('user')
      bndUser.addField({name:g18n.t('TgID'),code:'tgid',type:'string',flags:['system','logic','locked','template'],generator:'tg:id'})
      bndUser.addField({name:g18n.t('First name'),code:'firstName',type:'string',flags:['system','logic','locked','template','title'],generator:'tg:firstName'})
      bndUser.addField({name:g18n.t('Last name'),code:'lastName',type:'string',flags:['system','logic','locked','template'],generator:'tg:lastName'})
      bndUser.addField({name:g18n.t('Is Bot'),code:'isBot',type:'boolean',flags:['system','logic','locked','template'],generator:'tg:isBot',def:false})
      bndUser.addField({name:g18n.t('Private chat'),code:'privateChat',type:'relation',conf:{ops:[{source:'manual',manual:'tgchat'}]},flags:['system','logic','locked','template']})
      bndUser.addField({name:g18n.t('Public chats'),code:'publicChats',type:'relation',conf:{ops:[{source:'manual',manual:'tgchat'}]},flags:['system','logic','locked','template'],quantity:0})

      log.info('saving bundle',bndUser.code())
      $t.push(bndUser.save())
      
      log.info('modifying file bundle')
      let bndlFiles = this._core.module('bundle').bundle('file')
      bndlFiles.addField({name:g18n.t('File Telegram ID'),code:'tgid',type:'string',flags:['system','logic','title']})
      bndlFiles.addField({name:g18n.t('Telegram Info'),code:'tginfo',type:'any',flags:['system','logic']})

      log.info('saving bundle',bndlFiles.code())
      $t.push(bndlFiles.save())

      Rx.concat(...$t).subscribe(()=>{},err=>{
        log.err('installing telegram failed')
        console.log(err)
        obs.error(err)
      },()=>{
        log.success('installed')
        obs.next()
        obs.complete()
      })
    })
  }

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let install = {}
      let $t = []
      $t.push(this.M('core').consolePrompt(g18n.t('Telegram token []:')).pipe(RxO.map(val=>{install['telegram.token']=val})))
      $t.push(this.M('core').consolePrompt(g18n.t('Telegram username []:')).pipe(RxO.map(val=>{install['telegram.username']=val})))
      $t.push(this.M('core').consolePrompt(g18n.t('Google Analytics ID []:'),{default:''}).pipe(RxO.map(val=>{install['ga.id']=val})))
      $t.push(this.M('core').consolePrompt(g18n.t('Google Analytics app []:'),{default:''}).pipe(RxO.map(val=>{install['ga.app']=val})))

      Rx.concat(...$t).subscribe(()=>{},err=>this.obsErr(obs,err),_=>{
        let $tt = []
        //$tt.push(this.M('config').setConf('telegram.token',install['telegram.token'],'Telegram bot token',['system']))
        //$tt.push(this.M('config').setConf('telegram.username',install['telegram.username'],'Telegram bot username',['system','locked']))
        $tt.push(this._update011to012({
          token: install['telegram.token'],
          username: install['telegram.username']
        }))
        $tt.push(this.M('config').setConf('ga.id',install['ga.id'],'Google Analytics ID'))
        $tt.push(this.M('config').setConf('ga.app',install['ga.app'],'Google Analytics app'))

        Rx.concat(...$tt).subscribe(()=>{},err=>this.obsErr(obs,err),__=>{
          this.obsEnd(obs)
        })
      })

    })
  }

  _update01to011()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      log.info('updating tgmessage bundle')
      let bndlMessages = this._core.module('bundle').bundle('tgmessage')
      bndlMessages.addField({name:g18n.t('Sender'),code:'sender',type:'relation',conf:{ops:[{source:'manual',manual:'user'}]},flags:['system','logic']})
      bndlMessages.addField({name:g18n.t('Flags'),code:'flags',type:'string',flags:['system','logic'],quantity:0})
      bndlMessages.addField({name:g18n.t('Hashes'),code:'hashes',type:'string',flags:['system','required','logic'],quantity:0})
      bndlMessages.addField({name:g18n.t('Submessage'),code:'submessage',type:'relation',conf:{ops:[{source:'manual',manual:'tgmessage'}]},flags:['system','logic','template'],def:null})
      bndlMessages.updField({name:g18n.t('Raw'),code:'raw',type:'any',flags:['system']})

      log.info('saving bundle',bndlMessages.code())
      $t.push(bndlMessages.save())

      Rx.concat(...$t).subscribe(()=>{},err=>this.obsErr(obs,err),__=>{
        this.obsEnd(obs)
      })
    })
  }

  //also used at installer with installation process input config
  _update011to012(conf=null)
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      let defRouter = this.C('ifrouters').newNode()
      defRouter.f('name',g18n.t('Default telegram router'))
      defRouter.f('code','tgdefault')
      defRouter.f('type','telegrambotrouter')

      $t.push(this.C('ifrouters').findNode({code:'tgdefault'}).pipe(
        RxO.mergeMap(x=>{
          if (!x)
            return defRouter.save()
          return Rx.of(x)
        })
      ))

      let defIface = this.C('interfaces').newNode()
      defIface.f('enabled',true)
      defIface.f('status','unknown')
      defIface.f('name',g18n.t('Default telegram bot'))
      defIface.f('code','defbot')
      defIface.f('type','telegrambot')
      defIface.f('router','tgdefault')
      if (conf)
        defIface.f('config',conf)
      else
        defIface.f('config',{
          username: this.M('config').conf('telegram.username'),
          token: this.M('config').conf('telegram.token')
        })

      $t.push(defIface.save())

      log.info('moving default telegram bot to interface')
      Rx.concat(...$t).pipe(RxO.last()).subscribe(_=>this.obsEnd(obs))
    })
  }

  moduleUpdate(from,to)
  {
    return Rx.of(1).pipe(
      RxO.mergeMap(_=>{
        if (from=='0.1')
          return Rx.concat(this._update01to011(),this._update011to012())
        if (from=='0.1.1')
          return this._update011to012()
        return Rx.of(1)
      }),
      RxO.map(_=>to)
    )
  }
 



  dashboardRoutes(app)
  {
    return Rx.Observable.create(obs=>{

      let rtList = {
        url: '/api/chats/:category',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let query = {}
            let p = ctx.f('params')
            let cat = p.category || ''
            if (cat == 'public')
              query = { $or:[{type:'group'},{type:'supergroup'},{type:'channel'}]}
            //log.warn('b4 query')
            this.C('tgchat').findNodes(query).subscribe($list=>{
              //log.warn('a4 query')
              let ret = $list.map(o=>o.toObject())
              this.obsEnd(obs,ret)
            },err=>{
              log.err('fail loading roles')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }

      let rtOne = {
        url: '/api/chat/:chatId',
        access: 'auth.root',
        method: 'get',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let query = {}
            let p = ctx.f('params')
            let cat = p.chatId || ''
            this.C('tgchat').findNode({_id:cat}).subscribe($list=>{
              if (!$list)
                return this.obsEnd(obs,null)
              this.obsEnd(obs,$list.toObject())
            },err=>{
              log.err('fail loading roles')
              console.log(err)
              this.obsErr(obs,err)
            })
          })
        }
      }

      let rtSave = {
        url: '/api/chat/save',
        access: 'auth.root',
        method: 'post',
        fn: (ctx)=>{
          return Rx.Observable.create(obs=>{
            let query = {}
            let p = ctx.f('body')
            this.C('tgchat').findNode({_id:p.id}).subscribe($chat=>{
              if (!$chat)
              {
                log.err('search chat error')
                console.log(err)
                return this.obsEnd(obs,{success:false})
              }
              //console.log(p)
              if ($chat.f('type')=='private'&&p.conf['answer:delay']<1)
                p.conf['answer:delay'] = 1
              if ($chat.f('type')!='private'&&p.conf['answer:delay']<3)
                p.conf['answer:delay'] = 3
              $chat.f('conf',p.conf)
              $chat.f('priority',p.priority)
              $chat.f('memory',p.memory)
              $chat.f('history',p.history)
              $chat.field('allow').clear()
              p.allow.forEach(a=>$chat.field('allow').push(a))
              $chat.field('deny').clear()
              p.deny.forEach(a=>$chat.field('deny').push(a))
              $chat.save().subscribe(c=>{
                log.info('chat updated',c.f('title'))
                this.obsEnd(obs,{success:true})
              },err=>{
                log.err('chat save error')
                console.log(err)
                this.obsEnd(obs,{success:false})
              })
            },err=>{
              log.err('search chat error')
              console.log(err)
              this.obsEnd(obs,{success:false})
            })
            /*
            let cat = p.category || ''
            if (cat == 'public')
              query = { $or:[{type:'group'},{type:'supergroup'}]}
            this.C('tgchat').findNodes(query).subscribe($list=>{
              let ret = $list.map(o=>o.toObject())
              this.obsEnd(obs,ret)
            },err=>{
              log.err('fail loading roles')
              console.log(err)
              this.obsErr(obs,err)
            })
            /**/
          })
        }
      }

      obs.next([rtOne,rtSave,rtList])
      obs.complete()
    })
  }




}

module.exports = TelegramModule
