let Rx = require('rxjs')
let RxO = require('rxjs/operators')

const TJS = require('../includes')

let log = TJS.log.m('TutorialsModule')
let g18n = TJS.g18n

let TTField = require('./testfield')
let TestReaction = require('./testreaction')
let TestMsg = require('./testmsg')

class TutorialsModule extends TJS.CoreModule
{
  
  constructor(parent)
  {
    super(parent)
  }

  /**/
  moduleLoad()
  {
    return Rx.Observable.create(obs=>{
      this.M('field').registerFieldType(TTField)
      this.M('bot/reactions').registerReactionType(TestReaction)
      this.M('actions').registerAction(TestReaction)
      this.M('bot/messages').registerMessageType('tutorialtest',TestMsg)
      //log.test('loading')
      //this.moduleInstall().subscribe(_=>{
        this.obsEnd(obs)
      //})
    })
  }
  /**

  moduleInit()
  {
    return this.moduleInstall()
  }
  /**/

  moduleInstall()
  {

    return Rx.Observable.create(obs=>{
      let $t = []

      //lessons data bundle
      let bndLesson = this.M('bundle').bundle('tutorial_test',true)
      bndLesson.name(g18n.t('Tutorial test'))
      bndLesson.desc(g18n.t('Tutorial test contains some brief data with tests'))
      bndLesson.addFlag('system')
      bndLesson.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system','logic','title','template'],def:true})
      bndLesson.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','template','logic']})
      bndLesson.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','template','logic','title']})
      bndLesson.addField({name:g18n.t('Description'),code:'desc',type:'string',flags:['system','template','logic'],conf:{ops:[{source:'manual',manual:true}]}})
      bndLesson.addField({name:g18n.t('Brief'),code:'brief',type:'string',flags:['system','template','logic'],conf:{ops:[{source:'manual',manual:true}]}})
      bndLesson.addField({name:g18n.t('Access logic'),code:'access_logic',type:'logicfilter',flags:['system','logic']})
      bndLesson.addField({name:g18n.t('Test'),code:'testdata',type:'tutorialtest',flags:['system','logic','template']})
      bndLesson.addField({name:g18n.t('Users data'),code:'usersdata',type:'any',flags:['system','logic','template'],def:null})


      log.log('installing bundle',bndLesson.code())
      $t.push(bndLesson.save(true,true))


      /*
      //course data bundle
      let bndCourse = this.M('bundle').bundle('tutorial_course',true)
      bndCourse.name(g18n.t('Tutorial course'))
      bndCourse.desc(g18n.t('Tutorial course contains lessons with tests'))
      bndCourse.addFlag('system')
      bndCourse.addField({name:g18n.t('Enabled'),code:'enabled',type:'boolean',flags:['system','logic','title','template'],def:true})
      bndCourse.addField({name:g18n.t('Code'),code:'code',type:'string',flags:['system','template','logic']})
      bndCourse.addField({name:g18n.t('Name'),code:'name',type:'string',flags:['system','template','logic','title']})
      bndCourse.addField({name:g18n.t('Description'),code:'desc',type:'string',flags:['system','template','logic'],conf:{ops:[{source:'manual',manual:true}]}})
      bndCourse.addField({name:g18n.t('Brief'),code:'brief',type:'string',flags:['system','template','logic'],conf:{ops:[{source:'manual',manual:true}]}})
      bndCourse.addField({name:g18n.t('Access logic'),code:'access_logic',type:'logicfilter',flags:['system','logic']})
      bndCourse.addField({name:g18n.t('Lessons'),code:'lessons',type:'relation',flags:['system','template','logic'],conf:{ops:[{source:'manual',manual:'tutorial_lesson'}]},quantity:0})
      bndCourse.addField({name:g18n.t('Users data'),code:'usersdata',type:'any',flags:['system','logic','template'],def:null})

      log.log('installing bundle',bndCourse.code())
      $t.push(bndCourse.save(true,true))
      /**/

      //modify user bundle
      let bndUser = this._core.module('bundle').bundle('user')
      if(!bndUser.hasField('testsData'))
      {
        bndUser.addField({name:g18n.t('Tutorial tests data'),code:'testsData',type:'any', quantity:1, flags:['system','logic','template']})
        log.log('modifying user bundle')
        $t.push(bndUser.save())
      }

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        log.success('bundle installed')
        this.obsEnd(obs)
      })
 
    })
  }

  /*
  moduleInit()
  {
    return Rx.Observable.create(obs=>{
      this.C('http_blocks').setNodeClassFn(mdl=>{
        let type = mdl.type[0]
        if (this._classes[type])
          return this._classes[type]
        return AbstractHttpBlock
      })
      this.obsEnd(obs)
    })
  }
  /**/

  moduleDefaults()
  {
    return Rx.Observable.create(obs=>{
      let $t = []

      let contentBlock = this.C('http_blocks').newNode()
      contentBlock.f('enabled',true)
      contentBlock.f('code','content')
      contentBlock.f('name','Content')
      contentBlock.f('type','content')
      contentBlock.f('title','<none>')
      contentBlock.field('flags').all(['system'])
      $t.push(contentBlock.save())

      let tjsBlock = this.C('http_blocks').newNode()
      tjsBlock.f('enabled',true)
      tjsBlock.f('code','buildedwith')
      tjsBlock.f('name','Builded with')
      tjsBlock.f('type','text')
      tjsBlock.f('title','<none>')
      tjsBlock.f('data',{
        content: 'Builded with takatanJS'
      })
      $t.push(tjsBlock.save())

      Rx.concat(...$t).subscribe(()=>{},obs.error,()=>{
        log.success('default blocks installed')
        this.obsEnd(obs)
      })
    })
  }


}

module.exports = TutorialsModule
