let Rx = require('rxjs')
let RxO = require('rxjs/operators')

const TJS = require('../includes')

let log = TJS.log.m('TutorialTestsReaction')
let g18n = TJS.g18n

class TutorialTestReaction extends TJS.AbstractReaction
{
  
  constructor(...args)
  {
    super(...args)
  }

  static info()
  {
    return {
      code: 'tutorialtest',
      name: g18n.t('Tutorial test'),
      shortname: g18n.t('Test'),
      faicon: 'far fa-question-circle',
      uicolor: 'text-primary',
      uigroup: 'tutorials',
      routersLimit: ['telegrambotrouter'],
      mode: 'message', //message/reaction/any
      greedy: true,
      conflicts: ['*'],
      requireSend: false,
      doSend: false,
      confs: [{
        name: g18n.t('test'),
        code: 'target',
        desc: g18n.t('test'),
        allowManual: false,
        allowGenerators: false,
        type: 'relation',
        xtype: 'relation:tutorial_lesson'
      },{
        name: g18n.t('mode (inline/keyboard, inline default)'),
        code: 'mode',
        desc: g18n.t('mode (inline/keyboard, inline default)'),
        allowManual: true,
        allowGenerators: true,
        type: 'string'
      },{
        name: g18n.t('buttons per row (1 default)'),
        code: 'kbdrowsize',
        desc: g18n.t('buttons per row (1 default)'),
        allowManual: true,
        allowGenerators: true,
        type: 'number'
      },{
        name: g18n.t('keep history for inline (no default) NOT WORKS IN BETA'),
        code: 'history',
        desc: g18n.t('keep history for inline (no default) NOT WORKS IN BETA'),
        allowManual: true,
        allowGenerators: true,
        type: 'boolean'
      },{
        name: g18n.t("text in inline mode keep same question (no default)"),
        code: 'noinlinetext',
        desc: g18n.t('text in inline mode keep same question (no default)'),
        allowManual: true,
        allowGenerators: true,
        type: 'boolean'
      }],
      context: [{
        code: 'tutorialtest',
        name: g18n.t('Test status'),
        fields: [{
          code: 'state',
          name: g18n.t('Status'),
          desc: g18n.t('open/success/fail'),
          type: 'string'
        },{
          code: 'textOnInline',
          name: g18n.t('Text on inline'),
          desc: g18n.t('test uses inline mode but user send text'),
          type: 'boolean'
        },{
          code: 'qsend',
          name: g18n.t('Sended question number'),
          desc: g18n.t('starts from zero!'),
          type: 'number'
        },{
          code: 'qwait',
          name: g18n.t('Answered question number'),
          desc: g18n.t('starts from zero!'),
          type: 'number'
        },{
          code: 'araw',
          name: g18n.t('Answer'),
          desc: g18n.t('users answer'),
          type: 'string'
        },{
          code: 'ascore',
          name: g18n.t('Answer score'),
          desc: g18n.t('users answer score'),
          type: 'number'
        },{
          code: 'tscore',
          name: g18n.t('Final test score'),
          desc: g18n.t('exists when test completed'),
          type: 'number'
        },{
          code: 'trang',
          name: g18n.t('Final users rang'),
          desc: g18n.t('exists when test completed'),
          type: 'string'
        }]
      }]
    }
  }

  execute(ctx,$msg)
  {
    return this.loadConfs(ctx).pipe(RxO.mergeMap(cfg=>{
      if (typeof cfg.history != 'boolean')
        cfg.history = false

      if (typeof cfg.noinlinetext != 'boolean')
        cfg.noinlinetext = false

      cfg.kbdrowsize = parseInt(cfg.kbdrowsize)
      if (typeof cfg.kbdrowsize != 'number' || cfg.kbdrowsize < 2)
        cfg.kbdrowsize = 1

      cfg.ok = false
      if (!(cfg.target instanceof TJS.Node))
        return Rx.of(cfg)

      if (cfg.target.bundle()!='tutorial_lesson')
        return Rx.of(cfg)

      if (!cfg.target.f('enabled'))
        return Rx.of(cfg)

      ctx.addNode(cfg.target,'tutorial_lesson')
      return this.M('logic').testLogicDirectRx(cfg.target.f('access_logic'),ctx).pipe(RxO.map(res=>{
        cfg.ok = res
        return cfg
      }))
    }))
  }

}

module.exports = TutorialTestReaction
